ifeq ($(CSPICE_DIR),)
CSPICE_DIR=/usr/local/naif/cspice
endif

ifeq ($(CSPICE_LIB),)
SPICE_LIB = $(CSPICE_DIR)/lib/cspice.a
endif

export BUILD_DIR
export INST_NAT_LIB
export INST_EXT_LIB
export INST_INC

# Source Files ###############################################################

HDR_IN = JWavesSDSblk.h JWavesRdrBlk.h JWavesUsegBlk.h JWavesDcmpBlk.h \
 JWavesMP.h JWavesIDP.h JWavesStore.h JWavesProc.h JWaves.h JWavesDefs.h \
 JWavesExcept.h JWavesConv.h JWavesData.h JWavesLRSblk.h JWavesHRSblk.h \
 JWavesUpkBlk.h JWavesPkt.h JWavesUpkFilter.h JWavesUsegFilter.h \
 JWavesDcmpFilter.h JWavesFDMblk.h JWavesCCSDSblk.h JWavesFilter.h \
 JWavesSideBlk.h JWavesSideFilter.h JWavesHFWBRbands.h JWavesSciMP.h \
 JWavesMroMP.h JWavesMroBlk.h JWavesHrbhBlk.h JWavesSfduBlk.h \
 JWavesChanMgr.h

SRC_IN = JWavesSDSblk.c JWavesRdrBlk.c JWavesUsegBlk.c JWavesDcmpBlk.c \
 JWavesPkt.c JWavesMP.c JWavesIDP.c JWavesStore.c JWavesData.c JWavesConv.c \
 JWavesLRSblk.c JWavesHRSblk.c JWavesUpkBlk.c JWavesUpkFilter.c \
 JWavesUsegFilter.c JWavesDcmpFilter.c JWavesFDMblk.c JWavesCCSDSblk.c \
 JWavesFilter.c JWavesSideBlk.c JWavesSideFilter.c JWavesHFWBRbands.c \
 JWavesSciMP.c JWavesMroMP.c JWavesMroBlk.c JWavesHrbhBlk.c JWavesSfduBlk.c \
 JWavesChanMgr.c

# Source for the PPD plugin 
PPD_SRC = JWavesPPD.c

PY_IN=jwaves_util.py

SWIG_BUILD_IN = JWaves.i
SWIG_DEFS_IN = JWaves.i JWavesMultiFileSrc.i JWavesDefs.i JWavesRdrBlk.i \
 JWavesMroBlk.i
 
SWIG_HPP_IN = JWavesRdrBlk.hpp JWavesMroBlk.hpp
	 
TEST_IN = test_9bitflt test_getxydat

DOC_IN = telemetry_ref.odg waves_gs_fmts_stor.txt

# Flags ######################################################################

# THREAD_DEF is defined by some of the rules below, this just in time 
#     evaluation of variables is a useful, if odd feature of gnu make

DEFINES = -DLINUX -DCFG=$(INST_ETC)
INCFLAGS = -I. -I$(INST_INC)
WARNFLAGS = -Wall -Werror

CFLAGS = -fexceptions -rdynamic -fPIC -ggdb -std=c99 \
 $(INCFLAGS) $(WARNFLAGS) $(DEFINES) $(TH_CFLAGS)
 
CC=gcc

#LFLAGS = -lefence -L$(INST_NAT_LIB) $(TH_LFLAGS) -ldl -lm
LFLAGS = -L$(INST_NAT_LIB) $(TH_LFLAGS) -ldl -lm

#############################################################################

HDRS =         $(patsubst %,jwaves/%,$(HDR_IN))
INSTALL_HDRS = $(patsubst %,$(INST_INC)/jwaves/%,$(HDR_IN))

SRCS = $(patsubst %.c,jwaves/%.c,$(SRC_IN))
SWIG_SRC = $(patsubst %.i,$(BUILD_DIR)/%_wrap.cpp,$(SWIG_BUILD_IN))

SWIG_DEFS = $(patsubst %.i,jwaves/%.i,$(SWIG_DEFS_IN)) \
 $(patsubst %.hpp,jwaves/%.hpp,$(SWIG_HPP_IN)) 

INSTALL_SWIG = $(patsubst %.i,$(INST_INC)/jwaves/%.i,$(SWIG_DEFS_IN)) \
 $(patsubst %.hpp,$(INST_INC)/jwaves/%.hpp,$(SWIG_HPP_IN)) 

PY_SRCS = $(patsubst %.py,jwaves/%.py,$(PY_IN))
INSTALL_PYSRCS = $(patsubst %.py,$(INST_HOST_LIB)/%.py,$(PY_IN))

TEST_SRCS = $(patsubst %,test/%.c,$(TEST_IN))
OBJS = $(patsubst jwaves/%.c,$(BUILD_DIR)/%.o,$(SRCS))

TEST_PROGS = $(patsubst %,$(BUILD_DIR)/%,$(TEST_IN))

INSTALL_DOCS = $(patsubst %,$(INST_DOC)/%,$(DOC_IN))

# Pattern Rules #############################################################
.SUFFIXES:
.SUFFIXES: .c .o .h .py

# Pattern rule for the test executables
$(BUILD_DIR)/%:test/%.c
	$(CC) $(CFLAGS) $(LFLAGS) -o $@ $< $(BUILD_DIR)/libjwaves$(TH_SUF).a 

# Pattern rule for the library object files
$(BUILD_DIR)/%.o:jwaves/%.c | $(BUILD_DIR)
	$(CC) -c $(CFLAGS) -o $@ $<
	
# Pattern rule for generating python module C code from .i files
$(BUILD_DIR)/%_wrap.cpp $(BUILD_DIR)/%.py:jwaves/%.i $(SWIG_DEFS)
	swig -Wall -dump_classes -c++ -python -nodefaultctor -I. -I$(INST_INC) -o $(BUILD_DIR)/$*_wrap.cpp $< 
	swig -python -external-runtime $(BUILD_DIR)/swigpyrun.h

# Pattern rule for building shared objects
$(BUILD_DIR)/%.so:$(OBJS)
	gcc -rdynamic -ggdb -shared -o $@ $(OBJS) $(LFLAGS)

# Pattern rule for building archive files
$(BUILD_DIR)/%.a:$(OBJS)
	ar -crs $@ $(OBJS)

# Pattern rule for installing python extension libs from the builddir
$(INST_EXT_LIB)/%.so:$(BUILD_DIR)/%.so
	install -D -m 775 $< $@

# Pattern rule for installing python extension lib wrappers from the builddir
$(INST_EXT_LIB)/%.py:$(BUILD_DIR)/%.py
	install -D -m 664 $< $@
	
# Pattern rule for installing pure python code from the source dir
$(INST_HOST_LIB)/%.py:jwaves/%.py
	install -D -m 664 $< $@
	
# Pattern rule for installing files in the juno library location  
$(INST_NAT_LIB)/%:$(BUILD_DIR)/%
	install -D -m 664 $< $@

# Pattern rule for installing PPD plugin libraries
$(INST_NAT_LIB)/ppd/%.so:$(BUILD_DIR)/%.so
	install -D -m 775 $< $@

# Pattern rule for installing misc docs	
$(INST_DOC)/%:doc/%
	install -D -m 664 $< $@

# Pattern rule for installing header files
$(INST_INC)/jwaves/%:jwaves/%
	install -D -m 664 $< $@
	


# Explicit Rules #############################################################

# Direct make not to nuke the intermediate .o files
.SECONDARY: $(OBJS)

build:TH_LFLAGS= -lrpwgse_t -lfftw3 -lpthread
build:TH_CFLAGS=-DGSE_POSIX_THREADS
build:TH_SUF=_t

build:$(BUILD_DIR)/libjwaves_t.a $(TEST_PROGS) $(BUILD_DIR)/ppd_jwaves.so | $(BUILD_DIR)

$(BUILD_DIR):
	@if [ ! -e "$(BUILD_DIR)" ]; then mkdir $(BUILD_DIR); fi

nothreads:TH_LFLAGS=-lrpwgse -libfftw3
nothreads: $(BUILD_DIR)/libjwaves.a $(TEST_PROGS) $(BUILD_DIR)/ppd_jwaves.so | $(BUILD_DIR)
 

$(BUILD_DIR)/ppd_jwaves.so:$(BUILD_DIR)/libjwaves_t.a $(BUILD_DIR)/JWavesPPD.o 
	gcc -rdynamic -ggdb -shared $(LFLAGS) -o $@ $(BUILD_DIR)/JWavesPPD.o $(BUILD_DIR)/libjwaves_t.a

install_nothreads: $(INST_NAT_LIB)/libjwaves.a $(INST_INC)/jwaves $(INSTALL_HDRS)
	
install:build $(INST_NAT_LIB)/libjwaves_t.a $(INSTALL_HDRS) $(INST_NAT_LIB)/ppd/ppd_jwaves.so


$(BUILD_DIR)/_jwaves.so:$(HDRS) $(SWIG_SRC)
	python setup.py build_ext --inplace
	mv _jwaves.so $@

pylib:$(BUILD_DIR) install $(BUILD_DIR)/_jwaves.so $(BUILD_DIR)/jwaves.py $(PY_SRCS)

pylib_install: pylib $(INST_EXT_LIB)/_jwaves.so $(INST_EXT_LIB)/jwaves.py $(INSTALL_PYSRCS)

clean:
	rm $(OBJS)  $(TEST_PROGS)

distclean:
	rm -r -f $(BUILD_DIR) build
	
	
# Documentation Rules ########################################################
doc_install: html $(INSTALL_DOCS)
	if [ ! -e $(INST_DOC)/jwaves ]; then mkdir -p $(INST_DOC)/jwaves; fi
	cp -r html $(INST_DOC)/jwaves
	chmod -R g+w $(INST_DOC)/jwaves

html: $(HDRS)
	env PREFIX=$(PREFIX) REAL_PWD=`python -c "import os.path; print os.path.abspath('.')"` doxygen
