# Common Definitions ########################################################
PREFIX=C:/local/juno

CFG=$(PREFIX)/cfg
DOC=$(PREFIX)/doc
BIN=$(PREFIX)/bin
SRC=$(PREFIX)/src
LIB=$(PREFIX)/lib
PYLIB=$(LIB)/python
INC=$(PREFIX)/include
SHARE=$(PREFIX)/share

DATE=$(shell date "+%Y-%m-%d")
ARCH=$(shell uname -s)

BUILD_DIR=build.$(ARCH)

# Flags ######################################################################

# THREAD_DEF is defined by some of the rules below, this just in time 
#     evaluation of variables is a useful, if odd feature of gnu make

DEFINES = -DLINUX -DCFG=$(CFG)
INCFLAGS = -I. -I../librpwgse
WARNFLAGS = -Wall -Werror

CFLAGS = -fexceptions -ggdb $(INCFLAGS) $(WARNFLAGS) $(DEFINES) $(TH_CFLAGS)
 
CC=gcc

LFLAGS = $(TH_LFLAGS) ../librpwgse/$(BUILD_DIR)/librpwgse$(TH_SUF).a \
  C:/local/lib/dbghelp.lib -lKernel32 -lbfd -liberty -lm  

# Specific Definitions ######################################################

HDR_IN = JWavesSDSblk.h JWavesRdrBlk.h JWavesUsegBlk.h JWavesMP.h JWavesIDP.h\
 JWavesStore.h JWavesProc.h JWaves.h JWavesDefs.h JWavesExcept.h JWavesConv.h\
 JWavesLRSblk.h JWavesUpkBlk.h

SRC_IN = JWavesSDSblk.c JWavesRdrBlk.c JWavesUsegBlk.c JWavesPkt.c JWavesMP.c\
 JWavesIDP.c JWavesStore.c JWavesUnpack.c JWavesData.c JWavesConv.c\
 JWavesLRSblk.c JWavesUpkBlk.c JWavesDecomp.c
	 
TEST_IN = test_9bitflt test_getxydat

HDRS =         $(patsubst %,jwaves/%,$(HDR_IN))
INSTALL_HDRS = $(patsubst %,$(INC)/jwaves/%,$(HDR_IN))

SRCS = $(patsubst %.c,jwaves/%.c,$(SRC_IN))

TEST_SRCS = $(patsubst %,test/%.c,$(TEST_IN))
OBJS = $(patsubst jwaves/%.c,$(BUILD_DIR)/%.o,$(SRCS))

TEST_PROGS = $(patsubst %,$(BUILD_DIR)/%,$(TEST_IN))

# Pattern Rules #############################################################

# Pattern rule for the test executables
$(BUILD_DIR)/%:test/%.c
	$(CC) $(CFLAGS) $< $(BUILD_DIR)/libjwaves$(TH_SUF).a $(LFLAGS) -o $@

# Pattern rule for the library object files
$(BUILD_DIR)/%.o:jwaves/%.c
	$(CC) -c $(CFLAGS) -o $@ $<
	#chmod g+w $@
	
# Pattern rule for shared objects
$(BUILD_DIR)/%.dll:$(OBJS)
	gcc -rdynamic -ggdb -shared -o $@ $(OBJS) $(LFLAGS)
	
# Pattern rule for archive files
$(BUILD_DIR)/%.a:$(OBJS)
	ar -crs $@ $(OBJS)
	
# Generic Pattern rule for installing files in the juno library location  
$(LIB)/%:$(BUILD_DIR)/%
	if [ ! -e $(LIB) ]; then mkdir -p $(LIB); fi
	cp $< $@

# Generic pattern rule for installing header files
$(INC)/jwaves/%:jwaves/%
	if [ ! -e $(INC)/jwaves ]; then mkdir -p $(INC)/jwaves; fi
	cp $< $@

# Explicit Rules #############################################################

# Direct make not to nuke the intermediate .o files
.SECONDARY: $(OBJS)

build:TH_SUF=_t
build:TH_LFLAGS=-lpthread
build:TH_CFLAGS=-DGSE_POSIX_THREADS

build:$(BUILD_DIR) $(BUILD_DIR)/libjwaves_t.a $(TEST_PROGS)

nothreads:$(BUILD_DIR) $(BUILD_DIR)/libjwaves.a $(TEST_PROGS)


$(BUILD_DIR):
	@if [ ! -e "$(BUILD_DIR)" ]; then echo mkdir $(BUILD_DIR); \
        mkdir $(BUILD_DIR); fi

install_nothreads: $(LIB)/libjwaves.a $(INSTALL_HDRS)
	
 
install:$(LIB)/libjwaves_t.a $(INSTALL_HDRS)

clean:
	rm $(OBJS)  $(TEST_PROGS)

distclean:
	rm -r -f $(BUILD_DIR)
	
# Documentation Rules ########################################################
install_html: html
	if [ ! -e $(DOC)/jwaves ]; then mkdir -p $(DOC)/jwaves; fi
	cp -r html libjwaves.tag $(DOC)/jwaves
	chmod -R g+w $(DOC)/jwaves

html: $(HDRS)
	env PREFIX=$(PREFIX) doxygen
