/* Example of iterating over juno waves packets in a file and printing
   the original header bytes along with the data.
	
	Compile with the commands:
	
	gcc -Wall -fexceptions -rdynamic -fPIC -ggdb -DGSE_POSIX_THREADS \
       -I $PREFIX/include -o waves_orig.o -c waves_orig.c
	
	gcc -Wall -fexceptions -rdynamic -fPIC -ggdb waves_orig.o \
        $PREFIX/lib/libjwaves_t.a  $PREFIX/lib/librpwgse_t.a \
        -o waves_orig -lm -ldl -lpthread
 
  Where $PREFIX is the top level libjwaves and librpwgse libraries.
*/

/* Define our system clib conformance */
#define _POSIX_C_SOURCE 200112

#ifdef GSE_POSIX_THREADS
#include <pthread.h>
#endif

/* Source URL for librpwgse:
 * https://saturn.physics.uiowa.edu/svn/util/C/trunk/librpwgse
 */
#include <rpwgse/Gse.h>

/* Source URL for libjwaves:
 * https://ec.physics.uiowa.edu/control/juno_hw/GSE/trunk/libjwaves
 */
#include <jwaves/JWaves.h>

int main(int argc, char** argv)
{
	char*          sFile    = NULL;
	GsePktSrc*     pSrc     = NULL;
	GsePkt*        pPkt     = NULL;
	cbytebuf_t     header   = CBBUF_NULL;    /* a cbytebuf_t is the struct    */
	cbytebuf_t     data     = CBBUF_NULL;    /* {const byte* ptr; size_t len} */
	int            nPkt     = 0;
	
	gse_thread_init(GSE_NOTICE, NULL);              /* Always do this first */
	
	if(argc > 1)                                    /* Get the file name */
		sFile = argv[1];
	else
		sFile = jwaves_latest_file(".raw", GSE_CRITICAL);
	
	gse_notice_v("Reading from %s", sFile);         /* Newline auto appended */
	
	/* Detailed parsing is not needed for this example, so just use the generic
	   decoder, gse_generic_blocks */
	pSrc = (GsePktSrc*) new_GseFilePktSrc(sFile,gse_generic_blocks,at_eof_stop);
	
	while( (pPkt = pSrc->readNext(pSrc)) != NULL){
		nPkt += 1;
		fprintf(stdout, "Packet no: %d\n", nPkt);
		
		header = jwaves_inst_hdr(pPkt);
		data = jwaves_inst_data(pPkt);
		
		gse_fmt_hexprn(stdout, header, 4, 1);        /* Print in 1 byte groups */
		gse_fmt_hexprn(stdout, data, 4, 2);          /* Print in 2 byte groups */
		
		pPkt->del(pPkt);                             /* Free the packet */
	}
	return 0;	
}
