#define _POSIX_C_SOURCE 200112

#ifdef GSE_POSIX_THREADS
#include <pthread.h>
#endif

#include <rpwgse/GseConv.h>
#include <rpwgse/GseExcept.h>
#include <rpwgse/GseLog.h>

#include "JWavesCCSDSblk.h"


/****************************************************************************/
bytebuf_t JWavesCCSDSblk_encode(GseBlock* pBase, byte uExtraLen, byte* pOffset)
{	
	JWavesCCSDSblk* pThis = (JWavesCCSDSblk*)pBase;
	bytebuf_t buf;
	byte uOffset;

		
	buf = GseBlock_encode(pBase, 11, &uOffset);
	gse_assert( buf.len >= 11 + uOffset);
	
	buf.ptr[uOffset]      = (pThis->uVer << 5) & 0xE0;
	buf.ptr[uOffset]     |= (pThis->uPktType << 4) & 0x10;
	buf.ptr[uOffset]     |= (pThis->uSecHdr  << 3) & 0x08;
	buf.ptr[uOffset]     |= (pThis->uAPID >> 8) & 0x07;
	
	buf.ptr[uOffset + 1]  = pThis->uAPID & 0xFF;
	
	buf.ptr[uOffset + 2]  = (pThis->uGrouping << 6) & 0xC0;
	buf.ptr[uOffset + 2] |= (pThis->uPartNo >> 8) & 0x3F;
	
	buf.ptr[uOffset + 3]  = pThis->uPartNo & 0xFF;
	
	buf.ptr[uOffset + 4]  = (pThis->uPktLen >> 8) & 0xFF;
	
	buf.ptr[uOffset + 5]  = pThis->uPktLen & 0xFF;
	
	gse_u4ToMSBbytes(buf.ptr + uOffset + 6, pThis->uSCLK);
	
	buf.ptr[uOffset + 10] = pThis->uSubSec;
		
	if(pOffset != NULL) *pOffset = uOffset + 11;
	return buf;
}

/****************************************************************************/
byte JWavesCCSDSblk_decode(struct gse_block* pBase, cbytebuf_t buf)
{
	JWavesCCSDSblk* pThis = (JWavesCCSDSblk*)pBase;
	byte uOffset;
	
	uOffset = GseBlock_decode(pBase, buf);
	if(uOffset != 2)
		gse_throw(GSE_XCEPT, GSE_XLIBINCOMPAT);
	
	if(pBase->uType != JW_CCSDS_BLK_TYPE_ID)
		gse_throw_fm(GSE_XCEPT, GSE_XDECODE, "Expected 0x%hhX for the block type "
		             "ID, but recieved 0x%hhX", JW_CCSDS_BLK_TYPE_ID, pBase->uType);
	
	pThis->uVer     = (buf.ptr[uOffset] >> 5) & 0x7;
	
	pThis->uPktType = (buf.ptr[uOffset] >> 4) & 0x1;
	
	pThis->uSecHdr  = (buf.ptr[uOffset] >> 3) & 0x1;
	
	pThis->uAPID    = gse_u2FromMSBbytes(buf.ptr + uOffset);
	pThis->uAPID   &= 0x07FF;
	
	pThis->uGrouping = (buf.ptr[uOffset + 2] >> 6) & 0x3;
	
	pThis->uPartNo   = gse_u2FromMSBbytes(buf.ptr + uOffset + 2);
	pThis->uPartNo  &= 0x3FFF;
	
	pThis->uPktLen   = gse_u2FromMSBbytes(buf.ptr + uOffset + 4);
	
	pThis->uSCLK     = gse_u4FromMSBbytes(buf.ptr + uOffset + 6);
	
	pThis->uSubSec   = buf.ptr[uOffset + 10];
	
	return uOffset + 11;
}

/****************************************************************************/
GseBlock* JWavesCCSDSblk_copy(GseBlock* pBase, size_t uOverMalloc)
{
	JWavesCCSDSblk* pNew;
	JWavesCCSDSblk* pThis = (JWavesCCSDSblk*)pBase;	
	size_t uSize = 0;
	 
 	if(uOverMalloc > sizeof(JWavesCCSDSblk))
		uSize = uOverMalloc;
	else
		uSize = sizeof(JWavesCCSDSblk);
	
	pNew  = (JWavesCCSDSblk*)GseBlock_copy(pBase, uSize);

	pNew->uVer      = pThis->uVer;
	pNew->uPktType  = pThis->uPktType;
	pNew->uSecHdr   = pThis->uSecHdr;
	pNew->uAPID     = pThis->uAPID;
	pNew->uGrouping = pThis->uGrouping;
	pNew->uPartNo   = pThis->uPartNo;
	pNew->uPktLen   = pThis->uPktLen;
	pNew->uSCLK     = pThis->uSCLK;
	pNew->uSubSec   = pThis->uSubSec;

	pNew->del = pThis->del;
	
	return (GseBlock*)pNew;
}

/****************************************************************************/
JWavesCCSDSblk* new_JWavesCCSDSblk(cbytebuf_t buf){
	GseBlock* pBase = NULL;
	JWavesCCSDSblk* pThis = NULL;
	
	pBase = new_GseBlock(CBBUF_NULL, sizeof(JWavesCCSDSblk));
	pThis = (JWavesCCSDSblk*)pBase;
	
	/* Override base class items */
	/* No new dynalloc stuff, so just use base destructor */
	pBase->encode = JWavesCCSDSblk_encode;
	pBase->decode = JWavesCCSDSblk_decode;
	pBase->_sName = "Juno FDS CCSDS Header";
	pBase->uType  = JW_CCSDS_BLK_TYPE_ID;
	pBase->copy   = JWavesCCSDSblk_copy;
	
	pBase->_uMinLen = 13;
	
	/* New items */
	pThis->uVer      = 0;
	pThis->uPktType  = 0;
	pThis->uSecHdr   = 1;
	pThis->uAPID     = 0;
	pThis->uGrouping = 3;
	pThis->uPartNo   = 1; 
	pThis->uPktLen   = 4;
	pThis->uSCLK     = 0;
	pThis->uSubSec   = 0;
	
	pThis->del       = delete_GseBlock;
		
	if(buf.ptr == NULL)
		return pThis;
		
	except_cleanup_push(delete_GseBlock, pThis);
	
	JWavesCCSDSblk_decode(pBase, buf);

	except_cleanup_pop(except_no_call);
	return pThis;	  
}

