/** @file JWavesChanMgr.h Juno specific file channel manager */
#ifndef _JWavesChanMgr_H_
#define _JWavesChanMgr_H_

#include <rpwgse/GseMultiFileSrc.h>
#include <rpwgse/GseTreeWatch.h>

#ifdef __cplusplus
extern "C" {
#endif

/** @addtogroup mod_jwaves_store Storage  */
/*@{*/

#define S jwaves_dir_list
#define C JWavesDirList
#define I char

#include <rpwgse/listTplt.h>

#undef I
#undef C
#undef S

/* Helper for the channel manager */
typedef struct jwaves_file_chan{

	/* The list of file names to keep up2date */
	JWavesDirList* pFiles;
	
	/* What data channel are we */
	char* sType;
	
	/* Has a filesystem delete forced a change in the currenly accessed file */
	bool bValid;
	
	/* The file a client most recently asked for */
	char* sCurFile;
	
	/* The file channel needs sorting */
	bool bNeedsSort;
	
	/* Ye olde destructor */
	void (*del)(void* vpThis);
	
} JWavesFileChan;

JWavesFileChan* new_JWavesFileChan();

#define S jwaves_file_ch_ary
#define C JWavesFileChAry
#define T JWavesFileChan

#include <rpwgse/ptrAryTplt.h>

#undef T
#undef C
#undef S

/* ************************************************************************* */
/** Juno Waves Specific File Channel manager 
 *
 * @class JWavesChanMgr
 * @extends GseFileChanMgr
 */
typedef struct jwaves_chan_mgr{

	/** Base class is a generic file channel manager */
	GseFileChanMgr base;
	
	/* The tree watcher */
	GseTreeWatch* pWatch;
	
	/* The array of file channels */
	JWavesFileChAry* pChAry;
	
	/* The start and end times as given */
	char* sBegIdx;
	char* sEndIdx;
	
	/* The start and end times, as SCLKs */
	float8 dBegSclk;
	float8 dEndSclk;
	
	/* When derving sclks based off of filename assume this much extra 
	   data may be contained with in the file */
	float4 dSclkSlop;
	
	/* The top level data directory it watching (rpwgse or WAVES_REFDR) */
	char* sTop;
	
	/* Our ubiquitous destructor */
	void (*del)(void* vpThis);
	
} JWavesChanMgr;

/** Constructor for juno waves file channel manager.
 *
 * @param sBegTime The starting time point for data.  This may be a SCLK or SCET
 *        the function will figure it out based on the format.  As a special
 *        exception numbers begining with a -/+ sign are considered as time
 *        offsets from now.  This only works when GMT like sclks are in use.
 *               
 * @param sEndTime The stopping time point for data.  The same format rules apply
 *        here as to @a sStart.  In addition, if this value is NULL there is
 *        no stop point, so all data into the indefinite future will be 
 *        included.
 *
 * @param sTypes A list of data types.  Use 1-N of the type strings below seperated 
 *        by commas:
 *        - hsk.raw
 *        - lrs.raw
 *        - hrs_rec.raw 
 *        - hrs_bin.raw
 *        - lrs.ucal
 *        - hrs_rec.ucal 
 *        - hrs_bin.ucal
 *        - hsk.dat 
 *        - srv.pkt
 *        - rec.pkt
 *        - bst.pkt
 *
 * @param sGroup The ubiquitous data group, usually you want 'flight' but
 *        there are many other choices.
 *  
 * @returns An JWavesChanMgr allocated on the heap and ready to hand to
 *          a GseMultiFilePktSrc object constructor. 
 * @public
 * @memberof JWavesChanMgr
 */
JWavesChanMgr* new_JWavesChanMgr(const char* sBegTime, const char* sEndTime,
		                           const char* sTypes, const char* sGroup);

/*@}*/

#ifdef __cplusplus
}
#endif


#endif /* _JWavesChanMgr_H_ */
