#define _POSIX_C_SOURCE 200112

#ifdef GSE_POSIX_THREADS
#include <pthread.h>
#endif

#include <math.h>

#include <rpwgse/GsePkt.h>
#include <rpwgse/GseData.h>
#include <rpwgse/GseWrap.h>
#include <rpwgse/GseString.h>
#include <rpwgse/GseLog.h>
#include <rpwgse/GseMath.h>
#include <rpwgse/GseTime.h>

#include "JWavesExcept.h"
#include "JWavesSDSblk.h"
#include "JWavesConv.h"

#define _JWavesData_C_

#include "JWavesData.h"
#include "JWavesHFWBRbands.h"

float jw_LFR_lo_wfrm_X[65536];
float jw_LFR_hi_wfrm_X[65536];
float jw_WBR_mix_wfrm_X[65536];
float jw_WBR_base_wfrm_X[65536];
static bool jw_bNeedsInit = true;


const float jw_LFR_coeff_X[32] = {
	 1, 2, 3, 4, 5, 6, 7, 8, 9,10,11,12,13,14,15,16,
	17,18,19,20,21,22,23,24,25,26,27,28,29,30,31,32
};

const float jw_LFR_lo_spec_X[] = {
	   48.83, /*  0 */
	   97.65, /*  1 */
	  146.5,  /*  2 */
	  195.3,  /*  3 */
	  244.1,  /*  4 */
	  293.0,  /*  5 */
	  342.7,  /*  6 */
	  390.6,  /*  7 */
	  439.5,  /*  8 */
	  488.3,  /*  9 */
	  537.1,  /* 10 */
	  586.0,  /* 11 */
	  634.8,  /* 12 */
	  684.0,  /* 13 */
	  794.0,  /* 14 */
	  891.0,  /* 15 */
	 1000,    /* 16 */
	 1122,    /* 17 */
	 1259,    /* 18 */
	 1413,    /* 19 */
	 1585,    /* 20 */
	 1778,    /* 21 */
	 1995,    /* 22 */
	 2239,    /* 23 */
	 2512,    /* 24 */
	 2818,    /* 25 */
	 3162,    /* 26 */
	 3548,    /* 27 */
	 3981,    /* 28 */
	 4467,    /* 29 */
	 5012,    /* 30 */
	 5623,    /* 31 */
	 6310,    /* 32 */
	 7079,    /* 33 */
	 7943,    /* 34 */
	 8912,    /* 35 */
	10000,    /* 36 */
	11220,    /* 37 */
	12590,    /* 38 */
	14130,    /* 39 */
	15850,    /* 40 */
	17780,    /* 41 */
	19950 	 /* 42 */
};

const int4 jw_LFR_lo_spec_bins[] = {
	 1, /*  0 */
	 1, /*  1 */
	 1, /*  2 */
	 1, /*  3 */
	 1, /*  4 */
	 1, /*  5 */
	 1, /*  6 */
	 1, /*  7 */
	 1, /*  8 */
	 1, /*  9 */
	 1, /* 10 */
	 1, /* 11 */
	 1, /* 12 */
	 2, /* 13 */
	 2, /* 14 */
	 2, /* 15 */
	 2, /* 16 */
	 3, /* 17 */
	 3, /* 18 */
	 3, /* 19 */
	 4, /* 20 */
	 4, /* 21 */
	 5, /* 22 */
	 5, /* 23 */
	 6, /* 24 */
	 7, /* 25 */
	 7, /* 26 */
	 8, /* 27 */
	10, /* 28 */
	10, /* 29 */
	12, /* 30 */
	13, /* 31 */
	15, /* 32 */
	17, /* 33 */
	19, /* 34 */
	21, /* 35 */
	23, /* 36 */
	27, /* 37 */
	30, /* 38 */
	33, /* 39 */
	37, /* 40 */
	42, /* 41 */
	47  /* 42 */
};

const size_t jw_LFR_lo_spec_len = 43;
const char* jw_LFR_lo_spec_unit = "Hz";

const float jw_LFR_hi_spec_X[] = {
	 19952.6, /*  0 */
	 22387.2, /*  1 */
	 25118.9, /*  2 */
	 28183.8, /*  3 */
	 31622.8, /*  4 */
	 35481.4, /*  5 */
	 39810.7, /*  6 */
	 44668.4, /*  7 */
	 50118.7, /*  8 */
	 56234.2, /*  9 */
	 63095.8, /* 10 */
	 70794.6, /* 11 */
	 79432.9, /* 12 */
	 89125.1, /* 13 */
	100000,   /* 14 */
	112202,   /* 15 */
	125893,   /* 16 */
	141254    /* 17 */
};

const int4 jw_LFR_hi_spec_bins[] = {
	 6, /*  0 */
	 7, /*  1 */
	 8, /*  2 */
	 9, /*  3 */
	10, /*  4 */
	11, /*  5 */
	13, /*  6 */
	14, /*  7 */
	15, /*  8 */
	18, /*  9 */
	20, /* 10 */
	22, /* 11 */
	25, /* 12 */
	28, /* 13 */
	32, /* 14 */
	35, /* 15 */
	40, /* 16 */
	44  /* 17 */
};

const size_t jw_LFR_hi_spec_len = 18;
const char* jw_LFR_hi_spec_unit = "Hz";	
	
const float jw_HFR_base_spec_X[] = {
	 141.254,  /*  0 */
	 158.489,  /*  1 */
	 177.828,  /*  2 */
	 199.526,  /*  3 */
	 223.872,  /*  4 */
	 251.189,  /*  5 */
	 281.838,  /*  6 */
	 316.228,  /*  7 */
	 354.813,  /*  8 */
	 398.107,  /*  9 */
	 446.684,  /* 10 */
	 501.187,  /* 11 */
	 562.341,  /* 12 */
	 630.957,  /* 13 */
	 707.946,  /* 14 */
	 794.328,  /* 15 */
	 891.251,  /* 16 */
	1000.000,  /* 17 */
	1122.019,  /* 18 */
	1258.926,  /* 19 */
	1412.538,  /* 20 */
	1584.893,  /* 21 */
	1778.280,  /* 22 */
	1995.262,  /* 23 */
	2238.721,  /* 24 */
	2511.887,  /* 25 */
	2818.383   /* 26 */
};

const int jw_HFR_base_spec_bins[] = {
	 2, /*  0 */
	 3, /*  1 */
	 3, /*  2 */
	 3, /*  3 */
	 4, /*  4 */
	 4, /*  5 */
	 5, /*  6 */
	 5, /*  7 */
	 6, /*  8 */
	 7, /*  9 */
	 8, /* 10 */
	 8, /* 11 */
	10, /* 12 */
	10, /* 13 */
	12, /* 14 */
	14, /* 15 */
	15, /* 16 */
	16, /* 17 */
	19, /* 18 */
	22, /* 19 */
	23, /* 20 */
	27, /* 21 */
	30, /* 22 */
	34, /* 23 */
	37, /* 24 */
	43, /* 25 */
	47  /* 26 */ 
};

const size_t jw_HFR_base_spec_len = 27;
const char* jw_HFR_base_spec_unit = "kHz";
		
const float jw_HFR_spec_X[] = {
	 3500, /* 0*/
	 4500, /* 1*/
	 5500, /* 2*/
	 6500, /* 3*/
	 7500, /* 4*/
	 8500, /* 5*/
	 9500, /* 6*/
	10500, /* 7*/
	11500, /* 8*/
	12500, /* 9*/
	13500, /* 10*/
	14500, /* 11*/
	15500, /* 12*/
	16500, /* 13*/
	17500, /* 14*/
	18500, /* 15*/
	19500, /* 16*/
	20500, /* 17*/
	21500, /* 18*/
	22500, /* 19*/
	23500, /* 20*/
	24500, /* 21*/
	25500, /* 22*/
	26500, /* 23*/
	27500, /* 24*/
	28500, /* 25*/
	29500, /* 26*/
	30500, /* 27*/
	31500, /* 28*/
	32500, /* 29*/
	33500, /* 30*/
	34500, /* 31*/
	35500, /* 32*/
	36500, /* 33*/
	37500, /* 34*/
	38500, /* 35*/
	39500, /* 36*/
	40500  /* 37*/
/*	41500, *//* 38*/
/*	42500 */ /* 39*/
};
const size_t jw_HFR_spec_len = 38;
const char* jw_HFR_spec_unit = "kHz";

const float jw_PLL_oscillator_X[] = {
	 81, /*  0 */
	 83, /*  1 */
	 85, /*  2 */
	 87, /*  3 */
	 89, /*  4 */
	 91, /*  5 */
	 93, /*  6 */
	 97, /*  7 */
	 99, /*  8 */
	101, /*  9 */
	103, /* 10 */
	105, /* 11 */
	107, /* 12 */
	109, /* 13 */
	111, /* 14 */
	113, /* 15 */
	115, /* 16 */
	117, /* 17 */
	119, /* 18 */
	121, /* 19 */
	123, /* 20 */
	125, /* 21 */
	129, /* 22 */
	131, /* 23 */
	133, /* 24 */
	135, /* 25 */
	137, /* 26 */
	139, /* 27 */
	141, /* 28 */
	143, /* 29 */
	145, /* 30 */
	147, /* 31 */
	149, /* 32 */
	151, /* 33 */
	153, /* 34 */
	155, /* 35 */
	157, /* 36 */
	161  /* 37 */
	/*165,*/ /* (38 */
};

/* The Y data for the PLL detect packets is out of order, for each index
   i of the y data move it to jw_PLL_reorder_Y[i] */
const size_t jw_PLL_reorder_Y[] = {
	14, /*  0 */
	29, /*  1 */
	 3, /*  2 */
	10, /*  3 */
	18, /*  4 */
	25, /*  5 */
	33, /*  6 */
	 1, /*  7 */
	 5, /*  8 */
	 8, /*  9 */
	12, /* 10 */
	16, /* 11 */
	20, /* 12 */
	23, /* 13 */
	27, /* 14 */
	31, /* 15 */
	35, /* 16 */
	 0, /* 17 */
	 2, /* 18 */
	 4, /* 19 */
	 6, /* 20 */
	 7, /* 21 */
	 9, /* 22 */
	11, /* 23 */
	13, /* 24 */
	15, /* 25 */
	17, /* 26 */
	19, /* 27 */
	21, /* 28 */
	22, /* 29 */
	24, /* 30 */
	26, /* 31 */
	28, /* 32 */
	30, /* 33 */
	32, /* 34 */
	34, /* 35 */
	36, /* 36 */
	37  /* 37 */
	
};
	 
const size_t jw_PLL_oscillator_len = 38;
const char* jw_PLL_oscillator_unit = "MHz";


void jwaves_init_wfrm_tbls(){
	int i;
	
	float fLFRlo, fLFRhi, fHFWBRmix, fHFWBRbase;
	fLFRlo = 1/50.0;
	fLFRhi = 1/375.0;
	fHFWBRmix = 1/1312.5;
	fHFWBRbase = 1/7000.0;
	
	jw_LFR_lo_wfrm_X[0]  = 0;
	jw_LFR_hi_wfrm_X[0]  = 0;
	jw_WBR_mix_wfrm_X[0] = 0;
	jw_WBR_base_wfrm_X[0] = 0;
	
	for(i = 1; i<65536; i++){
		jw_LFR_lo_wfrm_X[i]   =     fLFRlo + jw_LFR_lo_wfrm_X[i-1];
		jw_LFR_hi_wfrm_X[i]   =     fLFRhi + jw_LFR_hi_wfrm_X[i-1];
		
		/* These are the sampling frequencies */
		jw_WBR_mix_wfrm_X[i]  =  fHFWBRmix * i;
		
		jw_WBR_base_wfrm_X[i] = fHFWBRbase * i;
	}
}

/* jw_wfrm_unit = "msec"; */


/****************************************************************************/
/* Holder for Juno waves table data of all sorts */
typedef struct jwaves_xy_data{
	GseXYdata base;
	
	const char* sChID;
	const char* sChName;
	char* sInfo;
	
	struct timeval utc;
	char* sSCLK;
	
	const float4* pX;
	float4buf_t   bufX;  /* This is used if pX is null */
	const char*  sXunit;
	gse_unittype_t Xtype;
	
	float4 hi_limit;
	float4 lo_limit;
	
	float4buf_t bufY;
	const char* sYunit;
	gse_unittype_t Ytype;
		
	char* sDynChID;
	char* sDynChName;
	
	uint4 uHints; 
	
} JWavesXYdata;

const char* JWavesXYdata_channel(const GseXYdata* pBase){
	const JWavesXYdata* pThis = (const JWavesXYdata*)pBase;
	
	return (const char*)(pThis->sDynChID ? pThis->sDynChID : pThis->sChID);
}
	
const char* JWavesXYdata_channelName(const GseXYdata* pBase){
	const JWavesXYdata* pThis = (const JWavesXYdata*)pBase;
	
	return (const char*)(pThis->sDynChName ? pThis->sDynChName : pThis->sChName);
}
	
const char* JWavesXYdata_channelInfo(const GseXYdata* pBase){
	return ((const JWavesXYdata*)pBase)->sInfo;
}
	
float4 JWavesXYdata_yHiLimit(const GseXYdata* pBase){
	return ((const JWavesXYdata*)pBase)->hi_limit;
}
	
float4 JWavesXYdata_yLoLimit(const GseXYdata* pBase){
	return ((const JWavesXYdata*)pBase)->lo_limit;
}
	
void JWavesXYdata_captureUTC(const GseXYdata* pBase, struct tm* pTm,
                             uint4* pNano){
	const JWavesXYdata* pThis = (const JWavesXYdata*)pBase;
#ifndef _WIN32
	gmtime_r(&(pThis->utc.tv_sec), pTm);
#else
	/* This is thread safe on windows */
	/* http://www.mail-archive.com/mingw-w64-public@lists.sourceforge.net/msg00179.html */
	struct tm* pRet;
	pRet = gmtime(&(pThis->utc.tv_sec));
	pTm->tm_year = pRet->tm_year;
	pTm->tm_mon  = pRet->tm_mon;
#endif
	pTm->tm_year += 1900;
	pTm->tm_mon += 1;
	
	*pNano = (pThis->utc.tv_usec) * 1000;
}
	
const char* JWavesXYdata_captureSCLK(const GseXYdata* pBase){
	return ((const JWavesXYdata*)pBase)->sSCLK;
}
	
size_t JWavesXYdata_length(const GseXYdata* pBase){
	return ((const JWavesXYdata*)pBase)->bufY.len;
}
	
const float4* JWavesXYdata_Xdata(const GseXYdata* pBase){
	const JWavesXYdata* pThis = (const JWavesXYdata*)pBase;
		
	return pThis->pX ? pThis->pX : pThis->bufX.ptr;
}

gse_unittype_t JWavesXYdata_Xtype(const GseXYdata* pBase){
	return ((const JWavesXYdata*)pBase)->Xtype;
};

const char* JWavesXYdata_Xunit(const GseXYdata* pBase){
	return ((const JWavesXYdata*)pBase)->sXunit;
}
	
const float4* JWavesXYdata_Ydata(const GseXYdata* pBase){
	return ((const JWavesXYdata*)pBase)->bufY.ptr;
}

gse_unittype_t JWavesXYdata_Ytype(const GseXYdata* pBase){
	return ((const JWavesXYdata*)pBase)->Ytype;
}

const char* JWavesXYdata_Yunit(const GseXYdata* pBase){
	return ((const JWavesXYdata*)pBase)->sYunit;
}
		
uint4 JWavesXYdata_hints(const GseXYdata* pBase){
	return ((const JWavesXYdata*)pBase)->uHints;
}
			
void delete_JWavesXYdata(void* vpThis){
	JWavesXYdata* pThis = (JWavesXYdata*)vpThis;
	
	if(pThis->bufX.ptr != NULL) free(pThis->bufX.ptr);
	if(pThis->bufY.ptr != NULL) free(pThis->bufY.ptr);
	if(pThis->sDynChID != NULL) free(pThis->sDynChID);
	if(pThis->sDynChName != NULL) free(pThis->sDynChName);
	if(pThis->sInfo    != NULL) free(pThis->sInfo);
	if(pThis->sSCLK    != NULL) free(pThis->sSCLK);
	free(pThis);
}

GseXYdata* JWavesXYdata_copy(const GseXYdata* pBase)
{
	JWavesXYdata* pThis = (JWavesXYdata*)pBase;
	JWavesXYdata* pNew;
	
	pNew = (JWavesXYdata*)gse_calloc(1, sizeof(JWavesXYdata));
	memcpy(pNew, pThis, sizeof(JWavesXYdata));
	
	/* Extra work for the Y data arrays */
	if(pNew->sInfo     != NULL) pNew->sInfo     = gse_strcopy(pThis->sInfo);
	if(pNew->sSCLK     != NULL) pNew->sSCLK     = gse_strcopy(pThis->sSCLK);
	if(pNew->sDynChID  != NULL) pNew->sDynChID  = gse_strcopy(pThis->sDynChID);
	if(pNew->sDynChName!= NULL) pNew->sDynChName= gse_strcopy(pThis->sDynChName);
	
	if(pNew->bufX.ptr != NULL){
		pNew->bufX.ptr = (float4*)gse_calloc(pNew->bufX.len, sizeof(float4));
		memcpy(pNew->bufX.ptr, pThis->bufX.ptr, sizeof(float4)*(pNew->bufX.len));
	}
	
	if(pNew->bufY.ptr != NULL){
		pNew->bufY.ptr = (float4*)gse_calloc(pNew->bufY.len, sizeof(float4));
		memcpy(pNew->bufY.ptr, pThis->bufY.ptr, sizeof(float4)*(pNew->bufY.len));
	}
	
	return (GseXYdata*)pNew;
}

/*****************************************************************************/
/* Constructor helpers */

char* jwaves_mk_attn_info(byte uPSID, byte uAttn){
	byte uPktID;
	int nAttn = 0;
	const char* sFA = NULL;
	char* sBA = NULL;
	
	uPktID = (uPSID >> 4) & 0xF;
	
	/* Handle preamp setting */
	if(uAttn & 0x20) sFA = "FATN";
	
	/* LFR Attn */
	if((uPktID == 0x9)||(uPktID == 0xA)||
		(uPSID == 0xF0)||(uPSID == 0xF4)||(uPSID == 0xF5)){
	
		if(uAttn & 0x10) sBA = CS("BATN");
	}
	/* HFR/HFWBR Attn */
	else{
		/* Front attn and back step attenuator */
		nAttn += (uAttn & 0xF) * 2;
		sBA = NS("Step %d dB", nAttn);
	}
	
	if((sFA == NULL)&&(sBA == NULL)) return CS("No attn");
	if((sFA == NULL)&&(sBA != NULL)) return sBA;
	if((sFA != NULL)&&(sBA == NULL)) return CS(sFA);
	
	return gse_string("%s, %s", sFA, sBA);
}

void JWavesXYdata_setLimits(JWavesXYdata* pThis, uint4 flags, byte uFmt){
	if(flags & JW_DATA_HFWBR_SIGNED){
		switch(uFmt){
			case 0x1B: /* 8-bit Y data */
				pThis->lo_limit = -128;     pThis->hi_limit = 128;   return;
				
			case 0x4:
			case 0x11: /* 8-bit data */
				pThis->lo_limit = -128;     pThis->hi_limit = 128;   return;
			case 0x6:
			case 0x7:
			case 0x8: /* 12-bit data */
				pThis->lo_limit = -2048;    pThis->hi_limit = 2048;  return;
				
			case 0x1C: /* Needless override of an existing format */

			case 0xC:
			case 0xD: /* 16-bit data */
				pThis->lo_limit = -32768;   pThis->hi_limit = 32768; return;
			case 0x3:
			case 0x12: /* 9-bit data */
				pThis->lo_limit = 9.54e-07; /* 0x000 -> converter */
				pThis->hi_limit = 1.65e+13; /* 0x1FF -> converter */
				return;
			case 0x2: /* 32-bit floats */
				/* pThis->lo_limit = 9.54e-07; *//* Reused 9-bit vals because no idea  */
				/* pThis->hi_limit = 1.65e+13; *//* what to put here */
				pThis->lo_limit = -32768.0;   
				pThis->hi_limit = 32768.0;
				return;
		}
	}
	else{
		switch(uFmt){
			case 0x1B: /* 8-bit Y data */
				pThis->lo_limit = 0;        pThis->hi_limit = 256;   return;
				
			case 0x4:
			case 0x11: /* 8-bit data */
				pThis->lo_limit = 0;        pThis->hi_limit = 256;   return;
			case 0x6:
			case 0x7:
			case 0x8: /* 12-bit data */
				pThis->lo_limit = 0;        pThis->hi_limit = 4096;  return;
				
			case 0x1C:  /* Needless override of an existing format */

			case 0xC:
			case 0xD: /* 16-bit data */
				pThis->lo_limit = 0;        pThis->hi_limit = 65536; return;
			
			case 0x5:
			case 0x3:
			case 0x12: /* 9-bit data */
				/* pThis->lo_limit = 9.54e-07; *//* 0x000 -> converter */
				/* pThis->hi_limit = 1.65e+13; *//* 0x1FF -> converter */
				pThis->lo_limit = 0;
				pThis->hi_limit = 65536.0;
				return;
				
			case 0x2: /* 32-bit floats */
				/* pThis->lo_limit = 9.54e-07; *//* Reused 9-bit vals because no idea  */
				/* pThis->hi_limit = 1.65e+13; *//* what to put here */
				
				pThis->lo_limit = 0;
				pThis->hi_limit = 65536.0;
				return;
		}
	}
	gse_throw_fm(JW_XCEPT, JW_XINVHDRFIELD, "Unrecognized data format value "
				             "0x%02X.", uFmt);
}

void JWavesXYdata_setupWBR(JWavesXYdata* pThis, const JWavesSDSblk* pSci, 
		                     const GsePkt* pPkt, const char* sRec,
                           char cChan, uint4 flags)
{
	uint4 convFlags = 0;
	char* sAttn;
	struct jwaves_float4_conv conv;
	init_jwaves_float4_conv(&conv);
	
	sAttn = jwaves_mk_attn_info(pSci->uPSID, pSci->uAttn);
	
	/* Three states: uMixer >g_uJWavesHFWBRbands: Error
	                 nVCO == -1:                  No mixing
		              nVCO > -1:                   Mixing
	 */
	if(pSci->uMixer > g_uJWavesHFWBRbands){
		pThis->sInfo = NS("%s  %c  %s  BAND ERROR", sRec, cChan, sAttn);
	}
	else{
		if(g_pJWavesHFWBRbands[pSci->uMixer].nVCO < 0)
			pThis->sInfo = NS("%s  %c  %s  Fltr: %s  Mix: OFF", sRec, cChan, sAttn,
			                  g_pJWavesHFWBRbands[pSci->uMixer].sFltr);
		else
			pThis->sInfo = NS("%s  %c  %s  Fltr: %s  Mix: %.2f MHz", sRec, cChan,
			                  sAttn, g_pJWavesHFWBRbands[pSci->uMixer].sFltr,
									g_pJWavesHFWBRbands[pSci->uMixer].fCent);
	}
	free(sAttn);

	pThis->pX = jw_WBR_mix_wfrm_X; /* TODO: make this dynamic */
	pThis->sXunit = "msec";
	pThis->Xtype  = TIME;
	
	jwaves_bytes2float4(pSci->uFmt, pPkt->getData(pPkt), convFlags, &conv);
	pThis->bufY.ptr = conv.ptr;
	pThis->bufY.len = conv.len;
	
	pThis->sYunit = "Counts";
	pThis->Ytype  = NUMBER;
}


void JWavesXYdata_setupSide(JWavesXYdata* pThis, const JWavesSDSblk* pSci, 
		                     const GsePkt* pPkt, const char* sRec, uint4 flags)
{
	uint4 convFlags = 0;
	char* sAttn;
	int i, nHalf;
	float4 fFreqStep, fCent;
	float8 dRange;
	
	struct jwaves_float4_conv conv;
	init_jwaves_float4_conv(&conv);
	
	sAttn = jwaves_mk_attn_info(pSci->uPSID, pSci->uAttn);
	
	/* Three states: uMixer >g_uJWavesHFWBRbands: Error
	                 nVCO == -1:                  No mixing
		              nVCO > -1:                   Mixing
	 */
	if(pSci->uMixer > g_uJWavesHFWBRbands){
		pThis->sInfo = NS("%s  %s  BAND ERROR", sRec, sAttn);
	}
	else{
		if(g_pJWavesHFWBRbands[pSci->uMixer].nVCO < 0){
			free(sAttn);
			gse_throw_m(GSE_XCEPT, GSE_XINCONCEIVABLE, "JWavesSideFilter is "
			            "combinding packets that were not Mixed!");
		}
		else
			pThis->sInfo = NS("%s  %s  Fltr: %s  Mix: %.2f MHz", sRec,
			                  sAttn, g_pJWavesHFWBRbands[pSci->uMixer].sFltr,
									g_pJWavesHFWBRbands[pSci->uMixer].fCent);
	}
	free(sAttn);
	
	/* Do the Y axis first, in dB from full range */
	jwaves_bytes2float4(pSci->uFmt, pPkt->getData(pPkt), convFlags, &conv);
	
	if(flags & JW_DATA_RAW_FLOATS){
		pThis->Ytype  = NUMBER;
		pThis->sYunit = "Counts";
	}	
	else{
		dRange = abs( pThis->hi_limit - pThis->lo_limit );
		for(i=0; i<conv.len; i++){	
			conv.ptr[i] = 20 * log10( conv.ptr[i] / dRange);
		}
		pThis->sYunit = "dB";
		pThis->Ytype  = DB;
		pThis->lo_limit = -90;
		pThis->hi_limit = 10;
	}
	pThis->bufY.ptr = conv.ptr;
	pThis->bufY.len = conv.len;
	
	/* The X axis */
	fFreqStep = 1312.5 / conv.len;  /* In KHz about mixing freq */
	fCent = g_pJWavesHFWBRbands[pSci->uMixer].fCent;
	
	gse_assert(conv.len % 2 == 0);
	nHalf = conv.len/2;
	
	pThis->bufX.len = conv.len;
	pThis->bufX.ptr = (float4*)gse_calloc(pThis->bufX.len, sizeof(float4));

	/* Do the lower sideband */
	for(i = nHalf-1; i >= 0; i--){
		pThis->bufX.ptr[i] = fCent*1000.0 - fFreqStep*( nHalf - i - 0.5);
	}
	
	/* Now the upper sideband */
	for(i = nHalf; i < conv.len; i++){
		pThis->bufX.ptr[i] = fCent*1000.0 + fFreqStep*( i - nHalf + 0.5);
	}
			
	pThis->sXunit = "kHz";
	pThis->Xtype  = FREQUENCY;
}

/****************************************************************************/
/* Have to sort the data, this is a pain in the a** */

struct jwaves_q_pair{
	byte factor;
	uint4 sclk;
};

int jwaves_q_comp(const void* p1, const void* p2)
{
	const struct jwaves_q_pair* q1 = (const struct jwaves_q_pair*)p1;
	const struct jwaves_q_pair* q2 = (const struct jwaves_q_pair*)p2;
	
	if(q1->sclk > q2->sclk) return 1;
	if(q1->sclk < q2->sclk) return -1;
	return 0;
}

void JWavesXYdata_setupQfact(JWavesXYdata* pThis, const GsePkt* pPkt,
                             uint4 flags)
{	
	const JWavesSDSblk* pSci;
	int i;
	cbytebuf_t cbuf;
	uint2 u2;
	uint2 u2UpperSclk;
	uint2 u2LowerSclk;
	uint4 u4;
	struct jwaves_q_pair* pPairs;
	uint4 u4Earliest = 0;
	
	pSci = (const JWavesSDSblk*) pPkt->getBlkByType(pPkt, JW_SDS_BLK_TYPE_ID, 0);
		
	if(pSci->uFmt != 0x1B)
		gse_throw_fm(GSE_XCEPT, GSE_XDECODE, "Expected format 0x1B for Q-Factor"
		             " data, recieved 0x%02uuX", pSci->uFmt);
	
	/* Allocate pair buffer (truncation is okay) */
	cbuf = pPkt->getData(pPkt);
	pPairs = (struct jwaves_q_pair*)gse_calloc(cbuf.len/3, sizeof(struct jwaves_q_pair));

			
	/* Loop through the data 3-bytes at a time an pull out the Y *and* X data */
	u2UpperSclk = (pSci->uColSCLK >> 16) & 0xFFFF;
	u2LowerSclk = pSci->uColSCLK & 0xFFFF;
	
	u4Earliest = 0xFFFFFFFF;
	for(i = 0; i<(cbuf.len - 2); i += 3)
	{
		pPairs[i/3].factor = cbuf.ptr[i];  /* Y data */
		
		u2 = cbuf.ptr[i+2] * 256 + cbuf.ptr[i+1];
		
		/* If u2 > lower 16 bits of sclk then dec upper bits 1st */
		if((u2 > u2LowerSclk)&&(u2UpperSclk > 0))
			u4 = ((uint4)(u2UpperSclk - 1))*65536 + u2;
		else
			u4 = ((uint4)(u2UpperSclk))*65536 + u2;
		
		pPairs[i/3].sclk = u4;             /* X data */
		
		/* Store the lowest sclk encontered */
		if(u4 < u4Earliest) u4Earliest = u4;
	}
	
	/* Sort the array because we don't know the order Willy will emit in */
	qsort(pPairs, cbuf.len/3, sizeof(struct jwaves_q_pair), jwaves_q_comp);
	
	/* Copy back into the standard layout */
	pThis->bufX.len = cbuf.len / 3;
	pThis->bufX.ptr = (float4*)gse_calloc(pThis->bufX.len, sizeof(float4));
	pThis->bufY.len = cbuf.len / 3;
	pThis->bufY.ptr = (float4*)gse_calloc(pThis->bufY.len, sizeof(float4));
	for(i=0; i<pThis->bufY.len; i++)
	{
		pThis->bufY.ptr[i] = (float4)pPairs[i].factor;
		pThis->bufX.ptr[i] = (float4)(pPairs[i].sclk - u4Earliest);
	}
	free(pPairs);
	
	/* Back the up the reported SCLK to the earliest time */
	if(pThis->sSCLK != NULL) free(pThis->sSCLK);
	pThis->sSCLK = NS("%u", u4Earliest);
		
	jwaves_sclk2tv((float8)u4Earliest, &(pThis->utc));
	pThis->sInfo = CS(" ");
	
	pThis->sXunit = "sec";
	pThis->Xtype = TIME;
	
	pThis->sYunit = "Q-Factor";
	pThis->Ytype  = NUMBER;
}


/****************************************************************************/
JWavesXYdata* new_JWavesXYdata(const GsePkt* pPkt, int part, uint4 flags,
                               int4 nPktIdx, size_t nOffset){
	
	GseXYdata* pBase = NULL;
	JWavesXYdata* pThis = NULL;
	
	const JWavesSDSblk* pSci = NULL;
	char* sAttn = NULL;
	const char* sRec = NULL;
	struct jwaves_float4_conv conv;
	cbytebuf_t cbuf = CBBUF_NULL;
	const byte* pBytes = NULL;
	uint4 convFlags = 0;
	uint2 u2Tmp = 0;
	size_t expected_len = 0;
	size_t offset = 0;
	char* sCh = NULL;
	int4 i;
	const int4* pNoBins = NULL;   /* The number of bins to div by in "Spectra" mode */
	
	init_jwaves_float4_conv(&conv);
	
	if(jw_bNeedsInit){
		jwaves_init_wfrm_tbls();
		jw_bNeedsInit = false;
	}
	
	if((pPkt->pHdr->uContent & 0xF) < 0x4)
		gse_throw_m(JW_XCEPT, JW_XPKTNOTREADY, "Packet data has not been "
		            "processed into a single capture, content type must be 0x4 "
		            "or higher in order to generate a GSE XY data structure");
	
	pSci = (const JWavesSDSblk*) pPkt->getBlkByType(pPkt, JW_SDS_BLK_TYPE_ID, 0);
				
	pThis = (JWavesXYdata*)gse_calloc(1, sizeof(JWavesXYdata));
	except_cleanup_push(delete_JWavesXYdata, pThis);
	pBase = (GseXYdata*)pThis;
	
	pBase->channel = JWavesXYdata_channel;
	pBase->channelName = JWavesXYdata_channelName;
	pBase->channelInfo = JWavesXYdata_channelInfo;
	pBase->yHiLimit = JWavesXYdata_yHiLimit;
	pBase->yLoLimit = JWavesXYdata_yLoLimit;
	pBase->captureSCLK = JWavesXYdata_captureSCLK;
	pBase->captureUTC = JWavesXYdata_captureUTC;
	pBase->length = JWavesXYdata_length;
	pBase->Xdata = JWavesXYdata_Xdata;
	pBase->Xtype = JWavesXYdata_Xtype;
	pBase->Xunit = JWavesXYdata_Xunit;
	pBase->Ydata = JWavesXYdata_Ydata;
	pBase->Ytype = JWavesXYdata_Ytype;
	pBase->Yunit = JWavesXYdata_Yunit;
	pBase->hints = JWavesXYdata_hints;
	pBase->del = delete_JWavesXYdata;
	pBase->copy = JWavesXYdata_copy;
	
	
	/* Get the capture times */
	pThis->sSCLK = NS("%.3f", (double)pSci->uColSCLK + pSci->uColRTI / 40.0);
	pSci->collectUTC(pSci, &(pThis->utc));
	JWavesXYdata_setLimits(pThis, flags, pSci->uFmt);
	
	sAttn = jwaves_mk_attn_info(pSci->uPSID, pSci->uAttn);
	except_cleanup_push(free, sAttn);
	
	if( !(pThis->sChID = jwaves_psid_sname(pSci->uPSID, part)) )
		pThis->sChID = "Unknown";
			
	if(!(pThis->sChName = jwaves_psid_lname(pSci->uPSID, part, pSci->uColSCLK)))
		pThis->sChName = "Unknown";
	
	pThis->uHints = 0;
	
	switch(pSci->uPSID){
		
		/* Q-Factor settings */
		case 0x21:
			JWavesXYdata_setupQfact(pThis, pPkt, flags); 
			break;
		
		
		/* AGC settings over time a number from 0 to 255 */
		case 0x26:
		case 0x27:
			if(pSci->uPSID == 0x26) pThis->sInfo = CS("HFWBR AGC setting");
			if(pSci->uPSID == 0x27) pThis->sInfo = CS("HFR AGC setting");
			
			jwaves_bytes2float4(pSci->uFmt, pPkt->getData(pPkt), convFlags, &conv);
			
			pThis->bufX.len = conv.len;
			pThis->bufX.ptr = (float4*) gse_calloc(conv.len, sizeof(float4));
					
			/* Read the MSF bytes to generate the X axis */
			if(pSci->uMSFlen < 2){
				gse_error_v("Packet %d at 0x%X: AGC Settings packet is missing"
						      "MSF bytes!", nPktIdx+1, nOffset);
			 	
				for(i=0; i< pThis->bufX.len; i++) pThis->bufX.ptr[i] = i;
				pThis->sXunit = "Sample No.";
				pThis->Xtype  = NUMBER;
			}
			else{
				u2Tmp = ((int)(pSci->MSFary[1])) * 256 + pSci->MSFary[0];
				
				for(i=0; i< pThis->bufX.len; i++)
					pThis->bufX.ptr[i] = i * u2Tmp * 0.025;
				
				pThis->sXunit = "sec";
				pThis->Xtype = TIME;
			}
				
			pThis->bufY.ptr = conv.ptr;
			pThis->bufY.len = conv.len;
			pThis->sYunit = "Counts";
			pThis->Ytype  = NUMBER;
			break;
		
		/* HFR Lock "spectra" */
		case 0x44: 
		case 0x45:
			if((pSci->uPSID == 0x44)||(pSci->uPSID == 0x24))
				pThis->sInfo = NS("HFWBR Lock  %s  Fltr 0.1 - 45 MHz  Sweeping  8bit", sAttn);
			else
				pThis->sInfo = NS("HFR Lock  %s  Fltr 0.1 - 45 MHz  Sweeping  8bit", sAttn);
		
			if(flags & JW_DATA_HFRLOCK_OSC_FREQ){
				pThis->pX = jw_PLL_oscillator_X;
				pThis->sXunit = jw_PLL_oscillator_unit;
			}
			else{
				pThis->pX = jw_HFR_spec_X;
				pThis->sXunit = jw_HFR_spec_unit;
			}
			pThis->Xtype  = FREQUENCY;
		
			if(jwaves_bytes2float4(pSci->uFmt, pPkt->getData(pPkt), convFlags, 
					                 &conv) != jw_HFR_spec_len)
				gse_error_v("Packet %d at 0x%X: Expected %d samples of %s data but "
				            "only received %d", nPktIdx+1, nOffset,
				            jw_HFR_spec_len, pThis->sChName, conv.len);
			pThis->bufY.ptr = conv.ptr;
			pThis->bufY.len = conv.len;
			
			if(flags & JW_DATA_HFRLOCK_OSC_FREQ){
				pThis->bufY.ptr = (float4*)gse_calloc(jw_HFR_spec_len, sizeof(float4));
				for(i = 0; i < jw_HFR_spec_len; i++)
					pThis->bufY.ptr[ jw_PLL_reorder_Y[i] ] = conv.ptr[i];
				free(conv.ptr);
				conv.ptr = NULL;
			}
			
			pThis->sYunit = "Counts";
			pThis->Ytype  = NUMBER;
			break;
		
		
		/* Info pattern is: Rec, Attn, Filter, Mixer, Converter, FFT */
		
		/* HFR Sweep spec */
		case 0x46: 
		case 0x47:
			if(pSci->uPSID == 0x46)
				pThis->sInfo = NS("HFWBR sweeps %s  Fltr 0.1 - 45 MHz", sAttn);
			else
				pThis->sInfo = NS("HFR sweeps %s  Fltr 0.1 - 45 MHz", sAttn);
		
			pThis->pX = jw_HFR_spec_X;
			pThis->sXunit = jw_HFR_spec_unit;
			pThis->Xtype  = FREQUENCY;
		
			if(jwaves_bytes2float4(pSci->uFmt, pPkt->getData(pPkt), convFlags, 
					                 &conv) != jw_HFR_spec_len)			
				gse_error_v("Packet %d at 0x%X: Expected %d samples of %s data but "
				            "only received %d", nPktIdx+1, nOffset,
				            jw_HFR_spec_len, pThis->sChName, conv.len);
			
			pThis->bufY.ptr = conv.ptr;
			pThis->bufY.len = conv.len;
			
			pThis->sYunit = "Counts";
			pThis->Ytype  = NUMBER;
			break;
			
		/* HFR baseband spec */	
		case 0x5F:
		case 0x5B:
			if(pSci->uPSID == 0x5F)
				pThis->sInfo = NS("HFR base  %s  Fltr 0.15-3.0 MHz",
						            sAttn);
			else
				pThis->sInfo = NS("HFWBR base  %s  Fltr 0.15-3.0 MHz",
						            sAttn);
			
			pThis->pX = jw_HFR_base_spec_X;
			pThis->sXunit = jw_HFR_base_spec_unit;
			pThis->Xtype  = FREQUENCY;
			
			if(jwaves_bytes2float4(pSci->uFmt, pPkt->getData(pPkt), convFlags, 
					                 &conv) != jw_HFR_base_spec_len)
				gse_error_v("Packet %d at 0x%X: Expected %d samples of %s data, "
				            "instead received %d", nPktIdx+1, nOffset,
				            jw_HFR_base_spec_len, pThis->sChName, conv.len);
			
			/* Do they want raw data? */
			if(JW_DATA_RAW_FLOATS & flags){
				pThis->Ytype = NUMBER;
				pThis->lo_limit = 0;
				pThis->hi_limit = 1.97132e+08; /* 2048*2048 * 47 */
				pThis->sYunit = "Raw Value";
			}
			else{
				for(i = 0; i < conv.len; i++){
					conv.ptr[i] = 2.0 * sqrt(conv.ptr[i]);           /* Get to counts */
					if(flags & JW_DATA_DIV_BANDWIDTH) 
						conv.ptr[i] /= jw_HFR_base_spec_bins[i];
					conv.ptr[i] = 20 * gse_log10_sub( conv.ptr[i]/32768, -10); /* Log */
				}
				
				pThis->sYunit = "dB";
				pThis->Ytype  = DB;
				pThis->lo_limit = -90;
				pThis->hi_limit = 10;
			}
			
			pThis->bufY.ptr = conv.ptr;
			pThis->bufY.len = conv.len;
			break;
		
		/* HFWBR baseband waveform */
		case 0x7B:
		case 0x7F:
		case 0x3B:
		case 0x3F:
			if((pSci->uPSID == 0x7B)||(pSci->uPSID == 0x3B))
				sRec = "HFWBR I";
			else
				sRec = "HFR I"; 
			
			if((pSci->uPSID == 0x3B)||(pSci->uPSID == 0x3F))
				pThis->sInfo = NS("%s  %s  Fltr 0.15 - 3.0 MHz  baseband 12bit DR",
							          sRec,sAttn);				
			else
				pThis->sInfo = NS("%s  %s  Fltr 0.15 - 3.0 MHz  baseband 12bit",
							          sRec,sAttn);
			
			pThis->pX = jw_WBR_base_wfrm_X;
			pThis->sXunit = "msec";
			pThis->Xtype  = TIME;
			
			jwaves_bytes2float4(pSci->uFmt, pPkt->getData(pPkt), convFlags, &conv);
			pThis->bufY.ptr = conv.ptr;
			pThis->bufY.len = conv.len;
			
			pThis->sYunit = "Counts";
			pThis->Ytype  = NUMBER;
			break;
		
		/* HFWBR Mixed Waveforms */
		case 0x88:
			JWavesXYdata_setupWBR(pThis, pSci, pPkt, "HFWBR", 'I', flags); break;
		case 0x89:
			JWavesXYdata_setupWBR(pThis, pSci, pPkt, "HFWBR", 'Q', flags); break;
		case 0x8C:
			JWavesXYdata_setupWBR(pThis, pSci, pPkt, "HFR", 'I', flags); break;
		case 0x8D:
			JWavesXYdata_setupWBR(pThis, pSci, pPkt, "HFR", 'Q', flags); break;
			
		/* Sideband recovered HFWBR Wavesforms */
		case 0xF1:
			JWavesXYdata_setupSide(pThis, pSci, pPkt, "HFWBR", flags); break;
		case 0xF2:
			JWavesXYdata_setupSide(pThis, pSci, pPkt, "HFR", flags); break;
								
		/* LFR Spectrums, and Noise Reduction Diagnostic Spectrum */
		case 0x91:
		case 0x92:
		case 0x93:
		case 0xF0:
		case 0xF4:
		case 0xF5:
		/*case 0x99:*/
			pThis->pX = jw_LFR_lo_spec_X;
			pThis->sXunit = jw_LFR_lo_spec_unit;
			expected_len = jw_LFR_lo_spec_len;
			pThis->Xtype  = FREQUENCY;
			pThis->uHints |= _GSE_HNT_SET_X_LOG;
			/* pThis->uHints |= _GSE_HNT_SET_Y_LOG; */
		
			pNoBins = jw_LFR_lo_spec_bins;
			
			/* LFR B Spectrum */
			if((pSci->uPSID == 0x91)||(pSci->uPSID == 0xF0))
				pThis->sInfo = NS("%s  Fltr .5 - 20 kHz  16bit  FFT", sAttn);
			
			/* LFR Lo E Spectrum */
			if((pSci->uPSID == 0x92)||(pSci->uPSID == 0xF4))
				pThis->sInfo = NS("%s  Fltr .5 - 20 kHz  16bit  FFT", sAttn);	
			
			/* LFR Hi E Spectrum */
			if((pSci->uPSID == 0x93)||(pSci->uPSID == 0xF5)){
				pThis->sInfo = NS("%s  Fltr 10 - 150 kHz  16bit  FFT", sAttn);
				pThis->pX = jw_LFR_hi_spec_X;
				pThis->sXunit = jw_LFR_hi_spec_unit;
				expected_len = jw_LFR_hi_spec_len;
				pNoBins = jw_LFR_hi_spec_bins;
			}
			
			/* Convert the data, note that for 0xF* data, the starting offset
			   and convert lengths are not obvious */
			if((pSci->uPSID & 0xF0) == 0x90)
				cbuf = pPkt->getData(pPkt);
			else
				cbuf = jwaves_loc_nrd_spec_data(pPkt, part);
						
			if( jwaves_bytes2float4(pSci->uFmt, cbuf, convFlags,
			                        &conv) != expected_len)
				gse_error_v("Packet %d at 0x%X: Expected %d samples of %s data "
				            "instead received %d", nPktIdx+1, nOffset,
				            expected_len, pThis->sChName, conv.len);
			
			pThis->bufY.ptr = conv.ptr;
			pThis->bufY.len = conv.len;
			
			
			/* Do they want raw data? */
			if(JW_DATA_RAW_FLOATS & flags){
				pThis->Ytype = NUMBER;
				pThis->lo_limit = 0;
				
				if((pSci->uPSID == 0x93)||(pSci->uPSID == 0xF5))
					pThis->hi_limit = 4.72446e+10; /* 32768*32768 * 44 */
				else
					pThis->hi_limit = 5.04659e+10; /* 32768*32768 * 47 */
				
				pThis->sYunit = "Raw Value";
			}
			else{
			
				/* Scale it */
				for(i = 0; i < pThis->bufY.len; i++){
					pThis->bufY.ptr[i] = 2.0 * sqrt(pThis->bufY.ptr[i]);          /* Get to counts */
					
					if(flags & JW_DATA_DIV_BANDWIDTH)               /* Div by band   */
						pThis->bufY.ptr[i] /= pNoBins[i];
					
					pThis->bufY.ptr[i] = 20*gse_log10_sub(pThis->bufY.ptr[i]/32768, -10); /* dB */
				}						
				pThis->lo_limit = -120;
				pThis->hi_limit = 10;
			
				pThis->sYunit = "Single Bin Response";
				pThis->Ytype  = DB;
			}
			break;
			
		/* LFR Waveforms, and Noise Reduction Diagnostic Wavesforms */
		case 0xA1:
		case 0xA2:
		case 0xA3:
		case 0xA0:
		case 0xA4:
		case 0xA5:
			/* LFR B Waveform */
			if((pSci->uPSID == 0xA1)||(pSci->uPSID == 0xA0)){
				pThis->sInfo = NS("%s  Fltr .5 - 20 kHz  16bit", sAttn);
				pThis->pX = jw_LFR_lo_wfrm_X;
			}
			/* LFR Lo E Waveform */
			if((pSci->uPSID == 0xA2)||(pSci->uPSID == 0xA4)){
				pThis->sInfo = NS("%s  Fltr .5 - 20 kHz  16bit", sAttn);
				pThis->pX = jw_LFR_lo_wfrm_X;
			}
			/* LFR Hi E Waveform */
			if((pSci->uPSID == 0xA3)||(pSci->uPSID == 0xA5)){ 
				pThis->sInfo = NS("%s  Fltr 10 - 150 kHz  16bit", sAttn);
				pThis->pX = jw_LFR_hi_wfrm_X;
			}
			
			/* Okay, part 4 (idx 3) of the NRD waveforms is something else altogether */
			if((part == 3)&&( 
			   (pSci->uPSID == 0xA0)||(pSci->uPSID == 0xA4)||(pSci->uPSID == 0xA5))){
				
				pThis->sXunit = "Coeff. Num.";
				pThis->Xtype = NUMBER;
				
				pThis->sYunit = "Coefficent";
				pThis->Ytype = NUMBER;
				
				cbuf = pPkt->getData(pPkt);
				pBytes = cbuf.ptr + cbuf.len - 128;
				
				pThis->bufY.len = 32;
				pThis->bufY.ptr = (float4*)gse_calloc(pThis->bufY.len, sizeof(float4));
				
				for(i=0;i<32;i++){
					pThis->bufY.ptr[i] = gse_f4FromLSBbytes(pBytes + i*4);
				}
				
				/* Override X axis to be the coefficent axis */
				pThis->pX = jw_LFR_coeff_X;
				pThis->lo_limit = 0.000001;   
				pThis->hi_limit = 10.0;
				/* pThis->uHints |= _GSE_HNT_SET_Y_LOG; */
			}
			else{	
				pThis->sXunit = "msec";
				pThis->Xtype  = TIME;
				
				jwaves_bytes2float4(pSci->uFmt, pPkt->getData(pPkt), convFlags, &conv);
				
				pThis->sYunit = "Counts";
				pThis->Ytype  = NUMBER;
				
				if((pSci->uPSID==0xA0)||(pSci->uPSID==0xA4)||(pSci->uPSID==0xA5)){
					/* Noise reduction diagnostics have three datasets per packet */
					if((part < 0)||(part > 2)) gse_throw(GSE_XCEPT, GSE_XBADARG);	
					
					if(conv.len % 3 != 0){
						free(conv.ptr);
						gse_throw_m(JW_XCEPT, JW_XNUMPTS, "Noise Reduction Diagnostic "
								       "packet number of samples is not a multiple of 3.");
					}	
					
					pThis->bufY.len = conv.len / 3;
					if(part == 2)
						pThis->bufY.len = pThis->bufY.len - 32; /* Coefficents are last part */
					
					offset = pThis->bufY.len * part;
					pThis->bufY.ptr = (float4*)gse_calloc(pThis->bufY.len, sizeof(float4));
				
					/*tricky mem copy, the pointers are to floats, but the count is bytes*/
					memcpy(pThis->bufY.ptr, conv.ptr + offset, pThis->bufY.len * 4); 
					free(conv.ptr);
				}
				else{
					pThis->bufY.ptr = conv.ptr;
					pThis->bufY.len = conv.len;
				}
			}
				
			break;
			
		default:
			gse_throw_fm(JW_XCEPT, JW_XPKTNOTSCI, "Packet ID 0x%X and Source ID "
			             "0x%X are not a recognized science data product.", 
			             (pSci->uPSID >> 4)& 0xF, pSci->uPSID & 0xF);
	}
	
	/* Flag setting, add ' dcmp' to packets that have been decompressed */ 
	if((flags & JW_DATA_MARK_DCMP)&&
	 pPkt->hasUniqueBlkByType(pPkt,GSE_PROGRAM_BLK_TYPE_ID,JW_DCMP_PROC_STEP_ID))
	{
		sCh = gse_string("%s dcmp", pBase->channel(pBase));
		if(pThis->sDynChID != NULL) free(pThis->sDynChID);
		pThis->sChID = NULL;
		pThis->sDynChID = sCh;
		
		sCh = gse_string("%s dcmp", pBase->channelName(pBase));
		if(pThis->sDynChName != NULL) free(pThis->sDynChName);
		pThis->sChName = NULL;
		pThis->sDynChName = sCh;

	}
	
	except_cleanup_pop(except_call);    /* sAttn */
	except_cleanup_pop(except_no_call); /* pThis */
	return pThis;
}

/****************************************************************************/
GseXYdataAry* fact_JWavesXYdataAry(const GsePkt* pPkt, uint4 flags,
                                   int4 nPktIdx, size_t nOffset)
{
	GseXYdataAry* pDatAry = NULL;
	const JWavesSDSblk*    pSci = NULL;
	
	if(! pPkt->hasUniqueBlkByType(pPkt, JW_SDS_BLK_TYPE_ID, 0) )
		return NULL;
	
	pDatAry = new_GseXYdataAry(0);
	except_cleanup_push(pDatAry->del, pDatAry);
	pSci = (const JWavesSDSblk*) pPkt->getBlkByType(pPkt, JW_SDS_BLK_TYPE_ID, 0);
	
	switch(pSci->uPSID){
		
	case 0xA0:
	case 0xA4:
	case 0xA5:
		/* KEEP DATA SETS IN ORDER OR waves_cal WILL FAIL!! */
		pDatAry->add(pDatAry, (GseXYdata*)new_JWavesXYdata(pPkt, 0, flags, nPktIdx, nOffset));
		pDatAry->add(pDatAry, (GseXYdata*)new_JWavesXYdata(pPkt, 1, flags, nPktIdx, nOffset));
		pDatAry->add(pDatAry, (GseXYdata*)new_JWavesXYdata(pPkt, 2, flags, nPktIdx, nOffset));
		pDatAry->add(pDatAry, (GseXYdata*)new_JWavesXYdata(pPkt, 3, flags, nPktIdx, nOffset));
		break;		
				
	case 0xF0:
	case 0xF4:
	case 0xF5:
		pDatAry->add(pDatAry, (GseXYdata*)new_JWavesXYdata(pPkt, 0, flags, nPktIdx, nOffset));
		pDatAry->add(pDatAry, (GseXYdata*)new_JWavesXYdata(pPkt, 1, flags, nPktIdx, nOffset));
		pDatAry->add(pDatAry, (GseXYdata*)new_JWavesXYdata(pPkt, 2, flags, nPktIdx, nOffset));
		break;

	case 0x21:  /* Q-Factors */
	case 0x26:  /* AGC Settings */
	case 0x27:
	case 0x44:  /* HFR PLL Lock Quality */
	case 0x45:
	case 0x46:  /* HFR Sweep Spectra */
	case 0x47:
	case 0x5F:  /* HFR Baseband Spectra */
	case 0x5B:
	case 0x7B:  /* HBWBR Baseband Waveforms */
	case 0x7F:
	case 0x3B:
	case 0x3F:
	case 0x88:  /* HFWBR I & Q "Waveforms" */
	case 0x89:
	case 0x8C:
	case 0x8D:
	case 0xF1:  /* HFWBR Narrow Band Spectra */
	case 0xF2:
	case 0x91:  /* LFR Spectra */
	case 0x92:
	case 0x93:
	case 0xA1:  /* LFR Waveforms */
	case 0xA2:
	case 0xA3:
		pDatAry->add(pDatAry, (GseXYdata*)new_JWavesXYdata(pPkt, -99, flags, nPktIdx, nOffset));
		break;
	
	/* Don't know what the heck this is, Willy must have added something new */
	default:
		pDatAry = NULL;
		break;
	}	
			
	except_cleanup_pop(except_no_call); 
	return pDatAry;
}















