#define _POSIX_C_SOURCE 200112

#ifdef GSE_POSIX_THREADS
#include <pthread.h>
#endif

#include <rpwgse/GseExcept.h>
#include <rpwgse/GseConv.h>

#include "JWavesDefs.h"
#include "JWavesDcmpBlk.h"

/*****************************************************************************/
bytebuf_t JWavesDcmpBlk_encode(GseBlock* pBase, byte uExtraLen, byte* pOffset)
{
	byte offset;
	bytebuf_t buf;

	JWavesDcmpBlk* pThis = (JWavesDcmpBlk*)pBase;
	GseProcBlk* pProc   = (GseProcBlk*)pBase;
	
	gse_overflowUinU(uExtraLen + 4, sizeof(byte)); 
	
	buf = GseProcBlk_encode(pBase, uExtraLen + 4, &offset);		
	
	/* Check base parsing */
	if(pProc->uStepID != JW_DCMP_PROC_STEP_ID)
		gse_throw_fm(GSE_XCEPT, GSE_XENCODE, "Processing step ID was %hhu"
				       "expected %hhu.", pProc->uStepID, JW_DCMP_PROC_STEP_ID);
	
	/* Fill in my own encoding */
	gse_f4ToMSBbytes(buf.ptr+offset, pThis->fCmpRatio);
	
	if(pOffset != NULL) *pOffset = offset + 4;
	return buf;
}

byte JWavesDcmpBlk_decode(GseBlock* pBase, cbytebuf_t buf)
{
	JWavesDcmpBlk* pThis = (JWavesDcmpBlk*)pBase;
	byte offset;
	
	
	if((offset = GseProcBlk_decode(pBase, buf)) != 12)
		gse_throw(GSE_XCEPT, GSE_XLIBINCOMPAT);
	 	
	if(pBase->uLen != 15)
		gse_throw_fm(GSE_XCEPT, GSE_XDECODE, "Expected 15 (i.e. 13 - 1) for the"
		             " Decomp block length, but recieved %hhu", pBase->uLen);
	
	/* Decoding */
	pThis->fCmpRatio = gse_f4FromMSBbytes(buf.ptr + offset);

	return offset + 4;		
}

bool JWavesDcmpBlk_merge(GseBlock* pBase, const GseBlock* pOther, uint4 uFlags)
{
	JWavesDcmpBlk* pThis = (JWavesDcmpBlk*)pBase;
	JWavesDcmpBlk* pDcmpOth = (JWavesDcmpBlk*)pOther;
			
	if(! GseProcBlk_merge(pBase, pOther, uFlags) ) return false;
	
	pThis->fCmpRatio = (pThis->fCmpRatio + pDcmpOth->fCmpRatio) / 2.0;
	
	return true;
}

GseBlock* JWavesDcmpBlk_copy(GseBlock* pBase, size_t uOverMalloc)
{
	JWavesDcmpBlk* pNew = NULL;
	JWavesDcmpBlk* pThis = (JWavesDcmpBlk*)pBase;
	size_t uSize = 0;
	 
 	if(uOverMalloc > sizeof(JWavesDcmpBlk))
		uSize = uOverMalloc;
	else
		uSize = sizeof(JWavesDcmpBlk);
	
	pNew = (JWavesDcmpBlk*)GseProcBlk_copy(pBase, uSize);
	pNew->fCmpRatio = pThis->fCmpRatio;
	pNew->del     = pThis->del;
	
	return (GseBlock*)pNew;
}

JWavesDcmpBlk* new_JWavesDcmpBlk(cbytebuf_t buf, size_t uOver)
{
	JWavesDcmpBlk* pThis = new_empty_JWavesDcmpBlk(uOver);
	
	if(buf.ptr == NULL) return pThis;
	
	except_cleanup_push(delete_GseBlock, pThis);
	
	JWavesDcmpBlk_decode((GseBlock*)pThis, buf);

	except_cleanup_pop(except_no_call);
	return pThis;
}

JWavesDcmpBlk* new_empty_JWavesDcmpBlk(size_t uOver)
{
	GseBlock* pBase = NULL;
	GseProcBlk* pProc = NULL;
	JWavesDcmpBlk* pThis = NULL;
	
	if(uOver < sizeof(JWavesDcmpBlk))
		pProc = new_GseProcBlk(CBBUF_NULL, sizeof(JWavesDcmpBlk));
	else
		pProc = new_GseProcBlk(CBBUF_NULL, uOver);
	
	pBase = (GseBlock*)pProc;
	pThis = (JWavesDcmpBlk*)pProc;
	
	/* Override a couple base class functions */
	pBase->encode = JWavesDcmpBlk_encode;
	pBase->decode = JWavesDcmpBlk_decode;
	pBase->_sName = "Waves Decompressor";
	pBase->copy   = JWavesDcmpBlk_copy;
	
	/* no need to override the base class "canMerge" as our data is 
	   always mergable */
	pBase->merge  = JWavesDcmpBlk_merge;
	
	pProc->uStepID = JW_DCMP_PROC_STEP_ID;
	
	pBase->_uMinLen += 4;
	
	/* Set new items */
	pThis->del = pProc->del;
	
	/* Rest of the defaults are all zero, so calloc gets this */
	return pThis;
}
