/** @file JWavesDcmpBlk.h */
#ifndef _JWavesDcmpBlk_H_
#define _JWavesDcmpBlk_H_

#include <rpwgse/GseProcBlk.h>

#ifdef __cplusplus
extern "C" {
#endif

/** @addtogroup mod_jwaves_pkt */
/*@{*/


/** Encoder/Decoder for Waves Decompression Processing Blocks.
 *
 * Processing Blocks record the details of operations on packets by ground
 * software.  The merge policy is to average the value of the compression
 * ratio.  Note that this only works well if only packets are combined.
 *
 * @class JWavesDcmpBlk
 * @extends GseProcBlk
 */
typedef struct jwaves_dcmp_blk{

	/** Base structure */
	GseProcBlk base;

	/** FIELD: The compression ratio
	 * 
	 * Contians the number of bytes in the un-compressed packet data section
	 * divided by the number of bytes in the up-stream rice compressed packet
	 * data section */
	float4 fCmpRatio;

	/** Destructor */
	void (*del)(void* vpThis);
	
} JWavesDcmpBlk;

/** Generate a new Juno Waves Decompressor processing block.
 * @public
 * @memberof JWavesDcmpBlk
 */
JWavesDcmpBlk* new_JWavesDcmpBlk(cbytebuf_t buf, size_t uOver);

/** Generate a new empty Juno Waves Decompressor Processing Block
 * @public
 * @memberof JWavesDcmpBlk
 */ 
JWavesDcmpBlk* new_empty_JWavesDcmpBlk(size_t uOver);

/*@}*/

#ifdef __cplusplus
}
#endif

#endif /* _JWavesDcmpBlk_H_ */
