/** @file JWavesDcmpFilter.h */
#ifndef _JWavesDcmpFilter_H_
#define _JWavesDcmpFilter_H_

#include <rpwgse/GsePktFilter.h>

#ifdef __cplusplus
extern "C" {
#endif

/** @addtogroup mod_jwaves_proc Processing */
/*@{*/

/** A Packet Source that decompresses packet data sections.
 *
 * The expected input is GSE packets of content type 0x83 that have been
 * unsegmented.  Packets that are:
 *
 *   -# Contain raw data.
 *   -# Segmented.
 *   -# Are not compressed.
 *
 * Are passed to the output unaltered.  Waves data is assumed to use
 * @ref gse_rice_decomp "Rice compression".
 *
 * @class JWavesDcmpFilter
 * @extends GsePktFilter
 */
typedef struct jwaves_dcmp_filter
{
	/** Enclosed base structure. */
	GsePktFilter base;
		
	/** Destructor */
	void (*del)(void* vpThis);
} JWavesDcmpFilter;

/** Constructor for a decompressing packet source.
 *
 * Generates a new decompressing packet source.  This filter decompresses
 * recognized packets before passing them on.
 *
 * @public
 * @memberof JWavesDcmpFilter
 *
 * @param pSrc A Source of Juno GSE packets with content type 0x83, other
 *        packet types maybe mixed in, but these are merely passed on 
 *        un-altered.  Segmented and Uncompressed packets are also passed on
 *
 * @param uProgID The program ID to attach to any generated packets.  All
 *        programs alterning Juno Waves data have a unique ID.
 *
 * @return A new GsePktSrc compatible structure that handles the 
 *        decompressing packets.
 *
 * @throws  varies  Almost any type of exception may be thrown during packet
 *         processing.
 */
JWavesDcmpFilter* new_JWavesDcmpFilter(GsePktSrc* pSrc, byte uProgID);

/*@}*/

#ifdef __cplusplus
}
#endif

#endif /* _JWavesDcmpFilter_H_ */
