#define _POSIX_C_SOURCE 200112

#ifdef GSE_POSIX_THREADS
#include <pthread.h>
#endif

#include <rpwgse/GseLog.h>
#include <rpwgse/GseRiceDecomp.h>
#include <rpwgse/GseWrap.h>
#include <rpwgse/GseConv.h>

#include "JWavesExcept.h"
#include "JWavesProc.h"
#include "JWavesSDSblk.h"
#include "JWavesConv.h"

/*****************************************************************************/
GsePkt* jwaves_decomp(const GsePkt* pPkt, byte uProgID)
{
	JWavesSDSblk* pSci;
	JWavesSDSblk* pNewSci;
	byte uDataKind, uDataCont;
	cbytebuf_t cbuf;
	int nBitWidth;
	struct rice_decomp_t decomp;
	GsePkt* pNewPkt;
	bytebuf_t newData;
	size_t i;
	
	uDataKind = (pPkt->pHdr->uContent >> 4) & 0x0F;
	uDataCont = pPkt->pHdr->uContent & 0x0F;

	if( uDataKind != JWAVES_KIND_SCI_DATA)
		gse_throw_fm(JW_XCEPT, JW_XPKTNOTSCI, "Non-science data sent to the "
		             "decompressor, Kind = 0x%02X", uDataKind);
	
	if( uDataCont == JWAVES_CNT_RAW_IDP)
		gse_throw_fm(JW_XCEPT, JW_XPKTNOTREADY, "Raw data sent to the "
		             "decompressor, Content = 0x%02X", uDataCont);

	if( uDataCont == JWAVES_CNT_CAPTURE){
		gse_debug("Data does not require decompression, content type "
		          "is all ready an uncalibrated capture."); 
		return NULL;
	}
	
	pSci = (JWavesSDSblk*) pPkt->getBlkByType(pPkt, JW_SDS_BLK_TYPE_ID, 0);
	
	if(pSci->isSegmented(pSci))
		gse_throw_m(JW_XCEPT, JW_XPKTNOTREADY, "Data must be unsegmented "
		             "before decompression");
	
	if(! pSci->isCompressed(pSci) ){
		gse_debug("Data is not compressed");
		return NULL;
	}
	
	/* Okay, data requires decompression, determine the bit mode */
	cbuf = pPkt->getData(pPkt);
	nBitWidth = jwaves_dataBlockBits(pSci->uFmt);
	if(! gse_rice_decomp(cbuf.ptr, cbuf.len, nBitWidth, &decomp) )
	{
		if(decomp.pMsg != NULL)
			gse_error_v("Decompression failed with error code %d, msg was: %s",
			            decomp.errorNum, decomp.pMsg);
		else
			gse_error_v("Decompression failed with error code %d, (no message)",
			            decomp.errorNum);
		
		return NULL;
	}
	
	except_cleanup_push(free, decomp.pDecomp);
	
	/* Make a new packet as a copy of the old one */
	pNewPkt = pPkt->copy(pPkt, 0);
	except_cleanup_push(pNewPkt->del, pNewPkt);
	
	newData.ptr = (byte*) gse_calloc(decomp.uSamples * 2, sizeof(byte));
	except_cleanup_push(free, newData.ptr);
	
	for(i = 0; i<decomp.uSamples; i++)
		gse_u2ToLSBbytes(newData.ptr + i*2, decomp.pDecomp[i]);
		
	pNewPkt->setData(pNewPkt, newData);
	except_cleanup_pop(except_no_call);  /* newData */
	
	/* Set the format */
	pNewSci = (JWavesSDSblk*)pNewPkt->getBlkByType(pNewPkt,JW_SDS_BLK_TYPE_ID,0);
	
	pNewSci->uFmt = jwaves_decompFmt(pSci->uFmt);
	pNewSci->setMPlen(pNewSci, decomp.uSamples * 2);
	
	except_cleanup_pop(except_no_call);  /* pNewPkt */
	except_cleanup_pop(except_call);     /* always delete decomp.pDecomp */
	
	return pNewPkt;
}








