/** @file JWavesDefs.h */
#ifndef _JWavesDefs_H_
#define _JWavesDefs_H_

/** @addtogroup mod_junogse */
/*@{*/
/** @defgroup mod_jwaves_defs Constants 
 *
 * Macro definitions for various fields in JWaves GSE Headers
 */
/*@}*/
/** @addtogroup mod_jwaves_defs */
/*@{*/

/** The Spice ID for the Juno Spacecraft */
#define JUNO_SPICE_ID -61


/** @defgroup mod_jwaves_defs_cnt Data Contents Indicator
 *
 * Constants for GseHdr::uContent
 */
/*@{*/

/** If the <b>upper 4 bits</b> contain this value the packet is Housekeeping
 * data */
#define JWAVES_KIND_HOUSEKEEPING 0x6

/** If the <b>upper 4 bits</b> contain this value the packet is process
 * data, (ex. Magnetometer readings) */
#define JWAVES_KIND_PROCESS 0x7

/** If the <b>upper 4 bits</b> contain this value the packet is science
 * data */
#define JWAVES_KIND_SCI_DATA 0x8

/** If the <b>upper 4 bits</b> contains this value the packet is MRO data */
#define JWAVES_KIND_MRO  0xE


/** If the <b>lower 4 bits</b> contain this value Packet data is Raw IDP */
#define JWAVES_CNT_RAW_IDP 0x1

/** If the <b>lower 4 bits</b> contain this value Packet data is one 
 * minipacket's data, and in may be segmented, compressed or glopped.
 * To see the current processing status consult the fields in the
 * JWavesSDSblk 
 */
#define JWAVES_CNT_MP_DATA 0x3

/** If the <b>lower 4 bits</b> contian this value packet data is a reassembled,
    decompressed capture ready for display or archiving */
#define JWAVES_CNT_CAPTURE 0x4

/** If the <b>lower 4 bits</b> contian this value packet data is ground,
    derived packet (a measurement) */
#define JWAVES_CNT_MEASUREMENT 0x5

/*@}*/



/** @defgroup mod_jwaves_defs_blktype Block Type IDs
 *
 * Constants for GseBlock::uType
 */
/*@{*/

/** Gse Block Type ID for Waves Science Data Status blocks */ 
#define JW_SDS_BLK_TYPE_ID  0x10 

/** Gse Block Type ID for waves MRO definition blocks */
#define JW_MRO_BLK_TYPE_ID 0x11


/** Gse Block Type ID for High Rate bin headers */
#define JW_HRS_BIN_HDR_ID 0x1A

/** Gse Block Type ID for Spacecraft Flight Data Systems attached CCSDS
    headers if present. */
#define JW_CCSDS_BLK_TYPE_ID 0x20

/*@}*/

/** @defgroup mod_jwaves_defs_process Processing Step
 *
 * Constants for GseProcBlk::uStepID
 */
/*@{*/

/** Generic Reader processing information block */
#define JW_RDR_PROC_STEP_ID 0x10

/** Generic Reader processing information block */
#define JW_SCELRS_PROC_STEP_ID 0x11

/** Generic Reader processing information block */
#define JW_SCEHRS_PROC_STEP_ID 0x12

/** FDM Reader processing information block */
#define JW_FDM_PROC_STEP_ID 0x13

/** SFDU Reader processing information block */
#define JW_SFDU_PROC_STEP_ID 0x14

/** Unpacker processing information block */
#define JW_UPACK_PROC_STEP_ID 0x20

/** Unsegmentor processing information block */
#define JW_USEG_PROC_STEP_ID 0x30

/** Decompressor processing information block */
#define JW_DCMP_PROC_STEP_ID 0x40

/** Deglopper processing information block */
#define JW_DGLOP_PROC_STEP_ID 0x50

/** Reformatter processing information block */
#define JW_REFMT_PROC_STEP_ID 0x60

/** HFWBR I/Q channel sideband recovery processing information block */
#define JW_SBR_PROC_STEP_ID 0x70

/** Calibrator processing information block */
#define JW_CAL_PROC_STEP_ID 0xE0


/*@}*/


/** @defgroup mod_jwaves_defs_telmsrc Telemetry Source
 *
 * Constants for upper 2 bits of JWavesSDSblk::uTsLo and JWavesSDSblk::uTsHi
 */
/*@{*/

/** Packet sent from the Waves Housekeeping telemetry task */
#define JW_TELEM_SRC_HSK  0x0 

/** Packet sent from the Waves Low Rate Science telemetry task */
#define JW_TELEM_SRC_LRS  0x1

/** Packet sent from the Waves High Rate Science telemetry task */
#define JW_TELEM_SRC_HRS  0x2
/*@}*/

/** @defgroup mod_jwaves_defs_fmt Data Formats
 *
 * Constants for the JWavesSDSblk::uFmt field
 */
/*@{*/

#define  JW_8BIT_RICE  				   0x14 
#define  JW_8BIT_IN_8BIT  			    0x4 
#define  JW_8BIT_IN_16BIT_lo 		   0x11 
   							  
#define  JW_12BIT_RICE 				   0x13 
#define  JW_12BIT_IN_8BIT_truc  	   0x10 
#define  JW_12BIT_IN_12BIT			    0x6 
#define  JW_12BIT_IN_16BIT_lo		    0x7 
#define  JW_12BIT_IN_32BIT_lo		    0x8 

#define  JW_16BIT_RICE 				    0x9 
#define  JW_16BIT_IN_8BIT_truc 	    0xA 
#define  JW_16BIT_IN_12BIT_truc	    0xB 
#define  JW_16BIT_IN_16BIT  		    0xC 
#define  JW_16BIT_IN_32BIT_lo  	    0xD 

#define  JW_9BIT_IN_12BIT_lo_RICE   0xE 
#define  JW_9BIT_IN_16BIT_lo_RICE   0xF 
#define  JW_9BIT_IN_9BIT  			    0x5 
#define  JW_9BIT_IN_12BIT_lo        0x15
#define  JW_9BIT_IN_16BIT_lo 		    0x3 
#define  JW_9BIT_IN_32BIT_lo 		   0x12 

#define  JW_32BIT_IN_32BIT			    0x2 

/*@}*/

/** @defgroup mod_jwaves_fltr_flags Filter Options
 * 
 * These all live in the same number space, and so any of these flags may
 * may be set for any filter in libjwaves and each filter will pick out 
 * just the settings they understand.
 */
/*@{*/

/** Direct JWavesUpkFilter to convert packet and source IDs
 *
 * We may have changed the offical packet ID and Source values for various
 * packet types.  This flag directs the unpacker to convert any old ID it
 * encounters to the newest versions. */
#define JW_FLTR_UPK_CONV_IDS    0x00001000

/* * Direct JWavesUpkFilter to echo a notice message for each packet
 * read from it's source.
 */
/* #define JW_FLTR_UPK_READ_NOTE   0x00002000 */

/** Direct JWavesSideFilter to retain I/Q packets in its output stream
 *
 * Normally when an I/Q waveform packet pair are encountered they are
 * converted to a single sideband spectrum.  This flag indicates that
 * both the derived packets and the input packets should be passed to the
 * output.
 */
#define JW_FLTR_SBR_KEEP_IQ     0x00010000

/** Synonym for @c true, useful when calling jwaves_check_idsrc() */
#define JWAVES_THROW true

/** Synonym for @c false, useful when calling jwaves_check_idsrc() */
#define JWAVES_LOG   false


/*@}*/

/*@}*/

#ifdef __cplusplus
extern "C" {
#endif

/** @addtogroup mod_jwaves_pkt */
/*@{*/

/** The telemetry source enumeration,
 * The values were derived form the Flight software assembly source at
 * https://saturn.physics.uiowa.edu/svn/juno_hw/DPU_FSW/tlm/lrsv2.asm
 * see line 340
 */
enum _jwaves_tlm_src{invalid = 0, hrs = 1, lrs = 2, hsk = 3};

/** The route the packets took out of the instrument */
typedef enum _jwaves_tlm_src jwaves_tlm_src;

/*@}*/

#ifdef __cplusplus
}
#endif

#endif /* _JWavesDefs_H_ */
