#define _POSIX_C_SOURCE 200112

#ifdef GSE_POSIX_THREADS
#include <pthread.h>
#endif

#include <rpwgse/GseExcept.h>
#include <rpwgse/GseConv.h>
#include <rpwgse/GseString.h>

#include "JWavesDefs.h"
#include "JWavesFDMblk.h"


bytebuf_t JWavesFDMblk_encode(GseBlock* pBlk, byte uExtraLen, byte* pOffset)
{
	size_t i;
	size_t nFNlen;
	byte offset;
	bytebuf_t buf;
	
	JWavesFDMblk* pThis = (JWavesFDMblk*)pBlk;
	GseProcBlk* pProc   = (GseProcBlk*)pBlk;
	
	/* Check prior to encode */
	if(pProc->uStepID != JW_FDM_PROC_STEP_ID)
		gse_throw_fm(GSE_XCEPT, GSE_XENCODE, "Processing step ID was %hhu"
				       "expected %hhu.", pProc->uStepID, JW_FDM_PROC_STEP_ID);
	
	gse_overflowUinU(uExtraLen + 47, sizeof(byte));
	
	buf = JWavesRdrBlk_encode(pBlk, uExtraLen + 47, &offset);
	gse_assert(buf.len >= offset + 47);
	
	if(pThis->uCharWidth > 15) pThis->uCharWidth = 15;
	
	nFNlen = strlen(pThis->sFileName);
	if(pThis->sFileName && (nFNlen > 45))
		buf.ptr[offset] = (pThis->uCharWidth | 0x10) & 0xFF;
	else
		buf.ptr[offset] = pThis->uCharWidth & 0xFF;
	
	
	for(i = 0; i < 45; i++){
		if(i < nFNlen)
			buf.ptr[offset + 1 + i ] = pThis->sFileName[i];
		else
			buf.ptr[offset + 1 + i ] = '\0';
	}
	
	buf.ptr[offset + 46] = '\0';
	
	if(pOffset != NULL) *pOffset = offset + 47;
	return buf;
}

byte JWavesFDMblk_decode(GseBlock* pBlk, cbytebuf_t buf)
{
	GseProcBlk*   pProc = (GseProcBlk*)pBlk;
	JWavesFDMblk* pThis = (JWavesFDMblk*)pBlk;
	
	size_t nFNlen;
	byte uOffset;
	
	uOffset = JWavesRdrBlk_decode(pBlk, buf);
	
	if(pProc->uStepID != JW_FDM_PROC_STEP_ID)
		gse_throw_fm(GSE_XCEPT, GSE_XDECODE, "Expected 0x%hhX for the block step "
		             "ID, but recieved 0x%hhX", JW_FDM_PROC_STEP_ID,
				       pProc->uStepID);
	
	if( (buf.ptr[uOffset] >> 4) & 0x1 )
		pThis->bTrunc = true;
	else
		pThis->bTrunc = false;
	
	pThis->uCharWidth = buf.ptr[uOffset] & 0x0F;
	if(pThis->uCharWidth != 1)
		gse_throw_m(GSE_XCEPT, GSE_XNOTYETIMP, "Handling multi-byte characters"
	               " is not yet implemented");
	
	nFNlen = strlen( (char*) (buf.ptr + uOffset + 1) );
	
	if(nFNlen > 0)
		pThis->sFileName = gse_strcopy( (char*) (buf.ptr + uOffset + 1) );
	else
		pThis->sFileName = NULL;
	
	return uOffset + 47;
}

bool JWavesFDMblk_canMerge(GseBlock* pBase, const GseBlock* pOther, uint4 uFlags)
{
	JWavesFDMblk* pThis = (JWavesFDMblk*)pBase;	
	JWavesFDMblk* pFDMoth = (JWavesFDMblk*)pOther;
	
	if(! JWavesRdrBlk_canMerge(pBase, pOther, uFlags) ) return false;
	
	if( (pThis->bTrunc     != pFDMoth->bTrunc) || 
	    (pThis->uCharWidth != pFDMoth->uCharWidth) )  return false;
		
	return strcmp(pThis->sFileName, pFDMoth->sFileName) == 0 ? true : false;   
}

/* Copy constructor */
GseBlock* JWavesFDMblk_copy(GseBlock* pBase, size_t uOver)
{
	JWavesFDMblk* pNew = NULL;
	JWavesFDMblk* pThis = (JWavesFDMblk*)pBase;
	
	if(uOver < sizeof(JWavesFDMblk))
		pNew = (JWavesFDMblk*) JWavesRdrBlk_copy(pBase, sizeof(JWavesFDMblk));
	else
		pNew = (JWavesFDMblk*) JWavesRdrBlk_copy(pBase, uOver); 
	
	pNew->bTrunc    = pThis->bTrunc;
	pNew->uCharWidth = pThis->uCharWidth;
	pNew->sFileName  = gse_strcopy(pThis->sFileName);
	
	pNew->del       = pThis->del;
	
	return (GseBlock*)pNew;
}

/* Get rid of our string */
void delete_JWavesFDMblk(void* vpThis)
{
	JWavesFDMblk* pThis = (JWavesFDMblk*)vpThis;
	JWavesRdrBlk* pBase = (JWavesRdrBlk*)pThis;
	
	free(pThis->sFileName);
	pBase->del(vpThis);
}

JWavesFDMblk* new_JWavesFDMblk(cbytebuf_t buf, size_t uOver)
{
	GseBlock*     pBase = NULL;
	GseProcBlk*   pProc = NULL;
	JWavesRdrBlk* pRdr  = NULL;
	JWavesFDMblk* pThis = NULL;
	
	if(uOver < sizeof(JWavesFDMblk))
		pRdr = new_empty_JWavesRdrBlk(sizeof(JWavesFDMblk));
	else
		pRdr = new_empty_JWavesRdrBlk(uOver); 
	
	pBase = (GseBlock*)pRdr;
	pBase->encode  = JWavesFDMblk_encode;
	pBase->decode  = JWavesFDMblk_decode;
	pBase->copy    = JWavesFDMblk_copy;
	pBase->del     = delete_JWavesFDMblk;
	
	pBase->_sName  = "Waves JPL-FDM Reader";
	pBase->_uMinLen += 47;
	
	pBase->canMerge = JWavesFDMblk_canMerge;
	/* Since merge can only happen if the fields are identical, no reason to 
	   have a merge override, it would only call the base clase anyway       */
	
	pProc = (GseProcBlk*)pRdr;
	pProc->uStepID = JW_FDM_PROC_STEP_ID;
	
	pRdr->uSource = 0x03;
	
	pThis = (JWavesFDMblk*)pRdr;
	pThis->del     = delete_JWavesFDMblk;
	pThis->uCharWidth = 0x01;
	
	/* Calloc in base makes the rest of this 0, as we want */
	
	if(buf.ptr == NULL) return pThis;
	
	except_cleanup_push(pThis->del, pThis);
	
	JWavesFDMblk_decode(pBase, buf);

	except_cleanup_pop(except_no_call);
	
	return pThis;
}
