#define _POSIX_C_SOURCE 200112

#ifdef GSE_POSIX_THREADS
#include <pthread.h>
#endif

#include <rpwgse/GseExcept.h>
#include <rpwgse/GseWrap.h>
#include <rpwgse/GseLog.h>
#include <rpwgse/GseString.h>

#include "JWavesDefs.h"
#include "JWavesFilter.h"
#include "JWavesUpkFilter.h"
#include "JWavesUsegFilter.h"
#include "JWavesDcmpFilter.h"
#include "JWavesSideFilter.h"

/****************************************************************************/
bool JWavesFilter_false(GsePktSrc* pBase)
{
	return false;
}

/****************************************************************************/
/* Basic error packet catcher setup */
void JWavesFilter_setCatcher(GsePktFilter* pBase,
                             gse_errpkt_hndlr_t pFunc, void* pCatchCtx)
{
	JWavesFilter* pThis = (JWavesFilter*) pBase;
	GsePktFilter* pFilter;
	int i;
	
	pBase->pCatcher = pFunc;
	pBase->pCatchCtx = pCatchCtx;
	
	for(i = 0; i<pThis->nFltrs; i++){
		pFilter = (GsePktFilter*)pThis->pFltrs[i];
		pFilter->setCatcher(pFilter, pFunc, pCatchCtx);
	}
		
}

/*****************************************************************************/
GsePkt* JWavesFilter_readNext(GsePktSrc* pBase){
	JWavesFilter* pThis = (JWavesFilter*)pBase;
	GsePktFilter* pGseFltr = (GsePktFilter*)pBase;
	
	GsePktSrc* pFltr = pThis->pFltrs[pThis->nFltrs - 1];
	GsePkt* pPkt;
	char* sNotice;
	
	pFltr = pThis->pFltrs[pThis->nFltrs - 1];  /* Call the top most filter */
	
	while( (pPkt = pFltr->readNext(pFltr)) != NULL){
		
		/* Pass on the two known final packet types */
		if((pPkt->pHdr->uContent == 0x85) || (pPkt->pHdr->uContent == 0xE4))
			break;
		
		if((pThis->uFlags & JW_FLTR_SBR_KEEP_IQ)&&(pPkt->pHdr->uContent == 0x84))
			break;
		
		/* Handle packets that are not final */
		if(pGseFltr->pCatcher != NULL){
			
			sNotice = NS("Passing non-final packet, number %d at offset 0x%X "
			            "in %s, to a straggler handler.", pBase->pktIdx(pBase),
			            pBase->offset(pBase), pBase->name(pBase) );
			
			gse_notice(sNotice);
			
			if(! pGseFltr->pCatcher(pGseFltr->pCatchCtx, pPkt, sNotice) ){
				pPkt = NULL;
				break;
			}
		}
		else{
			gse_error_v("Dropping non-final packet number %d at offset 0x%X in %s",
			            pBase->pktIdx(pBase), pBase->offset(pBase), 
			            pBase->name(pBase) );
					
			pPkt->del(pPkt);
		}		
	}
	
	return pPkt;
}

/*****************************************************************************/
void delete_JWavesFilter(void* vpThis)
{
	free(vpThis);
}

/*****************************************************************************/
JWavesFilter* new_JWavesFilter(GsePktSrc* pSrc, byte uProgID, uint4 uFlags)
{
	JWavesFilter* pThis = NULL;
	GsePktFilter* pBase = NULL;
	
	pThis = (JWavesFilter*)new_GsePktFilter(
			    pSrc, 
	          uProgID,
	          sizeof(JWavesFilter),
	          JWavesFilter_readNext,
	          delete_JWavesFilter
	        );
	pBase = (GsePktFilter*)pThis;
	
	
	/* Get my stuff */
	except_cleanup_push(delete_JWavesFilter, pThis);
	
	pBase->setCatcher     = JWavesFilter_setCatcher;
	
	pThis->uFlags         = uFlags;
	pThis->del            = delete_JWavesFilter;
	
	pThis->nFltrs = 4;
	pThis->pFltrs = (GsePktSrc**)gse_calloc(pThis->nFltrs, sizeof(GsePktSrc*));
	
	/* Unpacker */
	pThis->pFltrs[0] = (GsePktSrc*)new_JWavesUpkFilter(pSrc, uProgID, uFlags);
	
	/* Unsegmentor */
	pThis->pFltrs[1]=(GsePktSrc*)new_JWavesUsegFilter(pThis->pFltrs[0],uProgID);
	
	/* Decompressor */
	pThis->pFltrs[2]=(GsePktSrc*)new_JWavesDcmpFilter(pThis->pFltrs[1],uProgID);
		
	/* DeGlopper ??? */
	
	
	/* Sideband recovery */
	pThis->pFltrs[3]=(GsePktSrc*)new_JWavesSideFilter(pThis->pFltrs[2],uProgID,
			                                            uFlags);

		
	except_cleanup_pop(except_no_call);
	return pThis;		
}
