/** @file JWavesFilter.h */
#ifndef _JWavesFilter_H_
#define _JWavesFilter_H_

#include <rpwgse/GsePktFilter.h>

#ifdef __cplusplus
extern "C" {
#endif

/** @addtogroup mod_jwaves_proc Processing */
/*@{*/

/** Packet source filter that handles all Juno Waves ground proccessing steps.
 * 
 * 
 * @class JWavesFilter
 * @extends GsePktFilter
 */
typedef struct jwaves_filter
{
	/** Base Structure */
	GsePktFilter base;
	
	/** Processing flags see @ref mod_jwaves_fltr_flags. */
	uint4 uFlags;
	
	/* Private internal single stage processing filters */
	GsePktSrc** pFltrs;
	size_t nFltrs;
	
	/** Destructor */
	void (*del)(void* vpThis);
	
} JWavesFilter;

/** Constructor for the all-in-one packet filter
 *
 * @param pSrc A Source of Juno GSE packets with any content type.  Packets
 *        that do not require processing are merely passed on.
 *
 * @param uProgID The ID of calling program.  All programs altering Juno Waves
 *        data are required to have a unique ID.
 *
 * @param uFlags Processing flags see @ref mod_jwaves_fltr_flags
 *
 * @public
 * @memberof JWavesFilter
 * 
 */
JWavesFilter* new_JWavesFilter(GsePktSrc* pSrc, byte uProgID, uint4 uFlags);

/*@}*/

#ifdef __cplusplus
}
#endif

#endif /* _JWavesFilter_H_ */
