#define _POSIX_C_SOURCE 200112

#ifdef GSE_POSIX_THREADS
#include <pthread.h>
#endif

#include <rpwgse/GseConv.h>
#include <rpwgse/GseExcept.h>
#include <rpwgse/GseLog.h>

#include "JWavesHrbhBlk.h"


/****************************************************************************/
bytebuf_t JWavesHrbhBlk_encode(GseBlock* pBase, byte uExtraLen, byte* pOffset)
{
	JWavesHrbhBlk* pThis = (JWavesHrbhBlk*)pBase;
	int i;
	bytebuf_t buf;
	byte uOffset;

	buf = GseBlock_encode(pBase, 22, &uOffset);
	gse_assert( buf.len >= 22 + uOffset);
	
	gse_u2ToMSBbytes(buf.ptr + uOffset, pThis->uPktNo);
	gse_u4ToMSBbytes(buf.ptr + uOffset + 2, pThis->uQFsec);
	gse_u2ToMSBbytes(buf.ptr + uOffset + 6, pThis->uQFsubSec);
	gse_u2ToMSBbytes(buf.ptr + uOffset + 8, pThis->uJunoProcSubSec);
	gse_u4ToMSBbytes(buf.ptr + uOffset + 10, pThis->uJunoProcSec);
	gse_u4ToMSBbytes(buf.ptr + uOffset + 14, pThis->uPreEventWords);
	buf.ptr[uOffset + 18] = pThis->uQF;
	for(i=0; i<3; i++) buf.ptr[uOffset + 19 + i] = 0;
	
	/* Tell caller how many bytes I used up */
	if(pOffset != NULL) *pOffset = uOffset + 22;
	
	/* Return my encoded bytes */
	return buf;
}

/****************************************************************************/
byte JWavesHrbhBlk_decode(struct gse_block* pBase, cbytebuf_t buf)
{
	JWavesHrbhBlk* pThis = (JWavesHrbhBlk*)pBase;
	byte uOffset;
	
	uOffset = GseBlock_decode(pBase, buf);
	gse_assert(buf.len >= 22 + uOffset);
	
	pThis->uPktNo          = gse_u2FromMSBbytes(buf.ptr + uOffset);
	pThis->uQFsec          = gse_u4FromMSBbytes(buf.ptr + uOffset + 2);
	pThis->uQFsubSec       = gse_u2FromMSBbytes(buf.ptr + uOffset + 6);
	pThis->uJunoProcSubSec = gse_u2FromMSBbytes(buf.ptr + uOffset + 8);
	pThis->uJunoProcSec    = gse_u4FromMSBbytes(buf.ptr + uOffset + 10);
	pThis->uPreEventWords  = gse_u4FromMSBbytes(buf.ptr + uOffset + 14);
	pThis->uQF             = buf.ptr[uOffset + 18];
	
	/* Tell caller how many bytes wer read from the buffer */
	return uOffset + 22;
}

/****************************************************************************/
GseBlock* JWavesHrbhBlk_copy(GseBlock* pBase, size_t uOverMalloc)
{
	JWavesHrbhBlk* pNew;
	JWavesHrbhBlk* pThis = (JWavesHrbhBlk*)pBase;	
	size_t uSize = 0;
	 
 	if(uOverMalloc > sizeof(JWavesHrbhBlk))
		uSize = uOverMalloc;
	else
		uSize = sizeof(JWavesHrbhBlk);
	
	pNew  = (JWavesHrbhBlk*)GseBlock_copy(pBase, uSize);

	pNew->uPktNo          = pThis->uPktNo;
	pNew->uQFsec          = pThis->uQFsec;
	pNew->uQFsubSec       = pThis->uQFsubSec;
	pNew->uJunoProcSec    = pThis->uJunoProcSec;
	pNew->uJunoProcSubSec = pThis->uJunoProcSubSec;
	pNew->uPreEventWords  = pThis->uPreEventWords;
	pNew->uQF             = pThis->uQF;

	pNew->del = pThis->del;
	
	return (GseBlock*)pNew;
}

/****************************************************************************/
JWavesHrbhBlk* new_JWavesHrbhBlk(cbytebuf_t buf){
	GseBlock* pBase = NULL;
	JWavesHrbhBlk* pThis = NULL;
	
	pBase = new_GseBlock(CBBUF_NULL, sizeof(JWavesHrbhBlk));
	pThis = (JWavesHrbhBlk*)pBase;
	
	/* Override base class items */
	/* No new dynamic allocation so just use the base destructor */
	pBase->encode = JWavesHrbhBlk_encode;
	pBase->decode = JWavesHrbhBlk_decode;
	pBase->_sName = "Juno High Rate Bin Header";
	pBase->uType  = JW_HRS_BIN_HDR_ID;
	pBase->copy   = JWavesHrbhBlk_copy;
	
	pBase->_uMinLen = 24;
	
	/* New Items, all default to 0 */
	
	pThis->del    = delete_GseBlock;
	
	if(buf.ptr == NULL)
		return pThis;
	
	except_cleanup_push(delete_GseBlock, pThis);
	
	JWavesHrbhBlk_decode(pBase, buf);

	except_cleanup_pop(except_no_call);
	
	return pThis;
}
