/** @file JWavesHrbhBlk.h */
#ifndef _JWavesHrbhBlk_H_
#define _JWavesHrbhBlk_H_


#include <rpwgse/GseBlock.h>
#include <jwaves/JWavesDefs.h>


#ifdef __cplusplus
extern "C" {
#endif


/** @addtogroup mod_jwaves_pkt */
/*@{*/

/** Interface for Juno High Rate Bin Header blocks
 * These contain the Q-factor for binned data, amoung other fields.  See
 * section L3-Wav-788 of the Waves Instrument to Spacecraft ICD, table
 * 3.5-13 "Saved Bin Header Format and Content" for details of the 
 * fields herein.
 *
 * @class JWavesHrbhBlk
 * @extends GseBlock
 */
typedef struct jwaves_hrbh_blk{

	/** Base structure */
	GseBlock base;

	/** FIELD: The packet number within the bin, only available for
	           for burst data */
	uint2 uPktNo;
	
	/** FIELD: The seconds field of the timestamp sent by Waves with the
	  * quality factor that caused this bin to be saved.  My be 0 for
	  * high rate record mode data.
	  */
	uint4 uQFsec;
	
	/** FEILD: The subseconds field corresponding to @a uQFsec.  Each
	 *  count in this filed represents 1/(2^16) seconds.  May be 0 for
	 *  high rate record mode data.
	 */
	uint2 uQFsubSec;
		
	/** FIELD: The seconds field of the time when either 
	 *  
	 *     -# the spacecraft software processed the Waves quality factor that
	 *        caused the bin, or 
	 *     -# for bins with data with no quality factor, the time when the bin
	 *        completed filling.
	 */
	uint4 uJunoProcSec;
	
	/** FIELD: The subseconds field corresponding to @a uProcSec. Each
	 *  count in this filed represents 1/(2^16) seconds.
	 */
	uint2 uJunoProcSubSec;

	
	/** FIELD: Number of words in the saved bin preceding the quality point.
	 * Nominally will be the configured value, but may be fewer if there are
	 * fewer words in the bin when the Waves quality factor is received.  Will
	 * be 0 if the Waves quality factor is 0, indicating no quality factor.
	 */
	uint4 uPreEventWords;
	
	/** FIELD: Quality Factor associated with the saved bin as received from
	 *  Waves, or 0 if there is no quality factor (record mode data).
	 */
	byte uQF;
	

	/** Destructor */
	void (*del)(void* vpThis);

} JWavesHrbhBlk;


/** Generate a new Juno Waves HRBH (High Rate Bin Header) block.
 * @public
 * @memberof JWavesHrbhBlk
 */
JWavesHrbhBlk* new_JWavesHrbhBlk(cbytebuf_t buf);

/*@}*/

#ifdef __cplusplus
}
#endif



#endif /* _JWavesHrbhBlk_H_ */
