/** @file JWavesLRSblk.h */
#ifndef _JWavesLRLblk_H_
#define _JWavesLRLblk_H_

#include <jwaves/JWavesRdrBlk.h>

/** @addtogroup mod_jwaves_pkt */
/*@{*/

#ifdef __cplusplus
extern "C" {
#endif

/** Encoder/Decoder for Waves SCE, LRS reader blocks.
 * This block is present if the data was originally read from the Lockheed 
 * Martin Spacecraft Emulator.
 * @n
 * @class JWavesLRSblk
 * @extends JWavesRdrBlk
 */
typedef struct jwaves_lrs_blk{
	
	/** Base structure */
	JWavesRdrBlk base;
	
	/** FIELD: Lower 24 bits of the user UID on the system that read the data */
	uint4 uUID;
	
	/** FIELD: Remote (SCE) socket IP */
	uint4 uRemoteIP;
	
	/** FIELD: Local (Workstation) socket IP */
	uint4 uLocalIP;
	
	/** FIELD: The @ref ip_header which was output by Waves to the Juno FDS */
	byte IPhdr[20];
	
	/** FIELD: The @ref udp_header which wavs output by Waves to the Juno FDS */
	byte UDPhdr[8];
	
	/** FIELD: The @ref cip_header which was outpt by waves to the Juno FDS */
	byte CIPhdr[8];
	
	/** Destructor */
	void (*del)(void* vpThis);
	
} JWavesLRSblk;

/** Generate a new Juno Waves SCE LRS Reader Processing Block 
 * @public
 * @memberof JWavesLRSblk
 */
JWavesLRSblk* new_JWavesLRSblk(cbytebuf_t buf, size_t uOver);


/*@}*/
		
#ifdef __cplusplus
}
#endif


#endif /* _JWavesLRLblk_H_ */
