#define _POSIX_C_SOURCE 200112

#ifdef GSE_POSIX_THREADS
#include <pthread.h>
#endif

#include <rpwgse/GseConv.h>
#include <rpwgse/GseLog.h>
#include <rpwgse/GseWrap.h>

#include "JWavesConv.h"
#include "JWavesMP.h"

/*****************************************************************************/
cbytebuf_t JWavesMP_getHdr(JWavesMP* pThis){
	return const_bytebuf(pThis->_hdr);
}

cbytebuf_t JWavesMP_getData(JWavesMP* pThis){
	return const_bytebuf(pThis->_data);
}

bytebuf_t JWavesMP_takeData(JWavesMP* pThis)
{
	bytebuf_t buf;
	
	buf = pThis->_data;
	pThis->_data.ptr = NULL;
	pThis->_data.len = 0;
	return buf;
}

byte JWavesMP_getHdrLen(JWavesMP* pThis){
	byte uRet;
	gse_storeSinU(pThis->_hdr.len, &uRet, sizeof(uRet));
	return uRet;
}

uint2 JWavesMP_getDataLen(JWavesMP* pThis){
	uint2 uRet;
	gse_storeSinU(pThis->_data.len, &uRet, sizeof(uRet));
	return uRet;
}

uint4 JWavesMP_collectSCLK(JWavesMP* pThis, uint4 uRepSCLK){
	
	return jwaves_recover_sclk(pThis->uRTI, uRepSCLK);
}


/* Old Version of collectSCLK determination ...
uint4 JWavesMP_collectSCLK(JWavesMP* pThis, uint4 uRepSCLK)
{
	uint4 uSCLK_collect_lo_10bits;
	uint4 uSCLK_report_lo_10bits;
	uint4 uSCLK_report_hi_22bits;
	uint4 uSCLK_collect;
	
	/ * Git rid of the time quality field * /
	uint4 _uRepSCLK = 0xFFFFFFFE & uRepSCLK;
	
	uSCLK_collect_lo_10bits = pThis->uRTI / 40;
	
	delta = 
	
	uSCLK_report_lo_10bits = _uRepSCLK & 0x000003FF;
	uSCLK_report_hi_22bits = _uRepSCLK & 0xFFFFFC00;
	
	if(uSCLK_collect_lo_10bits <= uSCLK_report_lo_10bits)
		uSCLK_collect = uSCLK_report_hi_22bits | uSCLK_collect_lo_10bits;
	else
		uSCLK_collect = (uSCLK_report_hi_22bits - (0x3FF + 1)) | 
			              uSCLK_collect_lo_10bits;
	
	return uSCLK_collect;
} 
*/
		
/*****************************************************************************/
/* Decoding helper */
		
void JWavesMP_readCommon(JWavesMP* pThis, cbytebuf_t cbuf)
{
	int nMSF;
	int i;
	
	if(cbuf.len < 8)
		gse_throw_fm(GSE_XCEPT, GSE_XDECODE, "At least 8 bytes are required to "
		             "decode basic Minipacket header fields, but cbuf.len is "
		             "%zd", cbuf.len);
	
	/* Don't set the uType, that's up to the derived class */
	pThis->uID = (cbuf.ptr[0] >> 4)&0x0F;
	pThis->uLen = gse_u2FromMSBbytes(cbuf.ptr) & 0x0FFF;
	pThis->uRTI = gse_u2FromMSBbytes(cbuf.ptr + 2);
	
	nMSF = ( cbuf.ptr[7] >> 6) & 0x03;
	switch(nMSF){
		case 0: nMSF = 0; break;
		case 1: nMSF = 2; break;
		case 2: nMSF = 4; break;
		case 3: nMSF = 8; break;
	}
	
	if(nMSF == 0){
		pThis->bufMSF.ptr = NULL;
		pThis->bufMSF.len = 0;
		return;
	}
	
	if(cbuf.len < 8 + nMSF)
		gse_throw_fm(GSE_XCEPT, GSE_XDECODE, "Input cbuf is too short to "
		             "contain a standard header plus %d MSF bytes", nMSF);
	
	pThis->bufMSF.ptr = pThis->_msf;
	pThis->bufMSF.len = nMSF;
		
	for(i = 0; i<nMSF; i++)
		pThis->_msf[i] = cbuf.ptr[8 + i];

}

/*****************************************************************************/
void delete_JWavesMP(void* vpThis)
{
	JWavesMP* pThis = (JWavesMP*)vpThis;
	if(pThis->_hdr.ptr != NULL) free(pThis->_hdr.ptr);
	if(pThis->_data.ptr != NULL) free(pThis->_data.ptr);
	free(vpThis);
}


JWavesMP* new_JWavesMP(size_t uMalloc)
{
	JWavesMP* pThis;
	
	if(uMalloc > sizeof(JWavesMP))
		pThis = (JWavesMP*)gse_calloc(1, uMalloc);
	else
		pThis = (JWavesMP*)gse_calloc(1, sizeof(JWavesMP));
	
	pThis->del = delete_JWavesMP;
	pThis->getHdr = JWavesMP_getHdr;
	pThis->getHdrLen = JWavesMP_getHdrLen;
	pThis->getData = JWavesMP_getData;
	pThis->takeData = JWavesMP_takeData;
	pThis->getDataLen = JWavesMP_getDataLen;
	pThis->collectSCLK = JWavesMP_collectSCLK;
	
	return pThis;
}

/*****************************************************************************/
/* Instantiate the JWavesMPptrAry Template */

#include <assert.h>

#define C JWavesMPptrAry
#define T JWavesMP
#define CASCADE_DEL

#include <rpwgse/ptrAryTplt.c>

#undef CASCADE_DEL
#undef C
#undef T


