/** @file JWavesMroMP.h */
#ifndef _JWavesMroMP_H_
#define _JWavesMroMP_H_

#include <jwaves/JWavesMP.h>

#ifdef __cplusplus
extern "C" {
#endif

/** @addtogroup mod_jwaves_raw */
/*@{*/

/** Contains the header and data buffers for a memory readout data minipackets
 * @n
 * @class JWavesMroMP
 * @extends JWavesMP
 */

typedef struct jwaves_mro_mp{

	/** Base structure containing the common minipacket fields.
	 * @note The base.uID field should always be 0xE for MRO packets.
	 */
	JWavesMP base;

   /** FIELD: The lowest memory address of the dumped data.
	 * Since MRO packets are generated by the Y180, the valid range of
	 * memory addresses is: 0 to 0xFFFFF (20-bit memory range.)
	 */
	uint4 uAddress;
	
	/** FIELD: The WvFE memory frame number. 
	 * Due to the instrument memory architecture, this field is meaningful
	 * only if the @a uAddress field is greater than or equal to 0x80000.
	 */
	byte uFrame;
	
	/** FIELD: The Y180 memory region.
	 * Due to the instrument memory architecture, this field is meaningful
	 * only if the @a uAddress field is less than 0x80000.
	 */
	byte uReg;
	
	/** Destructor */
	void (*del)(void* vpThis);
	
} JWavesMroMP;


/** Constructor for new Memory Read Out Minipacket data holder
 *
 * @param bulkBuf a buffer containing the minipacket header in the 
 *        minipacket data.
 *
 * @param pOffset is used for both input and output.  Byte decoding starts
 *        @c *pOffset bytes into @c cbuf.ptr.  After decoding, 
 *        *pOffset is incremented by the number of bytes read.
 * 
 * @memberof JWavesMroMP
 */
JWavesMroMP* new_JWavesMroMP(cbytebuf_t bulkBuf, size_t* pOffset);

/*@}*/

#ifdef __cplusplus
}
#endif

#endif /* _JWavesMroMP_H_ */
