/* JWavesMultiFileSrc */

%feature("pythonappend") JWavesMultiFileSrc::next(void) %{
	if $result == None:
		raise StopIteration
%}

%inline %{

#include <rpwgse/GsePkt.hpp>

/* Definitions for the JWavesMultiFileSrc constructor */
#define AT_EOF_STOP   0
#define AT_EOF_RETRY  1
#define AT_EOF_BLOCK  2

class JWavesMultiFileSrc {
public:
	
	JWavesMultiFileSrc(const char* sBeg, const char* sEnd, const char* sTypes, 
	                   const char* sGroup, int nStopCondition);
	JWavesMultiFileSrc* __iter__();
	cGsePkt* nextPkt();
	~JWavesMultiFileSrc();
	
protected:
	GseFileChanMgr* m_pMgr;
	GsePktSrc* m_pSrc;
};


%}

/* Here is an example of wrapping an iteration object so that it supports
   the python iterator protocol. */

%extend JWavesMultiFileSrc {
	%pythoncode {
	def next(*args):
		ret = _jwaves.JWavesMultiFileSrc_nextPkt(*args)
		if ret == None:
			raise StopIteration
		return ret
	}
}

%header %{


JWavesMultiFileSrc::JWavesMultiFileSrc(
   const char* sBeg, const char* sEnd, const char* sTypes, const char* sGroup,
	int nStopCondition
){
	gse_eof_disp eof_disp = at_eof_stop;
	if(nStopCondition == AT_EOF_RETRY)
		eof_disp = at_eof_retry;
	if(nStopCondition == AT_EOF_BLOCK)
		eof_disp = at_eof_block;
	
	m_pMgr = (GseFileChanMgr*)new_JWavesChanMgr(sBeg, sEnd, sTypes, sGroup);
	
	m_pSrc = (GsePktSrc*)new_GseMultiFileSrc(m_pMgr, jwaves_blocks, 
	                                               eof_disp, 10);
}

JWavesMultiFileSrc::~JWavesMultiFileSrc(){ m_pSrc->del(m_pSrc); }

cGsePkt* JWavesMultiFileSrc::nextPkt(){
	GsePkt* pCpkt = m_pSrc->readNext(m_pSrc);
	if(pCpkt == NULL)
		return NULL;	
	
	return new cGsePkt(pCpkt);
}

JWavesMultiFileSrc* JWavesMultiFileSrc::__iter__(){ return this; }

%}



