/** @file JWavesPPD.h A Juno Waves Reader the Plugable Packet Display (PPD) */
#ifndef _JWavesPPD_H_
#define _JWavesPPD_H_

#include <rpwgse/GsePktSrc.h>
#include <ppd/prodThread.h>

#ifdef __cplusplus
extern "C" {
#endif

/** Produce #GseXYdata from Juno Waves files
 *
 * @class JWReader
 * @extends Producer
 */
typedef struct jw_reader_struct{
	/** Include generic producer as sub structure, base structure function 
	overrided handled in constructor */
	Producer base;
	
	/* The file source used to get new packets */
	GsePktSrc* pSrc;
	
	/* The Juno Waves data filter (handles useg/etc.) */
	/* JWavesFilter* pFilter; */
	
	/* The data group to user for opening data, this is needed if 
	   sFile is autohrs or autolrs */
	char* sGroup;
	
	/* The file to open during capture setup, if null then autoselect */
	char* sFile;
	
	/* If the file to open is no set, which type of file to lookup, HRS or LRS */
	int4 iWhichAuto;
	
	/* The most recient data collected, held here until makeWork is called */
	GseXYdataAry* pDataAry;
	
	/** Destructor */
	void (*del)(void* vpThis);
	
	/* Read direction */
	bool bForward;
	
	/* Flags for the libjwaves XY structure factory */
	uint4 uXYFlags;
	
	/* Skip to the end of the file before reading any data */
	bool bSkipToEnd;
	
	/* Start time, if using time based lookup */
	char* sStart;
	
	/* Stop time, if using time based lookup */
	char* sStop;
		
} JWReader;

/** Generate a new test interface producer
 * @param pFlag A reference to an integer, that if set to zero halts the
 *              reader.
 * @param nLevel The logging level for this thread see gse_thread_init for
 *        details
 * @param uOverMalloc Internal variable used for derived classes.
 * @note only one producer is allowed per application right now 
 * @public
 * @memberof JWReader
 */
JWReader* newJWReader(int* pFlag, int4 nLevel, size_t uOverMalloc);


/* Plugin Interface ******************************************************** */

/** Required plugin function, creates a new Producer Derived object */ 
ConfInter* newGVThreadObj(int* pFlag, int4 nLevel);

/** Required plugin function, returns a pointer to the Producer name */
const char* GVThreadObjName();

/** Required plugin funciton.
 * @returns 1, since this plugin is a producer.
 */
int GVThreadObjType();

/** Required plugin function. 
 * @returns the version as a new string allocated on the heap.
 */
char* GVThreadObjVer();


#ifdef __cplusplus
}
#endif

#endif /* _JWavesPPD_H_ */

