#define _POSIX_C_SOURCE 200112

#ifdef GSE_POSIX_THREADS
#include <pthread.h>
#endif

#include <assert.h>
#include <rpwgse/GseExcept.h>
#include <rpwgse/GseLog.h>
#include <rpwgse/GsePkt.h>

#include "JWavesDefs.h"
#include "JWavesSDSblk.h"
#include "JWavesMroBlk.h"
#include "JWavesUpkBlk.h"
#include "JWavesRdrBlk.h"
#include "JWavesLRSblk.h"
#include "JWavesHRSblk.h"
#include "JWavesFDMblk.h"
#include "JWavesCCSDSblk.h"
#include "JWavesHrbhBlk.h"
#include "JWavesUsegBlk.h"
#include "JWavesDcmpBlk.h"
#include "JWavesSideBlk.h"
#include "JWavesSfduBlk.h"

/****************************************************************************/
/* Pulling fields from a Blob O' Bytes, doesn't work in all situations      */

/* Structure def is: offset, size, bigendian, signed, shift, mask */

#define NA true

const gse_field_def JW_FD_MODIFIER       =  {18, 1,    NA, false,  0,      0};
const gse_field_def JW_FD_LO_TLM_SRC     =  {19, 1,    NA, false, -4,   0x0F};
const gse_field_def JW_FD_HI_TLM_SRC     =  {19, 1,    NA, false,  0,   0x0F};
const gse_field_def JW_FD_COL_SCLK       =  {20, 4,  true, false,  0,      0};
const gse_field_def JW_FD_COL_RTI        =  {24, 1,    NA, false,  0,      0};
const gse_field_def JW_FD_AVG            =  {25, 1,    NA, false,  0,      0};
const gse_field_def JW_FD_HI_SEQ_NO      =  {26, 2,  true, false,  0, 0x3FFF};
const gse_field_def JW_FD_HI_REP_SCLK    =  {28, 4,  true, false,  0,      0};
const gse_field_def JW_FD_IDP_CRC        =  {32, 2, false, false,  0,      0};
const gse_field_def JW_FD_LO_SEQ_NO      =  {34, 2,  true, false,  0, 0x3FFF};
const gse_field_def JW_FD_SEQ_NO         =  {34, 2,  true, false,  0, 0x3FFF};
const gse_field_def JW_FD_LO_REPORT_SCLK =  {36, 4,  true, false,  0,      0};
const gse_field_def JW_FD_REPORT_SCLK    =  {36, 4,  true, false,  0,      0};
const gse_field_def JW_FD_MP_ID          =  {40, 1,    NA, false, -4,   0x0F};
const gse_field_def JW_FD_MP_LEN         =  {40, 2,  true, false,  0, 0x0FFF};
const gse_field_def JW_FD_MP_RTI         =  {42, 2, false, false,  0,      0};
const gse_field_def JW_FD_MP_GLOP        =  {44, 1,    NA, false, -7,   0x01};
const gse_field_def JW_FD_MP_PATN        =  {44, 1,    NA, false, -6,   0x01};
const gse_field_def JW_FD_MP_BGAIN       =  {44, 1,    NA, false, -5,   0x01};
const gse_field_def JW_FD_MP_EOS         =  {44, 1,    NA, false, -4,   0x01};
const gse_field_def JW_FD_MP_SEG_0R_CYCLES= {44, 1,    NA, false,  0,   0x0F};
const gse_field_def JW_FD_MP_ATTN        =  {45, 1,    NA, false, -4,   0x0F};
const gse_field_def JW_FD_MP_SRC         =  {45, 1,    NA, false,  0,   0x0F};
const gse_field_def JW_FD_MP_BND_OR_CYCSEC= {46, 1,    NA, false,  0,      0};
const gse_field_def JW_FD_MP_MSF         =  {47, 1,    NA, false, -6,   0x03};
const gse_field_def JW_FD_MP_FMT         =  {47, 1,    NA, false,  0,   0x3F};
const gse_field_def JW_FD_WKS_TIME       =  {76, 4,  true, false,  0,      0};
const gse_field_def JW_FD_WKS_UID        =  {84, 4,  true, false,  0,0xFFFFFF};
const gse_field_def JW_FD_LRS_SCE_IP     =  {88, 4,  true, false,  0,      0};
const gse_field_def JW_FD_LRS_WKS_IP     =  {92, 4,  true, false,  0,      0};


/****************************************************************************/
/* Header Object creation via factory functions                             */

GseBlock* jwaves_blocks(int2 nSpaceCraft, byte uType, byte uProcess,
                        cbytebuf_t buf){
	if(nSpaceCraft != JUNO_SPICE_ID)
		gse_throw_fm(GSE_XCEPT, GSE_XBADARG, "Can only generate header "
				       "blocks for Juno Waves (Spice ID -61), spacecraft id was "
		             "%hd.", nSpaceCraft);
	
	/* Science Data Status Block */
	if(uType == JW_SDS_BLK_TYPE_ID)
		return (GseBlock*)new_JWavesSDSblk(buf);
	
	/* Memory Readout discription block */
	if(uType == JW_MRO_BLK_TYPE_ID)
		return (GseBlock*)new_JWavesMroBlk(buf);
	
	/* CCSDS Status Block */
	if(uType == JW_CCSDS_BLK_TYPE_ID)
		return (GseBlock*)new_JWavesCCSDSblk(buf);
	
	/* High Rate Bin Header Block */
	if(uType == JW_HRS_BIN_HDR_ID)
		return (GseBlock*)new_JWavesHrbhBlk(buf);
	
	/* Generic Reading */
	if((uType == GSE_PROGRAM_BLK_TYPE_ID)&&
		(uProcess == JW_RDR_PROC_STEP_ID))
		return (GseBlock*)new_JWavesRdrBlk(buf, 0);
	
	/* SCE LRS Reading */
	if((uType == GSE_PROGRAM_BLK_TYPE_ID)&&
		(uProcess == JW_SCELRS_PROC_STEP_ID))
		return (GseBlock*)new_JWavesLRSblk(buf, 0);
	
	/* SCE HRS Reading */
	if((uType == GSE_PROGRAM_BLK_TYPE_ID)&&
		(uProcess == JW_SCEHRS_PROC_STEP_ID))
		return (GseBlock*)new_JWavesHRSblk(buf, 0);
	
	/* SCE FDM Reading */
	if((uType == GSE_PROGRAM_BLK_TYPE_ID)&&
		(uProcess == JW_FDM_PROC_STEP_ID))
		return (GseBlock*)new_JWavesFDMblk(buf, 0);
	
	/* SFDU Reading */
	if((uType == GSE_PROGRAM_BLK_TYPE_ID)&&
		(uProcess == JW_SFDU_PROC_STEP_ID))
		return (GseBlock*)new_JWavesSfduBlk(buf, 0);
	
	/* Unpacking */
	if((uType == GSE_PROGRAM_BLK_TYPE_ID)&&
		(uProcess == JW_UPACK_PROC_STEP_ID))
		return (GseBlock*) new_JWavesUpkBlk(buf, 0);
	
	/* Unsegmenting */
	if((uType == GSE_PROGRAM_BLK_TYPE_ID)&&
		(uProcess == JW_USEG_PROC_STEP_ID))
		return (GseBlock*)new_JWavesUsegBlk(buf, 0);
	
	/* Decompressing */
	if((uType == GSE_PROGRAM_BLK_TYPE_ID)&&
		(uProcess == JW_DCMP_PROC_STEP_ID))
		return (GseBlock*)new_JWavesDcmpBlk(buf, 0);
	
	/* Sideband recovery */
	if((uType == GSE_PROGRAM_BLK_TYPE_ID)&&
		(uProcess == JW_SBR_PROC_STEP_ID))
		return (GseBlock*)new_JWavesSideBlk(buf, 0);
			
	
	gse_warning_v("Unknown block id: 0x%hhX or processing step id: 0x%hhu", 
			        uType, uProcess);
	
	/* Factory functions must always return at least a generic block if 
	   nothing else matches */
	return gse_generic_blocks(nSpaceCraft, uType, uProcess, buf);					 
}

/****************************************************************************/

/* WARNING: if there are multiple minipackets in a single IDP only the first
   header is returned */
cbytebuf_t jwaves_inst_hdr(GsePkt* pPkt)
{
	const GseBlock* pBlk = NULL;
	cbytebuf_t ret       = CBBUF_NULL;
	byte uCycles         = 0U;
	byte uMSF            = 0U;
	
	if(pPkt->pHdr->nSpaceCraft != -61)
		gse_throw_fm(GSE_XCEPT, GSE_XBADARG, "Can only generate header "
				       "blocks for Juno Waves (Spice ID -61), spacecraft id was "
		             "%hd.", pPkt->pHdr->nSpaceCraft);
	
	/* For .upk and up the headers are in SDS or MRO */
	if((pPkt->pHdr->uContent & 0xF) != 0x1){
		
		/* Look in the SDS, see waves_gs_fmts_stor.txt for details */
		if((pPkt->pHdr->uContent & 0xF0) == 0xE0)
			pBlk = pPkt->getBlkByType(pPkt, JW_MRO_BLK_TYPE_ID, 0);
		else
			pBlk = pPkt->getBlkByType(pPkt, JW_SDS_BLK_TYPE_ID, 0);
		
		ret = pBlk->original(pBlk);
		
		if((pPkt->pHdr->uContent & 0xF0) == 0xE0)
			ret.ptr += 8;   /* MRO starts at 8 down */
		else
			ret.ptr += 16;   /* SCI starts at 16 down */
	}
	/* For .raw data the headers start in the data section */
	else{
		ret = pPkt->getData(pPkt);
		if(ret.len < 16)
			gse_throw_fm(GSE_XCEPT, GSE_XDECODE, "Data buffer is too short.  "
					       "Expected at least 16 bytes, received %zd.", ret.len);
	}
		
	/* Calculate the length of the number of valid MPH bytes, can't use
	   JWavesMP because data may be disconnected, Can't use SDS because
		may not have it. */
	if( ((pPkt->pHdr->uContent & 0xF0) == 0x80) ||
		 ((pPkt->pHdr->uContent & 0xF0) == 0xE0)    ){
		
		uMSF = (ret.ptr[15] >> 6) & 0x03;
		switch(uMSF){
			case 1:   uMSF = 2; break;
			case 2:   uMSF = 4; break;
			case 3:   uMSF = 8; break;
			default:  uMSF = 0; break;
		}
	}
	
	/* Get glop bytes for SCI data */
	if((pPkt->pHdr->uContent & 0xF0) == 0x80){
		if( ret.ptr[12] & 0x80 )
			uCycles = (ret.ptr[12] & 0x0F);
	}
		
	if(ret.len < 16 + uCycles + uMSF)
		gse_throw_fm(GSE_XCEPT, GSE_XDECODE, "Data buffer is too short.  "
				       "Expected at least %zd bytes, received %zd.", 
		             16 + uCycles + uMSF, ret.len);
		
	ret.len = 16 + uCycles + uMSF;
	
	return ret;
}


/* This function returns "just data" only as long as multiple
   mini-packets are not packed into a single waves packet. */
cbytebuf_t jwaves_inst_data(GsePkt* pPkt)
{
	cbytebuf_t ret  = CBBUF_NULL;
	byte uCycles         = 0U;
	byte uMSF            = 0U;
	
	if(pPkt->pHdr->nSpaceCraft != -61)
		gse_throw_fm(GSE_XCEPT, GSE_XBADARG, "Can only generate header "
				       "blocks for Juno Waves (Spice ID -61), spacecraft id was "
		             "%hd.", pPkt->pHdr->nSpaceCraft);
	
	ret = pPkt->getData(pPkt);
	
	/* For anything but .raw data this is easy */
	if((pPkt->pHdr->uContent & 0xF) != 0x1)
		return ret;
	
	/* Okay, figure out the size of the header and skip over it, by default
	 * it is 16 bytes, but SCI and MRO headers may be longer. */

	if( ((pPkt->pHdr->uContent & 0xF0) == 0x80) ||
		 ((pPkt->pHdr->uContent & 0xF0) == 0xE0)    ){
		
		uMSF = (ret.ptr[15] >> 6) & 0x03;
		switch(uMSF){
			case 1:   uMSF = 2; break;
			case 2:   uMSF = 4; break;
			case 3:   uMSF = 8; break;
			default:  uMSF = 0; break;
		}
	}
	
	/* Get glop bytes for SCI data */
	if((pPkt->pHdr->uContent & 0xF0) == 0x80){
		if( ret.ptr[12] & 0x80 )
			uCycles = (ret.ptr[12] & 0x0F);
	}
	
	if(ret.len < 16 + uCycles + uMSF)
		gse_throw_fm(GSE_XCEPT, GSE_XDECODE, "Data buffer is too short.  "
				       "Expected at least %zd bytes, received %zd", 
		             16 + uCycles + uMSF, ret.len);
	
	ret.len -= 16 + uCycles + uMSF;
	ret.ptr += 16 + uCycles + uMSF;
	
	return ret;
}
