/** @file JWavesProc.h */
#ifndef _JWavesUnpack_H_
#define _JWavesUnpack_H_

#include <rpwgse/GsePkt.h>
#include <rpwgse/GseData.h>

#ifdef __cplusplus
extern "C" {
#endif

/** @addtogroup mod_junogse */
/*@{*/

/** @defgroup mod_jwaves_proc Processing 
 *
 * Handle Ground Processing Steps.
 *
 * An overview of the ground processing steps may be found in the
 * @ref page_spec_4_1 "Juno Waves Ground Processing and Storage Document"
 */
/*@}*/

/** @addtogroup mod_jwaves_proc */
/*@{*/

/** Change the format of the Data Section of a Waves GSE packet 
 *
 * @param pPkt a Waves GsePkt containing a @ref JWavesSDSblk "Juno Waves Science
 *        Data Status block" that does not contain compressed data.
 *
 * @param uOutFmt An uncompressed output format, see @ref mod_jwaves_conv
 *        for a listing of data formats.
 *
 * @param uProgID The ID of the calling program.  All programo altering Juno
 *        Waves data have a unique ID. 
 *
 * @return one of JW_16BIT_INT_IN_16BIT or JW_32BIT_FLT_IN_32BIT to indicate
 *        conversion of integers or floating point values. output was
 */
byte jwaves_reformat(GsePkt* pPkt, byte uOutFmt, byte uProgID);


/** Is a particular reformatting operation possible on particular packet 
 *
 * @throws nothing, this function does not throw.
 */ 
bool jwaves_can_refmt(const GsePkt* pPkt, byte uOutFmt);
		

/*@}*/

#ifdef __cplusplus
}
#endif

#endif /* _JWavesUnpack_H_ */
