/** \file JWavesRdrBlk.h */
#ifndef _JWavesRdrBlk_H_
#define _JWavesRdrBlk_H_

#include <rpwgse/GsePkt.h>
#include <rpwgse/GseProcBlk.h>

/** @addtogroup mod_jwaves_pkt */
/*@{*/

#ifdef __cplusplus
extern "C" {
#endif

/** Encoder/Decoder for Generic Juno Waves Reader Processing Blocks 
 * @class JWavesRdrBlk
 * @extends GseProcBlk
 */
typedef struct jwaves_rdr_blk{
	
	/** Base structure */
	GseProcBlk base;
	
	/** FIELD: The source of the data
	 * 0x01 - Lockheed Martin SCE LRS interface
    * 0x02 - Lockheed Martin SCE HRS interface
	 * 0x03 - An FDM output file
	 * Others defined as needed */
	byte uSource;
	
	/** Destructor */
	void (*del)(void* vpThis);
	
} JWavesRdrBlk;

/** Generate a new Juno Waves Generic Reader Processing Block 
 * @public
 * @memberof JWavesRdrBlk
 */
JWavesRdrBlk* new_JWavesRdrBlk(cbytebuf_t buf, size_t uOver);

/** Generate a new empty Juno Waves Generic Reader Processing Block
 * @public
 * @memberof JWavesRdrBlk
 */ 
JWavesRdrBlk* new_empty_JWavesRdrBlk(size_t uOver);

/** Same interface as GseBlock::decode, used by derived classes
 * @protected
 * @memberof JWavesRdrBlk
 */
byte JWavesRdrBlk_decode(struct gse_block* pBase, cbytebuf_t buf);

/** Same interface as GseBlock::encode, used by derived classes
 * @protected
 * @memberof JWavesRdrBlk
 */
bytebuf_t JWavesRdrBlk_encode(GseBlock* pBase, byte uExtraLen, byte* pOffset);

/** Same interface as GseBlock::copy, used by derived classes
 * @protected
 * @memberof JWavesRdrBlk
 */
GseBlock* JWavesRdrBlk_copy(GseBlock* pBase, size_t uOverMalloc);

/** Defines the jwaves reader block merge function to be the same as
 * it's base class.
 * Since we can't merge unless the uSource is the same, no special merging
 * functionality is needed 
 */
#define JWavesRdrBlk_merge GseProcBlk_merge

/** Override of GseBlock::canMerge
 * @protected
 * @memberof JWavesRdrBlk
 */
bool JWavesRdrBlk_canMerge(GseBlock* pBase, const GseBlock* pOther, uint4 uFlags);


/** Get any one of a number of defined reader blocks for juno.
 *
 * @returns a pointer to a JWavesRdrBlk, JWavesLRSblk, JWavesSRSblk, or
 *          JWavesFDMblk object.  Or NULL if no known reader block is
 *          present in the header.
 */
JWavesRdrBlk* jwaves_get_rdr_blk(GsePkt* pPkt);

		
/*@}*/
		
#ifdef __cplusplus
}
#endif

#endif /* _JWavesRdrBlk_H_ */

