#define _POSIX_C_SOURCE 200112

#ifdef GSE_POSIX_THREADS
#include <pthread.h>
#endif

#include <rpwgse/GseExcept.h>
#include <rpwgse/GseConv.h>

#include "JWavesSideBlk.h"

int4 JWavesSideBlk_addProgID(JWavesSideBlk* pThis, byte uProgID)
{
	GseProcBlk* pProc = (GseProcBlk*)pThis;
	int i, num = 0;
	
	if(uProgID == 0)
		gse_throw_m(GSE_XCEPT, GSE_XBADARG, "0 is not a legal program ID");
	
	for(i = 0; i<4; i++)
		if(pThis->uObsProgIDs[i] > 0) 
			num++;
	
	if(pProc->uProgID == uProgID)
		return num;
	
	for(i = 0; i<4; i++) 
		if(pThis->uObsProgIDs[i] == uProgID)
			return num;
	
	for(i = 0; i<4; i++){
		if(pThis->uObsProgIDs[i] == 0){
			pThis->uObsProgIDs[i] = uProgID;
			num++;
			break;
		}
	}
	
	return num;
}

bytebuf_t JWavesSideBlk_encode(GseBlock* pBase, byte uExtraLen, byte* pOffset)
{
	GseProcBlk* pProc = (GseProcBlk*)pBase;
	JWavesSideBlk* pThis = (JWavesSideBlk*)pBase;
	int i;
	byte offset;
	bytebuf_t buf;
	
	
	if(uExtraLen != 0)
		gse_throw_m(GSE_XCEPT, GSE_XBADARG, "The JWavesSideBlk class is final "
		            "(i.e. should not be derived from), thus uExtraLen should "
		            "always be 0.");
	
	/* Check the base values */
	if(pProc->uStepID != JW_SBR_PROC_STEP_ID)
		gse_throw_fm(GSE_XCEPT, GSE_XENCODE, "uStepID (PROCESS) != %hhu",
				     JW_SBR_PROC_STEP_ID);
	
	/* Check my values */
	for(i = 0; i<4; i++){
		if(pThis->uObsProgIDs[i] == pProc->uProgID)
			gse_throw_fm(GSE_XCEPT, GSE_XENCODE, "My process ID 0x%02uuX is "
			             "repeated in the 'Observed process IDs area'",
			              pProc->uProgID);
	}
	
	if(pThis->uNumISegs > 16)
		gse_throw_fm(GSE_XCEPT, GSE_XENCODE, "I channel invalid number of "
		             "sequences: %uud", pThis->uNumISegs);
				
	if(pThis->uNumQSegs > 16)
		gse_throw_fm(GSE_XCEPT, GSE_XENCODE, "Q channel invalid number of "
                   "sequences: %uud", pThis->uNumQSegs);
	
	
	buf = GseProcBlk_encode(pBase, 82, &offset);
	
	for(i=0; i<4; i++) buf.ptr[offset+i] = pThis->uObsProgIDs[i];
	
	gse_u4ToMSBbytes(buf.ptr + offset + 4, pThis->uLoReadSec);
	gse_u4ToMSBbytes(buf.ptr + offset + 8, pThis->uHiReadSec);
	gse_u2ToMSBbytes(buf.ptr + offset + 12, pThis->uLoReadUS);
	gse_u2ToMSBbytes(buf.ptr + offset + 14, pThis->uHiReadUS);
	/* No flags defined at this time */
	buf.ptr[offset + 17] = (((pThis->uNumISegs-1) << 4)&0xF0) |
	                       ((pThis->uNumQSegs-1) & 0x0F );
	
	for(i=0; i<pThis->uNumISegs; i++)
		gse_u2ToMSBbytes(buf.ptr + offset + 18 + i*2, pThis->uISeqTSary[i]);
	
	for(i=0; i<pThis->uNumISegs; i++)
		gse_u2ToMSBbytes(buf.ptr + offset + 50 + i*2, pThis->uQSeqTSary[i]);
		
	return buf; 
}

byte JWavesSideBlk_decode(GseBlock* pBase, cbytebuf_t buf)
{
	GseProcBlk* pProc = (GseProcBlk*)pBase;
	JWavesSideBlk* pThis = (JWavesSideBlk*)pBase;
	int i;
	size_t offset;
	
	offset = GseProcBlk_decode(pBase, buf);
	
	if(pProc->uStepID != JW_SBR_PROC_STEP_ID)
		gse_throw_fm(GSE_XCEPT, GSE_XDECODE, "Expected 0x%hhX for the block type "
		             "ID, but recieved 0x%hhX", JW_SBR_PROC_STEP_ID, pBase->uType);
	
	if(pBase->uLen != offset + 82 - 1)
		gse_throw_fm(GSE_XCEPT, GSE_XDECODE, "Expected %d (%d - 1) for the "
				       "Sideband Recovery block length, but recieved %hhu", 
				        offset - 1 + 82, offset + 82, pBase->uLen);
	
	for(i=0; i<4; i++) pThis->uObsProgIDs[i] = buf.ptr[offset+i];
	
	pThis->uLoReadSec = gse_u4FromMSBbytes(buf.ptr + offset + 4);
	pThis->uHiReadSec = gse_u4FromMSBbytes(buf.ptr + offset + 8);
	pThis->uLoReadUS  = gse_u2FromMSBbytes(buf.ptr + offset + 12);
	pThis->uHiReadUS  = gse_u2FromMSBbytes(buf.ptr + offset + 14);
   /* No flags defined at this time */
	
	pThis->uNumISegs = ((buf.ptr[offset + 17] >> 4)&0xF) + 1;
	pThis->uNumQSegs = ((buf.ptr[offset + 17])&0xF) + 1;
			
	
	if(pThis->uNumISegs > 16)
		gse_throw_fm(GSE_XCEPT, GSE_XDECODE, "I channel invalid number of "
		             "sequences: %uud", pThis->uNumISegs);
				
	if(pThis->uNumQSegs > 16)
		gse_throw_fm(GSE_XCEPT, GSE_XDECODE, "Q channel invalid number of "
                   "sequences: %uud", pThis->uNumQSegs);
		
	for(i=0; i<pThis->uNumISegs; i++)
		pThis->uISeqTSary[i] = gse_u2FromMSBbytes(buf.ptr + offset + 18 + i*2);
	
	for(i=0; i<pThis->uNumISegs; i++)
		pThis->uQSeqTSary[i] = gse_u2FromMSBbytes(buf.ptr + offset + 50 + i*2);
	
	return offset + 82;
}

bool JWavesSideBlk_canMerge(GseBlock* pBase, const GseBlock* pOther, uint4 uFlags)
{
	return false;	
}

GseBlock* JWavesSideBlk_copy(GseBlock* pBase, size_t uOverMalloc)
{
	JWavesSideBlk* pThis = (JWavesSideBlk*)pBase;
	JWavesSideBlk* pNew;
	size_t uMalloc;
	
	if(uOverMalloc > sizeof(JWavesSideBlk))
		uMalloc = uOverMalloc;
	else
		uMalloc = sizeof(JWavesSideBlk);
	
	pNew = (JWavesSideBlk*)GseProcBlk_copy(pBase, uMalloc);
	
	memcpy(pNew->uObsProgIDs, pThis->uObsProgIDs, 4);
	pNew->uLoReadSec = pThis->uLoReadSec;
	pNew->uHiReadSec = pThis->uHiReadSec;
	pNew->uLoReadUS  = pThis->uLoReadUS;
	pNew->uHiReadUS  = pThis->uHiReadUS;
	pNew->uNumISegs  = pThis->uNumISegs;
	pNew->uNumQSegs  = pThis->uNumQSegs;
	memcpy(pNew->uISeqTSary, pThis->uISeqTSary, 16*2);
	memcpy(pNew->uQSeqTSary, pThis->uQSeqTSary, 16*2);
	pNew->del        = pThis->del;
	pNew->addProgID  = pThis->addProgID;
	
	return (GseBlock*)pNew;
}

JWavesSideBlk* new_JWavesSideBlk(cbytebuf_t buf, size_t uOverMalloc){
	GseBlock* pBase = NULL;
	GseProcBlk* pProc = NULL;
	JWavesSideBlk* pThis = NULL;
	
	if(uOverMalloc > sizeof(JWavesSideBlk))
		pProc = new_GseProcBlk(buf, uOverMalloc);
	else
		pProc = new_GseProcBlk(buf, sizeof(JWavesSideBlk));
	
	pBase = (GseBlock*)pProc;
	pThis = (JWavesSideBlk*)pProc;
	
	/* Override a couple base class functions */
	pBase->encode = JWavesSideBlk_encode;
	pBase->decode = JWavesSideBlk_decode;
	pBase->_sName = "Unsegmentor";
	pBase->copy   = JWavesSideBlk_copy;
	pBase->canMerge = JWavesSideBlk_canMerge;/* These are not mergeable */
	
	pProc->uStepID = JW_SBR_PROC_STEP_ID;
	
	pBase->_uMinLen += 82;
	
	/* Set new items */
	pThis->del       = &delete_GseBlock;
	pThis->addProgID = JWavesSideBlk_addProgID;
	
	/* Rest of the defaults are all zero, so calloc gets this */ 
	if(buf.ptr == NULL)
		return pThis;

	except_cleanup_push(delete_GseBlock, pThis);
	
	JWavesSideBlk_decode(pBase, buf);

	except_cleanup_pop(except_no_call);
	return pThis;
}
