/** @file JWavesSideFilter.h */
#ifndef _JWavesSideFilter_H_
#define _JWavesSideFilter_H_

#ifdef __cplusplus
extern "C" {
#endif

#include <rpwgse/GseProcBlk.h>
#include <rpwgse/GsePktFilter.h>

#define JW_IQ_CAT_WI  0
#define JW_IQ_CAT_WIR 1
#define JW_IQ_CAT_WQ  2
#define JW_IQ_CAT_WQR 3
#define JW_IQ_CAT_HI  4
#define JW_IQ_CAT_HIR 5
#define JW_IQ_CAT_HQ  6
#define JW_IQ_CAT_HQR 7

#define JW_IQ_WAIT_CATEGORIES 8

/** @addtogroup mod_jwaves_proc Processing */
/*@{*/

/** A packet source that combinds I and Q data into single packets.
 *
 * The expected input is GSE packets with content type 0x84.  Packets with
 * a different content type are merely passed to the output un-altered.
 *
 * All final packets emitted from this source have a content byte of: 
 * @b 0x85 (one measurement).
 *
 * Packet matching is global from the start of the program till the
 * input returns Null.
 *
 * @class JWavesSideFilter
 * @extends GsePktFilter
 */
typedef struct jwaves_side_filter
{
	/** Base structure */
	GsePktFilter base;
	
	/* Holder for I or Q packets waiting for the other member of the pair
	 * 
	 * NOTE:  NO, ACTUALLY READ ME!!!
	 *
	 * If we get a problem with packets buffered in the instrument or
	 * the LockMart GDS rearranging the packet order, SWITCH from this
	 * simple waitlist to a wait dictonary with keys like:
	 *
	 *   "HIC2009-01-01T00:04:03.689"
	 *   "WQD2009-01-01T00:04:03.689"
	 */
	GsePkt* pWaiting[JW_IQ_WAIT_CATEGORIES];
	int4    pWaitIdx[JW_IQ_WAIT_CATEGORIES];
	
	/* Holder for the fft plan object plus IO */
	void* vpPlan;
	void* vpFTin;
	void* vpFTout;
	size_t nFFTlen;
	
	/* Holder for the hanning window */
	float8* pWnd;
	
	/* Holders for the inputs to a previous merge */
	GsePkt* pPktIMerged;
	GsePkt* pPktQMerged;
	
	uint4 uOpts;  /* Filter Options */
		
	/** Destructor */
	void (*del)(void* vpThis);
	
} JWavesSideFilter;


/** Constructer for new sideband recovery filters 
 *
 * @param pSrc A Source of Juno GSE packets with content type 0x84, other
 *        packet types maybe mixed in, but these are merely passed on 
 *        un-altered
 *
 * @param uProgID The program ID to attach to any generated packets.
 *
 * @param uFlags Filter operations flags, if this contains JW_FLTR_SBR_KEEP_IQ
 *        then no only will the sideband spectra be generated, the original I and
 *        Q packets will be emitted as well.
 *
 * @return a pointer to a new JWavesUpkFilter structure allocated on the heap.
 *
 * @public
 * @memberof JWavesSideFilter
 */
JWavesSideFilter* new_JWavesSideFilter(GsePktSrc* pSrc, byte uProgID, uint4 uFlags);


/*@}*/

#ifdef __cplusplus
}
#endif

#endif /* _JWavesSideFilter_H_ */
