#define _POSIX_C_SOURCE 200112

#ifdef GSE_POSIX_THREADS
#include <pthread.h>
#endif

#include <rpwgse/GseExcept.h>
#include <rpwgse/GseConv.h>

#include "JWavesDefs.h"
#include "JWavesUpkBlk.h"

bytebuf_t JWavesUpkBlk_encode(GseBlock* pBase, byte uExtraLen, byte* pOffset)
{
	byte offset;
	bytebuf_t buf;

	JWavesUpkBlk* pThis = (JWavesUpkBlk*)pBase;
	GseProcBlk* pProc   = (GseProcBlk*)pBase;
	
	gse_overflowUinU(uExtraLen + 4, sizeof(byte)); 
	
	buf = GseProcBlk_encode(pBase, uExtraLen + 4, &offset);		
	
	/* Check base parsing */
	if(pProc->uStepID != JW_UPACK_PROC_STEP_ID)
		gse_throw_fm(GSE_XCEPT, GSE_XENCODE, "Processing step ID was %hhu"
				       "expected %hhu.", pProc->uStepID, JW_RDR_PROC_STEP_ID);
	
	buf.ptr[offset] = pThis->uMiniPkts;
	buf.ptr[offset+1] = pThis->iMiniPkt;
	buf.ptr[offset+2] = 0;   /* The Bytes that don't do anything */
	buf.ptr[offset+3] = 0;   /* but save room for the future */
	
	if(pOffset != NULL) *pOffset = offset + 2;
	return buf;
}

byte JWavesUpkBlk_decode(GseBlock* pBase, cbytebuf_t buf)
{
	JWavesUpkBlk* pThis = (JWavesUpkBlk*)pBase;
	
	if(GseProcBlk_decode(pBase, buf) != 12)
		gse_throw(GSE_XCEPT, GSE_XLIBINCOMPAT);
	 
	/* It's okay if we don't get the last two zero bytes */
	if(pBase->uLen < 13)
		gse_throw_fm(GSE_XCEPT, GSE_XDECODE, "Expected at least 13 (i.e. 14 - 1) "
		             "for the Reader block length, but recieved %hhu",
		             pBase->uLen);
	
	/* Decoding */
	pThis->uMiniPkts = buf.ptr[12];
	pThis->iMiniPkt = buf.ptr[13];

	return 14;		
}

bool JWavesUpkBlk_merge(GseBlock* pBase, const GseBlock* pOther, uint4 uFlags)
{
	JWavesUpkBlk* pThis = (JWavesUpkBlk*)pBase;
	GseProcBlk* pProc   = (GseProcBlk*)pBase;
	
	JWavesUpkBlk* pUpkOth = (JWavesUpkBlk*)pOther;
	GseProcBlk* pProcOth  = (GseProcBlk*)pOther;
	
	byte uNewMiniPkts;
	
	uNewMiniPkts = pProc->microEpoch(pProc) < pProcOth->microEpoch(pProcOth) ?
			         pThis->uMiniPkts : pUpkOth->uMiniPkts;
	
	if( ! GseProcBlk_merge(pBase, pOther, GSE_PROC_MERGE_EARLY|uFlags) )
		return false;
	
	pThis->uMiniPkts = uNewMiniPkts;
	
	return true;
}

GseBlock* JWavesUpkBlk_copy(GseBlock* pBase, size_t uOverMalloc)
{
	JWavesUpkBlk* pNew = NULL;
	JWavesUpkBlk* pThis = (JWavesUpkBlk*)pBase;
	size_t uSize = 0;
	 
 	if(uOverMalloc > sizeof(JWavesUpkBlk))
		uSize = uOverMalloc;
	else
		uSize = sizeof(JWavesUpkBlk);
	
	pNew = (JWavesUpkBlk*)GseProcBlk_copy(pBase, uSize);
	pNew->uMiniPkts = pThis->uMiniPkts;
	pNew->iMiniPkt = pThis->iMiniPkt;
	pNew->del     = pThis->del;
	
	return (GseBlock*)pNew;
}

JWavesUpkBlk* new_JWavesUpkBlk(cbytebuf_t buf, size_t uOver)
{
	JWavesUpkBlk* pThis = new_empty_JWavesUpkBlk(uOver);
	
	if(buf.ptr == NULL) return pThis;
	
	except_cleanup_push(delete_GseBlock, pThis);
	
	JWavesUpkBlk_decode((GseBlock*)pThis, buf);

	except_cleanup_pop(except_no_call);
	return pThis;
}

JWavesUpkBlk* new_empty_JWavesUpkBlk(size_t uOver)
{
	GseBlock* pBase = NULL;
	GseProcBlk* pProc = NULL;
	JWavesUpkBlk* pThis = NULL;
	
	if(uOver < sizeof(JWavesUpkBlk))
		pProc = new_GseProcBlk(CBBUF_NULL, sizeof(JWavesUpkBlk));
	else
		pProc = new_GseProcBlk(CBBUF_NULL, uOver);
	
	pBase = (GseBlock*)pProc;
	pThis = (JWavesUpkBlk*)pProc;
	
	/* Override a couple base class functions */
	pBase->encode   = JWavesUpkBlk_encode;
	pBase->decode   = JWavesUpkBlk_decode;
	pBase->_sName   = "Waves Unpacker";
	pBase->copy     = JWavesUpkBlk_copy;
	pBase->merge    = JWavesUpkBlk_merge;
	
	pProc->uStepID = JW_UPACK_PROC_STEP_ID;
	
	pBase->_uMinLen += 4; /* Just want 2 extra bytes no current reason */
	
	/* Set new items */
	pThis->del = pProc->del;
	pThis->iMiniPkt = 0;
	
	/* Rest of the defaults are all zero, so calloc gets this */
	return pThis;
}
