/** \file JWavesUpkBlk.h */
#ifndef _JWavesUpkBlk_H_
#define _JWavesUpkBlk_H_

#include <rpwgse/GseProcBlk.h>

#ifdef __cplusplus
extern "C" {
#endif

/** @addtogroup mod_jwaves_pkt */
/*@{*/

/** Encoder/Decoder for Juno Waves Unpacking Processing Blocks.
 *
 * The merge policy for this block type is:
 *   # Expect to be the most derived class
 *   # Accept the default merge of the GseProcBlk (earliest time wins)
 *   # Save the number of packets in the version with the earliest proc time
 *     (i.e. stay consistant with GseProcBlk)
 * 
 * @class JWavesUpkBlk
 * @extends GseProcBlk
 */
typedef struct jwaves_upk_blk{
	
	/** Base structure */
	GseProcBlk base;
	
	/** FIELD: The number of minipackets in the original IDP packet.
	 * Prior to unpacking, there is one IDP packet per GSE packet, after
	 * unpacking there is only the minipacket data in the GSE packet. This
	 * field "remembers" the original number of "siblings" for this minipackets.
	 */
	byte uMiniPkts;
	
	/** FIELD: The index of this minipacket in the original waves packet */
	byte iMiniPkt;
	
	/** Destructor */
	void (*del)(void* vpThis);
	
} JWavesUpkBlk;

/** Generate a new Juno Waves Unpacker Processing Block 
 * @public
 * @memberof JWavesUpkBlk
 */
JWavesUpkBlk* new_JWavesUpkBlk(cbytebuf_t buf, size_t uOver);

/** Generate a new empty Juno Waves Generic Reader Processing Block
 * @public
 * @memberof JWavesUpkBlk
 */ 
JWavesUpkBlk* new_empty_JWavesUpkBlk(size_t uOver);

/** Same interface as GseBlock::decode, used by derived classes
 * @protected
 * @memberof JWavesUpkBlk
 */
byte JWavesUpkBlk_decode(struct gse_block* pBase, cbytebuf_t buf);

/** Same interface as GseBlock::encode, used by derived classes
 * @protected
 * @memberof JWavesUpkBlk
 */
bytebuf_t JWavesUpkBlk_encode(GseBlock* pBase, byte uExtraLen, byte* pOffset);

/** Same interface as GseBlock::copy, used by derived classes
 * @protected
 * @memberof JWavesUpkBlk
 */
GseBlock* JWavesUpkBlk_copy(GseBlock* pBase, size_t uOverMalloc);
		
/*@}*/
		
#ifdef __cplusplus
}
#endif

#endif /* _JWavesUpkBlk_H_ */

