#define _POSIX_C_SOURCE 200112

#ifdef GSE_POSIX_THREADS
#include <pthread.h>
#endif

#include <rpwgse/GseExcept.h>
#include <rpwgse/GseConv.h>

#include "JWavesUsegBlk.h"

bytebuf_t JWavesUsegBlk_encode(GseBlock* pBase, byte uExtraLen, byte* pOffset)
{
	int i, nFilled;
	uint2* pTmp;
	GseProcBlk* pProc = (GseProcBlk*)pBase;
	JWavesUsegBlk* pThis = (JWavesUsegBlk*)pBase;
	bytebuf_t buf;
	byte offset;
	
	if(uExtraLen != 0)
		gse_throw_m(GSE_XCEPT, GSE_XBADARG, "The JWavesUsegBlk class is final "
		            "(i.e. should not be derived from), thus uExtraLen should "
		            "always be 0.");
	
	/* Check the base values */
	if(pProc->uStepID != JW_USEG_PROC_STEP_ID)
		gse_throw_fm(GSE_XCEPT, GSE_XENCODE, "uStepID (PROCESS) != %hhu",
				     JW_USEG_PROC_STEP_ID);
	
	/* Check my values */
	if(pThis->uNumSegs < 2)
		gse_throw_m(GSE_XCEPT, GSE_XENCODE, "uNumSegs (FLAGS|NoSeg) < 2");
	
	if((pThis->uLoUnpakID == 0)||(pThis->uLoUnpakID > pThis->uHiUnpakID )||
		(pThis->uHiUnpakID == 0))
		gse_throw_fm(GSE_XCEPT, GSE_XENCODE, 
			"Invalid value for uLoUnpakID (LO_SEQ_NO): %hhu, or uHiUnpakID "
			"(HI_SEQ_NO): %hhu", pThis->uLoUnpakID, pThis->uHiUnpakID);
		
	if((pThis->uLoRdrID == 0)||(pThis->uLoRdrID > pThis->uHiRdrID)||
		(pThis->uHiRdrID == 0))
		gse_throw_fm(GSE_XCEPT, GSE_XENCODE, 
			"Invalid value for uLoRdrID (LO_RP_ID): %hhu, or uHiRdrID (HI_RP_ID) "
			": %hhu", pThis->uLoRdrID, pThis->uHiRdrID);
	
	if((pThis->uLoSource == 0)||(pThis->uLoSource > pThis->uHiSource)||
		(pThis->uHiSource == 0))
		gse_throw_fm(GSE_XCEPT, GSE_XENCODE, 
			"Invalid value for uLoSource (LO_SRC): %hhu, or uHiSource (HI_SRC) "
			": %hhu", pThis->uLoSource, pThis->uHiSource);

	if((pThis->uLoReadSec == 0)||(pThis->uLoReadSec > pThis->uHiReadSec)||
		(pThis->uHiReadSec == 0))
		gse_throw_fm(GSE_XCEPT, GSE_XENCODE, 
			"Invalid value for uLoReadSec (LO_READ_BEGIN_TIME_SEC): %hhu, or h"
			"uHiReadSec (HI_READ_BEGIN_TIME_SEC) : %hhu", pThis->uLoReadSec, 
			pThis->uHiReadSec);
		
	nFilled = 0;
	for(i=0; i<pThis->uNumSegs; i++)
		if(pThis->uSeqTSary[i] != 0) nFilled++;
	
	/* Note, one Zero is okay due to sequence roll over */
	if((nFilled != pThis->uNumSegs)&&(nFilled != pThis->uNumSegs - 1)){
		pTmp = pThis->uSeqTSary;
		gse_throw_fm(GSE_XCEPT, GSE_XENCODE,
			"Number of non-zero entries in uSeqTSary (Seq & TS No. 1 through 16) "
			"does not equal the value of uNumSegs (FLAGS|NoSeg).  uSeqTSary dump:\n"
			"   %02hX %02hX %02hX %02hX %02hX %02hX %02hX %02hX %02hX %02hX "
			"%02hX %02hX %02hX %02hX %02hX %02hX",
			pTmp, pTmp + 1, pTmp + 2, pTmp + 3, pTmp + 4, pTmp + 5, pTmp + 6, 
			pTmp + 7, pTmp + 8, pTmp + 9, pTmp + 10, pTmp + 11, pTmp + 12, 
			pTmp + 13, pTmp + 14, pTmp + 15);
	}
	
	/* Now actually encode it */
	buf = GseProcBlk_encode(pBase, 52, &offset);
	
	buf.ptr[offset] = (pThis->uNumSegs << 4) & 0xF0;
	buf.ptr[offset + 1] = pThis->uLoUnpakID;
	buf.ptr[offset + 2] = pThis->uHiUnpakID;
	buf.ptr[offset + 4] = pThis->uLoRdrID;
	buf.ptr[offset + 5] = pThis->uLoSource;
	gse_u4ToMSBbytes(buf.ptr + offset + 6, pThis->uLoReadSec);
	gse_u2ToMSBbytes(buf.ptr + offset + 10, pThis->uLoReadUS);
	buf.ptr[offset + 12] = pThis->uHiRdrID;
	buf.ptr[offset + 13] = pThis->uHiSource;
	gse_u4ToMSBbytes(buf.ptr + offset + 14, pThis->uHiReadSec);
	gse_u2ToMSBbytes(buf.ptr + offset + 18, pThis->uHiReadUS);
	
	for(i = 0; i < 16; i++)
		gse_u2ToMSBbytes(buf.ptr + offset + 20 + i*2, pThis->uSeqTSary[i]);
	
	if(pOffset != NULL) *pOffset = offset + 52;
	return buf;
}

byte JWavesUsegBlk_decode(struct gse_block* pBase, cbytebuf_t buf)
{
	int i;
	GseProcBlk*   pProc = (GseProcBlk*)pBase;
	JWavesUsegBlk* pThis = (JWavesUsegBlk*)pBase;
	size_t offset;
	
	offset = GseProcBlk_decode(pBase, buf);
	 
	if(pProc->uStepID != JW_USEG_PROC_STEP_ID)
		gse_throw_fm(GSE_XCEPT, GSE_XDECODE, "Expected 0x%hhX for the block type "
		             "ID, but recieved 0x%hhX", JW_USEG_PROC_STEP_ID, pBase->uType);
	
	if(pBase->uLen != offset + 52 - 1)
		gse_throw_fm(GSE_XCEPT, GSE_XDECODE, "Expected %d (%d - 1) for the "
				       "Unsegmentor block length, but recieved %hhu", offset-1 + 52, 
		              offset + 52, pBase->uLen);
	
	pThis->uNumSegs = (buf.ptr[offset] >> 4) & 0x0F;
	pThis->uLoUnpakID = buf.ptr[offset + 1];
	pThis->uHiUnpakID = buf.ptr[offset + 2];
	pThis->uLoRdrID = buf.ptr[offset + 4];
	pThis->uLoSource = buf.ptr[offset + 5];
	pThis->uLoReadSec = gse_u4FromMSBbytes(buf.ptr + offset + 6);
	pThis->uLoReadUS = gse_u2FromMSBbytes(buf.ptr + offset + 10);
	pThis->uHiRdrID = buf.ptr[offset + 12];
	pThis->uHiSource = buf.ptr[offset + 13];
	pThis->uHiReadSec = gse_u4FromMSBbytes(buf.ptr + offset + 14);
	pThis->uHiReadUS = gse_u2FromMSBbytes(buf.ptr + offset + 18);
	
	for(i = 0; i < 16; i++)
		pThis->uSeqTSary[i] = gse_u2FromMSBbytes(buf.ptr + offset + 20 + i*2);
	
	return offset + 52;
}

bool JWavesUsegBlk_canMerge(GseBlock* pBase, const GseBlock* pOther, uint4 uFlags)
{
	return false;	
}

GseBlock* JWavesUsegBlk_copy(GseBlock* pBase, size_t uOverMalloc)
{
	JWavesUsegBlk* pNew = NULL;
	JWavesUsegBlk* pThis = (JWavesUsegBlk*)pBase;
	size_t uSize = 0;
	int i;
	 
 	if(uOverMalloc > sizeof(JWavesUsegBlk))
		uSize = uOverMalloc;
	else
		uSize = sizeof(JWavesUsegBlk);
	
	pNew = (JWavesUsegBlk*)GseProcBlk_copy(pBase, uSize);
	
	pNew->uNumSegs = pThis->uNumSegs;
	pNew->uLoUnpakID = pThis->uLoUnpakID;
	pNew->uHiUnpakID = pThis->uHiUnpakID;
	pNew->uLoRdrID = pThis->uLoRdrID;
	pNew->uLoSource = pThis->uLoSource;
	pNew->uLoReadSec = pThis->uLoReadSec;
	pNew->uLoReadUS = pThis->uLoReadUS;
	pNew->uHiRdrID = pThis->uHiRdrID;
	pNew->uHiSource = pThis->uHiSource;
	pNew->uHiReadSec = pThis->uHiReadSec;
	pNew->uHiReadUS = pThis->uHiReadUS;
	for(i=0; i<16; i++)pNew->uSeqTSary[i] = pThis->uSeqTSary[i];
	pNew->del = pThis->del;
	
	return (GseBlock*)pNew;
}

JWavesUsegBlk* new_JWavesUsegBlk(cbytebuf_t buf, size_t uOverMalloc){
	GseBlock* pBase = NULL;
	GseProcBlk* pProc = NULL;
	JWavesUsegBlk* pThis = NULL;
	size_t uSize;

	if(uOverMalloc > sizeof(JWavesUsegBlk))
		uSize = uOverMalloc;
	else
		uSize = sizeof(JWavesUsegBlk);
	
	pProc = new_GseProcBlk(buf, uSize);
	pBase = (GseBlock*)pProc;
	pThis = (JWavesUsegBlk*)pProc;
	
	/* Override a couple base class functions */
	pBase->encode = JWavesUsegBlk_encode;
	pBase->decode = JWavesUsegBlk_decode;
	pBase->_sName = "Unsegmentor";
	pBase->copy   = JWavesUsegBlk_copy;
	pBase->canMerge = JWavesUsegBlk_canMerge;/* These are not mergeable */
	pProc->uStepID = JW_USEG_PROC_STEP_ID;
	
	pBase->_uMinLen = 64;
	
	/* Set new items */
	pThis->del = &delete_GseBlock;
	
	/* Rest of the defaults are all zero, so calloc gets this */ 
	if(buf.ptr == NULL)
		return pThis;

	except_cleanup_push(delete_GseBlock, pThis);
	
	JWavesUsegBlk_decode(pBase, buf);

	except_cleanup_pop(except_no_call);
	return pThis;
}
