/** @file JWavesUsegFilter.h */
#ifndef _JWavesUsegFilter_H_
#define _JWavesUsegFilter_H_

#include <rpwgse/GsePktFilter.h>

#ifdef __cplusplus
extern "C" {
#endif

/** @addtogroup mod_jwaves_proc Processing */
/*@{*/

/** A Packet Source that consolidates segmented data.
 *
 * The expected input is GSE packets with content type 0x83.  Packets with
 * a different content type are merely passed to the output un-altered.
 * Packets that have values of JWavesSDSblk::bEOC and JWavesSDSblk::uSegNo
 * which indicate that a packet is part of a group are not passed to the
 * output, but instead are collected into a single packet, and that is
 * set to the output of @ref GsePktSrc::readNext "readNext". @n @n
 *
 * @class JWavesUsegFilter
 * @extends GsePktFilter
 */
typedef struct jwaves_useg_filter
{
	/** Enclosed base structure. */
	GsePktFilter base;
	
	
	/* The internal packet channel array, the index order
	   is PSID,SEGNO, the values are packet pointers */
	GsePkt* channel[256][16];
	
	/** Destructor */
	void (*del)(void* vpThis);
	
} JWavesUsegFilter;


/** Constructor for an unsegmenting packet source.
 * Generates a new unsegmenting packet source.  This filter unpacks recoginzed
 * packets before passing them on.
 * @public
 * @memberof JWavesUsegFilter
 *
 * @param pSrc A Source of Juno GSE packets with content type 0x83, other
 *        packet types maybe mixed in, but these are merely passed on 
 *        un-altered
 *
 * @param uProgID The program ID to attach to any generated packets.
 */
JWavesUsegFilter* new_JWavesUsegFilter(GsePktSrc* pSrc, byte uProgID);


/*@}*/

#ifdef __cplusplus
}
#endif

#endif /* _JWavesUsegFilter_H_ */
