import logging
import os
import os.path

from copy import deepcopy
from os.path import join as pjoin


##############################################################################
# Global, used to keep track of names that have already been handed out.
g_dStaticOut = {}           # used in getOutputFile()

##############################################################################
def staticOutFile(log, tTime, sType, sInterface):
	"""Return an output file object based on the UTC time, data type and
	   interface for a packet.  The inputs are:
		
		log - A logger object from the standard python logging module.
		
	   tTime - a time tuple (nYear, nMonth, nDay, nHour, nMin, nSec[, ...])
	   sType - one of 'SCI', 'HSK', 'BST', 'PRC', a hex digit string.
	   sInterface - one of 'HR', 'LR'
	
	   The returned file object is cached so that the same one can be 
		returned over and over for the life of the program.
	"""
	
	global g_dStaticOut
	
	# If the file pointer is already open, then reuse it, for the life of
	# the program, this is different from dynOutFile()
	if g_dStaticOut.has_key( (sType, sInterface) ):
		return g_dStaticOut[ (sType, sInterface) ]
	
	sTopDir = os.getenv("WAVES_DATA")

	if sTopDir == None:
		raise RuntimeError("Environment variable WAVES_DATA is not defined")
	
	if not os.path.exists(sTopDir):
		raise RuntimeError("Output directory: %s does not exist"%sTopDir)
	
	sYear  = "%04d"%tTime[0]
	sMonth = "%02d"%tTime[1]
	sDay   = "%02d"%tTime[2]
	
	sOutDir = pjoin(sTopDir, "%s_%s"%(sYear, sMonth), sDay)
	
	sHour  = "%02d"%tTime[3]
	sMin   = "%02d"%tTime[4]
	sSec   = "%02d"%tTime[5]
	
	sSuffix = None
	if sType.lower() == 'sci':
		if sInterface.lower() == 'hr':
			sSuffix = '.hrs.raw'
		elif sInterface.lower() == 'lr':
			sSuffix = '.lrs.raw'
		else:
			raise ValueError("sInterface == '%s', expected either 'HR' or 'LR'."%sInterface)
	else:
		sSuffix = '.%s.raw'%sType.lower()
	
	sOutFile = "%s_%s_%sT%s_%s_%s"%(sYear, sMonth, sDay, sHour, sMin, sSec,
	                                sSuffix)
	
	sMonthDir = os.path.dirname(sOutDir)
	if not os.path.exists( sMonthDir ):
		os.makedirs( sMonthDir )
		os.chmod(sMonthDir, stat.S_ISGID | stat.S_IRWXU | stat.S_IRWXG )
	
	if not os.path.exists( sOutDir ):
		os.makedirs( sOutDir )
		os.chmod(sOutDir, stat.S_ISGID | stat.S_IRWXU | stat.S_IRWXG )
	
	# make the output file
	log.info("Opening %s for output"%pjoin(sOutDir, sOutFile))
		
	g_dStaticOut[ (sType, sInterface) ] = file(pjoin(sOutDir, sOutFile), 'wb')
	
	return g_dStaticOut[ (sType, sInterface) ]

	
