#define _POSIX_C_SOURCE 200112

#ifdef GSE_POSIX_THREADS
#include <pthread.h>
#endif


#include <rpwgse/Gse.h>
#include "../jwaves/JWavesDefs.h"
#include "../jwaves/JWavesData.h"
#include "../jwaves/JWavesPkt.h"
#include "../jwaves/JWavesSDSblk.h"


/*****************************************************************************/
void print_sciblk(JWavesSDSblk* pSci){
	
	printf("Sequence No:   %d\n", pSci->uSeqNoLo);
	printf("Collect SCLK:  %d.%03.0f\n", pSci->uColSCLK, 
			                           ((pSci->uColRTI)/40.0)*1000);
	printf("Packet ID:     0x%X\n", (pSci->uPSID >> 4)&0xF);
	printf("Packet Src:    0x%X\n", (pSci->uPSID)&0xF);
	printf("Data Format:   0x%02X\n", pSci->uFmt);
	if(pSci->isSegmented(pSci))
		printf("Segment Num:   0x%02X\n", pSci->uSegNo);
	else
		printf("Segment Num:  (not segmented)\n");
}

/*****************************************************************************/
void print_data(GseXYdata* pData)
{
	struct tm utc;
	uint4 uNano;
	const float4* pY; 
	const float4* pX;
	size_t i, nPrn;
	
	printf("Channel ID:   %s\n", pData->channel(pData));
	printf("Chan Name:    %s\n", pData->channelName(pData));
	printf("Info:         %s\n", pData->channelInfo(pData));
	printf("Y Limits:     %g to %g\n", pData->yLoLimit(pData), 
			                            pData->yHiLimit(pData));
	printf("Capture SCLK: %s\n", pData->captureSCLK(pData));
	
	pData->captureUTC(pData, &utc, &uNano);
	
	printf("Capture UTC:  %04d-%02d-%02dT%02d:%02d:%06.3f\n", utc.tm_year,
			  utc.tm_mon, utc.tm_mday, utc.tm_hour, utc.tm_min, 
			  (double)utc.tm_sec + (uNano/1000000000.0));
	
	printf("Units:        %s,%s (X,Y)\n", pData->Xunit(pData),
	        pData->Yunit(pData));
	
	nPrn = pData->length(pData);
	printf("Data points:  %zd\n", nPrn);
	if(nPrn > 3) nPrn = 3;
	
	pX = pData->Xdata(pData);
	pY = pData->Ydata(pData);
	
	for(i = 0; i < nPrn; i++) 
		printf("(%.3g, %.3g)  ", pX[i], pY[i]); 
	printf("\n\n");
}	

/*****************************************************************************/
int main(int argc, char** argv)
{
	GsePktSrc* pSrc;
	GsePkt* pPkt;
	GseXYdataAry* pDatAry = NULL;
	GseXYdata* pData = NULL;
	JWavesSDSblk* pSci = NULL;
	size_t nReadOffset = 0;
	int i;
	
	static const except_id_t catch_spec[] = {{XCEPT_GROUP_ANY, XCEPT_CODE_ANY}};
	except_t* pExcept;
	
	gse_thread_init(GSE_WARNING, NULL);
	
	if((argc < 2)||(strcmp(argv[1], "-h") == 0)){
		fprintf(stderr, "Tests converting packets to XY data structures\n"
		        "USAGE:  %s INPUT_FILE\n"
		        "\n"
		        "    where INPUT_FILE contains at least one Juno Waves packet\n"
		        "    that has completed ground processing\n\n", argv[0]);
		return 1;
	}
	
	pSrc = (GsePktSrc*)new_GseFilePktSrc(argv[1], jwaves_blocks, at_eof_block);
	
	while( (pPkt = pSrc->readNext(pSrc)) != NULL){
		
		nReadOffset = pSrc->offset(pSrc) - pPkt->pHdr->uTotalLen;
		
		if( (pPkt->pHdr->uContent & 0x0F) != JWAVES_CNT_CAPTURE){
			pPkt->del(pPkt);
			continue;
		}
		
		except_try_push(catch_spec, 1, &pExcept);
		/* Will return to here if an exception is thrown */
		
		if(pExcept == NULL){
			pSci = (JWavesSDSblk*) pPkt->getBlkByType(pPkt, JW_SDS_BLK_TYPE_ID, 0);
			print_sciblk(pSci);
			pDatAry = fact_JWavesXYdataAry(pPkt, 0, pSrc->pktIdx(pSrc), nReadOffset);
			for(i = 0; i < pDatAry->num; i++)
				print_data(pDatAry->pAry[i]);
			pData->del(pData);
			pData = NULL;
		}
		else{
			gse_log_except(GSE_ERROR, pExcept);
		}
		except_try_pop();
	}
	
	return 0;
	
}

