# Common Definitions #########################################################

#PYVER=$(shell python -c "import sys; print '.'.join( sys.version.split()[0].split('.')[:2] )")
#PYLIB=$(INST_NAT_LIB)/python$(PYVER)


DATE=$(shell date "+%Y-%m-%d")

# Flags ######################################################################

DEFINES = -DCFG=$(INST_ETC)
INCFLAGS = -I. -I./$(BUILD_DIR)
WARNFLAGS = -Wall 

CFLAGS = -fexceptions -rdynamic -fPIC -ggdb -std=c99 \
 $(INCFLAGS) $(WARNFLAGS) $(DEFINES) -DGSE_POSIX_THREADS -DUSE_CUPS

CC=gcc

#LFLAGS = -lefence -lcups -lpthread -ldl -lm 
LFLAGS = -lcups -lpthread -ldl -lm 

# Input Files ################################################################

HDR_IN = except.h list.h dict.h GseTypes.h GseExcept.h GseLog.h GseInit.h \
 GseWrap.h GseFile.h GseConv.h GseString.h GseFileStr.h GseSys.h GseTime.h \
 GseMath.h GseRunAvg.h aryTplt.h ptrAryTplt.h aryTplt.c ptrAryTplt.c \
 dictTplt.h ptrDictTplt.h dictTplt.c ptrDictTplt.c GseArrays.h GseData.h \
 GseBlock.h GseProcBlk.h GseHdr.h GsePkt.h GsePktSrc.h GseFilePktSrc.h \
 GsePktSink.h GseFilePktSink.h GsePktFilter.h GseRiceDecomp.h GseProp.h \
 GseField.h GseParser.h GseFieldPrn.h GsePrnSvc.h Gse.h listTplt.c listTplt.h \
 GseMD5.h GseCrc.h GseMultiFileSink.h GseTreeWatch.h GseMultiFileSrc.h
 
SRC_IN = except.c list.c dict.c GseTypes.c GseExcept.c GseLog.c GseInit.c \
 GseWrap.c GseFile.c GseConv.c GseString.c GseFileStr.c GseSys.c GseTime.c \
 GseMath.c GseRunAvg.c GseArrays.c GseData.c GseBlock.c GseProcBlk.c \
 GseHdr.c GsePkt.c GseFilePktSrc.c GseFilePktSink.c GsePktFilter.c \
 GseRiceDecomp.c GseProp.c GseField.c GseParser.c GseFieldPrn.c GsePrnSvc.c \
 GsePktSrc.c GseMD5.c GseCrc.c GseMultiFileSink.c GseTreeWatch_linux.c \
 GseMultiFileSrc.c

# Running SWIG on Gse.i causes the creation of _rpwgse.so and rpwgse.py
SWIG_BUILD_IN = Gse.i

SWIG_DEFS_IN = Gse.i GseTypes.i GseExcept.i GseLog.i GseString.i GsePkt.i \
 GseFilePktSrc.i GseField.i GseProp.i GseParser.i GseHdr.i GseBlock.i \
 GseProcBlk.i
 
SWIG_HPP_IN = Gse.hpp GsePkt.hpp GseBlock.hpp GseProcBlk.hpp GseProp.hpp \
 GseField.hpp GseParser.hpp

BISON_IN = GsePsrBldr.y
#BISON_IN=

LEX_IN = GsePsrBldrLxr.l
#LEX_IN=

PY_IN = rpwgse_util.py rpwgse_pkt.py

TEST_IN = test_throw test_alloc test_object sizes test_read test_tar test_untar\
 test_segcatch test_time test_dict test_psrbldr test_md5sum test_treewatch\
 test_abspath
 
PAGES_IN = aryTplt.page  main.page  ptrAryTplt.page  spec.page

MAN5_IN = gse_parser_conf.5

EXAMPLE_IN = except1.c hash1.c

# Patterns form input files ##################################################

HDRS = $(patsubst %,rpwgse/%,$(HDR_IN)) \
               $(patsubst %.y,$(BUILD_DIR)/rpwgse/%.h,$(BISON_IN)) \
               $(patsubst %.l,$(BUILD_DIR)/rpwgse/%.h,$(LEX_IN)) 

INSTALL_HDRS = $(patsubst %,$(INST_INC)/rpwgse/%,$(HDR_IN)) \
               $(patsubst %.y,$(INST_INC)/rpwgse/%.h,$(BISON_IN)) \
               $(patsubst %.l,$(INST_INC)/rpwgse/%.h,$(LEX_IN)) \

SRCS = $(patsubst %.c,rpwgse/%.c,$(SRC_IN))

LEX_SRCS = $(patsubst %.l,$(BUILD_DIR)/%.c,$(LEX_IN)) 

BISON_SRCS = $(patsubst %.y,$(BUILD_DIR)/%.c,$(BISON_IN))

SWIG_SRC = $(patsubst %.i,$(BUILD_DIR)/%_wrap.cpp,$(SWIG_BUILD_IN))

SWIG_DEFS = $(patsubst %.i,rpwgse/%.i,$(SWIG_DEFS_IN)) \
 $(patsubst %.hpp,rpwgse/%.hpp,$(SWIG_HPP_IN)) 
 
INSTALL_SWIG = $(patsubst %.i,$(INST_INC)/rpwgse/%.i,$(SWIG_DEFS_IN)) \
 $(patsubst %.hpp,$(INST_INC)/rpwgse/%.hpp,$(SWIG_HPP_IN)) 

PY_SRCS = $(patsubst %.py,rpwgse/%.py,$(PY_IN))
INSTALL_PYSRCS = $(patsubst %.py,$(INST_HOST_LIB)/%.py,$(PY_IN))

TEST_SRCS = $(patsubst %,test/%.c,$(TEST_IN))

OBJS := $(patsubst rpwgse/%.c,$(BUILD_DIR)/%.o,$(SRCS)) \
        $(patsubst $(BUILD_DIR)/%.c,$(BUILD_DIR)/%.o,$(BISON_SRCS)) \
	$(patsubst $(BUILD_DIR)/%.c,$(BUILD_DIR)/%.o,$(LEX_SRCS))

TEST_PROGS = $(patsubst %,$(BUILD_DIR)/%,$(TEST_IN))

INSTALL_MAN = $(patsubst %,$(INST_MAN)/man3/%,$(MAN3_IN)) \
              $(patsubst %,$(INST_MAN)/man5/%,$(MAN5_IN))

INSTALL_PAGES = $(patsubst %,$(INST_DOC)/rpwgse/page/%,$(PAGES_IN))

INSTALL_EXAMPLES = $(patsubst %,$(INST_DOC)/rpwgse/examples/%,$(EXAMPLE_IN))

# Pattern Rules #############################################################

# Cancel the built in gmake rules that depend on the file sufficies
.SUFFIXES:
.SUFFIXES: .c .o .h .py

# Pattern rule for the test executables
$(BUILD_DIR)/%:test/%.c $(BUILD_DIR)/librpwgse_t.so
	$(CC) $(CFLAGS) -L$(INST_NAT_LIB) -lrpwgse_t $(LFLAGS) -o $@ $<
#	$(CC) $(CFLAGS) $(LFLAGS) -o $@ $< $(BUILD_DIR)/librpwgse_t.a


# Pattern rule for the library object files
$(BUILD_DIR)/%.o:rpwgse/%.c
	$(CC) -c $(CFLAGS) -o $@ $<
	-chmod g+w $@

# Pattern rule for generating python module C code from .i files
$(BUILD_DIR)/%_wrap.cpp $(BUILD_DIR)/%.py:rpwgse/%.i $(SWIG_DEFS)
	swig -Wall -dump_classes -c++ -python -nodefaultctor -I. -o $(BUILD_DIR)/$*_wrap.cpp $<
	swig -python -external-runtime $(BUILD_DIR)/swigpyrun.h
	
# Pattern rule for installing python code from the builddir
$(INST_HOST_LIB)/%:$(BUILD_DIR)/%
	-if [ ! -e $(INST_HOST_LIB) ]; then mkdir -p $(INST_HOST_LIB); chmod g+w $(INST_HOST_LIB); fi
	cp $< $@

# Pattern rule for installing pure python code directly from the source dir
$(INST_HOST_LIB)/%:rpwgse/%
	install -D -m 664 $< $@
	
	 	
# Pattern rule for generating C and H files from flex input files
$(BUILD_DIR)/rpwgse/%.h $(BUILD_DIR)/%.c:rpwgse/%.l
	flex -o $(BUILD_DIR)/$*.c --header-file=$(BUILD_DIR)/rpwgse/$*.h $< 
	
# Pattern rule for generationg C and H files from bison input files
$(BUILD_DIR)/rpwgse/%.h $(BUILD_DIR)/%.c:rpwgse/%.y
	bison -v -o $(BUILD_DIR)/$*.c --defines=$(BUILD_DIR)/rpwgse/$*.h $<
	
# Generic Pattern rule for installing files in the juno library location  
$(INST_NAT_LIB)/%:$(BUILD_DIR)/%
	install -D -m 775 $< $@
	
# Generic pattern rule for installing header files, or other items in the
# include area such as swig interface definitions
$(INST_INC)/rpwgse/%:rpwgse/%
	cp $< $@
	-chmod g+w $@
	
# Generic pattern rule for installing generated header files
$(INST_INC)/rpwgse/%:$(BUILD_DIR)/rpwgse/%
	cp $< $@
	-chmod g+w $@
		
# Generic pattern rule for installing extra doxygen pages
$(INST_DOC)/rpwgse/page/%.page:doc/%.page
	cp $< $@
	-chmod g+w $@
	
# Generic pattern rules for installing man pages
$(INST_MAN)/man5/%.5:doc/%.5
	-if [ ! -e $(INST_MAN)/man5 ] ; then mkdir -p $(INST_MAN)/man5; chmod g+w $(INST_MAN)/man5; fi
	cp $< $@

$(INST_MAN)/man3/%.3:doc/%.3
	-if [ ! -e $(INST_MAN)/man5 ] ; then mkdir -p $(INST_MAN)/man5; chmod g+w $(INST_MAN)/man5; fi
	cp $< $@

# Generic pattern rule for installing code examples
$(INST_DOC)/rpwgse/examples/%:doc/examples/%
	if [ ! -e $(INST_DOC)/rpwgse/examples ]; then mkdir -p $(INST_DOC)/rpwgse/examples; fi
	cp $< $@
	-chmod g+w $@

# Explicit Rules #############################################################

# Direct make not to nuke the intermediate .o files
.SECONDARY: $(OBJS) $(LEX_SRCS) $(BISON_SRCS)

build:$(BUILD_DIR) $(BUILD_DIR)/librpwgse_t.so

test:$(TEST_PROGS)

$(BUILD_DIR)/librpwgse_t.a:$(OBJS)
	ar -crs $@ $(OBJS)

$(BUILD_DIR)/librpwgse_t.so:$(OBJS)
	gcc -rdynamic -ggdb -shared -o $@ $(OBJS) $(LFLAGS)

nothreads:$(BUILD_DIR) $(BUILD_DIR)/librpwgse.a $(TEST_PROGS)
	
$(BUILD_DIR):
	@if [ ! -e "$(BUILD_DIR)/rpwgse" ]; then \
	  echo mkdir -p $(BUILD_DIR)/rpwgse; \
	  mkdir -p $(BUILD_DIR)/rpwgse; chmod -R g+w $(BUILD_DIR); fi

install_nothreads: $(INST_NAT_LIB)/librpwgse.a $(INST_INC)/rpwgse $(INSTALL_HDRS)
	

install: build $(INST_NAT_LIB)/librpwgse_t.so $(INST_INC)/rpwgse $(INSTALL_HDRS) 

prn_objs:
	@for obj in $(OBJS); do echo $$obj; done


$(INST_INC)/rpwgse:
	if [ ! -e $@ ]; then mkdir -p $@; chmod g+w $@; fi
	
$(BUILD_DIR)/_rpwgse.so:$(HDRS) $(SWIG_SRC)
	env INST_INC=$(INST_INC) INST_NAT_LIB=$(INST_NAT_LIB) python setup.py build_ext --inplace
	mv _rpwgse.so $@

$(INST_EXT_LIB)/_rpwgse.so:$(BUILD_DIR)/_rpwgse.so
	install -D -m 775 $< $@

$(INST_EXT_LIB)/rpwgse.py:$(BUILD_DIR)/rpwgse.py
	install -D -m 664 $< $@

pylib:$(BUILD_DIR) install $(BUILD_DIR)/_rpwgse.so $(BUILD_DIR)/rpwgse.py \
      $(PY_SRCS) $(SWIG_DEFS)

pylib_install: pylib $(INST_EXT_LIB)/_rpwgse.so $(INST_EXT_LIB)/rpwgse.py \
      $(INSTALL_PYSRCS) $(INSTALL_SWIG)
	
clean:
	rm $(OBJS)  $(TEST_PROGS)

distclean:
	rm -r -f $(BUILD_DIR) build html librpwgse.tag

# Documentation Rules ########################################################
doc_install: $(INST_DOC)/rpwgse/page $(INSTALL_PAGES) html $(INSTALL_MAN) $(INSTALL_EXAMPLES)
	if [ ! -e $(INST_DOC)/rpwgse ]; then mkdir -p $(INST_DOC)/rpwgse; fi
	cp --remove-destination -r html librpwgse.tag $(INST_DOC)/rpwgse
	chmod -R g+w $(INST_DOC)/rpwgse

# make the doc/rpwgse/pages dir
$(INST_DOC)/rpwgse/page:
	if [ ! -e $@ ]; then mkdir -p $@; chmod g+w $@; fi	

html:$(HDRS) Doxyfile
	env PREFIX=$(PREFIX) doxygen




