# Common Definitions #########################################################

#SHELL=/opt/csw/bin/bash

# Pick a default install location, if user doesn't have one defiend
ifeq ($(PREFIX),)
PREFIX=/opt/project/juno/SunOS.sun4u
endif

CFG=$(PREFIX)/cfg
DOC=$(PREFIX)/doc
BIN=$(PREFIX)/bin
SRC=$(PREFIX)/src

PYVER=$(shell python -c "import sys; print '.'.join( sys.version.split()[0].split('.')[:2] )")

PYLIB=$(LIB)/python$(PYVER)
INC=$(PREFIX)/include
SHARE=$(PREFIX)/share

DATE=$(shell date "+%Y-%m-%d")
ARCH=$(shell uname -s).$(shell uname -m)

BUILD_DIR=build.$(ARCH)

# Flags ######################################################################

DEFINES = -DCFG=$(CFG)
INCFLAGS = -I. -I./$(BUILD_DIR)
WARNFLAGS = -errwarn

CFLAGS = -xcode=pic32 -g -xc99 \
 $(INCFLAGS) $(WARNFLAGS) $(DEFINES) $(TH_CFLAGS)

CC=/opt/SUNWspro/bin/cc

LFLAGS = $(TH_LFLAGS) -ldl -lm

# Input Files ################################################################

HDR_IN = GseInit.h except.h list.h dict.h GseBlock.h GseExcept.h GseWrap.h \
 GseProcBlk.h GseConv.h GseHdr.h GsePkt.h GsePktSrc.h GseFilePktSrc.h \
 GseString.h GseLog.h GseFile.h GsePktSink.h GseFilePktSink.h GseBufAry.h \
 aryTplt.h aryTplt.c ptrAryTplt.h ptrAryTplt.c Gse.h GseTypes.h GseFileStr.h \
 GseData.h GseRiceDecomp.h GseExtorProps.h GseExtor.h GseSys.h GseSys.h \
 GseSimpMath.h GseRunAvg.h
 
SRC_IN = except.c dict.c list.c GseTypes.c GseInit.c GseBlock.c GseExcept.c GseWrap.c \
 GseProcBlk.c GseConv.c GseHdr.c GsePkt.c GseFilePktSrc.c GseString.c GseLog.c \
 GseFile.c GseFilePktSink.c GseBufAry.c GseFileStr.c GseData.c GseRiceDecomp.c \
 GseExtorProps.c GseExtor.c GseSys.c GseSimpMath.c GseRunAvg.c

# Running SWIG on Gse.i causes the creation of _rpwgse.so and rpwgse.py
SWIG_IN = gse.i

BISON_IN = GseExtorParser.y

LEX_IN = GseExtorLexer.l

PY_IN = rpwgse_util.py rpwgse_pkt.py

TEST_IN = test_throw test_alloc test_object sizes test_read test_tar test_untar\
 test_segcatch test_lexer test_parser test_time

# Patterns form input files ##################################################

INSTALL_HDRS = $(patsubst %,$(INC)/rpwgse/%,$(HDR_IN)) \
               $(patsubst %.y,$(INC)/rpwgse/%.h,$(BISON_IN)) \
               $(patsubst %.l,$(INC)/rpwgse/%.h,$(LEX_IN)) 

SRCS = $(patsubst %.c,rpwgse/%.c,$(SRC_IN))

LEX_SRCS = $(patsubst %.l,$(BUILD_DIR)/%.c,$(LEX_IN)) 

BISON_SRCS = $(patsubst %.y,$(BUILD_DIR)/%.c,$(BISON_IN))

SWIG_IFACE = $(patsubst %.i,rpwgse/%.i,$(SWIG_IN))
SWIG_SRCS = $(patsubst %.i,$(BUILD_DIR)/%_wrap.c,$(SWIG_IN))

PY_SRCS = $(patsubst %.py,rpwgse/%.py,$(PY_IN))
INSTALL_PYSRCS = $(patsubst %.py,$(PYLIB)/%.py,$(PY_IN))

TEST_SRCS = $(patsubst %,test/%.c,$(TEST_IN))

OBJS := $(patsubst rpwgse/%.c,$(BUILD_DIR)/%.o,$(SRCS)) \
        $(patsubst $(BUILD_DIR)/%.c,$(BUILD_DIR)/%.o,$(BISON_SRCS)) \
	$(patsubst $(BUILD_DIR)/%.c,$(BUILD_DIR)/%.o,$(LEX_SRCS))

TEST_PROGS = $(patsubst %,$(BUILD_DIR)/%,$(TEST_IN))

PAGES = aryTplt.page  main.page  ptrAryTplt.page  spec.page

INSTALL_PAGES = $(patsubst %,$(DOC)/rpwgse/page/%,$(PAGES))

# Pattern Rules #############################################################

# Cancel the built in gmake rules that depend on the file sufficies
.SUFFIXES:
.SUFFIXES: .c .o .h .py

# Pattern rule for the test executables
$(BUILD_DIR)/%:test/%.c
	$(CC) $(CFLAGS) -L$(BUILD_DIR) -lrpwgse$(TH_SUF) $(LFLAGS) -o $@ $<

# Pattern rule for the library object files
$(BUILD_DIR)/%.o:rpwgse/%.c
	$(CC) -c $(CFLAGS) -o $@ $<
	@chmod g+w $@

# Pattern rule for generating python module C code from .i files
$(BUILD_DIR)/%_wrap.c $(BUILD_DIR)/%.py:rpwgse/%.i
	swig -python -o $(BUILD_DIR)/$*_wrap.c $< 
	
# Pattern rule for installing python code from the builddir
$(PYLIB)/%:$(BUILD_DIR)/%
	if [ ! -e $(PYLIB) ]; then mkdir -p $(PYLIB); chmod g+w $(PYLIB); fi
	cp $< $@

# Pattern rule for installing pure python code directly from the source dir
$(PYLIB)/%:rpwgse/%
	if [ ! -e $(PYLIB) ]; then mkdir -p $(PYLIB); chmod g+w $(PYLIB); fi
	cp $< $@
	 	
# Pattern rule for generating C and H files from flex input files
$(BUILD_DIR)/rpwgse/%.h $(BUILD_DIR)/%.c:rpwgse/%.l
	flex -o $(BUILD_DIR)/$*.c --header-file=$(BUILD_DIR)/rpwgse/$*.h $< 
	
# Pattern rule for generationg C and H files from bison input files
$(BUILD_DIR)/rpwgse/%.h $(BUILD_DIR)/%.c:rpwgse/%.y
	bison -v -o $(BUILD_DIR)/$*.c --defines=$(BUILD_DIR)/rpwgse/$*.h $<
	
# Pattern rule for shared objects
$(BUILD_DIR)/%.so:$(OBJS)
	gcc -rdynamic -ggdb -shared -o $@ $(OBJS) $(LFLAGS)
	
# Pattern rule for archive files
$(BUILD_DIR)/%.a:$(OBJS)
	ar -crs $@ $(OBJS)

# Generic Pattern rule for installing files in the juno library location  
$(LIB)/%:$(BUILD_DIR)/%
	if [ ! -e $(LIB) ]; then mkdir -p $(LIB); chmod g+w $(LIB); fi
	cp $< $@_$(DATE)
	if [ -e $@ ]; then rm $@; fi
	ln -s $@_$(DATE) $@
	
# Generic pattern rule for installing header files
$(INC)/rpwgse/%:rpwgse/%
	cp $< $@
	chmod g+w $@
	
# Generic pattern rule for installing generated header files
$(INC)/rpwgse/%:$(BUILD_DIR)/rpwgse/%
	cp $< $@
	chmod g+w $@
	
# Generic pattern rule for installing extra doxygen pages
$(DOC)/rpwgse/page/%.page:doc/%.page
	cp $< $@
	chmod g+w $@

# Explicit Rules #############################################################

# Direct make not to nuke the intermediate .o files
.SECONDARY: $(OBJS) $(LEX_SRCS) $(BISON_SRCS)

build:TH_SUF=_t
build:TH_CFLAGS=-DGSE_POSIX_THREADS
build:TH_LFLAGS=-lpthread

build:$(BUILD_DIR) $(BUILD_DIR)/librpwgse_t.so $(BUILD_DIR)/librpwgse_t.a \
 $(TEST_PROGS)

nothreads:$(BUILD_DIR) $(BUILD_DIR)/librpwgse.so $(BUILD_DIR)/librpwgse.a\
 $(TEST_PROGS)
	
$(BUILD_DIR):
	if test ! -e "$(BUILD_DIR)/rpwgse" ; then \
	  echo mkdir -p $(BUILD_DIR)/rpwgse; \
	  mkdir -p $(BUILD_DIR)/rpwgse; chmod -R g+w $(BUILD_DIR); fi

install_nothreads: $(LIB)/librpwgse.a $(INC)/rpwgse $(INSTALL_HDRS)
	

install: build $(LIB)/librpwgse_t.a $(INC)/rpwgse $(INSTALL_HDRS) 

prn_objs:
	@for obj in $(OBJS); do echo $$obj; done


$(INC)/rpwgse:
	if test ! -e $@ ; then mkdir -p $@; chmod g+w $@; fi
	
$(BUILD_DIR)/_rpwgse.so:$(HDRS) $(SWIG_SRCS)
	python setup.py build_ext --inplace
	mv _rpwgse.so $@

pylib:$(BUILD_DIR) install $(BUILD_DIR)/_rpwgse.so $(BUILD_DIR)/rpwgse.py $(PYSRCS)

pylib_install: pylib $(PYLIB)/_rpwgse.so $(PYLIB)/rpwgse.py $(INSTALL_PYSRCS)
	
clean:
	rm $(OBJS)  $(TEST_PROGS)

distclean:
	rm -r -f $(BUILD_DIR) build html librpwgse.tag

# Documentation Rules ########################################################
install_doc: $(DOC)/rpwgse/page $(INSTALL_PAGES) html
	if test ! -e $(DOC)/rpwgse ; then mkdir -p $(DOC)/rpwgse; fi
	cp --remove-destination -r html librpwgse.tag $(DOC)/rpwgse
	chmod -R g+w $(DOC)/rpwgse

# make the doc/rpwgse/pages dir
$(DOC)/rpwgse/page:
	if [ ! -e $@ ]; then mkdir -p $@; chmod g+w $@; fi	

html:$(HDRS) Doxyfile
	env PREFIX=$(PREFIX) doxygen




