# Common Definitions ########################################################
PREFIX=C:/local/juno

CFG=$(PREFIX)/cfg
DOC=$(PREFIX)/doc
BIN=$(PREFIX)/bin
SRC=$(PREFIX)/src
LIB=$(PREFIX)/lib
PYLIB=$(LIB)/python
INC=$(PREFIX)/include
SHARE=$(PREFIX)/share

DATE=$(shell date "+%Y-%m-%d")
ARCH=$(shell uname -s)

BUILD=build.$(ARCH)

# Flags ######################################################################

DEFINES = -DCFG=$(CFG)
INCFLAGS = -I. -IC:/local/include
WARNFLAGS = -Wall -Werror

# NOTE: -std=c99 is not supported in windows, just triggers __STRICT_ANSII
#                macro which turns off alot of standard functions such as
#                strcasecmp
CFLAGS = -fexceptions -ggdb $(INCFLAGS) $(WARNFLAGS) $(DEFINES) $(TH_CFLAGS)

CC=gcc

LFLAGS = $(TH_LFLAGS) C:/local/lib/dbghelp.lib -lKernel32 -lbfd -liberty -lm

# Specific Definitions ######################################################

HDR_IN = GseInit.h except.h list.h GseBlock.h GseWinSvc.h GseExcept.h GseWrap.h \
 GseProcBlk.h GseConv.h GseHdr.h GsePkt.h GsePktSrc.h GseFilePktSrc.h \
 GseString.h GseLog.h GseFile.h GsePktSink.h GseFilePktSink.h GseBufAry.h \
 aryTplt.h aryTplt.c ptrAryTplt.h ptrAryTplt.c Gse.h GseTypes.h GseFileStr.h \
 GseData.h GseRiceDecomp.h GseExtorProps.h GseExtor.h GseSys.h GseSimpMath.h

#things that took more work to get ported from linux are at the end
SRC_IN = except.c list.c GseTypes.c GseBlock.c GseWrap.c GseWinSvc.c \
 GseProcBlk.c GseConv.c GseHdr.c GsePkt.c GseFilePktSrc.c GseString.c GseLog.c \
 GseFile.c GseFilePktSink.c GseBufAry.c GseFileStr.c GseData.c GseRiceDecomp.c \
 GseInit.c GseExcept.c BfdContext.c GseExtorProps.c GseExtor.c GseSys.c \
 GseSimpMath.c GseRunAvg.c

BISON_IN = GseExtorParser.y

LEX_IN = GseExtorLexer.l
 
TEST_IN = test_throw test_alloc test_object sizes test_read test_tar test_untar \
          test_segcatch test_lexer test_parser test_time

# Patterns form input files ##################################################

INSTALL_HDRS = $(patsubst %,$(INC)/rpwgse/%,$(HDR_IN)) \
               $(patsubst %.y,$(INC)/rpwgse/%.h,$(BISON_IN)) \
               $(patsubst %.l,$(INC)/rpwgse/%.h,$(LEX_IN)) 

SRCS = $(patsubst %.c,rpwgse/%.c,$(SRC_IN))

LEX_SRCS = $(patsubst %.l,$(BUILD)/%.c,$(LEX_IN)) 

BISON_SRCS = $(patsubst %.y,$(BUILD)/%.c,$(BISON_IN))

TEST_SRCS = $(patsubst %,test/%.c,$(TEST_IN))

OBJS := $(patsubst rpwgse/%.c,$(BUILD)/%.o,$(SRCS)) \
        $(patsubst $(BUILD)/%.c,$(BUILD)/%.o,$(BISON_SRCS)) \
	     $(patsubst $(BUILD)/%.c,$(BUILD)/%.o,$(LEX_SRCS))

TEST_PROGS = $(patsubst %,$(BUILD)/%,$(TEST_IN))

PAGES = aryTplt.page  main.page  ptrAryTplt.page  spec.page

INSTALL_PAGES = $(patsubst %,$(DOC)/rpwgse/page/%,$(PAGES))

# Pattern Rules #############################################################

# Cancel the built in gmake rules that depend on the file sufficies
.SUFFIXES:
.SUFFIXES: .c .o .h

# Pattern rule for the test executables
$(BUILD)/%:test/%.c
	$(CC) $(CFLAGS) $< $(BUILD)/librpwgse$(TH_SUF).a $(LFLAGS) -o $@

# Pattern rule for the rpwgse library object files
$(BUILD)/%.o:rpwgse/%.c
	$(CC) -c $(CFLAGS) -o $@ $<
	
# Pattern rule for generating C and H files from flex input files
$(BUILD)/rpwgse/%.h $(BUILD)/%.c:rpwgse/%.l $(BUILD)
	flex -o $(BUILD)/$*.c --header-file=$(BUILD)/rpwgse/$*.h $< 
	
# Pattern rule for generationg C and H files from bison input files
$(BUILD)/rpwgse/%.h $(BUILD)/%.c:rpwgse/%.y $(BUILD)
	bison -v -o $(BUILD)/$*.c --defines=$(BUILD)/rpwgse/$*.h $<

# Pattern rule for archive files
$(BUILD)/%.a:$(OBJS)
	ar -crs $@ $(OBJS)
	
# Generic Pattern rule for installing files in the juno library location  
$(LIB)/%:$(BUILD)/%
	if [ ! -e $(LIB) ]; then mkdir -p $(LIB); fi
	cp $< $@
	
# Generic pattern rule for installing header files
$(INC)/rpwgse/%:rpwgse/%
	if [ ! -e $(INC)/rpwgse ]; then mkdir -p $(INC)/rpwgse; fi
	cp $< $@

# Generic pattern rule for installing generated header files
$(INC)/rpwgse/%:$(BUILD)/rpwgse/%
	cp $< $@
	
# Generic pattern rule for installing extra doxygen pages
$(DOC)/rpwgse/page/%.page:doc/%.page
	if [ ! -e $(DOC)/rpwgse/page ]; then mkdir -p $(DOC)/rpwgse/page; fi
	cp $< $@

# Explicit Rules #############################################################

# Direct make not to nuke the intermediate .o files
.SECONDARY: $(OBJS) $(LEX_SRCS) $(BISON_SRCS)

build:TH_SUF=_t
build:TH_CFLAGS=-DGSE_POSIX_THREADS
build:TH_LFLAGS=-lpthread

build:$(BUILD) $(BUILD)/librpwgse_t.a $(TEST_PROGS)
	
nothreads:$(BUILD) $(BUILD)/librpwgse.a $(TEST_PROGS)
	
$(BUILD):
	@if [ ! -e "$(BUILD)/rpwgse" ]; then \
	  echo mkdir -p $(BUILD)/rpwgse; \
	  mkdir -p $(BUILD)/rpwgse; fi

install_nothreads: $(LIB)/librpwgse.a $(INC)/rpwgse $(INSTALL_HDRS)
	

install: $(LIB)/librpwgse_t.a $(INC)/rpwgse $(INSTALL_HDRS)

prn_objs:
	@for obj in $(OBJS); do echo $$obj; done


$(INC)/rpwgse:
	if [ ! -e $@ ]; then mkdir -p $@; fi
	
clean:
	rm $(OBJS)  $(TEST_PROGS)

distclean:
	rm -r -f $(BUILD) html librpwgse.tag

# Documentation Rules ########################################################
install_doc: $(DOC)/rpwgse/page $(INSTALL_PAGES) html
	if [ ! -e $(DOC)/rpwgse ]; then mkdir -p $(DOC)/rpwgse; fi
	cp -r html librpwgse.tag $(DOC)/rpwgse

# make the doc/rpwgse/pages dir
$(DOC)/rpwgse/page:
	if [ ! -e $@ ]; then mkdir -p $@; fi	

html:$(HDRS)
	env PREFIX=$(PREFIX) doxygen

##############################################################################	
# Explicit rule for making a DLL, not necessarily a good idea, would have
# to define EXPOPT every where and make a symbol file.  Too much work for
# little perceptable gain, the rule are here so they are not forgotten
$(BUILD)/rpwgse.dll:DLL_FLAG=-DBUILD_DLL
$(BUILD)/rpwgse.dll:$(OBJS)
	gcc -ggdb -shared -o $@ $(OBJS) $(LFLAGS) -Wl,--out-implib,$(BUILD)/librpwgse.a

