/** file except1.c */
#include <stdlib.h>
#include <assert.h>
#include <rpwgse/GseExcept.h>

#define MY_GROUP 42 
#define MY_CODE 1 

static void func_that_throws(void){
  except_throw(MY_GROUP, MY_CODE, "this is an exception");
}

static void func_that_cleans_up(void){
	void *local_data = malloc(10); 
	except_cleanup_push(free, local_data); 
	func_that_throws(); 
	except_checked_cleanup_pop(free, except_call); 
}

void func_that_catches(void){
	/* catch specification */
	static const except_id_t catch_spec[] = { 
	  {MY_GROUP, XCEPT_CODE_ANY}
	}; 
	/* exception handle */
	except_t *exc; 
	
	except_try_push(catch_spec, 1, &exc); 
	/*
	 * Start of try-catch region: when exception is
	 * thrown, control returns here. 
	 */
	
	if (exc == 0){
		/* try code that may throw an exception */
		func_that_cleans_up(); 
	}
	else{
		/* handle exception that was thrown */ 
		assert (except_group(exc) == MY_GROUP); 
		printf("exception caught: %s %ld %ld\n", 
			except_message(exc), 
			except_group(exc), 
			except_code(exc)); 
			
		goto terminate; /* ERROR! jumping out of try-catch */ 
	} 
	
	/* end of try-catch region */
	except_try_pop(); 
terminate: 
	; 
} 
