#!/usr/bin/env python 

import sys
import re
import os
import os.path

##############################################################################
# globals

# These are replacable includes

g_lReplaceables = [
	('ptrAryTplt.h', re.compile(r'(#include\s)(")(\S*ptrAryTplt.h)(")') ),
	('aryTplt.h', re.compile(r'(#include\s)(")(\S*aryTplt.h)(")') ),
	('ptrDictTplt.h', re.compile(r'(#include\s)(")(\S*ptrDictTplt.h)(")') ),
	('dictTplt.h', re.compile(r'(#include\s)(")(\S*dictTplt.h)(")') ),
	('listTplt.h', re.compile(r'(#include\s)(")(\S*listTplt.h)(")') )
]



##############################################################################
def main(argv):

	prnerr = sys.stderr.write
	
	if len(argv) == 0 or argv[1] == '-h' or argv[1].endswith('help'):
		prnerr("Doxygen input filter for header files that contain C style safe")
		prnerr("macro templates.  The contents of the #included file are inserted" )
		prnerr("in the place of the include statement since Doxygen doesn't offer")
		prnerr("this capibility.  The recognized macro template includes are:")
		prnerr("")		
		for tup in g_lReplaceables:
			prnerr("  %s"%tup[0])
		prnerr("")
		prnerr("Usage %s [FILENAME]"%os.path.basename(argv[0]))
		return 1
		
	sDir = os.path.dirname(argv[1])
	if sDir != None:
		sFile = os.path.basename(argv[1])
		os.chdir(sDir)
	else:
		sFile = argv[1]
	
	for sLine in file(sFile,'rb'):
		
		match = None
		
		for (s, regex) in g_lReplaceables:
			match = regex.search(sLine)
			if match != None:
				break
		
		if match == None:
			sys.stdout.write(sLine)
		else:
			print "/* From", match.group(3), "*/"
			fRep = file(match.group(3), 'rb')
			sys.stdout.write(fRep.read())

	return 0

##############################################################################
if __name__ == "__main__":
	sys.exit(main(sys.argv))
