/** @file BfdContext.h - A C version of Edd Dawson's bfd_context class */
#ifndef _BfdContext_H_
#define _BfdContext_H_

#ifdef __cplusplus
extern "C" {
#endif

/* On linux this is in the binutils-devel rpm */
#include <bfd.h>

/** @addtogroup mod_win_svc */
/*@{*/

/** Lookup function names in a file and cache the results.  
 *
 * Used by the windows backtrace() replacement. This class is not used in
 * the Linux and Solaris versions of the library.
 * 
 * @warning On Windows, a single BfdContext instance is used for all non-dll
 * symbol lookup for the entire application.  This is not thread safe.
 *
 * @class BfdContext
 */
typedef struct bfd_context{
	
	/** Get the name of a function given it's offset
	 *
	 * @param pThis A context structure.
	 * @param offset The offset to the symbol in question, presumably found
	 *        via the backtrace() function.
	 * @return A NULL terminated string allocated via calloc containing the
	 *         symbol name, or NULL if no symbol could be found for the
	 *         given offset. 
	 */
	const char* (*getFuncName)(struct bfd_context* pThis, unsigned long offset);
	
	/** Destructor */
	void (*del)(void* vpThis);
	
	bfd* abfd_;
	asection* sec_;
	asymbol** symbol_table_;
	
} BfdContext;

/** Make a new name lookup context on the Heap.
 *
 * @param sFileName An executable file.
 * @return A new BfdContext that can be used to lookup symbol names by thier
 *         addresses for the given file.
 * @public
 * @memberof BfdContext
 */
BfdContext* new_BfdContext(const char* sFileName);

/*@}*/

#ifdef __cplusplus
}
#endif

#endif

