/* Swig python definitions for librpwgse wrappers */
%feature("autodoc", "1");

%module rpwgse
%include "typemaps.i"
		
%header %{
#include <rpwgse/Gse.h>

#ifdef __cplusplus
extern "C" {
#endif

static PyObject* g_pGseError;
static const except_id_t g_gse_catch_spec[] = { {XCEPT_GROUP_ANY, XCEPT_CODE_ANY} }; 

/* WARNING! WARNING! WARNING! This is so not thread safe, don't use the
   rpwgse library from more than one python thread!!! */
static except_t* g_pGseExcept = NULL; 
static char g_gse_xcept_buf[1024];

#ifdef __cplusplus
}
#endif	
%}

/* Define a general exception bouncer from rpwgse to python */
%exception {
	
	except_try_push(g_gse_catch_spec, 1, &g_pGseExcept);
	
	if( g_pGseExcept == 0){
		$action
	}
	else{
		g_gse_xcept_buf[1023] = '\0';
		gse_shortprn_except( (char*)g_gse_xcept_buf, 1023, g_pGseExcept);
		PyErr_SetString(g_pGseError, g_gse_xcept_buf);
		return NULL;	
	}
	
	except_try_pop();
}

/****************************************************************************/
/* The modules, make sure that SWIG doesn't define it's own consturctors,   */
/* these are objects and that's my job */

%nodefaultctor;

%include GseTypes.i
			
%include GseExcept.i
%include GseLog.i

/* %include GseInit.i 
   Nothing needed here unless multi-thread support is needed for the
   python wrapper, since I've never written any multi-threaded python
	code this is not needed for now. -cwp 2009-05-27 */

%include GseString.i

%include GseBlock.i
%include GseProcBlk.i
%include GseHdr.i
%include GsePkt.i


%include GseFilePktSrc.i

%include GseProp.i
%include GseField.i
%include GseParser.i



/* Python Module Initilazation ***********************************************/

%init %{
	g_pGseError = PyErr_NewException("rpwgse.error", PyExc_StandardError, NULL);
	Py_INCREF(g_pGseError);
	PyModule_AddObject(m, "error", g_pGseError);
	
	/* Call Init on module load */
	gse_thread_init(GSE_WARNING, NULL);
%}

/* Additional items for the generated rpwgse.py file, add in some pure python
   code */

%pythoncode %{
from rpwgse_util import *
from rpwgse_pkt  import *
	
def gse_trace(sMsg):
	gse_log_msg(GSE_TRACE, sMsg)
	
def gse_debug(sMsg):
	gse_log_msg(GSE_DEBUG, sMsg)

def gse_notice(sMsg):
	gse_log_msg(GSE_NOTICE, sMsg)
	
def gse_info(sMsg):
	gse_log_msg(GSE_NOTICE, sMsg)

def gse_warning(sMsg):
	gse_log_msg(GSE_WARNING, sMsg)

def gse_error(sMsg):
	gse_log_msg(GSE_ERROR, sMsg)

def gse_critical(sMsg):
	gse_log_msg(GSE_CRITICAL, sMsg)
%}
