#define _POSIX_C_SOURCE 200112

#ifdef GSE_POSIX_THREADS
#include <pthread.h>
#endif

#include "GseCrc.h"

/*****************************************************************************/
/* CRC Generator */

#define                 P_CCITT     0x1021
static bool             gse_bCrcCcittInit = false;
static unsigned short   gse_CrcTabCcitt[256];


#ifdef GSE_POSIX_THREADS
static pthread_mutex_t gse_CrcCcittMtx = PTHREAD_MUTEX_INITIALIZER;
#endif


static void gse_init_crcccitt_tab(){
   int i, j;
   unsigned short crc, c;
	
#ifdef GSE_POSIX_THREADS
	pthread_mutex_lock(&gse_CrcCcittMtx);

	if(gse_bCrcCcittInit == true){ 
		pthread_mutex_unlock(&gse_CrcCcittMtx);
		return;
	}
#else
	if(gse_bCrcCcittInit == true) return;
#endif

   for(i=0; i<256; i++) {
      crc = 0;
      c   = ((unsigned short) i) << 8;

      for(j=0; j<8; j++) {

         if( (crc ^ c) & 0x8000 ) 
            crc = ( crc << 1 ) ^ P_CCITT;
         else
            crc =   crc << 1;

          c = c << 1;
      }
      gse_CrcTabCcitt[i] = crc;
   }
   gse_bCrcCcittInit = true;
	
#ifdef GSE_POSIX_THREADS	
	pthread_mutex_unlock(&gse_CrcCcittMtx);
#endif
}

/* Calc CRC-CCITT on arbitrary len set of char data */
uint2 gse_crc_ccitt(const byte* sData, size_t szLen){
   int i;
   uint2 usIdx, usChar, usCRC;
   
   gse_init_crcccitt_tab();
    
   usCRC = 0;
   for(i = 0; i<szLen; i++){
      usChar  = 0x00ff & (unsigned short) sData[i];
      
      usIdx = (usCRC >> 8) ^ usChar;
      usCRC = (usCRC << 8) ^ gse_CrcTabCcitt[usIdx];
    }
      
   return usCRC;
}

uint2 gse_update_crc_ccitt(uint2 crc, byte b){
	unsigned short usIdx, short_c;
	
	short_c  = 0x00ff & (unsigned short) b;

	gse_init_crcccitt_tab();

	usIdx = (crc >> 8) ^ short_c;
	crc = (crc << 8) ^ gse_CrcTabCcitt[usIdx];

	return crc;
}
