#ifndef MINGW
#define _POSIX_C_SOURCE 200112L
#else
#define WINVER        WindowsXP
#define _WIN32_WINNT  WindowsXP
#endif

#ifdef GSE_POSIX_THREADS
#include <pthread.h>
#endif

#include <strings.h>

#include "GseData.h"
#include "GseExcept.h"
#include "GseString.h"
#include "GseWrap.h"

const char* gse_str_unittype(gse_unittype_t type)
{
	switch(type){
		case NUMBER:          return "Value";                  break;
		case TIME:            return "Time";                   break;
		case FREQUENCY:       return "Frequency";              break;
		case DB:              return "dB";                     break;
		case INV_DB:          return "1/dB";                   break;
		case POWER_PER_HZ:    return "Spectral Density";       break;
		case POWER_PER_SEC:    return "Instantaneous Power";   break;
	}
	gse_throw_m(GSE_XCEPT,GSE_XNOTYETIMP,"String for unit type not listed.");
	return "Unknown";
}


gse_unittype_t gse_unittype_invert(gse_unittype_t type)
{
	switch(type){
		case NUMBER: return NUMBER;
		case TIME: return FREQUENCY;
		case FREQUENCY: return TIME;
		case DB: return INV_DB;
		case INV_DB: return DB;
		case POWER_PER_HZ: return POWER_PER_SEC;
		case POWER_PER_SEC: return POWER_PER_HZ;
	}
	gse_throw_m(GSE_XCEPT,GSE_XNOTYETIMP,"Inversion for unit type not implimented.");
	return NUMBER;
}

/** Given a unit string, try to make its inverse unit string in a sane way.
 * @return a new unit string allocated on the heap.
 */
/*static const char* gse_pre_in[] = {
	"T", "G", "M", "k", 0, "m", "micro", "nano", "pico"
};

static const char* gse_pre_out[] = {
	"pico", "nano", "micro", "m", 0, "k", "M", "G", "T"
};*/

/* Make butt simple for now, improve later */
char* gse_invert_units(const char* sUnit){
	
	if((sUnit == NULL)||(sUnit[0] == '\0'))
		gse_throw(GSE_XCEPT, GSE_XBADARG);
	
	if(strcasecmp(sUnit, "msec") == 0) return gse_strcopy("kHz");
	if(strcasecmp(sUnit, "kHz") == 0) return gse_strcopy("msec");
	
	if(strcasecmp(sUnit, "sec") == 0) return gse_strcopy("Hz");
	if(strcasecmp(sUnit, "Hz") == 0) return gse_strcopy("sec");
	
	if(strcasecmp(sUnit, "micosec") ==0) return gse_strcopy("MHz");
	if(strcasecmp(sUnit, "MHz") ==0) return gse_strcopy("microsec");
	
	if(strcasecmp(sUnit, "nanosec") == 0) return gse_strcopy("GHz");
	if(strcasecmp(sUnit, "GHz") == 0) return gse_strcopy("nanosec");
	
	/* Fallback */
	return gse_string("1/(%s)", sUnit);
}

/*****************************************************************************/
/* Data Structure Array instantiation */

#define C GseXYdataAry
#define T GseXYdata
#define CASCADE_DEL

#include "ptrAryTplt.c"

#undef CASCADE_DEL
#undef C
#undef T 
