/* GseField.i */

%rename(GseField) cGseField;
%newobject cGseField::first() const;
%newobject cGseField::next(const cGseProp* pProp) const;
%newobject cGseField::getProp(const char* sName) const;

%header %{

#include <rpwgse/GseField.hpp>

cGseField::cGseField(const GseField* pWrap){
	if(pWrap == NULL)
		gse_throw_m(GSE_XCEPT,GSE_XBADARG,"Null pointer to cGseField constructor");
	m_pField = pWrap;
}

cGseField::~cGseField(){ }

const char* cGseField::name() const{ return m_pField->name(m_pField); }
const char* cGseField::desc() const{ return m_pField->desc(m_pField); }
char cGseField::valType() const{     return m_pField->valType(m_pField); }
const char* cGseField::sval() const{ return m_pField->sval(m_pField); }
float8 cGseField::fval() const{      return m_pField->fval(m_pField); }
uint8 cGseField::uval() const{       return m_pField->uval(m_pField); }
int8 cGseField::ival() const{        return m_pField->ival(m_pField); }
gse_alarm_stat_t cGseField::alarm() const{ return m_pField->alarm(m_pField); }
bool cGseField::hasProp(const char* sName) const{
	return m_pField->hasProp(m_pField, sName);
}
	
cGseProp* cGseField::first() const{ 
	/* Okay to cast, not going to be the owner anyway */
	GseProp* pCobj = (GseProp*)m_pField->first(m_pField);
	if(pCobj == NULL) return NULL;
	cGseProp* pRet = NULL;
	switch(pCobj->ptype){
	case pt_int:
	case pt_str:
	case pt_flt:  pRet = (cGseProp*)new cGsePropSimple(pCobj); break;
	case pt_list: pRet = (cGseProp*)new cGsePropList(pCobj); break;
	default:      pRet = (cGseProp*)new cGsePropEnum(pCobj); break;
	}
	
	pRet->m_bOwner = false;  /* Be explicit about this! */
	return pRet;
}

cGseProp* cGseField::next(const cGseProp* pProp) const{
	/* Okay to cast, not going to be the owner anyway */
	GseProp* pCobj = (GseProp*)m_pField->next(m_pField, pProp->m_pProp);
	if(pCobj == NULL) return NULL;
	cGseProp* pRet = NULL;
	switch(pCobj->ptype){
	case pt_int:
	case pt_str:
	case pt_flt:  pRet = (cGseProp*)new cGsePropSimple(pCobj); break;
	case pt_list: pRet = (cGseProp*)new cGsePropList(pCobj); break;
	default:      pRet = (cGseProp*)new cGsePropEnum(pCobj); break;
	}
	
	pRet->m_bOwner = false;  /* Be explicit about this! */
	return pRet;
}

gse_prop_type_t cGseField::getPropType(const char* sName) const{
	GseProp* pObj = (GseProp*)m_pField->getProp(m_pField, sName);
	if(pObj == NULL) return pt_invalid;
	else return pObj->ptype;
}

cGseProp* cGseField::getProp(const char* sName) const{
	/* Okay to cast, not going to be the owner anyway */
	GseProp* pCobj = (GseProp*)m_pField->getProp(m_pField, sName);
	if(pCobj == NULL) return NULL;
	cGseProp* pRet = NULL;
	switch(pCobj->ptype){
	case pt_int:
	case pt_str:
	case pt_flt:  pRet = (cGseProp*)new cGsePropSimple(pCobj); break;
	case pt_list: pRet = (cGseProp*)new cGsePropList(pCobj); break;
	default:      pRet = (cGseProp*)new cGsePropEnum(pCobj); break;
	}
	
	pRet->m_bOwner = false;  /* Be explicit about this! */
	return pRet;
}

%}

typedef enum gse_alarm_stat_t{
	no_alarm=0, err_lo, warn_lo, warn_hi, err_hi, inv_alarm_stat
}gse_alarm_stat_t;

%include rpwgse/GseField.hpp


