/* @file GseFieldPrn.h */
#ifndef _GseFieldPrn_H_
#define _GseFieldPrn_H_

#include <rpwgse/GseField.h>

#ifdef __cplusplus
extern "C" {
#endif

/** @addtogroup mod_bin_parser */
/*@{*/

/** Checking for a field printing function */
bool gse_has_field_printer(const char* sName);

/** Print a field's val using a named printer */
char* gse_print_field(const char* sName, const GseField* pField);


/** Validate a format property using a named printer */
char* gse_validate_fmt(const char* sFunc, const GseField* pField);

typedef char* (gse_field_printer)(const GseField* pField);

/** A validator can expect to recieve NULL for the pFmt if non is supplied.
 * It may choose to issue or not issue an error in this case. 
 */
typedef char* (gse_format_validator)(const GseField* pField);

/** Adding new field printers and validators to the library.
 * The printing functions are held in a global library.
 *
 * @warning This function is @b NOT multithread safe.
 */
void gse_add_field_printer(const char* sName, gse_field_printer* pPrinter, 
		                     gse_format_validator* pValidator);

/*@}*/

#ifdef __cplusplus
}
#endif

#endif /* _GseField_H_ */
