#define _POSIX_C_SOURCE 200112

#ifdef GSE_POSIX_THREADS
#include <pthread.h>
#include <signal.h>
#endif

#include <string.h>
#include <unistd.h>
#include <sys/fcntl.h>

#ifdef __MINGW32__
#include <windows.h>
/* Use windows API millisecond sleep */
#define sleep(n) Sleep(1000 * n)
#endif

#include "GseFilePktSrc.h"
#include "GseWrap.h"
#include "GseFile.h"
#include "GseExcept.h"
#include "GseString.h"
#include "GseConv.h"
#include "GseLog.h"


#define GSE_READ_POLLING

int4 GseFilePktSrc_ptkIdx(GsePktSrc* pBase){
	return ((GseFilePktSrc*)pBase)->_pktIdx;
}

size_t GseFilePktSrc_offset(GsePktSrc* pBase){
	return gse_lseek( ((GseFilePktSrc*)pBase)->_fd, 0, SEEK_CUR);
}

const char* GseFilePktSrc_name(GsePktSrc* pBase){
	return ((GseFilePktSrc*)pBase)->_sName;
}

int4 GseFilePktSrc_numPkts(GsePktSrc* pBase){
	return ((GseFilePktSrc*)pBase)->_nPkts;
}

bool GseFilePktSrc_retTrue(const GsePktSrc* pBase){ return true; }

/****************************************************************************/
size_t GseFilePktSrc_read(GseFilePktSrc* pSrc, byte* pDest,size_t uBytes)
{
	size_t uRead;
	size_t uPos;
	
	uPos = gse_lseek(pSrc->_fd, 0, SEEK_CUR);
	uRead = gse_read(pSrc->_fd, pDest, uBytes, &(pSrc->_bRestartReads));
	
	while((uRead < uBytes) && (pSrc->_eof_disp == at_eof_block) &&
			 pSrc->_bRestartReads )
	{
#ifdef GSE_READ_POLLING
		sleep(1);
		gse_lseek(pSrc->_fd, uPos, SEEK_SET);
		uRead = gse_read(pSrc->_fd, pDest, uBytes, &(pSrc->_bRestartReads));
#else
#error non polling version not yet implimented
#endif
	}
	
	return uRead;
}


/* Seach for a sync pattern in a byte stream, if only get part of the 
 * sync pattern, back up the potential start of it.
 *
 * WARNING!  This algorithim depends on the sync pattern not having
 *           ANY repeated characters.
 */
bool GseFilePktSrc_search(GseFilePktSrc* pSrc, size_t* pnPrefix)
{
	
	byte uCur;
	size_t uStart, uPtrnStart, uPos;
	bool bInPtrn = false;
	bool bAllPtrn = false;
	int8 iPtrn;
	static const int nSyncLen = 4;
	
	
	uPos = gse_lseek(pSrc->_fd, 0, SEEK_CUR);
	uStart = uPos;
	
	/* do this butt simple and inefficent */
	while(GseFilePktSrc_read(pSrc, &uCur, 1) == 1){
				
		uPos++;
		
		if(uCur == gse_sSyncPtrn[0] ){
			bInPtrn = true;
			uPtrnStart = uPos - 1;
			continue;
		}
		
		if(!bInPtrn)
			continue;
		
		iPtrn = uPos - 1 - uPtrnStart;
		
		if(uCur != gse_sSyncPtrn[iPtrn]){
			bInPtrn = false;
			continue;	
		}

		/* I'm in a subsequent member of the match is it the last? */
		if(iPtrn + 1 == nSyncLen){
			bAllPtrn = true;
			break;
		}
	}
		
	/* Save the prefix stuff */
	if(bInPtrn)
		*pnPrefix = uPtrnStart - uStart;
	else
		*pnPrefix = uPos - uStart;
	

	/* Set file pointer on the way out */
	if(bInPtrn){                                  
		if(!bAllPtrn)   
			gse_lseek(pSrc->_fd, uPtrnStart, SEEK_SET);   /* At start of ptrn */
		else
			gse_lseek(pSrc->_fd, uPos, SEEK_SET);         /* after the ptrn */
	}

	return bAllPtrn;
}

/****************************************************************************/
/* if have to bail early then reset the file pointer back to the
 * start of the PACKET */


GsePkt* GseFilePktSrc_readNextImp(GsePktSrc* pBase, size_t* pnPrefix)
{
	size_t uOffBeg;
	byte sPrefix[4 + 12];
	char sPtrnBuf[3*4 + 1];
	
	int i;
	uint2 uSzHdr;
	bytebuf_t bufPkt;
	GsePkt* pPkt = NULL;
	GseFilePktSrc* pSrc = (GseFilePktSrc*)pBase;
	
	
	/* If I'm in stop at eof mode, and the num of packets is known and
	 * the current packet indicates this is the last one, don't bother to read */
	if( (pSrc->_eof_disp == at_eof_stop)&&(pSrc->_nPkts != -1) &&
		 (pSrc->_nPkts == pSrc->_pktIdx + 1))
		return NULL;
	
	uOffBeg = gse_ltell(pSrc->_fd);
	
	
	
	/* Read the sync pattern */
	if(! GseFilePktSrc_search(pSrc, pnPrefix) ){
		
		if(pSrc->_eof_disp == at_eof_stop){
			pSrc->_nPkts = pSrc->_pktIdx + 1;
			if( !gse_at_eof(pSrc->_fd) )
				gse_error_v("Couldn't find packet start pattern:%s in %s "
				          "starting at offset %d", 
				          gse_hexprn(gse_sSyncPtrn, 4, sPtrnBuf, 3*4 + 1),
							 pBase->name(pBase), uOffBeg);
		}
		
		if(pSrc->_eof_disp == at_eof_block)
			gse_notice_v("Read interupted at offset %d of %s", 
			             gse_ltell(pSrc->_fd), pSrc->_sName);
		
		gse_lseek(pSrc->_fd, uOffBeg, SEEK_SET);
		return NULL;
	}
	
	for(i = 0; i<4; i++)	sPrefix[i] = gse_sSyncPtrn[i];
	
	
	
	/* Read the 16 byte prefix */
	if( GseFilePktSrc_read(pSrc, sPrefix + 4, 12) != 12){
		
		if(pSrc->_eof_disp == at_eof_stop){
			pSrc->_nPkts = pSrc->_pktIdx + 1;
		
			gse_error_v("Couldn't get remainder of packet prefix in %s at offset "
					      "0x%06X", pSrc->_sName, uOffBeg + 4);
		}
		
		if(pSrc->_eof_disp == at_eof_block)
			gse_notice_v("Read interupted at offset %d of %s",
			             gse_ltell(pSrc->_fd), pSrc->_sName);
		
		gse_lseek(pSrc->_fd, uOffBeg, SEEK_SET);
		return NULL;
	}
	
	gse_storeUinS( gse_u4FromMSBbytes(sPrefix + 8), 
	               &(bufPkt.len), sizeof(bufPkt.len) );
	
	if((bufPkt.len < 20)||(bufPkt.len > pBase->uMaxPktSz)){
		gse_throw_fm(GSE_XCEPT, GSE_XDECODE, "Unrealistic packet size %zd "
		             "encontered.", bufPkt.len);
	}
			
	uSzHdr = gse_u2FromMSBbytes(sPrefix + 6);
	
	bufPkt.ptr = gse_calloc(bufPkt.len, 1);
	except_cleanup_push(free, bufPkt.ptr);
	
	memcpy(bufPkt.ptr, sPrefix, 16);
	
	
	
	/* Read the rest of the packet */
	 if(GseFilePktSrc_read(pSrc, bufPkt.ptr + 16, bufPkt.len - 16) !=
		bufPkt.len - 16){
		
		if(pSrc->_eof_disp == at_eof_stop){
			pSrc->_nPkts = pSrc->_pktIdx + 1;
			gse_error_v("Couldn't get %u packet bytes starting at offset 0x%06X"
					      " of %s", bufPkt.len, uOffBeg, pBase->name(pBase));
		}
		
		if(pSrc->_eof_disp == at_eof_block)
			gse_notice_v("Read interupted at offset %d of %s",
			             gse_ltell(pSrc->_fd), pSrc->_sName);
			
		gse_lseek(pSrc->_fd, uOffBeg, SEEK_SET);
		return NULL;
	}
	
	pSrc->_pktIdx += 1;
	
	/* parse it, the Pkt copies the bytes!!! it DOES NOT take ownership of them */
	pPkt = new_GsePkt(const_bytebuf(bufPkt), pSrc->_factory, 0);
	
	except_cleanup_pop(except_call);  /* Free the byte buf */
	return pPkt;
}

#ifdef GSE_POSIX_THREADS
/* For sources that don't stop at EOF, we need a way to close from a different
 * thread, kinda tricky,  This is almost useless for a single
 * threaded program. */
void GseFilePktSrc_interrupt(GsePktSrc* pBase, pthread_t threadID){
	((GseFilePktSrc*)pBase)->_bRestartReads = false;
	pthread_kill(threadID, SIGUSR2);
}
#endif


GsePkt* GseFilePktSrc_readNext(GsePktSrc* pBase)
{
	static const except_id_t catch_spec[] = {{XCEPT_GROUP_ANY, XCEPT_CODE_ANY}};
	except_t* pExcept;
	GsePkt* pPkt = NULL;
	GseFilePktSrc* pThis = (GseFilePktSrc*)pBase;
	size_t nCurOff, nPrefix;
	
	
	while(true){
		nCurOff = gse_ltell(pThis->_fd);
		
		except_try_push(catch_spec, 1, &pExcept);
		
		/* Will return to here if an exception is thrown */
		if(pExcept == NULL){
			nPrefix = 0;
			pPkt = GseFilePktSrc_readNextImp(pBase, &nPrefix);
		}
		else{
			gse_log_except(GSE_ERROR, pExcept);
			
			/** Advance the file pointer past the sync bytes, and unset 
			 * num packets if was set */
			gse_lseek_bnds(pThis->_fd, nCurOff + 4 + nPrefix, SEEK_SET); 
			
			if((!gse_at_eof(pThis->_fd)) && (pThis->_eof_disp == at_eof_stop))
				pThis->_nPkts = -1;
		}
	
		except_try_pop();
		
		if(pPkt != NULL)
			break;
		
		/* If I'm supposed to stop at the end of the file, and I've found *and
		   am at* the end of file, get out. */
		if( (pThis->_eof_disp == at_eof_stop)&&(pThis->_nPkts != -1)
			 &&(pThis->_nPkts == pThis->_pktIdx + 1))
			break;
		
		/* If I'm supposed to just return and not loop here, get out. */
		if(pThis->_eof_disp == at_eof_retry)
			break;
		
		/* Okay, look again */
	}
	
	if(pPkt != NULL)
		gse_debug_v("Read packet %d from %s", pThis->_pktIdx, pThis->_sName);
	
	return pPkt;
}

/****************************************************************************/
/* This function is *much* less cautious, should only be used on data that  */
/* is pretty much known to be good                                          */
/* Assumptions:                                                             */
/*   Assume I'm at the beginning of a packet.                               */
/*   All packets have uncorrupted headers                                   */


/* skip forward one packet, return true if at start of next packet */
bool GseFilePktSrc_skipFore(GseFilePktSrc* pThis)
{
	GsePktSrc* pBase = (GsePktSrc*)pThis;
	int4 i;
	byte buf[16];
	size_t nPktSize;
	
	/* Read 1st 16 bytes */
	if( gse_read(pThis->_fd, (byte*)buf, 16, &(pThis->_bRestartReads)) < 16)
		return false;
	
	/* Check to see if in the sync pattern */
	for(i = 0; i<4; i++)
		if(buf[i] != gse_sSyncPtrn[i])
			return false;
	
	/* Get the packet size */
	gse_storeUinS( gse_u4FromMSBbytes((byte*)buf + 8), 
	               &nPktSize, sizeof(nPktSize) );
	
	/* sanity check the packet size */
	if((nPktSize < 20)||(nPktSize > pBase->uMaxPktSz))
		gse_throw_fm(GSE_XCEPT, GSE_XDECODE, "Unrealistic packet size %zd "
		             "encontered.", nPktSize);
	
	/* Seek forward that many bytes, allow throws */
	gse_lseek(pThis->_fd, nPktSize - 16, SEEK_CUR);
	
	return true;
}

/* skip backward one packet, return true if at the start of the next packet */
bool GseFilePktSrc_skipBack(GseFilePktSrc* pThis)
{
	GsePktSrc* pBase = (GsePktSrc*)pThis;
	byte buf[4];
	size_t nPktSize;
	
	/* rewind and read 4 bytes */
	gse_lseek(pThis->_fd, -4, SEEK_CUR);
	if( gse_read(pThis->_fd, (byte*)buf, 4, &(pThis->_bRestartReads)) < 4)
		return false;
	
	gse_storeUinS( gse_u4FromMSBbytes((byte*)buf), &nPktSize, sizeof(nPktSize) );
	
	/* sanity check the packet size */
	if((nPktSize < 20)||(nPktSize > pBase->uMaxPktSz))
		gse_throw_fm(GSE_XCEPT, GSE_XDECODE, "Unrealistic packet size %zd "
		             "encontered.", nPktSize);
	
	/* Seek backward that many bytes, allow throws */
	gse_lseek(pThis->_fd, 4 - nPktSize, SEEK_CUR);
	
	return true;
}


int4 GseFilePktSrc_skip(GsePktSrc* pBase, int nSkip)
{
	static const except_id_t catch_spec[] = {{XCEPT_GROUP_ANY, XCEPT_CODE_ANY}};
	except_t* pExcept;
	GseFilePktSrc* pThis = (GseFilePktSrc*)pBase;
	size_t nCurOff;
	int4 nOldIdx, nSkipped;
	
	if(nSkip == 0) return 0;
	
	nCurOff = gse_ltell(pThis->_fd);
	nOldIdx = pThis->_pktIdx;
	
	except_try_push(catch_spec, 1, &pExcept);
	
	if(pExcept == NULL){
		
		nSkipped = 0;
		while(nSkipped != nSkip){
			
			if(nSkip > 0){
				if( GseFilePktSrc_skipFore(pThis) ){
					nSkipped += 1;
					pThis->_pktIdx += 1;
					nCurOff = gse_ltell(pThis->_fd);
				}
				else{
					gse_lseek(pThis->_fd, nCurOff, SEEK_SET);
					break;
				}
			}
			else{
				if( GseFilePktSrc_skipBack(pThis) ){
					nSkipped -= 1;
					pThis->_pktIdx -= 1;
					nCurOff = gse_ltell(pThis->_fd);
				}
				else{
					gse_lseek(pThis->_fd, nCurOff, SEEK_SET);
					break;
				}
			}
		}

	}
	else{
		gse_log_except(GSE_ERROR, pExcept);
		gse_lseek_bnds(pThis->_fd, nCurOff, SEEK_SET);
	}
	
	except_try_pop();
	
	
	gse_debug_v("Skipped %d from index %d", nSkipped, nOldIdx);
	
	return nSkipped;
}

int4 GseFilePktSrc_skipToEnd(GsePktSrc* pBase)
{
	static const except_id_t catch_spec[] = {{XCEPT_GROUP_ANY, XCEPT_CODE_ANY}};
	except_t* pExcept;
	GseFilePktSrc* pThis = (GseFilePktSrc*)pBase;
	size_t nCurOff, nOldIdx;
	int nSkipped;
	
	nSkipped = 0;
	
	nCurOff = gse_ltell(pThis->_fd);
	nOldIdx = pThis->_pktIdx;
	
	except_try_push(catch_spec, 1, &pExcept);
	/* Will return to here on an exception */
	
	if(pExcept == NULL){
		
		while(true){
			if( GseFilePktSrc_skipFore(pThis) ){
				nSkipped += 1;
				pThis->_pktIdx += 1;
				nCurOff = gse_ltell(pThis->_fd);
			}
			else{
				gse_lseek(pThis->_fd, nCurOff, SEEK_SET);
				break;
			}
		}
		
	}
	else{
		gse_log_except(GSE_ERROR, pExcept);
		gse_lseek_bnds(pThis->_fd, nCurOff, SEEK_SET);
	}
	
	except_try_pop();
	
	gse_debug_v("Skipped %d from index %d", nSkipped, nOldIdx);
	
	return nSkipped;
}


/****************************************************************************/
void GseFilePktSrc_rewind(GsePktSrc* pBase)
{
	GseFilePktSrc* pThis = (GseFilePktSrc*)pBase;
	
	gse_lseek(pThis->_fd, 0, SEEK_SET);
	pThis->_pktIdx = -1;
}

/****************************************************************************/
void delete_GseFilePktSrc(void* vpThis)
{
	GseFilePktSrc* pThis = (GseFilePktSrc*)vpThis;
	
	if(pThis->_sName != NULL) free(pThis->_sName);	
	if(pThis->_fd != 0) close(pThis->_fd);
	
	free(vpThis);
	/* No base class stuff to do, it is just an interface */
}

/****************************************************************************/
GseFilePktSrc* new_GseFilePktSrc(const char* sFilename, gse_blk_factory factory,
                                 gse_eof_disp disp)
{
	GsePktSrc* pBase;
	GseFilePktSrc* pThis;
	
	pBase = new_GsePktSrc(sizeof(GseFilePktSrc));
	pThis = (GseFilePktSrc*) pBase;
	
	except_cleanup_push(delete_GseFilePktSrc, pThis);
	
	/* interface implimentation */
	pBase->_pIfaceOwner  = (void*)pThis;
	pBase->pktIdx   = GseFilePktSrc_ptkIdx;
	pBase->offset   = GseFilePktSrc_offset;
	pBase->name     = GseFilePktSrc_name;
	pBase->numPkts  = GseFilePktSrc_numPkts;
	pBase->readNext = GseFilePktSrc_readNext;
	
#ifdef GSE_POSIX_THREADS	
	pBase->interrupt    = GseFilePktSrc_interrupt;
#endif
	
	/* Let support previous default to false for now, add it in later */
	
	pBase->supportsRewind = GseFilePktSrc_retTrue;
	pBase->rewind         = GseFilePktSrc_rewind;
	
	pBase->supportsSkipAhead = GseFilePktSrc_retTrue;
	pBase->supportsSkipBack  = GseFilePktSrc_retTrue;
	pBase->skip              = GseFilePktSrc_skip;
	pBase->skipToEnd         = GseFilePktSrc_skipToEnd;
	
	/* New stuff */
	pThis->del = delete_GseFilePktSrc;
	pBase->del = delete_GseFilePktSrc;
	
	pThis->_eof_disp = disp;
	pThis->_pktIdx = -1;
	pThis->_bRestartReads = true;
	pThis->_nPkts = -1;
	pThis->_factory = factory;
	if(factory == NULL)
		gse_throw_m(GSE_XCEPT, GSE_XBADARG, "Null factory pointer");
	
	/* TODO  CONVERT NAME TO ABS PATH !!!!! */
	pThis->_sName = CS(sFilename);
		
	pThis->_fd = gse_open_ro(sFilename);
	
	except_cleanup_pop(except_no_call);
	return pThis;
}
