#define _POSIX_C_SOURCE 200112

#ifdef GSE_POSIX_THREADS
#include <pthread.h>
#endif

#include <string.h>
#include <errno.h>

#include "GseExcept.h"
#include "GseFileStr.h"
#include "GseString.h"
#include "GseWrap.h"


void GseFileStr_makeroom(GseFileStr* pThis, size_t more)
{	
	size_t add = more;
	
	if(more < 1) return;
	
	if(add < 10) add = 10;
	
	if(pThis->_buf == NULL){
		pThis->_buf = (char*)gse_calloc(add + 1, sizeof(char));
		pThis->_len = add;
	}
	
	if(pThis->_idx + more > pThis->_len - 2){	
		pThis->_buf = gse_realloc(pThis->_buf, pThis->_len + add);
		memset(pThis->_buf + pThis->_len, '\0', add);
		pThis->_len += add;
	}
}

const char* GseFileStr_getstr(GseFileStr* pThis){
	return (const char*)pThis->_buf;
}

char* GseFileStr_takestr(GseFileStr* pThis){
	char* sRet;
	
	sRet = pThis->_buf;
	pThis->_buf = NULL;
	pThis->_idx = 0;
	pThis->_len = 0;
	
	return sRet;
}

void GseFileStr_clear(GseFileStr* pThis)
{
	memset(pThis->_buf, 0, pThis->_len);
	pThis->_idx = 0;
}

/** Take the data string away, but start a new one. */
char* GseFileStr_reuse(GseFileStr* pThis, size_t nInitialBuf)
{
	char* sRet;
	
	sRet = pThis->_buf;
	
	pThis->_idx   = 0;
	
	if(nInitialBuf < 10)
		pThis->_len = 10;
	else
		pThis->_len = nInitialBuf;
	
	pThis->_buf = (char*)gse_calloc(pThis->_len + 1, sizeof(char));
	
	return sRet;
}

void GseFileStr_putchar(GseFileStr* pThis, char c)
{	
	GseFileStr_makeroom(pThis, 1);
	
	pThis->_buf[pThis->_idx] = c;
	pThis->_idx += 1;
}

void GseFileStr_putstr(GseFileStr* pThis, const char* s){
	size_t nStr;
	if(s == NULL) return;
	
	nStr = strlen(s);
	if(nStr < 1) return;
	
	GseFileStr_makeroom(pThis, nStr);
	
	strcpy(pThis->_buf + pThis->_idx, s);
	pThis->_idx += nStr;
}

void GseFileStr_write(GseFileStr* pThis, const byte* ptr, size_t num)
{	
	GseFileStr_makeroom(pThis, num);
	
	memcpy(pThis->_buf + pThis->_idx, ptr, num);
	pThis->_idx += num;
}

int GseFileStr_printf(GseFileStr* pThis, const char* fmt, ...){
	size_t nLen;
	char* s;
	va_list ap;
	
	va_start(ap, fmt);
	s = gse_vstring(fmt, ap);
	va_end(ap);
	nLen = strlen(s);
	
	except_cleanup_push(free, s);
	GseFileStr_putstr(pThis, s);
	except_cleanup_pop(except_call);
	
	return nLen;
}

void GseFileStr_seek(GseFileStr* pThis, size_t offset, int whence){
	size_t idx;
	size_t len;
	
	switch(whence){
		case SEEK_SET: idx = offset; break;
		case SEEK_CUR: idx = pThis->_idx + offset; break;
		case SEEK_END: idx = pThis->_len - 2 + offset; break;
		default:
			gse_throw(GSE_XCEPT, GSE_XBADARG);
			break;
	}
	if(idx < 0)
		gse_throw(CLIB_XCEPT, EINVAL);
	
	len = idx + 2;
	
	GseFileStr_makeroom(pThis, len - pThis->_len);
 
	pThis->_idx = idx;
}

size_t GseFileStr_tell(GseFileStr* pThis){
	return pThis->_idx;
}

size_t GseFileStr_strlen(GseFileStr* pThis){
	return strlen(pThis->_buf);
}

size_t GseFileStr_buflen(GseFileStr* pThis){
	return pThis->_idx;
}
	
void delete_GseFileStr(void* vpThis){
	GseFileStr* pThis = (GseFileStr*)vpThis;
	if(pThis->_buf != NULL) free(pThis->_buf);
	free(vpThis);
}

GseFileStr* new_GseFileStr(size_t nInitialBuf){
	GseFileStr* pThis;
	
	pThis = (GseFileStr*)gse_calloc(1, sizeof(GseFileStr));
	except_cleanup_push(delete_GseFileStr, pThis);
	init_GseFileStr(pThis, nInitialBuf);
	except_cleanup_pop(except_no_call);
	return pThis;
}

/*****************************************************************************/
/* for static vars */

void fin_GseFileStr(void* vpThis)
{
	GseFileStr* pThis = (GseFileStr*)vpThis;
	if(pThis->_buf != NULL) free(pThis->_buf);
}

void init_GseFileStr(GseFileStr* pThis, size_t nInitialBuf)
{
	pThis->getstr = GseFileStr_getstr;
	pThis->takestr= GseFileStr_takestr;
	pThis->clear  = GseFileStr_clear;
	pThis->reuse  = GseFileStr_reuse;
	pThis->putchar= GseFileStr_putchar;
	pThis->putstr = GseFileStr_putstr;
	pThis->printf = GseFileStr_printf;
	pThis->write  = GseFileStr_write;
	pThis->seek   = GseFileStr_seek;
	pThis->tell   = GseFileStr_tell;
	pThis->strlen = GseFileStr_strlen;
	pThis->buflen = GseFileStr_buflen;
	pThis->del    = delete_GseFileStr;

	pThis->_idx   = 0;
	
	if(nInitialBuf < 10)
		pThis->_len = 10;
	else
		pThis->_len = nInitialBuf;
	
	pThis->_buf = (char*)gse_calloc(pThis->_len + 1, sizeof(char));
}
