#define _POSIX_C_SOURCE 200112

#ifdef GSE_POSIX_THREADS
#include <pthread.h>
#endif

#include <stdlib.h>
#include <string.h>

#include "GseHdr.h"
#include "GsePkt.h"
#include "GseExcept.h"
#include "GseString.h"
#include "GseWrap.h"
#include "GseArrays.h"

/*****************************************************************************/
size_t GseHdr_numBlocks(GseHdr* pThis){ return pThis->_pBlocks->num; }

GseBlock* GseHdr_getBlockAt(GseHdr* pThis, size_t idx)
{
	if((idx < 0)||(idx >= pThis->_pBlocks->num))
		gse_throw(GSE_XCEPT, GSE_XARYINDEX);
	
	return pThis->_pBlocks->pAry[idx];
}

GseBlock* GseHdr_replaceBlockAt(GseHdr* pThis, size_t idx, GseBlock* pNew)
{
	GseBlock* pOld;
	
	if((idx < 0)||(idx >= pThis->_pBlocks->num))
		gse_throw(GSE_XCEPT, GSE_XARYINDEX);
	
	pOld = pThis->_pBlocks->pAry[idx];
	
	pThis->_pBlocks->pAry[idx] = pNew;
	return pOld;
}
			
size_t GseHdr_addBlock(GseHdr* pThis, GseBlock* pNewBlk)
{
	if(pNewBlk != NULL)
		pThis->_pBlocks->add(pThis->_pBlocks, pNewBlk);
	
	return pThis->_pBlocks->num;
}

/*****************************************************************************/
/* By Type functions */
GseBlock* GseHdr_getBlkByType(GseHdr* pThis, byte uType, byte uProc)
{
	GseBlock* pRet = NULL;
	int i, num = 0;
	GseBlock** p = pThis->_pBlocks->pAry;
	
	if(p == NULL)
		gse_throw_m(GSE_XCEPT, GSE_XASSERT, "GSE Header contains no blocks."); 
	
	for(i = 0; i<pThis->_pBlocks->num; i++){
		
		if(p[i] == NULL) continue;
		
		if(p[i]->uType == uType){
			if(uType != GSE_PROGRAM_BLK_TYPE_ID){
				num++;
				pRet = p[i];
			}
			else{
				if( ((GseProcBlk*)(p[i]))->uStepID == uProc){
					num++;
					pRet = p[i];
				}
			}
		}
	}
	
	if(pRet == NULL){
		if(uProc != GSE_PROGRAM_BLK_TYPE_ID)
			gse_throw_fm(GSE_XCEPT, GSE_XNOTUNIQUE, "GSE Header contians no "
			             "blocks of type 0x%02hhX", uType);
		else
			gse_throw_fm(GSE_XCEPT, GSE_XNOTUNIQUE, "GSE Header contians no "
			             "processing blocks for step ID 0x%02hhX", uProc);
	}
	
	if(num > 1){
		if(uProc != GSE_PROGRAM_BLK_TYPE_ID)
			gse_throw_fm(GSE_XCEPT, GSE_XNOTUNIQUE, "GSE Header has %d blocks "
			             "of type 0x%02hhX", num, uType);
		else
			gse_throw_fm(GSE_XCEPT, GSE_XNOTUNIQUE, "GSE Header has %d "
			             "processing blocks for step ID 0x%02hhX", num, uProc);
	}
	
	return pRet;
}

/*****************************************************************************/
/* By Type functions */
GseBlock* GseHdr_getBlkByTypeIdx(GseHdr* pThis, byte uType, byte uProc, int4 idx)
{
	int i, iBlk = -1;
	GseBlock** p = pThis->_pBlocks->pAry;
	
	if(p == NULL)
		gse_throw_m(GSE_XCEPT, GSE_XASSERT, "GSE Header contains no blocks."); 
	
	for(i = 0; i<pThis->_pBlocks->num; i++){
		
		if(p[i] == NULL) continue;
		
		if(p[i]->uType == uType){
			if(uType != GSE_PROGRAM_BLK_TYPE_ID){
				iBlk++;
				if(iBlk == idx)
					return p[i];
			}
			else{
				if( ((GseProcBlk*)(p[i]))->uStepID == uProc){
					iBlk++;
					if(iBlk == idx)
						return p[i];
				}
			}
		}
	}
	
	if(iBlk == 0){
		if(uProc != GSE_PROGRAM_BLK_TYPE_ID){
			gse_throw_fm(GSE_XCEPT, GSE_XARYINDEX, "GSE Header contians no "
			             "blocks of type 0x%02hhX", uType);
		}
		else{
			gse_throw_fm(GSE_XCEPT, GSE_XARYINDEX, "GSE Header contians no "
			             "processing blocks for step ID 0x%02hhX", uProc);
		}
	}
	
	if(uProc != GSE_PROGRAM_BLK_TYPE_ID){
		gse_throw_fm(GSE_XCEPT, GSE_XARYINDEX, "GSE Header contians only "
		             "%d blocks of type 0x%02hhX, idx %d is invalid.", 
				       iBlk + 1, uType, idx);
	}
	else{
		gse_throw_fm(GSE_XCEPT, GSE_XARYINDEX, "GSE Header contians only"
		             "%d processing blocks for step ID 0x%02hhX, idx %d is ",
				       "invalid", iBlk + 1, uProc, idx);
	}
			
	return NULL;
}


size_t GseHdr_numBlksByType(GseHdr* pThis, byte uType, byte uProc)
{
	int i, nRet = 0;
	GseBlock** p = pThis->_pBlocks->pAry;
	
	if(p == NULL) 
		return nRet;
	
	for(i = 0; i<pThis->_pBlocks->num; i++){
		if(p[i] == NULL) continue;
		
		if(p[i]->uType == uType){
			if(uType != GSE_PROGRAM_BLK_TYPE_ID)
				nRet++;
			else
				if(  ((GseProcBlk*)(p[i]))->uStepID == uProc)
					nRet++;
		}
	}
	
	return nRet;
}

/*****************************************************************************/
int GseHdr_blockSort(const void* ppFirst, const void* ppSecond)
{
	const GseBlock* p1;
	const GseBlock* p2;
	const GseProcBlk* pProc1;
	const GseProcBlk* pProc2;
	
	/* Cast appropriantly */
	p1 = *((GseBlock**)ppFirst);
	p2 = *((GseBlock**)ppSecond);
	
	/* Handle NULLs */
	if((p1 == NULL) && (p2 != NULL)) return -1;
	if((p1 != NULL) && (p2 == NULL)) return 1;
	if((p1 == NULL) && (p2 == NULL)) return 0;
	
	if(p1->uType != p2->uType){
		if(p1->uType > p2->uType)
			return 1;
		else
			return -1;
	}
	
	/* Secondary compares */
	if(p1->uType == GSE_PROGRAM_BLK_TYPE_ID){
		pProc1 = (GseProcBlk*)p1;
		pProc2 = (GseProcBlk*)p2;
		
		if(pProc1->uStepID > pProc2->uStepID) 
			return 1;
		else
			if(pProc1->uStepID < pProc2->uStepID)
				return -1;
		
		return 0;
	}
	
	/* no more secondary compares left */
	return 0;
}

/****************************************************************************/

/* Helper an array of arrays */
#define S gse_uint2_ary_ary
#define C GseUint2AryAry
#define T GseUint2Ary

#include "ptrAryTplt.h"

#undef S
#undef C
#undef T

#define C GseUint2AryAry
#define T GseUint2Ary
#define CASCADE_DEL

#include "ptrAryTplt.c"

#undef CASCADE_DEL
#undef C
#undef T

/* How it works:  Makes an array of arrays of uint2's in memory like:
   
	+-----+-----+-----+-----+
   | T S | idx1| idx2| 3rd |
	+-----+-----+-----+-----+
	+-----+-----+
   | T S | 1st |
	+-----+-----+
	+-----+-----+-----+
   | T S | 1st | 2nd |
	+-----+-----+-----+
	
	for each list that is larger than 1, iterrate through members
	1 through N-1 and try to merge them into the 1st member.
*/

void GseHdr_addType(GseUint2AryAry* pML, byte uType, byte uStep, int iIdx)
{
	int i;
	uint2 uTS;
	
	/* This doesn't work in gcc? Why not? */
	/* uTS = (((uint2)uType) << 8) | uStep; */
	
	/* Well try this */
	uTS = uType;
	uTS = uTS << 8;
	uTS |= uStep & 0x00FF;
	
	for(i = 0; i< pML->num; i++){
		
		/* Add to and existing array */
		if((pML->pAry[i]->num > 0) && (pML->pAry[i]->pAry[0] == uTS)){
			pML->pAry[i]->add( pML->pAry[i], iIdx);
			return;
		}
	}
	
	/* Make a new array */
	i = pML->add( pML, new_GseUint2Ary()) - 1;
	
	pML->pAry[i]->add( pML->pAry[i], uTS);
	pML->pAry[i]->add( pML->pAry[i], iIdx);
	
}

size_t GseHdr_mergeBlocks(GseHdr* pThis, byte uType, byte uStep)
{
	int iDest, iSrc, i, j, nMerged = 0;
	
	GseUint2AryAry* pML;
	
	GseBlock*   pBlk;
	GseProcBlk* pProc;
	
	GseBlock*   pDest;
	GseBlock*   pSrc;
	
	GseBlkAry* pRmBlks = NULL;
	
	pML = new_GseUint2AryAry(0);
	
	/* Setup the array of items to merge */
	for(i = 0; i<pThis->_pBlocks->num; i++){
		pBlk = pThis->_pBlocks->pAry[i];
		if(pBlk == NULL) continue;
		
		if(pBlk->uType == GSE_PROC_BLK_TYPE){
			pProc = (GseProcBlk*)pBlk;
			
			if((uType == 0) ||
			   ((uType == GSE_PROC_BLK_TYPE)&&(uStep == 0)) ||
				((uType == GSE_PROC_BLK_TYPE)&&(uStep == pProc->uStepID)) )
			{
				GseHdr_addType(pML, pBlk->uType, pProc->uStepID, i);
			}
			
		}
		else{
			if((uType == 0) || (uType == GSE_PROC_BLK_TYPE)){
				GseHdr_addType(pML, pBlk->uType, 0, i);	
			}
		}
	}
	
	/* Now go over the list of items added, and try to merge */
	pRmBlks = new_GseBlkAry(0);
	
	for(i=0; i<pML->num; i++){
		
		for(j=2; j<pML->pAry[i]->num; j++){
			iDest = pML->pAry[i]->pAry[1];
			iSrc  = pML->pAry[i]->pAry[j];
			
			pDest = pThis->_pBlocks->pAry[iDest];
			pSrc  = pThis->_pBlocks->pAry[iSrc];
			
			if( pDest->merge(pDest, pSrc, 0) ){ 
				pRmBlks->add(pRmBlks, pSrc);
				nMerged += 1;
			}
		}
	}
	
	/* Now remove all merged blocks */
	for(i=0; i<pRmBlks->num; i++){
		pThis->_pBlocks->rm_ptr(pThis->_pBlocks, pRmBlks->pAry[i]);		
		pRmBlks->pAry[i] = NULL;  /* Avoid a double free */
	}
	
	pRmBlks->del(pRmBlks);
	pML->del(pML);
	
	return nMerged;
}


/****************************************************************************/

bytebuf_t GseHdr_encode(GseHdr* pThis, uint2 uMinLen, uint2 uMaxLen)
{
	int i;
	GseBufAry* pBufs;
	bytebuf_t bufHdr; 
	size_t nOffset = 16;
	GseBlock* pBlk;
	
	bufHdr.ptr = NULL; 
	bufHdr.len = 16;
	
	/* Sort the blocks by type id, and then by proc id */
	pThis->_pBlocks->sort(pThis->_pBlocks, GseHdr_blockSort);
	
	
	/* get all the block bytes */
	pBufs = new_GseBufAry();
	except_cleanup_push(GseBufAry_cleanup, pBufs);
	
	for(i = 0; i<pThis->_pBlocks->num; i++){
		pBlk = pThis->_pBlocks->pAry[i];
		if(pBlk == NULL) continue;
		
		pBufs->add(pBufs, pBlk->encode(pBlk, 0, NULL) );
		bufHdr.len += pBufs->pAry[i].len;
	}
	
	if(bufHdr.len > uMaxLen)
		gse_throw_fm(GSE_XCEPT, GSE_XENCODE, "Header encodes to %d bytes"
		             " which is larger than the maximum of %hu bytes", 
		             bufHdr.len, uMaxLen);
	
	
	/* Allow the min-length field to cause padding and allow the uHeadTailLen
	   to cause padding as well */
	if(uMinLen > bufHdr.len)
		bufHdr.len = uMinLen;
	else
		/* Danm unsigned values, don't subtract from them as they might be
		   zero!!! */ 
		if( pThis->uHeadTailLen > (bufHdr.len + 4))
			bufHdr.len = pThis->uHeadTailLen - 4;
	
	bufHdr.ptr = (byte*)gse_calloc(bufHdr.len, sizeof(byte));
	except_cleanup_push(free, bufHdr.ptr);
	
	/* Copy all the block buffers into one output buffer */
	nOffset = 16;
	for(i = 0; i < pBufs->num; i++){
		memcpy(bufHdr.ptr + nOffset, pBufs->pAry[i].ptr, pBufs->pAry[i].len);
		nOffset += pBufs->pAry[i].len;
	}
	
	/* Encode the prefix */
	for(i=0; i<4; i++) bufHdr.ptr[i] = gse_sSyncPtrn[i];
	
	pThis->uHeadTailLen = bufHdr.len + 4;
	gse_u2ToMSBbytes(bufHdr.ptr + 6, pThis->uHeadTailLen);
	gse_i2ToMSBbytes(bufHdr.ptr + 12, pThis->nSpaceCraft);
	*(bufHdr.ptr + 14) = pThis->uHdrVer;
	*(bufHdr.ptr + 15) = pThis->uContent;
	
	except_cleanup_pop(except_no_call);  /* Only free return buffer on execpt */
	except_cleanup_pop(except_call);     /* Always free the block buf array   */
	return bufHdr;
}

/*****************************************************************************/
void delete_GseHdr(void* vpThis)
{
	GseHdr* pThis = (GseHdr*)vpThis;
	if(pThis->_pBlocks != NULL) 
		pThis->_pBlocks->del(pThis->_pBlocks);
	free(pThis);
}

GseHdr* GseHdr_copy(const GseHdr* pThis, size_t uOver)
{
	size_t i;
	GseHdr* pNew = NULL;
	
	if(uOver > sizeof(GsePkt))
		pNew = (GseHdr*) gse_calloc(1, uOver);
	else
		pNew = (GseHdr*) gse_calloc(1, sizeof(GseHdr));
	
	/* Don't use memcpy, it's too easy to miss a pointer and then have our
	   destructor take out some other objects memory.  Better to try and 
		difference NULL and find the error that way */
	pNew->uCRC         = pThis->uCRC;
	pNew->uHeadTailLen = pThis->uHeadTailLen;
	pNew->uTotalLen    = pThis->uTotalLen ;
	pNew->nSpaceCraft  = pThis->nSpaceCraft ;
	pNew->uHdrVer      = pThis->uHdrVer ;
	pNew->uContent     = pThis->uContent ;
	
	pNew->copy           = pThis->copy ;
	pNew->original       = pThis->original ;
	pNew->addBlock       = pThis->addBlock ;
	pNew->replaceBlockAt = pThis->replaceBlockAt ;
	pNew->getBlockAt     = pThis->getBlockAt ;
	pNew->numBlocks      = pThis->numBlocks ;
	pNew->getBlkByType   = pThis->getBlkByType ;
	pNew->getBlkByTypeIdx= pThis->getBlkByTypeIdx ;
	pNew->numBlksByType  = pThis->numBlksByType ;
	pNew->mergeBlocks    = pThis->mergeBlocks ;
	pNew->encode         = pThis->encode ;
	pNew->del            = pThis->del ;

	pNew->_pBlocks = new_GseBlkAry(pThis->_pBlocks->num);
	
	for(i = 0; i < pThis->_pBlocks->num; i++)
		pNew->_pBlocks->pAry[i] = 
		   pThis->_pBlocks->pAry[i]->copy(pThis->_pBlocks->pAry[i], 0);
	
	return pNew;
}


GseHdr* new_GseHdr(cbytebuf_t buf, gse_blk_factory factory, size_t uOverMalloc)
{
	char sPrnBuf1[3*4 + 1];
	char sPrnBuf2[3*4 + 1];
	
	cbytebuf_t blkBuf;
	int nBlk;
	byte uBlkType, u1stData;
	uint2 uHdrLen;
	
	GseHdr* pThis = new_empty_GseHdr(uOverMalloc);
	
	except_cleanup_push(delete_GseHdr, pThis);
	
	/* Handle decoding my stuff */
	if(buf.len < 16)
		gse_throw_fm(GSE_XCEPT, GSE_XDECODE, "At least 16 bytes are required "
		              "to make a Gse Header, but header length was: %d", buf.len);
	
	if(memcmp(buf.ptr, gse_sSyncPtrn, 4) != 0)
		gse_throw_fm(GSE_XCEPT, GSE_XDECODE, 
			"1st %d bytes were %s instead of the sync pattern %s", 
			gse_hexprn(buf.ptr,       4, sPrnBuf1, 3*4 + 1),
			gse_hexprn(gse_sSyncPtrn, 4, sPrnBuf2, 3*4 + 1)
		);
	
	pThis->uCRC         = gse_u2FromMSBbytes(buf.ptr + 4);
	pThis->uHeadTailLen = gse_u2FromMSBbytes(buf.ptr + 6);
	pThis->uTotalLen    = gse_u4FromMSBbytes(buf.ptr + 8);
	pThis->nSpaceCraft  = gse_i2FromMSBbytes(buf.ptr + 12);
	pThis->uHdrVer      = *(buf.ptr + 14);
	pThis->uContent     = *(buf.ptr + 15);
	
	/* Check lengths */
	uHdrLen = pThis->uHeadTailLen - 4;
	if(uHdrLen > buf.len)
		gse_throw_fm(GSE_XCEPT, GSE_XDECODE, 
			"Length inconsistancy: The encoded header length %u, is greater than "
			" the number of bytes sent to new_GseHdr, %u.", uHdrLen, buf.len); 
	
	/* Look for blocks and pass parsing off to the block factory */
	blkBuf.ptr = buf.ptr + 16;
	nBlk = 0;
	/* Rem: blocks must be at least 3 bytes long */
	while(blkBuf.ptr < (buf.ptr + uHdrLen - 2)){
		uBlkType = *(blkBuf.ptr);
		
		/* If we hit block type 0x00, we should be into the padding */
		if(uBlkType == 0)
			break;
		else
			nBlk++;
		 
		blkBuf.len = *(blkBuf.ptr + 1) + 1;
		u1stData = *(blkBuf.ptr + 2);
	
		if(blkBuf.ptr + blkBuf.len > buf.ptr + buf.len)
			gse_throw_fm(GSE_XCEPT, GSE_XDECODE, 
			   "Header Block %d starts at byte %u and ends at byte %u, which "
			   "is past the end of the %u byte Header.", nBlk, blkBuf.ptr - buf.ptr,
			   (blkBuf.ptr - buf.ptr) + blkBuf.len, buf.len
			);
	
		pThis->_pBlocks->add(pThis->_pBlocks,
	   	factory(pThis->nSpaceCraft, uBlkType, u1stData, blkBuf)
		);
		
		blkBuf.ptr += blkBuf.len;
	}
	
	except_cleanup_pop(except_no_call);
	return pThis;
}

GseHdr* new_empty_GseHdr(size_t uOverMalloc)
{
	GseHdr* pThis = NULL;
	size_t uSize = 0;
	 
   if(uOverMalloc > sizeof(GseHdr))
		uSize = uOverMalloc;
	else
		uSize = sizeof(GseHdr);
	
	pThis = (GseHdr*)gse_calloc(1, uSize);
	
	except_cleanup_push(delete_GseHdr, pThis);
	
	pThis->_pBlocks = new_GseBlkAry(0);
	
	pThis->copy           = GseHdr_copy;
	pThis->del            = delete_GseHdr;
	pThis->addBlock       = GseHdr_addBlock;
	pThis->encode         = GseHdr_encode;
	pThis->getBlkByType   = GseHdr_getBlkByType;
	pThis->getBlkByTypeIdx= GseHdr_getBlkByTypeIdx;
	pThis->numBlksByType  = GseHdr_numBlksByType;
	pThis->numBlocks      = GseHdr_numBlocks;
	pThis->getBlockAt     = GseHdr_getBlockAt;
	pThis->replaceBlockAt = GseHdr_replaceBlockAt;
	pThis->addBlock       = GseHdr_addBlock;
	pThis->mergeBlocks    = GseHdr_mergeBlocks;
	
	pThis->uHdrVer = 0x04;
	
	except_cleanup_pop(except_no_call);
	return pThis;
}
