/** @file GseInit.h Initialization of GSE lib thread-global data. */
#ifndef _GseInit_H_
#define _GseInit_H_
 
#include <rpwgse/GseLog.h>

#ifdef __cplusplus
extern "C" {
#endif

/**  @addtogroup mod_rpwgse */
/*@{*/
 
/** Initialize thread global structures.
 *
 * On success, thread global structures are setup to hold jumpbufs and cleanup
 * handlers for stack and heap unwinding.  If the underlying Kazlib exception
 * component cannot be configured then @c exit(134) is called.  After exception 
 * handling is setup, all other errors throw exceptions. 
 * 
 * @param level Messages at this level or higher will be logged, use
 *        0 to use the default log level of GSE_WARNING. See @ref mod_log
 *        for details.
 *
 * @param handler Installs a new log message handler function.  Use 
 *        NULL to keep the default handler. See @ref mod_log for detials. 
 *
 * @throws (GSE_XCEPT, GSE_XBADARG) if @a level is out of the range
 *         GSE_NOTHING to GSE_EVERYTHING 
 * @throws (GSE_XCEPT,GSE_XINITFAIL) if some other error occurs.
 */
void gse_thread_init(int level, gse_log_handler_t handler);

/** Future Capability: Define new exceptions for this thread */
void gse_thread_xcept_def(int group, const struct gse_except_def* newdefs);

/** Free thread global structures */
void gse_thread_deinit(void);

/*@}*/

#ifdef __cplusplus
}
#endif

#endif /* _GseInit_H_ */
