#define _POSIX_C_SOURCE 200112

#ifdef GSE_POSIX_THREADS
#include <pthread.h>
#endif

#include "GseExcept.h"
#include "GseWrap.h"
#include "GsePktFilter.h"


/****************************************************************************/
bool GsePktFilter_false(GsePktSrc* pBase)
{
	return false;
}

/****************************************************************************/
/* Basic error packet catcher setup */
void GsePktFilter_setCatcher(GsePktFilter* pThis, gse_errpkt_hndlr_t pFunc,
                             void* pCatchCtx)
{
	pThis->pCatcher = pFunc;
	pThis->pCatchCtx = pCatchCtx;
}

gse_errpkt_hndlr_t GsePktFilter_getCatcher(const GsePktFilter* pThis)
{
	return pThis->pCatcher;
}

void* GsePktFilter_getCatchCtx(const GsePktFilter* pThis)
{
	return pThis->pCatchCtx;
}

/****************************************************************************/
/* Hey bub, I'm just a filter, go ask the upstream source these questions.  */

int4 GsePktFilter_pktIdx(GsePktSrc* pBase)
{
	GsePktSrc* pSrc = ((GsePktFilter*)pBase)->pUpSrc;
	return pSrc->pktIdx(pSrc);
}

int4 GsePktFilter_numPkts(GsePktSrc* pBase)
{
	GsePktSrc* pSrc = ((GsePktFilter*)pBase)->pUpSrc;
	return pSrc->numPkts(pSrc);
}

size_t GsePktFilter_offset(GsePktSrc* pBase)
{
	GsePktSrc* pSrc = ((GsePktFilter*)pBase)->pUpSrc;
	return pSrc->offset(pSrc);
}
const char* GsePktFilter_name(GsePktSrc* pBase)
{
	GsePktSrc* pSrc = ((GsePktFilter*)pBase)->pUpSrc;	
	return pSrc->name(pSrc);
}
	
#ifdef GSE_POSIX_THREADS
void GsePktFilter_interrupt(GsePktSrc* pBase, pthread_t threadID)
{
	GsePktSrc* pSrc = ((GsePktFilter*)pBase)->pUpSrc;	
	return pSrc->interrupt(pSrc, threadID);
}
#endif

/****************************************************************************/

GsePktFilter* new_GsePktFilter(GsePktSrc* pSrc, byte uProgID, size_t uMalloc,
		                       GsePkt* (*readNext)(GsePktSrc*), void (*del)(void*)
		                      )
{
	GsePktFilter* pThis;
	GsePktSrc*    pBase;
	
	if(uMalloc <= sizeof(GsePktFilter))
		gse_throw_m(GSE_XCEPT, GSE_XBADARG, "GsePktFilter is a virtual base, "
				      "thus uMalloc must be greater than sizeof(GsePktFilter)");
	
	if(pSrc == NULL)
		gse_throw_m(GSE_XCEPT, GSE_XBADARG, "Upstream source cannot be NULL");
	
	if(uProgID == 0)
		gse_throw_m(GSE_XCEPT, GSE_XBADARG, "0 is not a legal program ID");
	
	pBase = new_GsePktSrc(uMalloc);
	pThis = (GsePktFilter*)pBase;
	
	/* Init the likely pass through items */
	pBase->pktIdx    = GsePktFilter_pktIdx;
	pBase->offset    = GsePktFilter_offset;
	pBase->name      = GsePktFilter_name;
	pBase->numPkts   = GsePktFilter_numPkts;
#ifdef GSE_POSIX_THREADS	
	pBase->interrupt = GsePktFilter_interrupt;
#endif
	
	/* Most agorithms don't work so easy going backwards, default to no
	   rewind support */
	
	/* Store the readnext function an the delete function */
	pBase->del            = del;
	pBase->readNext       = readNext;
	
	/* Store the upstream pointer and the program ID */
	pThis->uProgID     = uProgID;
	pThis->pUpSrc      = pSrc;
	pThis->setCatcher  = GsePktFilter_setCatcher;
	pThis->getCatcher  = GsePktFilter_getCatcher;
	pThis->getCatchCtx = GsePktFilter_getCatchCtx;
	pThis->pCatcher    = NULL;
	pThis->pCatchCtx   = NULL;
	pThis->del         = del;
	
	
	return pThis;
}
