/** @file GsePktSink.h Inteface definition for Packet sinks */
#ifndef _GsePktSink_H_
#define _GsePktSink_H_

#include <rpwgse/GsePkt.h>

#ifdef __cplusplus
extern "C" {
#endif

/** @addtogroup mod_processing Processing */
/*@{*/

/** Packet sink inteface, all things that support flushing packets impliment 
 * this.
 * @interface GsePktSink
 * 
 * @todo Consider adding a filter function setting interface 
 */
typedef struct gse_pkt_sink{
		
	/** @publicsection How many packets has this sink recieved */
	int4 (*received)(const struct gse_pkt_sink* pThis);
	
	/** How many packets has this sink sent to it's output */
	int4 (*flushed)(const struct gse_pkt_sink* pThis);
	
	/** Recieve a packet and delete it.
	 * The packet destrutor is only called after it's written.  So if an
	 * exception is thrown, it is up to the caller to figure out what to
	 * do with the packet.  
	 *
	 * Unlike #writeNext, this is always available.
	 *
	 * @throws Varies Throws various exceptions based on the underlying
	 * implimentation. @see GseFilePktSink
	 */
	void (*consumeNext)(struct gse_pkt_sink* pThis, GsePkt* pPkt);
	
	/** Recieve a packet but don't delete it. 
	 * Use #supportsWrite to see if this function is available.
	 *
	 * @throws Varies Throws various exceptions based on the underlying
	 * implimentation. @see GseFilePktSink
	 */
	void (*writeNext)(struct gse_pkt_sink* pThis, GsePkt* pPkt);
	
	/** Returns true if the #writeNext function is implimented */
	bool (*supportsWrite)(const struct gse_pkt_sink* pThis);
	
	/** Destructor function pointer:
	 * @warning Check to see if this is NULL.  Classes supporting multiple 
	 * interfaces will not implement the destructor at the interface level. */
	void (*del)(void* vpThis);
	
	/** @protected Owner Pointer, used to access the containing structure */
	void* _pIfaceOwner;
	
} GsePktSink;

/*@}*/

#ifdef __cplusplus
}
#endif

#endif /* _GsePktSink_H_ */
