/** @file GseProp.hpp C++ interface for GseProp and friends */
#ifndef _GseProp_HPP_
#define _GseProp_HPP_

#include <rpwgse/GseProp.h>


/* Basically useless except as a base class */
class cGseProp {
public:
	friend class cGseField;
	friend class cGsePropList;
	friend class cGseParser;
	
	const char* name() const;
	gse_prop_type_t type() const;
	const char* confName() const;
	int4 confLine() const;
	
	virtual cGseProp* copy() const;
	virtual ~cGseProp();
	
	cGseProp(GseProp* pWrap);  /* Would like to have protected but don't know 
	                              the name of the SWIG wrapper function in 
											order to 'friend' it */
protected:
	GseProp* m_pProp;
	bool m_bOwner;
};

/* Handles C structs, GsePropInt, GsePropFloat, and GsePropStr */
class cGsePropSimple : public cGseProp {
public:
	friend class cGseField;
	friend class cGsePropList;
	friend class cGseParser;
	
	int8 getInt() const;
	float8 getFloat() const;
	const char* getStr() const;
	virtual cGseProp* copy() const;

	cGsePropSimple(GseProp* pWrap);  /* protect if solve friend issue */
};

class cGsePropList : public cGseProp {
public:
	friend class cGseField;
	friend class cGseParser;
	
	int4 length() const;
	cGseProp* getByName(const char* sName) const;
	cGseProp* get(int4 idx) const;
	gse_prop_type_t getType(int4 idx) const;
	gse_prop_type_t getTypeByName(const char* sName) const;
	
	virtual cGseProp* copy() const;
	cGsePropList(GseProp* pWrap);  /* protect if solve friend issue */
};

class cGsePropEnum : public cGseProp {
public:
	friend class cGseField;
	friend class cGsePropList;
	friend class cGseParser;
	
	int4 length() const;
	bool hasDefault() const;
	const char* getDefault() const;
	const char* get(int8 INPUT) const;
	
	virtual cGseProp* copy() const;
	cGsePropEnum(GseProp* pWrap);  /* protect if solve friend issue */
};

#endif /* _GseProp_HPP_ */
