/* GseProp.i */

%rename(GseProp) cGseProp;
%rename(GsePropSimple) cGsePropSimple;
%rename(GsePropList) cGsePropList;
%rename(GsePropEnum) cGsePropEnum;

%newobject cGseProp::copy();
%newobject cGsePropSimple::copy();
%newobject cGsePropList::copy();
%newobject cGsePropEnum::copy();

%header %{
#include <rpwgse/GseProp.hpp>

/* Base Type *****************************************************************/
cGseProp::cGseProp(GseProp* pWrap){
	if(pWrap == NULL) 
		gse_throw_m(GSE_XCEPT,GSE_XBADARG,"Null pointer to cGseProp::cGseProp");
		
	m_pProp = pWrap;
	m_bOwner = false;
}

const char* cGseProp::name() const{ return m_pProp->sName; }
gse_prop_type_t cGseProp::type() const{ return m_pProp->ptype; }
const char* cGseProp::confName() const { return m_pProp->sConf; }
int4 cGseProp::confLine() const { return m_pProp->nLine; }
cGseProp::~cGseProp(){ if(m_bOwner) m_pProp->del(m_pProp); }

cGseProp* cGseProp::copy() const {
	GseProp* pCobj = m_pProp->copy(m_pProp);
	cGseProp* pNew = new cGseProp(pCobj);
	pNew->m_bOwner = true;
	return pNew;
}

/* Single value properties **************************************************/
cGsePropSimple::cGsePropSimple(GseProp* pWrap):cGseProp(pWrap){
	gse_prop_type_t ptype = pWrap->type(pWrap);
	
	if((ptype != pt_int)&&(ptype != pt_flt)&&(ptype != pt_str))
		gse_throw_m(GSE_XCEPT,GSE_XBADARG,"cGsePropSimple can't wrap "
		            "composite property types");
}

int8 cGsePropSimple::getInt() const {
	if( m_pProp->ptype != pt_int)
		gse_throw_m(GSE_XCEPT,GSE_XBADARG,"getInt called on non-integer GseProp");
	
	return ((GsePropInt*)m_pProp)->nVal;
}

float8 cGsePropSimple::getFloat() const {
	if( m_pProp->ptype != pt_flt)
		gse_throw_m(GSE_XCEPT,GSE_XBADARG,"getFloat called on non-float GseProp");
	
	return ((GsePropFloat*)m_pProp)->fVal;
}

const char* cGsePropSimple::getStr() const {
	if( m_pProp->ptype != pt_str)
		gse_throw_m(GSE_XCEPT,GSE_XBADARG,"getStr called on not-string GseProp");
	
	return ((GsePropStr*)m_pProp)->sVal;
}

cGseProp* cGsePropSimple::copy() const {
	GseProp* pCobj = m_pProp->copy(m_pProp);
	cGsePropSimple* pNew = new cGsePropSimple(pCobj);
	pNew->m_bOwner = true;
	return (cGseProp*) pNew;
}

/* List Properties ***********************************************************/

cGsePropList::cGsePropList(GseProp* pWrap):cGseProp(pWrap){
	if(pWrap->type(pWrap) != pt_list)
		gse_throw_m(GSE_XCEPT,GSE_XBADARG,"cGsePropList can't wrap non-list "
		             "C structures");
}

int4 cGsePropList::length() const{
	GsePropList* pThis = (GsePropList*) m_pProp;
	return pThis->length(pThis);
}

cGseProp* cGsePropList::getByName(const char* sName) const{
	GsePropList* pThis = (GsePropList*) m_pProp;
	GseProp* pCobj = pThis->getByName(pThis, sName);
	
	if(pCobj == NULL) return NULL;
	
	cGseProp* pRet = NULL;
	switch(pCobj->type(pCobj)){
	case pt_int:
	case pt_flt:
	case pt_str:
		pRet = (cGseProp*) (new cGsePropSimple(pCobj) );
		break;
	
	case pt_list:
		pRet = (cGseProp*) (new cGsePropList(pCobj) );
		break;
		
	case pt_enum:
		pRet = (cGseProp*) (new cGsePropEnum(pCobj) );
		break;
		
	default:
		gse_throw(GSE_XCEPT,GSE_XINCONCEIVABLE);
		break;
	}
	
	pRet->m_bOwner = false;
	return pRet;
}

cGseProp* cGsePropList::get(int4 idx) const{
	GsePropList* pThis = (GsePropList*) m_pProp;
	GseProp* pCobj = pThis->get(pThis, idx);
	
	if(pCobj == NULL) return NULL;
	
	cGseProp* pRet = NULL;
	switch(pCobj->type(pCobj)){
	case pt_int:
	case pt_flt:
	case pt_str:
		pRet = new cGsePropSimple(pCobj);
		break;
	
	case pt_list:
		pRet = new cGsePropList(pCobj);
		break;
		
	case pt_enum:
		pRet = new cGsePropEnum(pCobj);
		break;
		
	default:
		gse_throw(GSE_XCEPT,GSE_XINCONCEIVABLE);
		break;
	}
	
	pRet->m_bOwner = false;
	return pRet;
}

gse_prop_type_t cGsePropList::getType(int4 idx) const{
	GsePropList* pThis = (GsePropList*) m_pProp;
	GseProp* pProp = pThis->get(pThis, idx);
	if(pProp == NULL) return pt_invalid;
	return pProp->ptype;
}

gse_prop_type_t cGsePropList::getTypeByName(const char* sName) const{
	GsePropList* pThis = (GsePropList*) m_pProp;
	GseProp* pProp = pThis->getByName(pThis, sName);
	if(pProp == NULL) return pt_invalid;
	return pProp->ptype;
}


cGseProp* cGsePropList::copy() const{
	GseProp* pCobj = m_pProp->copy(m_pProp);
	cGsePropList* pNew = new cGsePropList(pCobj);
	return (cGseProp*) pNew;
}

/* Enum Properties ***********************************************************/

cGsePropEnum::cGsePropEnum(GseProp* pWrap):cGseProp(pWrap){
	if(pWrap->type(pWrap) != pt_enum)
		gse_throw_m(GSE_XCEPT,GSE_XBADARG,"cGsePropEnum can't wrap non-enum "
		             "C structures");
}

int4 cGsePropEnum::length() const{
	GsePropEnum* pThis = (GsePropEnum*)m_pProp;
	return pThis->length(pThis);
}

bool cGsePropEnum::hasDefault() const{
	GsePropEnum* pThis = (GsePropEnum*)m_pProp;
	return pThis->hasDefault(pThis);
}

const char* cGsePropEnum::getDefault() const{
	GsePropEnum* pThis = (GsePropEnum*)m_pProp;
	return (const char*)pThis->sDefault;
}

const char* cGsePropEnum::get(int8 INPUT) const{
	GsePropEnum* pThis = (GsePropEnum*)m_pProp;
	return pThis->get(pThis, INPUT);
}

cGseProp* cGsePropEnum::copy() const{
	GseProp* pCobj = m_pProp->copy(m_pProp);
	cGsePropEnum* pNew = new cGsePropEnum(pCobj);
	return (cGseProp*) pNew;
}

/* And finally the castors, I don't know how you improve SWIG so that these
   aren't needed */
cGsePropSimple* cast_GsePropSimple(cGseProp* pProp){
	gse_prop_type_t ptype = pProp->type();
	
	if((ptype != pt_int)&&(ptype != pt_flt)&&(ptype != pt_str))
		gse_throw_m(GSE_XCEPT,GSE_XBADARG,"Invalid cast to GsePropSimple");
		
	return (cGsePropSimple*)pProp;
}

cGsePropList* cast_GsePropList(cGseProp* pProp){
	gse_prop_type_t ptype = pProp->type();
	
	if(ptype != pt_list)
		gse_throw_m(GSE_XCEPT,GSE_XBADARG,"Invalid cast to GsePropList");
		
	return (cGsePropList*)pProp;
}

cGsePropEnum* cast_GsePropEnum(cGseProp* pProp){
	gse_prop_type_t ptype = pProp->type();
	
	if(ptype != pt_enum)
		gse_throw_m(GSE_XCEPT,GSE_XBADARG,"Invalid cast to GsePropEnum");
		
	return (cGsePropEnum*)pProp;
}


%}

typedef enum _gse_prop_type{ 
	pt_invalid = 0, pt_int, pt_flt, pt_str, pt_list, pt_enum
} gse_prop_type_t;

%include rpwgse/GseProp.hpp


/* Castors, since swig can't handle generator functions */

cGsePropSimple* cast_GsePropSimple(cGseProp* pProp);
cGsePropList* cast_GsePropList(cGseProp* pProp);
cGsePropEnum* cast_GsePropEnum(cGseProp* pProp);



