#ifdef GSE_POSIX_THREADS
#include <pthread.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "GseWrap.h"
#include "GseExcept.h"
#include "GseRiceDecomp.h"


/*****************************************************************************/
/* Forward Declarations of Implementation Functions                          */
/*****************************************************************************/

void getReference(int4 nBitSz, uint2* pComp, int4* compFileIndex, uint2* pDcmp,
                  int4* decompFileIndex, int4 count, int4* end, int4* position);
		
int4 getCodeID(int4 nBitSz, uint2* pComp, int4 *compFileIndex, int4 count,
               int4* maxValue, int4* half, int4* end, int4* position);

void setSplitSeqSz();

int4 getFS(int4 nBitSz, uint2* pComp, int4 *compFileIndex, int4 count,
           int4* end, int4* position);

void decodeMappedDiff(uint2* pDcmp, int4* decompFileIndex, uint2* pMapD,
        int4* MDIndex,int4* half,int4* maxValue,int4* currIndex);

void getBackup(int4 nBitSz, uint2* pComp, int4 *compFileIndex, int4 count,
               uint2* pMapD, int4* MDIndex, int4* end, int4* position);

uint2 getSplitSeqBits(uint2*, int4*,int4,int4*,int4*,int4*);

void joinFSandSS(int4,int4,uint2*,int4*,int4*);

int4 decompressPkt(int4,uint2 *,uint2*,int4*,int4,uint2*, int4*);

int4 decompressPkt(int4 nBitSz, uint2 *pComp, uint2*pDcmp, int4* decompFileIndex,
		             int4 count, uint2* pMapD, int4* MDIndex);

void fsToMd(int4 fundSeq, uint2*, int4*);


/*****************************************************************************/
/* Puts the reference value in the first element of decomp                   */
/*****************************************************************************/

void getReference(int4 nBitSz, uint2* pComp, int4* compFileIndex, uint2* pDcmp,
                  int4* decompFileIndex, int4 count, int4* end, int4* position)
{
	/*save off reference sample*/
	if(nBitSz==12){
		pDcmp[*decompFileIndex] = (pComp[*compFileIndex]>>4) & 0x0FFF;/*reference*/
		*position =12;
		return;
	}
	
	if(nBitSz==16){
		pDcmp[*decompFileIndex] = pComp[*compFileIndex]; /*reference*/
		(*compFileIndex)++;
		if(*compFileIndex>=count){
			*end=1;
		}
		return;
 	}
	
   gse_throw_fm(GSE_XCEPT, GSE_XBADARG, "nBitSz %d is not supported", nBitSz);
}

/*****************************************************************************/
/* Grabs the CodeID or the "PSI" value and places                            */
/* it in the working varaiable CodeID                                        */
/*****************************************************************************/

int4 getCodeID(int4 nBitSz, uint2* pComp, int4 *compFileIndex, int4 count,
               int4* maxValue, int4* half, int4* end, int4* position)
{
	int4 codeID=0;
	int4 loop;
	uint2 codeIDhold;
	int4 flag=0;
	uint2 Mask =0xF, overlapUpperMask =0, overlapLowerMask =0;
	
	if (nBitSz==12){
		if (*compFileIndex==0){
			codeID=(pComp[*compFileIndex]&0x000F);
			(*compFileIndex)++;
			if(*compFileIndex>=count){
				/* If the number of bytes is more than was passed in then exit */
				*end=1;
			}
			*position=0;
		}
		else{
			/* determine if split sequence bits are all contained in one element of 
			   array*/
			if(12-(*position)<0){
				flag = 1;
			}
			
			/*CodeID Bits all contained within current element*/
			if (flag==0){

				codeIDhold=(pComp[*compFileIndex]>>(12-(*position)))&Mask ;
				codeID=codeIDhold;
				*position = (*position) + 4;
			}
			
			/*CodeID Bits are split between [Current] and [Current+1] elements*/
			if (flag==1){
				overlapUpperMask = pow(2, 16 - (*position)) - 1;
				overlapLowerMask = pow(2, abs(4 - (16-(*position))) ) - 1;
				codeID = pComp[*compFileIndex] & overlapUpperMask;
				codeID = codeID << abs(12 - (*position));
				
				(*compFileIndex)++;
				
				if(*compFileIndex>=count){
					/* If the number of bytes is more than was passed in then exit */
					*end=1; 
				}
				codeID |= pComp[*compFileIndex] >> (16-(abs(12-(*position)))) & overlapLowerMask;
				
				*position = abs(12-(*position));
			}
		}
	}
	
	else{
		if(nBitSz == 16){
			if(*compFileIndex == 1){  		
				codeID = (pComp[*compFileIndex]>>12) & 0x000F;
				*position = 4;
			}
			else{
				/* Determine if split sequence bits are all contained in one element of 
				   array */
				if((12-(*position))<0){
					flag = 1; /*bits are in two adjacent elements*/
				}
				
				/*CodeID Bits all contained within current element*/
				if (flag==0){ 
					codeIDhold = (pComp[*compFileIndex]>>(12-(*position)))&Mask ;
					codeID = codeIDhold;
					*position = (*position) + 4;
				}
				
				/*CodeID Bits are split between [Current] and [Current+1] elements*/
				if(flag == 1){
	
					overlapUpperMask = pow(2, 16 - (*position) ) - 1;
					overlapLowerMask = pow(2, abs(4 - (16-(*position)) ) )-1;
					codeID = pComp[*compFileIndex] & overlapUpperMask;
					codeID = codeID << (abs(12-(*position)));
					(*compFileIndex)++;
	
					if(*compFileIndex >= count){
						/*if the number of bytes is more than was passed in by caller then exit*/
						*end=1;
					}
					
					codeID |= pComp[*compFileIndex] >> (16-(abs(12-(*position)))) & overlapLowerMask;
					*position = abs(12-(*position));  /*cwp is this correct? */
				}
			}
		}
	}
	
	for(loop=0;loop<nBitSz;loop++){
		(*maxValue) *= 2;
	}

	*half = (*maxValue)/2;

	return codeID;
}

/*****************************************************************************/
/* Sets number of bits in Split Sequence based on the PSI value              */
/*****************************************************************************/

void setSplitSeqSz(int4* codeID)  /*cwp This function has no effect!! */
{
	int4 splitSequenceSize;
	 
	switch(*codeID){
		
	case 0: case 15:
		splitSequenceSize=0;
		break;
		
	default:
		splitSequenceSize=(*codeID);
		break;
        
	}
	return;
}

int4 getFS(int4 nBitSz, uint2* pComp, int4 *compFileIndex, int4 count,
           int4* end, int4* position)
{
	int4 FundSeq=0;
	int4 loop=0;
	uint2 Mask=32768;
	uint2 temp;
	
	
	if( (*position) > 16)
		gse_throw(GSE_XCEPT, GSE_XBADARG);
	
		
	if((*position) == 16){
		(*position) = 0;
		(*compFileIndex)++;
		if(*compFileIndex>=count){
	   	*end=1;
		}
	}

	for(loop=0; loop<(count*16); loop++){  			 
		
		temp = pComp[*compFileIndex] << (*position);
		
		if((temp & Mask) == Mask){
			FundSeq=loop;
			(*position)++;
			if((*position) >= 16){
				(*position) = 0;
				(*compFileIndex)++;
				if(*compFileIndex >= count){
					*end=1;
					break;
				}
			}
			break;
		}
		else{
			(*position)++;
			if((*position) >= 16){
				(*position) = 0;
				(*compFileIndex)++;
				if(*compFileIndex >= count){
  					*end=1;
  					break;
				}
			}
		}
	}
	
	return FundSeq;
}
uint2 getSplitSeqBits(uint2* pComp, int4 *compFileIndex, int4 count,
                           int4* codeID, int4 *end, int4* position)
{
	uint2 splitSequenceSetHold;
	uint2 splitSequenceBits;
	int4 splitSequenceSize=(*codeID);
	int4 flag=0;
	uint2 Mask, overlapUpperMask =0, overlapLowerMask =0;

	if(splitSequenceSize>0){
		Mask=(pow(2,splitSequenceSize)-1); /*Create Mask*/
	}

	/*determine if split sequence bits are all contained in one element of array*/   
	if(((16-splitSequenceSize)-((*position)))<0){
		flag = 1;
	}
   
	/*Split Sequence Bits all contained within current element*/
	if(flag==0){
		splitSequenceSetHold = (pComp[*compFileIndex]>>((16-splitSequenceSize)-((*position))))&Mask ;
		splitSequenceBits = splitSequenceSetHold;
		*position = (*position)+splitSequenceSize;
	}

	/*Split Sequence Bits are split between [Current] and [Current+1] elements*/    
	if(flag==1){
		
		overlapUpperMask = pow(2,((16-(*position))))-1;
		overlapLowerMask = pow(2,abs(splitSequenceSize-(16-(*position))))-1;
		splitSequenceBits = pComp[*compFileIndex]& overlapUpperMask;
		splitSequenceBits = splitSequenceBits<<(abs((16-splitSequenceSize)-((*position))));
		(*compFileIndex)++;
		
		/*if the number of bytes is more than was passed in by caller then exit*/
		if(*compFileIndex>=count){
			*end=1;
		}
		
		splitSequenceBits |= pComp[*compFileIndex]>> (16-(abs((16-splitSequenceSize)-((*position))))) & overlapLowerMask;

		*position = abs((16-splitSequenceSize)-((*position)));
	}
	
	return splitSequenceBits;
}

/*****************************************************************************/
/*Do the mapping function in reverse                                         */
/*****************************************************************************/

void decodeMappedDiff(uint2* pDcmp, int4* decompFileIndex, uint2* pMapD,
        int4* MDIndex,int4* half,int4* maxValue,int4* currIndex)
{
	uint2 current[18];

	if((*MDIndex)<=1)
	return;	 
    
	/*If previous value is  <2047 for 12-bit and <32767 for 16-bit*/   
	if(pDcmp[*decompFileIndex] < (*half)){
		if(pMapD[(*MDIndex)-1]==0){
			current[*currIndex]=pDcmp[*decompFileIndex];
		}
		else{
			if(pMapD[(*MDIndex)-1]>2*pDcmp[*decompFileIndex]){
				current[*currIndex]=pMapD[(*MDIndex)-1];
			}
			else{
				if((pMapD[(*MDIndex)-1]&1)==0){
					current[*currIndex]=(pDcmp[*decompFileIndex]+(pMapD[(*MDIndex)-1]>>1));
				}
				else{
					current[*currIndex]=(pDcmp[*decompFileIndex]-((1+pMapD[(*MDIndex)-1])>>1));
				}
			}   
		} 
	}
	
	/*if previous value is <4095 for 12- bit and <65535 for 16- bit*/
	else{
		if(pMapD[(*MDIndex)-1]==0){
			current[*currIndex]=pDcmp[*decompFileIndex];
		}
		else{
			if(pMapD[(*MDIndex)-1]>(2*((*maxValue)-1-pDcmp[*decompFileIndex]))){
				current[*currIndex]=((*maxValue)-1-pMapD[(*MDIndex)-1]);
			}
			else{
				if((pMapD[(*MDIndex)-1]&1)==0){
					current[*currIndex]=(pDcmp[*decompFileIndex]+(pMapD[(*MDIndex)-1]>>1));
				}
				else{
					current[*currIndex]=(pDcmp[*decompFileIndex]-((1+pMapD[(*MDIndex)-1])>>1));
				}
			}
		}
	}
    
	pDcmp[*decompFileIndex+1]=current[*currIndex];
	(*currIndex)++;
	(*decompFileIndex)++; 
}

/*****************************************************************************/
/*For the case when the PSI value is equal to fifteen (backup operator)      */
/*****************************************************************************/

void getBackup(int4 nBitSz, uint2* pComp, int4 *compFileIndex, int4 count,
               uint2* pMapD, int4* MDIndex, int4* end, int4* position)
{
	uint2 backupHold;
	int4 flag=0;
	uint2 Mask=0x0FFF, overlapUpperMask =0, overlapLowerMask =0;
    
	if(*compFileIndex>count)
		return;
		
	/*split data into 12-bit blocks to get mapD's*/
	if(nBitSz==12){
		
		/*determine if backup bits are all contained in one element of array*/
		if((4-(*position))<0){
			flag = 1;     /*backup Bits are split*/
		}

		/*backup Bits all contained within current element*/
		if(flag==0){
			backupHold = (pComp[*compFileIndex]>>(4-(*position)))&Mask;
			pMapD[(*MDIndex)] = backupHold;
			*position = (*position)+12;
		}
        
		/*backup Bits are split between [Current] and [Current+1] elements*/
		if(flag==1){
			overlapUpperMask  = pow(2, 16 - (*position) ) - 1;
			overlapLowerMask  = pow(2, abs( 12 - (16-(*position)) ) ) - 1;
			
			pMapD[(*MDIndex)] = pComp[*compFileIndex] & overlapUpperMask;
			pMapD[(*MDIndex)] = pMapD[(*MDIndex)] << (abs(4-(*position)));
			
			(*compFileIndex)++;
			
			/*if the number of bytes is more than was passed in then exit*/
			if(*compFileIndex >= count){
				*end=1;  
			}
			pMapD[(*MDIndex)] = pMapD[(*MDIndex)] |
					             (pComp[*compFileIndex]>>(16-(abs(4-(*position)))) & overlapLowerMask);
			*position = abs(4-(*position));
		}
		
		(*MDIndex)++;
		return;
	}
	
	/*16 bit case*/
	if(nBitSz==16){
		
		/*determine if backup bits are all contained in one element of array*/
		if((*position)>0){
			flag = 1;/*backup Bits are split*/
		}
		
		/*backup Bits all contained within current element*/
		if(flag == 0){
			pMapD[(*MDIndex)]=pComp[*compFileIndex];
			(*position)=((*position)+16);
		}
      
		/*backup Bits are split between [Current] and [Current+1] elements*/
		if(flag == 1){
			overlapUpperMask=((pow(2,((16-(*position))))-1)); 
			overlapLowerMask=(pow(2,(*position))-1);
			pMapD[(*MDIndex)]=(pComp[*compFileIndex]& overlapUpperMask);
			pMapD[(*MDIndex)]=(pMapD[(*MDIndex)]<<(*position));
			(*compFileIndex)++;
			
			/*if the number of bytes is more than was passed in by caller then exit*/
			if(*compFileIndex>=count){
				*end=1;
			}
			
			pMapD[(*MDIndex)] |= ( (pComp[*compFileIndex] >> (16-(*position))) ) & overlapLowerMask;
		}
		(*MDIndex)++;
		return;
	}
	
	gse_throw_m(GSE_XCEPT, GSE_XBADARG, "Expected bitsize to be 12 or 16");
}

void fsToMd(int4 fundSeq, uint2* pMapD, int4* MDIndex)
{
    pMapD[(*MDIndex)]=fundSeq;
    (*MDIndex)++;
}

void joinFSandSS(int4 fundSeq, int4 splitSequenceBits, uint2* pMapD,
		           int4* MDIndex,int4* codeID)
{
    pMapD[(*MDIndex)] = (fundSeq<<(*codeID))|(splitSequenceBits);
    (*MDIndex)++;
}

/*****************************************************************************/
int4 decompressPkt(int4 nBitSz, uint2* pComp, uint2* pDcmp, int4* pDcmpIdx,
		             int4 count, uint2* pMapD, int4* pMDIdx)
{
	int4 fundSeq,loop2;
	int4 iCompIdx = 0;
	int4 maxVal   = 1;
	int4 half     = 0;
	int4 codeID;
	int4 end      = 0;
	int4 curIdx   = 1;
	int4 position = 0;
	uint2 k_bit_sample;
	
	getReference(nBitSz, pComp, &iCompIdx, pDcmp, pDcmpIdx, count, &end, &position);
    
	while(iCompIdx < count-1){
		 
		codeID = getCodeID(nBitSz, pComp, &iCompIdx, count, &maxVal, &half, &end,
				             &position);
		setSplitSeqSz(&codeID);
		loop2 = 0;

		switch(codeID){
		case 0:
			while(loop2 < 16){
				fundSeq = getFS(nBitSz, pComp, &iCompIdx, count, &end, &position);
				
				fsToMd(fundSeq, pMapD, pMDIdx);
				
				decodeMappedDiff(pDcmp, pDcmpIdx, pMapD, pMDIdx, &half, &maxVal, &curIdx);
				
				if(end == 1) break;
				
				loop2++;
			}
			break;
        
		case 15:
			while(loop2<16){
				getBackup(nBitSz, pComp, &iCompIdx, count, pMapD, pMDIdx, &end, &position);
				
				decodeMappedDiff(pDcmp, pDcmpIdx, pMapD, pMDIdx, &half, &maxVal, &curIdx);
				
				if(end==1) break;

				loop2++;
			}
			break;
        
		default:
			while(loop2<16){
				fundSeq = getFS(nBitSz, pComp, &iCompIdx, count, &end, &position);
				
				k_bit_sample = getSplitSeqBits(pComp, &iCompIdx, count, &codeID,
						                         &end, &position);
				
				joinFSandSS(fundSeq, k_bit_sample, pMapD, pMDIdx, &codeID);
				
				decodeMappedDiff(pDcmp, pDcmpIdx, pMapD, pMDIdx, &half, &maxVal, &curIdx);
				
				if(end==1) break;
				
				loop2++;
			}
			break; 
		}

		half=0;
		maxVal=1;
		curIdx=1;
	}
    
	return 0;
 }

/******************************************************************************/ 
/* Main Interface function                                                    */
/******************************************************************************/
bool gse_rice_decomp(const byte* pData, size_t uBytes, int4 bitMode,
                    struct rice_decomp_t* pResults)
 {
	uint2* pComp;
	uint2* pDcmp;
	uint2* pMapD;
	int4 MDIndex =0;
	int4 decompFileIndex=0;
	int4 count=0;
#if __BYTE_ORDER == __LITTLE_ENDIAN
	size_t u;
#endif
	
	/* Zero out the results struct */
	pResults->pDecomp  = NULL;
	pResults->uSamples = 0;
	pResults->pMsg 	 = NULL;
	pResults->bDynMsg  = false;
	pResults->errorNum = 0;

	if(pResults == NULL)
  		gse_throw_m(GSE_XCEPT, GSE_XBADARG, "Results pointer is NULL");

	if((pData == NULL)||(uBytes < 1))
		gse_throw_m(GSE_XCEPT, GSE_XBADARG, "Data pointer is NULL, or uBytes < 1");

	if((bitMode != 12)&&(bitMode != 16))
		gse_throw_m(GSE_XCEPT, GSE_XBADARG, "bitMode must be 12 or 16");
   
	if(uBytes % 2 == 0)                  /*cwp is this okay for 12 bit data? */
		count = uBytes/2;
	else
		count = (uBytes / 2) + 1;
	
	pComp = (uint2*) gse_calloc(count,    sizeof(uint2));
	pDcmp = (uint2*) gse_calloc(count*13, sizeof(uint2));
	pMapD  = (uint2*) gse_calloc(count*13, sizeof(uint2));

	/* Push cleanup handlers incase of exception */
	except_cleanup_push(free, pComp);
	except_cleanup_push(free, pDcmp);
	except_cleanup_push(free, pMapD);
	
	/* Temp hack to test recieve byte endianness, load as if data was big endian */
#if __BYTE_ORDER == __LITTLE_ENDIAN
	/* if(bitMode == 16){ */
		for(u = 0; u < uBytes; u += 2){ 
			*( ((byte*)(pComp + u/2)) + 1) = pData[u];
			
			if(u + 1 < uBytes) 
				*( ((byte*)(pComp + u/2)) ) = pData[u+1];
			else
				*( ((byte*)(pComp + u/2)) ) = 0; 
		}
	/* }
	else{
		memcpy(pComp, pData, uBytes);
	} */
#else
	memcpy(pComp, pData, uBytes);
#endif

	decompressPkt(bitMode, pComp, pDcmp, &decompFileIndex, count+1, pMapD,
   			     &MDIndex);

	except_cleanup_pop(except_call); 	 /* Always free pComp */
	except_cleanup_pop(except_no_call);  /* only free pDcmp on an exception */
	except_cleanup_pop(except_call); 	 /* Always free pMapD */

	pResults->pDecomp  = pDcmp;
	pResults->uSamples = decompFileIndex;
	
	/* Another hack make this value always % 16 */
	pResults->uSamples &= 0xFFFFFFF0;
	/* End hack */
	
	pResults->errorNum = 0; 

 	return (pResults->errorNum == 0);
 }
