/** @file GseRiceDecomp.h */
#ifndef _GseRiceDecomp_H_
#define _GseRiceDecomp_H_

#include <rpwgse/GseTypes.h>

#ifdef __cplusplus
extern "C" {
#endif

/** @addtogroup mod_rpwgse */
/*@{*/

/** @defgroup mod_compression Compression
 *
 * Functions for compressing and decompressing data.
 */

/*@}*/

/** @addtogroup mod_compression */
/*@{*/

/** Structure to hold the Rice Data Decompression function's working area as
 * well as the results of the decompression. */
 
struct rice_decomp_t{
	
	/** Pointer to results memory. This memory must be allocated by the
	 * caller to rice_decompress
	 */
	uint2* pDecomp;
	
	/** Number of decompressed samples in pDecomp, This will be filled in 
	 * by a succesful call to rice_decompress.  
	 */
	uint4 uSamples;
	
	/** Error message generated by rice_decompress() if any */
	const char* pMsg;
	
	/** True if the calling program is responsible for freeing the
	 *  error message memory */
	bool bDynMsg;
        
	/** Error indicator.
	 * 0 if succesful , if not, an error integer in the range 1 to 255 
	 * (defined below) and the @a pMsg string may be set. 
	 *
	 * @note the caller may be responsible for freeing the error string, see
	 * @a bDynMsg.
	 */
	int4 errorNum;
};

/** Rice Data Decompression
 *
 * The input data is expected to consit of a reference sample followed
 * by (operator, differences) blocks.  The reference sample should be 
 * in MSB first order.
 *
 * @todo Explain rice decompression in more detail here, and/or reference
 *        other explainations in a locatable manner.
 * 
 * @param  pData a pointer to the bytes that need to be decompressed.
 * @param  uBytes the number of bytes to decompress
 * @param  bitMode the bitsize of the uncompressed data eg. 12, or 16
 * @param  pResults pointer to a struct rice_decomp16 that will contain
 *         the decompressed samples.
 *
 * @throws (GSE_XCEPT, GSE_XBADARG) if no data, results pointer is NULL
 *         or the bitMode is not one of 12 or 16.
 */
bool gse_rice_decomp(const byte* pData, size_t uBytes, int bitMode,
                     struct rice_decomp_t* pResults); 
							  
/*@}*/
							  
#ifdef __cplusplus
}
#endif

#endif /* _GseRiceDecomp_H_ */
