#define _POSIX_C_SOURCE 200112

#ifdef GSE_POSIX_THREADS
#include <pthread.h>
#endif

#ifdef WIN32
#error Win32 version of GseSys.c is not yet implemented
#else
#include <unistd.h>
#include <sys/types.h>
#include <pwd.h>
#include <errno.h>
#include <time.h>
#endif

#include "GseSys.h"
#include "GseExcept.h"
#include "GseString.h"
#include "GseWrap.h"
#include "GseFile.h"

/*****************************************************************************/
procid_t gse_sopen(const char* sCmd, FILE** ppStdin, FILE** ppStdout, 
                   FILE** ppStderr)
{
	gse_throw_m(GSE_XCEPT, GSE_XNOTYETIMP, "gse_sopen is not yet implemented");
	
	return 0;
}


#ifndef WIN32
char* gse_username()
{
	struct passwd pwbuf;
	char strback[128];
	struct passwd* pResult;
	int nErr;
	
	if( ((nErr = getpwuid_r(getuid(), &pwbuf, strback, 127, &pResult)) != 0) ||
	    (pResult == NULL))
	{
		if(nErr != 0)
			gse_throw_m(CLIB_XCEPT, nErr, "Couldn't get the passwd entry");
		else
			gse_throw(GSE_XCEPT, GSE_XINCONCEIVABLE);
	}
	
	return gse_string(pResult->pw_gecos);
}

char* gse_username_s(char* buf, size_t len)
{
	struct passwd pwbuf;
	char strback[128];
	struct passwd* pResult;
	int nErr;
	
	if((buf == NULL) || (len < 2))
		gse_throw_m(GSE_XCEPT, GSE_XBADARG, "Buffer is null, or less than two "
		            "characters long.");
	
	if( ((nErr = getpwuid_r(getuid(), &pwbuf, strback, 127, &pResult)) != 0) ||
	    (pResult == NULL))
	{
		if(nErr != 0)
			gse_throw_m(CLIB_XCEPT, nErr, "Couldn't get the passwd entry");
		else
			gse_throw(GSE_XCEPT, GSE_XINCONCEIVABLE);
	}
	
	strncpy(buf, pResult->pw_gecos, len-1);
	buf[len - 1] = '\0';
	return buf;
}

/* Freekin linux removed this because it can be a security hole.  Dude
 * I'm not writing the login process for HAL 9000, just leave the damn
 * routine in the lib and warn everyone.  --cwp
 */
char* gse_tempname(const char* sSuffix)
{
	char* sBase = NULL;
	char* sPath = NULL;
	const char* _sSuffix;
	const char* sDir = "/tmp";
	
	pid_t pid = getpid();
	
	if(sSuffix != NULL) _sSuffix = sSuffix;
	else _sSuffix = ".tmp";
	
#ifdef GSE_POSIX_THREADS
	pthread_t tid = pthread_self();
	sBase = NS("%u-%u", pid, tid);
#else
	sBase = NS("%u", pid);
#endif
	
	/* TODO: Use random number generator to make this faster
	         in the case of many temporary files. */
	
	int i = 0;
	while(sPath == NULL){		
		sPath = NS("%s/%s-%03d%s", sDir, sBase, i, _sSuffix);
		
		if( gse_exists(sPath) )
			free(sPath);
		
		i += 1;
	}
	
	free(sBase);
	return sPath;
}


#else
#error TODO: Windows versions of GseSys functions are not yet implemented
#endif











