/** @file GseWinSvc.h Don't include this file usless you are building on
 *  Windows!!!
 */
#ifndef _GseWinSvc_H_
#define _GseWinSvc_H_

#include <rpwgse/GseTypes.h>

#ifdef __cplusplus
extern "C" {
#endif

/** @addtogroup mod_rpwgse */
/*@{*/

/** @defgroup mod_win_svc Windows Helpers
 * Provides some needed functionality that is no available by default from
 * the Win32 API 
 */
/*@}*/

/** @addtogroup mod_win_svc */
/*@{*/

/** Define the uid_t type */
typedef uint4 uid_t;

/** Get the current user's Relative Security ID (RID)
 *
 * This in a POSIX workalike replacement, your milage may vary.
 * Note that on windows, the RID is just one componet of the user id, which is
 * called the SID (security ID).
 * 
 * @return The current users RID, which can sorta be used like a user id
 *         on POSIX systems.
 *
 * @throws (WIN32_XCEPT, Last_Error) If the current process can't get it's SID.
 */
uid_t getuid();
 
 /* Threads intending to use backtrace() or dladdr() must call this first,
  * Called automatically from gse_thread_init() on Windows.
  */
 void gse_winsvc_init();
 
 /* Threads that will no longer use backtrace() or dladdr() should call this.
  * Called automatically from gse_thread_deinit() on Windows.
  */
 void gse_winsvc_deinit();
 
 
/** A Win32 implementation of the handy GNU backtrace function. 
 *
 * @param buffer a void pointer array to contain the function addresses.
 * @param size the maximum number of function address to store in buffer.
 * @return The number of function addresses returned in buffer.
 */
int backtrace(void** buffer, int size);


/** definition of the return data for dladdr().
 */
typedef struct {
	/** Filename of defining DLL or EXE */
	const char *dli_fname;
	
	/** Load address of the DLL or EXE that defines the object */
	void *dli_fbase;
	
	/** Name of nearest Symbol, string memory allocated possibly allocated
	 *  on the heap. See #dli_salloc;
	 */
	const char *dli_sname;
	
	/** Exact value of nearest symbol (Not implemented on Windows) */
	void *dli_saddr;
	
	/** Non-zero if the memory for dli_sname was allocated on the heap */
	int dli_salloc;
	
} Dl_info;

/** A Win32 implementation of the handy GNU dladdr function. 
 *
 * @warning This windows replacement allocates memory for dli_fname and
 *          dli_sname on the heap via malloc.  Caller is responsible for freeing
 *          memory if either the dli_fname and dli_sname are not null!
 *
 * @param addr The function address in question.
 * @param info A Dl_info structure pointer.
 * @returns 0 on error, non-zero on success (boolean).
 */ 
int dladdr(void *addr, Dl_info *info);


/*@}*/

#ifdef __cplusplus
}
#endif

#endif /* _GseWinSvc_H_ */
