#ifndef _MASH_DEF
#define _MASH_DEF
#define _MASH_IN(x, y)  x ## y
#define _MASH(x, y) _MASH_IN(x, y)
#endif /* _MASH_DEF */


/* Handle the de-alloc type selection */
#ifndef FREE_VALS
#ifndef DEL_VALS
#error Dictionary Value memory management method not defined
#endif
#endif

#ifndef KEY_FMT
#define KEY_FMT "address %zX"
#endif


typedef struct _MASH(C, _node_t) {
	dnode_t dnode;
	K key;
	bool noDel;
} _MASH(C, _node_t) ;

/*****************************************************************************/
/* Non Modifing usage functions */

const char* _MASH(name_, C) (const C* pThis)
{
	return (const char*)pThis->sName;
}

uint4 _MASH(length_, C) (const C* pThis)
{
	return (uint4) dict_count(&(pThis->dict));
}

bool _MASH(isEmpty, C) (const C* pThis)
{
	return dict_isempty( &(pThis->dict) ) ? true : false;
}

bool _MASH(hasKey_, C) (const C* pThis, K key)
{
	K _key = key;
	
	return dict_lookup((dict_t*)(&(pThis->dict)), &_key ) ? true : false;
}

const K* _MASH(first_, C) (const C* pThis)
{
	dnode_t* pNode;
	if( (pNode = dict_first((dict_t*)(&(pThis->dict)) ) ) != NULL)
		return (const K*) dnode_getkey(pNode);
	else
		return NULL;
}

const K* _MASH(last_, C) (const C* pThis)
{
	dnode_t* pNode;
	if( (pNode = dict_last((dict_t*)(&(pThis->dict)) ) ) != NULL)
		return (const K*) dnode_getkey(pNode);
	else
		return NULL;
}

const K* _MASH(next_, C) (const C* pThis, K key)
{
	dnode_t* pCur;
	dnode_t* pNext;
	K _key = key;
	
	pCur = dict_lookup( (dict_t*)(&(pThis->dict)), (const void*)(&_key));
		
	if(pCur == NULL)
		gse_throw_fm(GSE_XCEPT, GSE_XNOKEY, "Key "  KEY_FMT " is not in the %s"
				       " dictionary", key, pThis->sName);
	
	if( (pNext = dict_next( (dict_t*)(&(pThis->dict)), pCur)) == NULL)
		return NULL;
		
	return (const K*) dnode_getkey(pNext);
}

const K* _MASH(prev_, C) (const C* pThis, K key)
{
	dnode_t* pCur;
	dnode_t* pPrev;
	K _key = key;
		
	pCur = dict_lookup( (dict_t*)(&(pThis->dict)), (const void*)(&_key));
	
	if(pCur == NULL)
		gse_throw_fm(GSE_XCEPT, GSE_XNOKEY, "Key "  KEY_FMT " is not in the %s"
				       " dictionary", key, pThis->sName);
	
	if( (pPrev = dict_next( (dict_t*)(&(pThis->dict)), pCur)) == NULL)
		return NULL;
	
	return (const K*) dnode_getkey(pPrev);
}

const V* _MASH(get_, C) (const C* pThis, K key)
{
	dnode_t* pNode;
	K _key = key;
		
	if( (pNode = dict_lookup( (dict_t*)(&(pThis->dict)), (const void*)(&_key))) == NULL)
		return NULL;
	
	return (const V*) dnode_get(pNode);
}

/*****************************************************************************/
/* Modifing usage functions */

K _MASH(insert_, C) (C* pThis, K key, V* pVal)
{
	_MASH(C, _node_t)* pNode = NULL;
	K _key = key;
	
	pNode = (_MASH(C, _node_t) *) dict_lookup( &(pThis->dict), (const void*)(&_key) );
			
	if(pNode != NULL)
		gse_throw_fm(GSE_XCEPT, GSE_XNOKEY, "Key "  KEY_FMT " is already in the "
				       "%s dictionary", key, pThis->sName);
	
	/* WARNING! We must make our own node here, in order to store the
	   key somewhere, do *NOT* use dict_alloc_insert */
	
	pNode = (_MASH(C, _node_t) *) gse_calloc(1, sizeof( _MASH(C, _node_t) ) );
	
	dnode_init( (dnode_t*)pNode, pVal);  /* Handle dict module stuff */
	
	pNode->key = key;
	
	dict_insert(&(pThis->dict), (dnode_t*)pNode, (const void*) (&(pNode->key)) );	
	
	return key;
}

V* _MASH(getMutable_, C) (C* pThis, K key)
{
	dnode_t* pNode;
	K _key = key;
	
	if( (pNode = dict_lookup(&(pThis->dict), (const void*)(&_key) )) == NULL)
		gse_throw_fm(GSE_XCEPT, GSE_XNOKEY, "Key "  KEY_FMT " is not in the %s"
				       " dictionary", key, pThis->sName);
	
	return (V*) dnode_get(pNode);
}
		
V* _MASH(take_, C) (C* pThis, K key)
{
	dnode_t* pNode;
	K _key = key;
	 _MASH(C, _node_t)* pDNode;
	V* pVal;
	
	pNode = dict_lookup(&(pThis->dict), (const void*)(&_key) );
	
	/* Get the node */
	if( pNode == NULL)
		gse_throw_fm(GSE_XCEPT, GSE_XNOKEY, "Key "  KEY_FMT " is not in the %s"
				       " dictionary", key, pThis->sName);
	
	pDNode = ( _MASH(C, _node_t) *) pNode;
	
	pDNode->noDel = 1;  /* Mark as no delete value */
	
	pVal = (V*)dnode_get(pNode);
	
	dict_delete_free(&(pThis->dict), pNode);  /* Key is gone at this point */
	
	return pVal;
}

V* _MASH(replace_, C) (C* pThis, K key, V* pNewVal)
{
	dnode_t* pNode;
	K _key = key;
	V* pRet;
	
	pNode = dict_lookup(&(pThis->dict), (const void*)(&_key));
	
	/* Get the node */
	if(pNode == NULL)
		gse_throw_fm(GSE_XCEPT, GSE_XNOKEY, "Key "  KEY_FMT " is not in the %s"
				       " dictionary", key, pThis->sName);
	
	pRet = (V*)dnode_get(pNode);
	
	dnode_put(pNode, pNewVal);
	
	return pRet;
}

void _MASH(delPair_, C) (C* pThis, K key)
{
	dnode_t* pNode;
	K _key = key;
	
	pNode = dict_lookup(&(pThis->dict), (const void*)(&_key));
	
	/* Get the node */
	if(pNode == NULL)
		gse_throw_fm(GSE_XCEPT, GSE_XNOKEY, "Key "  KEY_FMT " is not in the %s"
				       " dictionary", key, pThis->sName);
	
	dict_delete_free(&(pThis->dict), pNode);  /* Key is gone at this point */
}

/*****************************************************************************/
/* Own/Disown values */

V* _MASH(disownVal, C) (C* pThis, K key)
{
	_MASH(C, _node_t) * pNode;
	K _key = key;
	
	pNode = ( _MASH(C, _node_t)* )dict_lookup(&(pThis->dict),(const void*)(&_key));
	
	if( pNode == NULL)
		gse_throw_fm(GSE_XCEPT, GSE_XNOKEY, "Key "  KEY_FMT " is not in the %s"
				       " dictionary", key, pThis->sName);
	
	pNode->noDel = 1;
	return (V*) dnode_get((dnode_t*)pNode);
}

void _MASH(disownAllVals, C) (C* pThis)
{
	pThis->bOwnsVals = false;
}


/*****************************************************************************/
/* Create/Destroy Nodes */

dnode_t* _MASH(allocNode_, C) (void* vpThis)
{
	dnode_t* pNew;
	pNew = (dnode_t*) gse_calloc(1, sizeof( _MASH(C, _node_t) ));
	return pNew;
}

void _MASH(freeNode_, C) (dnode_t* pNode, void* vpThis)
{
	V* pVal  = NULL;
	
	C* pThis = (C*)vpThis;
	_MASH(C, _node_t) * pN = ( _MASH(C, _node_t) *)pNode;
	
	bool bKeepVal = pN->noDel || (! pThis->bOwnsVals);
		
	if(!bKeepVal){
		pVal = (V*)dnode_get(pNode);
#ifdef DEL_VALS
		if(pVal != NULL)
			pVal->del(pVal);
#else
		free(pVal);
#endif
	}
	
	free(pN);
} 

/*****************************************************************************/
/* Create/Destroy Dictionaries */

void _MASH(finish_, C) (C* pThis)
{
	free(pThis->sName);
	dict_free_nodes(&(pThis->dict));
}

void _MASH(delete_, C) (void* vpThis)
{
	C* pThis = (C*)vpThis;
	
	_MASH(finish_, C) (pThis);
	
	free(vpThis);
}

int _MASH(compare_, C) (const void* k1, const void* k2)
{
	K* tk1 = (K*)k1;
	K* tk2 = (K*)k2;
	
	if(*tk1 > *tk2) return 1;
	if(*tk1 < *tk2) return -1;
	
	return 0;
}


void _MASH(init_, C)(C* pThis, const char* sName)
{
	pThis->name          = _MASH(name_, C);
	pThis->isEmpty       = _MASH(isEmpty, C);
	pThis->hasKey        = _MASH(hasKey_, C);
	pThis->length        = _MASH(length_, C);
	pThis->first         = _MASH(first_, C);
	pThis->last          = _MASH(last_, C);
	pThis->next          = _MASH(next_, C);
	pThis->prev          = _MASH(prev_, C);
	pThis->get           = _MASH(get_, C);
	
	pThis->insert        = _MASH(insert_, C);
	pThis->getMutable    = _MASH(getMutable_, C);
	pThis->take          = _MASH(take_, C);
	pThis->replace       = _MASH(replace_, C);
	pThis->delPair       = _MASH(delPair_, C);
	
	pThis->disownVal     = _MASH(disownVal, C);
	pThis->disownAllVals = _MASH(disownAllVals, C);
	pThis->del           = _MASH(delete_, C);
	
	if(sName != NULL)
		pThis->sName      = gse_strcopy(sName);
	else
		pThis->sName      = gse_strcopy("unnamed");
	
	pThis->bOwnsVals     = true;
	
	dict_init(&(pThis->dict), UINT_MAX, _MASH(compare_, C) );
	dict_set_allocator(&(pThis->dict), _MASH(allocNode_, C), 
			             _MASH(freeNode_, C), (void*)pThis );
}

C* _MASH(new_, C) (const char* sName){
	C* pThis = (C*) gse_calloc(1, sizeof(C));
	_MASH(init_, C)(pThis, sName);	
	return pThis;
}
