#ifndef _MASH_DEF
#define _MASH_DEF
#define _MASH_IN(x, y)  x ## y
#define _MASH(x, y) _MASH_IN(x, y)
#endif /* _MASH */

/* handle the cascade types */
#if ! defined(CASCADE_DEL) && ! defined (CASCADE_FREE) && ! defined (CASCADE_FUNC)
#error Cascading memory management method was not defined!
#endif

/* Flag value defs, can imagine alloc optimization flags in the future */
#define NODE_OWNS_ITEM 0x1

/*****************************************************************************/
int4 _MASH(C, _length)(const C* pThis)
{
	return pThis->nSz;
}

/*****************************************************************************/
/* Appending */

void _MASH(C, _append)(C* pThis, I* pItem)
{
	_MASH(C, Node) * pNode;
	
	pNode = ( _MASH(C, Node) * ) gse_calloc(1, sizeof( _MASH(C, Node) ));
	pNode->pItem = pItem;
	pNode->uFlags = NODE_OWNS_ITEM;
	
	if(pThis->pTail != NULL)
		pThis->pTail->pNext = pNode;  /* Previous tail points to me */
	
	/* Point to the previous tail, if any */
	pNode->pPrev = pThis->pTail;
	
	pThis->pTail = pNode;         /* I'm the new tail */
		
	if(pThis->nSz == 0)           /* If this 1st item then it's also the head */
		pThis->pHead = pNode;
	
	pThis->pCur = pNode;
	pThis->nSz += 1;	
}

void _MASH(C, _appendRef)(C* pThis, I* pItem)
{
	pThis->append(pThis, pItem);
	pThis->pCur->uFlags = pThis->pCur->uFlags & (~NODE_OWNS_ITEM);
}

/*****************************************************************************/
/* getting */

I* _MASH(C, _get)(C* pThis, int4 idx)
{
	int4 iIndex, i;
	
	if(pThis->nSz == 0){
		/* gse_throw_m(GSE_XCEPT, GSE_XARYINDEX, "Can not get elements from an "
		            "empty list"); */
		return NULL;
	}
	
	if( idx < 0) iIndex = pThis->nSz + idx;
	else         iIndex = idx;
	
	if((iIndex < 0)||(iIndex > pThis->nSz - 1))
		gse_throw_fm(GSE_XCEPT, GSE_XARYINDEX, "Real index %d (relative index %d) "
		             "is outside the list bounds, 0 to %d", iIndex, idx,
				       pThis->nSz - 1);
		
	/* If idx is closer to the back, search from that direction, else start
	   from the front */
	if(iIndex > pThis->nSz/2){
		i = pThis->nSz - 1;
		pThis->pCur = pThis->pTail;
		while(i > iIndex){
			pThis->pCur = pThis->pCur->pPrev;
			i--;
		}
	}
	else{
		i = 0;
		pThis->pCur = pThis->pHead;
		while(i< iIndex){
			pThis->pCur = pThis->pCur->pNext;
			i++;
		}
	}
	
	return pThis->pCur->pItem;
}

/*****************************************************************************/
/* Iterating */

I* _MASH(C, _next)(C* pThis)
{
	_MASH(C, Node) * pNode;
	
	if(pThis->nSz == 0) return NULL;
	
	if(pThis->pCur == NULL)
		gse_throw_m(GSE_XCEPT, GSE_XCANTITERATE, "Current list item is not defined");
	
	pNode = pThis->pCur->pNext;
	if(pNode != NULL){
		pThis->pCur = pNode;
		return pThis->pCur->pItem;
	}
	
	return NULL;
}

I* _MASH(C, _prev)(C* pThis)
{
	_MASH(C, Node) * pNode;
	
	if(pThis->nSz == 0) return NULL;
	
	if(pThis->pCur == NULL)
		gse_throw_m(GSE_XCEPT, GSE_XCANTITERATE, "Current list item is not defined");
	
	pNode = pThis->pCur->pPrev;
	if(pNode != NULL){
		pThis->pCur = pNode;
		return pThis->pCur->pItem;
	}
	return NULL;
}

/*****************************************************************************/
/* Inserting */

void _MASH(C, _insert)(C* pThis, I* pItem, int4 idx)
{
	_MASH(C, Node) * pNode;
	int4 iIndex;
	
	if(pThis->nSz == 0)
		gse_throw_m(GSE_XCEPT, GSE_XARYINDEX, "Use append to add elements to a size 0 list");
	
	if( idx < 0) iIndex = pThis->nSz - idx;
	else         iIndex = idx;
	if((iIndex < 0)||(iIndex > pThis->nSz - 1))
		gse_throw_fm(GSE_XCEPT, GSE_XARYINDEX, "Real index %d (relative index %d) "
		             "is outside the list bounds, 0 to %d", iIndex, idx, pThis->nSz);

	/* Treat an insert after the last position as an append */
	if(iIndex == pThis->nSz - 1){
		pThis->append(pThis, pItem);
		return;		
	}
	
	/* Make the node */
	pNode = ( _MASH(C, Node) * ) gse_calloc(1, sizeof( _MASH(C, Node) ));
	pNode->pItem = pItem;
	pNode->uFlags = NODE_OWNS_ITEM;
	
	/* Set the current item pointer */
	pThis->get(pThis, idx);
	
	/* hook in the new node */
	pThis->pCur->pNext->pPrev = pNode;  /* Next node's prev is me */
	pNode->pNext = pThis->pCur->pNext;  /* Set my next */
	
	pThis->pCur->pNext = pNode;         /* Cur node's next is me */
	pNode->pPrev = pThis->pCur;         /* set my prev */
	
	pThis->pCur = pNode;
	pThis->nSz += 1;
}

void _MASH(C, _insertRef)(C* pThis, I* pItem, int4 idx)
{
	pThis->insert(pThis, pItem, idx);
	pThis->pCur->uFlags = pThis->pCur->uFlags & (~NODE_OWNS_ITEM);
}

/*****************************************************************************/
/* Pop-ing, removing */

I* _MASH(C, _popHelper)(C* pThis, int4 idx, uint4* pFlags)
{
	I* pRet;
	
	/* Make the current item the one at idx */
	pRet = pThis->get(pThis, idx);
	
	/* Handle size = 1 differently */
	if(pThis->nSz == 1){
		pThis->pHead = NULL;
		pThis->pTail = NULL;
	}
	else{
	
		/* and the node before me must now point to the item after me */
		if(pThis->pCur->pPrev != NULL)
			pThis->pCur->pPrev->pNext = pThis->pCur->pNext;
		
		/* The node after me must now point to the item before me */	
		if(pThis->pCur->pNext != NULL)
			pThis->pCur->pNext->pPrev = pThis->pCur->pPrev;
			
		/* if I was the head, the item after must be the head */
		if(pThis->pHead == pThis->pCur)
			pThis->pHead = pThis->pCur->pNext;
	
		/* if I was the tail the item before me must be the tail */
		if(pThis->pTail == pThis->pCur)
			pThis->pTail = pThis->pCur->pPrev;
	}
	
	*pFlags = pThis->pCur->uFlags;
	
	pThis->nSz -= 1;
	free(pThis->pCur);
	pThis->pCur = NULL;
	return pRet;
}

I* _MASH(C, _pop)(C* pThis, int4 idx)
{
	uint4 uFlags;
	return pThis->popHelper(pThis, idx, &uFlags);
}

void _MASH(C, _remove)(C* pThis, int4 idx)
{
	I* pItem;
	uint4 uFlags;
	
	pItem = pThis->popHelper(pThis, idx, &uFlags);
	
	if(uFlags & NODE_OWNS_ITEM){
#ifdef CASCADE_DEL
		pItem->del(pItem);
#else
#ifdef CASCADE_FREE
		free(pItem);
#else
		CASCADE_FUNC(pItem);
#endif
#endif
	}
}

/*****************************************************************************/
/* Selecting a subset */

C* _MASH(C, _getWhere)(C* pThis,  bool (*selector)(void* pCtx, const I*), 
		                 void* pCtx)
{
	C* pNew = NULL;
	I* pItem = NULL;
	
	for(pItem = pThis->get(pThis, 0); pItem != NULL; pItem = pThis->next(pThis))
	{
		if( selector(pCtx, pItem) ){
			if(pNew == NULL)
				pNew = _MASH(new_, C) ();
			pNew->appendRef(pNew, pItem);
		}
	}
	return pNew;
}

/*****************************************************************************/
/* Removing a subset */

int4 _MASH(C, _rmWhere)(C* pThis, bool (*selector)(void* pCtx, const I*),
                        void* pCtx){
	_MASH(C, Node) * pNode;
	_MASH(C, Node) * pTmp;
	
	int4 nRm = 0;
	
	/* Start at the head */
	pNode = pThis->pHead;
			
	while(pNode != NULL){
		
		if( selector(pCtx, pNode->pItem) ){
			pTmp = pNode;
					
			/* set the next item now, I'm about to dissapear */
			pNode = pNode->pNext;
			
			/* If I was the head of the list, get a new head */
			if(pThis->pHead == pTmp) pThis->pHead = pTmp->pNext;
			
			/* If I was the tail of the list, get a new tail */
			if(pThis->pTail == pTmp) pThis->pTail = pTmp->pPrev;
			
			/* If I had a previous item, it's next now points to my next */
			if(pTmp->pPrev) pTmp->pPrev->pNext = pTmp->pNext;
			
			/* If I had a next item, it's prev now points to my prev */
			if(pTmp->pNext) pTmp->pNext->pPrev = pTmp->pPrev;
			
			if(pTmp->uFlags & NODE_OWNS_ITEM){
#ifdef CASCADE_DEL
				pTmp->pItem->del(pTmp->pItem);
#else
#ifdef CASCADE_FREE
				free(pTmp->pItem);
#else
				CASCADE_FUNC(pTmp->pItem);		
#endif
#endif
			}			
			free(pTmp);
			nRm++;
			pThis->nSz -= 1;
		}
		else{
			pNode = pNode->pNext;
		}
	}
	
	/* Set the current item to the head of the list, if it still exists */
	pThis->pCur = pThis->pHead;
	
	return nRm;
}


/*****************************************************************************/
/* inplace sorting, following notice is taken from the original source:
 *
 * This file is copyright 2001 Simon Tatham.
 * 
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT.  IN NO EVENT SHALL SIMON TATHAM BE LIABLE FOR
 * ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
 * CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

void _MASH(C, _sort)(C* pThis, int4(*compare)(const I* p1, const I* p2))
{
	_MASH(C, Node) *pList, *qList, *pNode;
	int nInnerSz, nMerges, nPSize, nQSize, i;
	
	if(pThis->nSz == 0) return;
	
	nInnerSz = 1;
	
	while(true){
		pList = pThis->pHead;
		pThis->pHead = NULL;
		pThis->pTail = NULL;
		
		nMerges = 0;            /* count number of merges we do in this pass */
		while(pList != NULL){
			nMerges++;           /* there exists a merge to be done */
			
			/* step 'nInnerSz' places along from pList */
			qList = pList;
			nPSize = 0;
			for(i = 0; i < nInnerSz; i++){
				nPSize++;
				qList = qList->pNext;
				if(qList == NULL) break;
			}
		
			/* if qList hasn't fallen off end, we have two lists to merge */
			nQSize = nInnerSz;
			
			/* Now we have two lists; merge them */
			while (nPSize > 0 || (nQSize > 0 && (qList != NULL))) {
				
				/* decide whether next element of merge comes from pList or qList */
				if (nPSize == 0) {

					/* pList is empty; pNode must come from qList. */
					pNode = qList;
					qList = qList->pNext;
					nQSize--;
					
				} else if (nQSize == 0 || !qList) {

					/* qList is empty; pNode must come from pList. */
					pNode = pList;
					pList = pList->pNext;
					nPSize--;
					
				} else if (compare(pList->pItem, qList->pItem) <= 0) {

					/* First element of pList is lower (or same);
					 * pNode must come from pList. */
					pNode = pList;
					pList = pList->pNext;
					nPSize--;
					
				} else {

					/* First element of qList is lower; pNode must come from qList. */
					pNode = qList;
					qList = qList->pNext;
					nQSize--;
				}

				/* add the next element to the merged list */
				if (pThis->pTail) {
					pThis->pTail->pNext = pNode;
				} else {
					pThis->pHead = pNode;
				}
				
				/* Maintain reverse pointers in a doubly linked list. */
				pNode->pPrev = pThis->pTail;
	
				pThis->pTail = pNode;
			}
		
			/* now pList has stepped 'nInnerSz' places along, and qList has too */
			pList = qList;
		}
		
		pThis->pTail->pNext = NULL;
		
		/* If we have done only one merge, we're finished. */
		if(nMerges <= 1)       /* allow for nMerges==0, the empty list case */
			break;
		
		/* Otherwise repeat, merging lists twice the size */
		nInnerSz *= 2;
	}
	
	pThis->pCur = pThis->pHead;
}

/*****************************************************************************/
/* Destructor */

void _MASH(delete_, C) (void* vpThis)
{
	C* pThis = (C*) vpThis;
	
	_MASH(C, Node) * pNode;
	_MASH(C, Node) * pTmp;
	
	pNode = pThis->pHead;
	
	while(pNode != NULL){
	
		if(pNode->uFlags & NODE_OWNS_ITEM){
#ifdef CASCADE_DEL
			pNode->pItem->del(pNode->pItem);
#else
#ifdef CASCADE_FREE
			free(pNode->pItem);
#else
			CASCADE_FUNC(pNode->pItem);
#endif
#endif
		}
		
		pTmp = pNode;
		pNode = pNode->pNext;
		free(pTmp);
	}

	free(vpThis);
}

/*****************************************************************************/
/* Constructor */

C* _MASH(new_, C) (void)
{
	C* pThis = NULL;
	
	pThis = (C*) gse_calloc(1, sizeof(C));
	
	pThis->length    = _MASH(C, _length);
	pThis->append    = _MASH(C, _append);
	pThis->insert    = _MASH(C, _insert);
	pThis->appendRef = _MASH(C, _appendRef);
	pThis->insertRef = _MASH(C, _insertRef);
	pThis->get       = _MASH(C, _get);
	pThis->getWhere  = _MASH(C, _getWhere);
	pThis->pop       = _MASH(C, _pop);
	pThis->remove    = _MASH(C, _remove);
	pThis->rmWhere   = _MASH(C, _rmWhere);
	pThis->popHelper = _MASH(C, _popHelper);
	pThis->next      = _MASH(C, _next);
	pThis->prev      = _MASH(C, _prev);
	pThis->sort      = _MASH(C, _sort);
	pThis->del       = _MASH(delete_, C);
	
	return pThis;
}

