#ifndef _MASH_DEF
#define _MASH_DEF
#define _MASH_IN(x, y) x ## y
#define _MASH(x, y) _MASH_IN(x, y)
#endif /* _MASH_DEF */

typedef struct _MASH(S, _node_t){

	struct _MASH(S, _node_t)* pPrev;
	struct _MASH(S, _node_t)* pNext;
	uint4 uFlags;
	I* pItem;
} _MASH(C, Node);

/** Generic typed object doubly-linked list structure.  
 *
 * Convienence functions have been added to use this in a manner similar
 * to an array, but don't be fooled,  a list is much slower than an array
 * when preforming random access.  To speed up iteration, this list object
 * mantians the concept of a current item, i.e. the last one retrieved using
 * @a get.
 *
 * Memory management.  By default the list is assumed to own all items
 * it contains.  Use the @a disown function to change the ownership state
 * of a single item.  Use appendRef and insertRef functions to add items in
 * the disowned state.
 *
 * This Class was generated from the @ref page_listTplt.
 * @n @n
 */
typedef struct S{

	/** Get the number of items in this list */
	int4 (*length)(const struct S* pThis);

	/** Add an item to the end of the list.
	 * The appended item becomes the current item.
	 */
	void (*append)(struct S* pThis, I* pItem);
	
	/** Add an item to the end of the list and set it in the disowned state. 
	 * The appended item becomes the current item.
	 */
	void (*appendRef)(struct S* pThis, I* pItem);
	
	/** Add an item after a given index, which becomes the current item. */
	void (*insert)(struct S* pThis, I* pItem, int4 idx);
	
	/** Add an item after a given index in the disowned state.
	 * The inserted item becomes the current item.
	 */
	void (*insertRef)(struct S* pThis, I* pItem, int4 idx);
		
	/** get an item at a particular index, O(N/2) operation,
	 * Here idx can be negative.  If there is no node at index
	 * idx, NULL is returned. */
	I* (*get)(struct S* pThis, int4 idx);
	
	
	/** Get items based on a selector.
	 * The new list does not own any of the selected items.
	 *
	 * @param pThis the list object contianing the data to be seleceted.
	 * @param selector A call back that returns @a true if an item matched
	 *        the selection criteria.
	 * @param pCtx a context pointer to supply to the @a selector.
	 *
	 * @returns A new list object allocated on the heap, or NULL if none
	 *          of the items matched the selection criteria.
	 */
	struct S* (*getWhere)(struct S* pThis, 
			     bool (*selector)(void* pCtx, const I*), void* pCtx);
	
	/** Remove and return an item from the list, O(N/2) max operation.
	 * Negative numbers can be used in the python style to state counting
	 * from the end of the list.  So -1 is the last item of the list.
	 */
	I* (*pop)(struct S* pThis, int4 idx);
	
	/** Remove and destroy an item in the list.
	 *
	 * If the item is owned by this list it is deleted.
	 */
	void (*remove)(struct S* pThis, int4 idx);
	
	/** Remove and destroy items matching some selection rule.
	 *
	 * @param pThis the list object contianing the data to be seleceted.
	 *
	 * @param selector A callback that returns @a true if the item is to
	 *        be removed.
	 *
	 * @param pCtx a context pointer to supply to the @a selector.
	 *
	 * @returns The number of items deleted.
	 */
	int4 (*rmWhere)(struct S* pThis, 
			          bool (*selector)(void* pCtx, const I*), void* pCtx);
	
	/** Get the next item from the list, this works in O(1) time.
	 *
	 * To setup for iteration over the whole list, call get with
	 * index 0
	 */
	I* (*next)(struct S* pThis);
	
	/** Get the previous item from the list, this works in O(1) time.
	 *
	 * To setup for a reverse iteration, call get with index -1.
	 */
	I* (*prev)(struct S* pThis);
	
	/** An inplace sort of the items in the list.  
	 *
	 * This uses a merge sort algorithm, from Simon Tatham,
	 *  O(N log(N)) 
	 */
	void (*sort)(struct S* pThis, int4 (*compare)(const I* p1, const I* p2));
	
	/** delete the list object */
	void (*del)(void* vpThis);
	
	I* (*popHelper)(struct S* pThis, int4 idx, uint4* pFlags);
	
	_MASH(C, Node) * pHead;
	_MASH(C, Node) * pTail;
	
	_MASH(C, Node) * pCur;
	
	int4 nSz;
	
} C;

/** Creates a new dynamic array structure on the heap. */
C* _MASH(new_, C) ();
