#ifndef _MASH_DEF
#define _MASH_DEF
#define _MASH_IN(x, y)  x ## y
#define _MASH(x, y) _MASH_IN(x, y)
#endif /* _MASH */


/** Generic typed pointer array structure.
 * This Class was generated from the @ref page_ptrAryTplt. @n
 */
typedef struct S{
	/** number of items pointed to, automatically updated by #add and #rm_ptr */
	int num;
	
	/** pointer to an array of pointers to items of type T, automatically
	    maintained by #add #rm_ptr and #del */
	T** pAry;
	
	/** Add a pointer to a thing of type T to the array. 
	 *
	 * @throws (CLIB_XCEPT, ENOMEM)
	 */
	int (*add)(struct S* pThis, T* pItem);
	
	/** remove a pointer to a thing of type T from the array.
	 * @throws (CLIB_XCEPT, ENOMEM)
	 */
	int (*rm_ptr)(struct S* pThis, T* pItem);
	
	/** Sort the array in place.
	 * 
	 * @note The compare function is compatible with qsort, and thus it will 
	 * recieve pointers to array items.  However the array items are themselves
	 * pointers.  Here is an example of the casting needed to get @c T* pointers
	 * inside @a compare:
	 * @code
	 * int compare_T(const void** ppFirst, const void** ppSecond)
	 * {
	 *    const T* pFirst;
	 *    const T* pSecond;
	 * 
	 *    pFirst  = *((T**)ppFirst);
	 *    pSecond = *((T**)ppSecond);
	 *
	 *    // Continue with compare ... 
	 * 
	 * @endcode 
	 *
	 * @param pThis the dynamic array structure.
	 *
	 * @param compare_func  Fuction should return > 0 if the first item 
	 *        is larger than the second, < 0 if the first item is less
	 *        than the second and 0 if they are equal.
	 */
	void (*sort)(struct S* pThis,
	             int compare(const void* ppFirst, const void* ppSecond));
	
	/** free the pointer array, maybe free items pointed to.
	
	\b NOTE: if the statement \#define CASCADE_DEL or \#define CASCADE_FREE
	exists before \#include	ptrAryTplt.c then calling \a del will result in
	a cascading pT->del calls or cascading free(pT) calls on all items.
	See documentation for ptrAryTplt.c */
	void (*del)(void* vpThis);
} C;

/** Creates a new dynamic pointer array structure on the heap.
 * @param uLen the initial length of the array, if > 0 will be initialized
 *        with all zeros.
 */
C* _MASH(new_, C) (size_t uLen);

/** Initilizing a stack object, see new_* version for more details */
void _MASH(init_, C)(C* pThis, size_t uLen);

