#ifndef _MASH_DEF
#define _MASH_DEF
#define _MASH_IN(x, y)  x ## y
#define _MASH(x, y) _MASH_IN(x, y)
#endif /* _MASH_DEF */


/* Handle the de-alloc type selection */
#ifndef FREE_KEYS
#ifndef DEL_KEYS
#error Dictionary Key memory management method not defined
#endif
#endif

#ifndef FREE_VALS
#ifndef DEL_VALS
#error Dictionary Value memory management method not defined
#endif
#endif

/*****************************************************************************/
/* Error handling disposition */

void _MASH(C, _setRetNull) (C* pThis) 
{
	pThis->bThrows = false;
}

void _MASH(C, _setThrow) (C* pThis)
{
	pThis->bThrows = true;
}

/*****************************************************************************/
/* Non Modifing usage functions */

const char* _MASH(C, _name) (const C* pThis)
{
	return (const char*)pThis->sName;
}

uint4 _MASH(C, _length) (const C* pThis)
{
	return (uint4) dict_count(&(pThis->dict));
}

bool _MASH(C, _isEmpty) (const C* pThis)
{
	return dict_isempty( &(pThis->dict) ) ? true : false;
}

bool _MASH(C, _hasKey) (const C* pThis, const K* pKey)
{
	return dict_lookup((dict_t*)(&(pThis->dict)), (K*)pKey) ? true : false;
}

const K* _MASH(C, _first) (const C* pThis)
{
	dnode_t* pNode;
	if( (pNode = dict_first((dict_t*)(&(pThis->dict)) ) ) != NULL)
		return (const K*) dnode_getkey(pNode);
	else
		return NULL;
}

const K* _MASH(C, _last) (const C* pThis)
{
	dnode_t* pNode;
	if( (pNode = dict_last((dict_t*)(&(pThis->dict)) ) ) != NULL)
		return (const K*) dnode_getkey(pNode);
	else
		return NULL;
}

const K* _MASH(C, _next) (const C* pThis, const K* pKey)
{
	dnode_t* pCur;
	dnode_t* pNext;
	
	if(pKey == NULL)
		gse_throw_m(GSE_XCEPT, GSE_XBADARG, "pKey is NULL.");
	
	if( (pCur = dict_lookup( (dict_t*)(&(pThis->dict)) , pKey)) == NULL){
		if(pThis->bThrows)
			gse_throw_fm(GSE_XCEPT, GSE_XNOKEY, "Key %zX is not in the %s "
			             "dictionary", pKey, pThis->sName);
		else
			return NULL;
	}

	
	if( (pNext = dict_next( (dict_t*)(&(pThis->dict)), pCur)) != NULL)
		return (const K*) dnode_getkey(pNext);
	else
		return NULL;
}

const K* _MASH(C, _prev) (const C* pThis, const K* pKey)
{
	dnode_t* pCur;
	dnode_t* pPrev;
	
	if(pKey == NULL)
		gse_throw_m(GSE_XCEPT, GSE_XBADARG, "pKey is NULL.");
	
	if( (pCur = dict_lookup( (dict_t*)(&(pThis->dict)), pKey)) == NULL){
		if(pThis->bThrows)
			gse_throw_fm(GSE_XCEPT, GSE_XNOKEY, "Key %zX is not in the %s dictionary",
					       pKey, pThis->sName);
		else
			return NULL;
	}
	
	if( (pPrev = dict_prev( (dict_t*)(&(pThis->dict)), pCur)) != NULL)
		return (const K*) dnode_getkey(pPrev);
	else
		return NULL;
}

const V* _MASH(C, _get) (const C* pThis, const K* pKey)
{
	dnode_t* pNode;
	
	if(pKey == NULL)
		gse_throw_m(GSE_XCEPT, GSE_XBADARG, "pKey is NULL.");
	
	if( (pNode = dict_lookup( (dict_t*)(&(pThis->dict)), pKey)) == NULL){
		if(pThis->bThrows)
			gse_throw_fm(GSE_XCEPT, GSE_XNOKEY, "Key %zX is not in the %s dictionary",
					       pKey, pThis->sName);
		else
			return NULL;
	}
	
	return (const V*) dnode_get(pNode);
}

/*****************************************************************************/
/* Modifing usage functions */

void _MASH(C, _setName) (C* pThis, const char* sName)
{
	free(pThis->sName);
	pThis->sName = CS(sName);
}

const K* _MASH(C, _insert) (C* pThis, K* pKey, V* pVal)
{
	dnode_t* pNode;
	
	if(pKey == NULL)
		gse_throw_m(GSE_XCEPT, GSE_XBADARG, "pKey is NULL.");
	
	if( (pNode = dict_lookup(&(pThis->dict), pKey)) != NULL)
		gse_throw_fm(GSE_XCEPT, GSE_XDUPKEY, "Key %zX is already in the %s "
		             "dictionary", pKey, pThis->sName);
	
	if(dict_alloc_insert(&(pThis->dict), pKey, pVal) == 0)
		gse_throw(GSE_XCEPT, GSE_XINCONCEIVABLE);
	
	return (const K*)pKey;
}

V* _MASH(C, _getMutable) (C* pThis, const K* pKey)
{
	dnode_t* pNode;
	
	if(pKey == NULL)
		gse_throw_m(GSE_XCEPT, GSE_XBADARG, "pKey is NULL.");
	
	if( (pNode = dict_lookup(&(pThis->dict), pKey)) == NULL){
		if(pThis->bThrows)
			gse_throw_fm(GSE_XCEPT, GSE_XNOKEY, "Key %zX is not in the %s dictionary",
					       pKey, pThis->sName);
		else
			return NULL;
	}
	
	return (V*) dnode_get(pNode);
}
		
V* _MASH(C, _take) (C* pThis, const K* pKey)
{
	dnode_t* pNode;
	gse_flagnode_t* pGNode;
	V* pVal;
	
	if(pKey == NULL)
		gse_throw_m(GSE_XCEPT, GSE_XBADARG, "pKey is NULL.");
	
	/* Get the node */
	if( (pNode = dict_lookup(&(pThis->dict), pKey)) == NULL)
		gse_throw_fm(GSE_XCEPT, GSE_XNOKEY, "Key %zX is not in the %s dictionary",
				       pKey, pThis->sName);
	
	pGNode = (gse_flagnode_t*)pNode;
	
	pGNode->noDelVal = 1;  /* Mark as no delete value */
	
	pVal = (V*)dnode_get(pNode);
	
	dict_delete_free(&(pThis->dict), pNode);  /* Key is gone at this point */
	
	return pVal;
}

V* _MASH(C, _replace) (C* pThis, K* pKey, V* pNewVal)
{
	dnode_t* pNode;
	V* pRet;
	
	if(pKey == NULL)
		gse_throw_m(GSE_XCEPT, GSE_XBADARG, "pKey is NULL.");
	
	/* Get the node */
	if( (pNode = dict_lookup(&(pThis->dict), pKey)) == NULL)
		gse_throw_fm(GSE_XCEPT, GSE_XNOKEY, "Key %zX is not in the %s dictionary",
				       pKey, pThis->sName);
		
	pRet = (V*)dnode_get(pNode);
	
	/* WARNING: Just because two keys compare the same does NOT mean that
   	         they point to the same memory location.  Since the old memory
					location may become invalid, update it to the supplied one. */
	
	if( (! ((gse_flagnode_t*)pNode)->noDelKey ) && pThis->bOwnsKeys){
#ifdef DEL_KEYS
		pKey->del(pNode->dict_key);
#else
		free((void*)(pNode->dict_key));
#endif
	}
	pNode->dict_key = pKey;

	/* Don't delete the value, just return it */	
	pNode->dict_data = pNewVal;
	
	return pRet;
}

void _MASH(C, _delPair) (C* pThis, const K* pKey)
{
	dnode_t* pNode;
	
	if(pKey == NULL)
		gse_throw_m(GSE_XCEPT, GSE_XBADARG, "pKey is NULL.");
	
	/* Get the node */
	if( (pNode = dict_lookup(&(pThis->dict), pKey)) == NULL)
		gse_throw_fm(GSE_XCEPT, GSE_XNOKEY, "Key %zX is not in the %s dictionary",
				       pKey, pThis->sName);
	
	dict_delete_free(&(pThis->dict), pNode);  /* Key is gone at this point */
}


/*****************************************************************************/
/* Delete Where implementation */
		 
struct _MASH(C, _del_ctx){
	bool (*keep)(void* pCtx, const K* pKey, const V* pVal);
	int4 nRemoved;
	void* pClientCtx;
};

		 
void _MASH(C, _del_proc)(dict_t* pDict, dnode_t* pNode, void* pCtx)
{
	struct  _MASH(C, _del_ctx) * pDelCtx = (struct  _MASH(C, _del_ctx)* )pCtx;
	const K* pKey;
	const V* pVal;
	
	pKey = (K*) dnode_getkey(pNode);
	pVal = (V*) dnode_get(pNode);
	
	if( (pDelCtx->keep)(pDelCtx->pClientCtx, pKey, pVal) == false){
		
		dict_delete_free(pDict, pNode);
		pDelCtx->nRemoved += 1;
	}	
}
	
int4 _MASH(C, _delWhere) (C* pThis, bool (*keep)(void* pCtx, const K* pKey, const V* pVal),
	                 void* pCtx)
{	
	struct _MASH(C, _del_ctx) ctx;
	
	ctx.keep = keep;
	ctx.nRemoved = 0;
	ctx.pClientCtx = pCtx;

	if(keep == NULL)
		gse_throw_m(GSE_XCEPT, GSE_XBADARG, "'keep' function pointer is null");
	
	dict_process(&(pThis->dict), &ctx, _MASH(C, _del_proc) );
	
	return ctx.nRemoved;
}

/*****************************************************************************/
/* Own/Disown Keys and values */

K* _MASH(C, _disownKey) (C* pThis, const K* pKey)
{
	gse_flagnode_t* pNode;
	
	if( (pNode = (gse_flagnode_t*) dict_lookup(&(pThis->dict), pKey)) == NULL)
		gse_throw_fm(GSE_XCEPT, GSE_XNOKEY, "Key %zX is not in the %s dictionary",
				       pKey, pThis->sName);
	
	pNode->noDelKey = 1;
	return (K*) dnode_getkey((dnode_t*)pNode);
}

void _MASH(C, _disownAllKeys) (C* pThis)
{
	pThis->bOwnsKeys = false;
}

V* _MASH(C, _disownVal) (C* pThis, const K* pKey)
{
	gse_flagnode_t* pNode;
	
	if( (pNode = (gse_flagnode_t*) dict_lookup(&(pThis->dict), pKey)) == NULL)
		gse_throw_fm(GSE_XCEPT, GSE_XNOKEY, "Key %zX is not in the %s dictionary",
				       pKey, pThis->sName);
	
	pNode->noDelVal = 1;
	return (V*) dnode_get((dnode_t*)pNode);
}

void _MASH(C, _disownAllVals) (C* pThis)
{
	pThis->bOwnsVals = false;
}


/*****************************************************************************/
/* Create/Destroy Nodes */

dnode_t* _MASH(C, _allocNode) (void* vpThis)
{
	dnode_t* pNew;
	pNew = (dnode_t*) gse_calloc(1, sizeof(gse_flagnode_t));
	return pNew;
}

void _MASH(C, _freeNode) (dnode_t* pNode, void* vpThis)
{
	K* pKey  = NULL;
	V* pVal  = NULL;
	
	C* pThis = (C*)vpThis;
	gse_flagnode_t* pN = (gse_flagnode_t*)pNode;
	
	bool bKeepKey = pN->noDelKey || (! pThis->bOwnsKeys);
	bool bKeepVal = pN->noDelVal || (! pThis->bOwnsVals);
			
	if(!bKeepKey){
		pKey = (K*)dnode_getkey(pNode);
#ifdef DEL_KEYS
		pKey->del(pKey);
#else
		free(pKey);
#endif
	}
	
	if(!bKeepVal){
		pVal = (V*)dnode_get(pNode);
#ifdef DEL_VALS
		if(pVal != NULL)
			pVal->del(pVal);
#else
		free(pVal);
#endif
	}
	
	free(pN);
} 

/*****************************************************************************/
/* Create/Destroy Dictionaries */

void _MASH(finish_, C) (C* pThis)
{
	free(pThis->sName);
	dict_free_nodes(&(pThis->dict));
}

void _MASH(delete_, C) (void* vpThis)
{
	C* pThis = (C*)vpThis;
	
	_MASH(finish_, C) (pThis);
	
	free(vpThis);
}

int _MASH(C, _compare) (const void* k1, const void* k2)
{
	const K *tk1 = (const K *)k1;
	const K *tk2 = (const K *)k2;
	
#ifdef KEY_COMPARE
	return KEY_COMPARE(tk1, tk2);
#else
	if(*tk1 > *tk2) return 1;
	if(*tk1 < *tk2) return -1;
	
	return 0;
#endif
}


void _MASH(init_, C)(C* pThis, const char* sName)
{
	pThis->setRetNull    = _MASH(C, _setRetNull);
	pThis->setThrow      = _MASH(C, _setThrow);
	
	pThis->name          = _MASH(C, _name);
	pThis->hasKey        = _MASH(C, _hasKey);
	pThis->length        = _MASH(C, _length);
	pThis->first         = _MASH(C, _first);
	pThis->last          = _MASH(C, _last);
	pThis->next          = _MASH(C, _next);
	pThis->prev          = _MASH(C, _prev);
	pThis->get           = _MASH(C, _get);
	
	pThis->setName       = _MASH(C, _setName);
	pThis->insert        = _MASH(C, _insert);
	pThis->getMutable    = _MASH(C, _getMutable);
	pThis->take          = _MASH(C, _take);
	pThis->replace       = _MASH(C, _replace);
	pThis->delPair       = _MASH(C, _delPair);
	pThis->delWhere      = _MASH(C, _delWhere);
	
	pThis->disownKey     = _MASH(C, _disownKey);
	pThis->disownAllKeys = _MASH(C, _disownAllKeys);
	pThis->disownVal     = _MASH(C, _disownVal);
	pThis->disownAllVals = _MASH(C, _disownAllVals);
	pThis->del           = _MASH(delete_, C);
	
	if(sName != NULL)
		pThis->sName      = gse_strcopy(sName);
	else
		pThis->sName      = gse_strcopy("unnamed");
	 
	pThis->bOwnsKeys     = true;
	pThis->bOwnsVals     = true;
	pThis->bThrows       = false;
	
	dict_init(&(pThis->dict), UINT_MAX, _MASH(C, _compare) );
	dict_set_allocator(&(pThis->dict), _MASH(C, _allocNode), 
			             _MASH(C, _freeNode), (void*)pThis );
}

C* _MASH(new_, C) (const char* sName){
	C* pThis = (C*) gse_calloc(1, sizeof(C));
	_MASH(init_, C)(pThis, sName);	
	return pThis;
}
