#ifndef _MASH_DEF
#define _MASH_DEF
#define _MASH_IN(x, y)  x ## y
#define _MASH(x, y) _MASH_IN(x, y)
#endif /* _MASH_DEF */


#ifndef GSE_FLAG_NODE
#define GSE_FLAG_NODE

/* Behind the scenes node type, allows for flaging 'No delete of a key/val',
 * This is not per/type, only needed once.  Note, the flags are defined
 * such that a calloc (wich 0's all memory, will set them to the default
 * values. 
 */
typedef struct gse_flagnode_t{
	dnode_t dnode;
	byte noDelKey;
	byte noDelVal;
}gse_flagnode_t;

#endif /* GSE_FLAG_NODE */

/* Put this inside a .page file describing using the ptrDictTplt
 *
 *  - S = Struct Definition
 *  - C = Class Type
 *  - K = Key Type
 *  - D = Data Type
 *
 * The declaration include, "ptrDictTplt.h" requires that dict.h
 * and GseTypes.h have already been included.
 *
 * The instatiation include, "ptrDictTplt.c" requires that GseWrap.h
 * and GseExcept.h have already been included.
 */

/** Generic typed 1 to 1 dictionary structure.
 *
 * @todo Add rmWhere support.
 * 
 * This dictionary assumes ownership of all key and data pointers, and 
 * frees or deletes them when the @a del member is called.  To prevent
 * a specific key or data item from being deleted use @a disown_key, 
 * or @a disown_data.  To prevent an instance of this dictionary from
 * managing the memory for any current and all future keys or data
 * use @a disown_all_keys and @a disown_all_data respectively.
 *
 * Generated from the Pointer Dictionary Template in "ptrDictTplt.h"
 * @n @n
 */
typedef struct S{

	/** Get the dictionary name, as defined during construction.
	 * @param pThis A pointer to the dictionary instance.
	 * @return The user defined name of the dictionary, or a pointer to
	 *         the string "unnamed".  Never returns NULL.
	 */
	const char* (*name)(const struct S* pThis);
	
	void (*setName)(struct S* pThis, const char* sName);
	
	/** Is the current dictionary empty */
	bool (*isEmpty)(const struct S* pThis);
	
	/** Get the number of entries in the dictionary */
	uint4 (*length)(const struct S* pThis);
	
	/** Does the dictionary contain a specific key */
	bool (*hasKey)(const struct S* pThis, const K* pKey);
	
	/** Get the first key in a dictionary
	 *
	 * @param pThis A pointer to the dictionary instance.
	 * @return The first key in the dictionary, which may also be the
	 *         last key in the dictionary if @a size is 1.  For a zero
	 *         length dictionary, NULL in returned.
	 */
	const K* (*first)(const struct S* pThis);
	
	/** Get the last key in a dictionary
	 *
	 * @param pThis A pointer to the dictionary instance.
	 * @return The last key in the dictionary, which may also be the
	 *         first key in the dictionary if @a size is 1.  For a zero
	 *         length dictionary, NULL in returned.
	 */
	const K* (*last)(const struct S* pThis);
	
	/** Get the next key
	 *
	 * @param pThis A pointer to the dictionary instance.
	 * @param pKey the reference key, the key after this one is returned.
	 *
	 * @return Returns NULL if @a pKey references the last value in the 
    *         this dictionary.
	 */
	const K* (*next)(const struct S* pThis, const K* pKey);
	
	/** Get the previous key.
	 * @returns NULL if @a pKey references the first value in this dictionary.
	 */
	const K* (*prev)(const struct S* pThis, const K* pKey);
	
	/** Get a constant pointer to a value.
	 * @param pThis The dictionary instance.
	 * @param pKey The key for the item to retrieve.
	 * @return a constant pointer to the value, or NULL if the value
	 *         does not exist.
	 */
	const V* (*get)(const struct S* pThis, const K* pKey);
	
	/** Sets this dictionary to return NULL when requesting non-existing items.
	 * Returning NULL is the default, this function call is not needed unless
	 * setThrow() has been called.
	 *
	 * This affects the calls to:
	 *  - next
	 *  - prev
	 *  - get
	 *  - getMutable
	 */
	void (*setRetNull)(struct S* pThis);
	
	
	/** Sets this dictionary to throw an exception when requesting non-existing 
	 * items.  
	 *
	 * The default is to return NULL for undefined keys.
	 *
	 * This affects the calls to:
	 *  - next
	 *  - prev
	 *  - get
	 *  - getMutable
	 */
	void (*setThrow)(struct S* pThis);
	
	/** Add a key value pair to the dictionary 
	 *
	 * @param pThis A pointer to the dictionary instance.
	 * @param pKey A pointer to the key.  Unless @a disownKey or @a disownAllKeys
	 *        is called, the memory associated with the key will be deleted when
	 *        the dictionary is deleted.
	 * @param pVal A pointer to the key.  Unless @a disownVal or @a disownAllVals
	 *        is called, the memory associated with the key will be deleted when
	 *        the dictionary is deleted.
	 *
	 * @returns the key pointer cast as constant.
	 *
	 * @throws (GSE_XCEPT,GSE_XDUPKEY) If a key is already in the dictionary.
	 * @throws (GSE_XCEPT,GSE_XBADARG) If @a pKef is null
	 */
	const K* (*insert)(struct S* pThis, K* pKey, V* pVal);
	
	/** Get a mutable pointer to a value.
	 *
	 * @warning Careful how you use this pointer.  It is ment for
	 *          mutating dictionary values but @b NOT deleting them.
	 *          Deleting the return value will cause a "double free"
	 *          error in Clib when the @a del member is invoke.
	 *
	 * @param pThis The dictionary instance.
	 * @param pKey The lookup key for the new item.
	 * @returns a pointer to the value, or NULL if the value
	 *         does not exist.
	 *
	 * @throws (GSE_XCEPT,GSE_XNOKEY) if @a pKey is not in the dictionary.
	 */
	V* (*getMutable)(struct S* pThis, const K* pKey);
	
	/** Take a value out of the dictionary, and delete it's key. 
	 *
	 * @note Taking an item causes it's key to be deleted, unless
	 *       @a disownKey or @a disownAllKeys has been called.
	 * 
	 * @param pThis The dictionary instance.
	 * @param pKey The key for the item to take.
	 *
	 * @returns The requested value.
	 *
	 * @throws (GSE_XCEPT,GSE_XNOKEY) if @a pKey is not in the dictionary.
	 */
	V* (*take)(struct S* pThis, const K* pKey);
		
	/** Replace one value with another, keeping the key.
	 *
	 * The new value takes on the ownership disposition of the old value.
	 * Thus if the old value was owned by the dictionary then the new value
	 * is as well, and vice versa.
	 
	 * @warning The caller is expected to take ownership of the memory
	 *          occupied by the returned value.  The old value is 
	 *          @b NOT deleted by this call.
	 *
	 * @param pThis The dictionary instance.
	 * @param pKey the key for the location of the new value.
	 * @param pNewVal The new value, this may be null.
	 *
	 * @returns a Pointer to the old value.  The caller is now reponsible
	 *          for the old value.
	 *
	 * @throws (GSE_XCEPT,GSE_XNOKEY) if @a pKey is not in the dictionary.
	 */
	V* (*replace)(struct S* pThis, K* pKey, V* pNewVal);
	
	
	/** Delete a key,value pair from the dictionary.
	 *
	 * @note If the key and/or value have been disowned, thier memory is
	 *       not freed.
	 *
	 * @param pThis The dictionary instance.
	 * @param pKey The key for the pair to delete.
	 * 
	 * @throws (GSE_XCEPT,GSE_XNOKEY) if @a pKey is not in the dictionary.
	 */
	void (*delPair)(struct S* pThis, const K* pKey);
	
	/** Delete multiple key,value pairs form the dictionary, based on a selection
	 * rule.
	 *
	 * @param pThis The dictionary instance.
	 * @param selector A function that returns false if the item is to be
	 *        deleted, true if it item is to be kept.
	 * @param A context pointer passed to the selector function.
	 *
	 * @returns The number of entries removed from the dictionary.
	 */
	int4 (*delWhere)(struct S* pThis, 
	                 bool (*keep)(void* pCtx, const K* pKey, const V* pVal),
	                 void* pCtx);
	
	/** Do not delete a specific key when the destructor is invoked
	 *
	 * @param pThis the dictionary instance.
	 * @param pKey the key that should not be deleted. 
	 * @returns A pointer to the key that has been disowned.
	 *
	 * @throws (GSE_XCEPT,GSE_XNOKEY) if pKey is not in the dictionary.
	 */
	K* (*disownKey)(struct S* pThis, const K* pKey);
	
	/** Do not delete any keys when the destructor is invoked.
	 * This setting also affects key deletion when @a remove or @a take
	 * is invoked.
	 */
	void (*disownAllKeys)(struct S* pThis);
	
	/** Do not delete a specific value item when the destructor is invoked 
	 *
	 * @param pThis the dictionary instance.
	 * @param pKey the key that should not be deleted. 
	 * @returns A pointer to the val that has been disowned.
	 *
	 * @throws (GSE_XCEPT,GSE_XNOKEY) if pKey is not in the dictionary.	 
	 */
	V* (*disownVal)(struct S* pThis, const K* pKey);
	
	/** Do not delete any value items when the destructor is invoked */
	void (*disownAllVals)(struct S* pThis);
	
	/** Destructor.
	 * Recusively deletes all keys and values unless they have been 
	 * disowned.
	 */
   void (*del)(void* vpThis);
	
	char* sName;
	
	bool bOwnsKeys;
	
	bool bOwnsVals;
	
	bool bThrows;
	
	dict_t dict;

} C;

/** Creates a new dynamic typed dictionary structure on the heap. */
C* _MASH(new_, C) (const char* sName);

/** Initilizing a stack dictionary object */
void _MASH(init_, C)(C* pThis, const char* sName);

/** Finishes a stack dictionary object */
void _MASH(finish_, C)(C* pThis);

