import sys
import struct
import time


##############################################################################
def readUntil(fIn, nSz, bGrowing):

	sIn = ""
	while len(sIn) < nSz:

		sIn += fIn.read(nSz - len(sIn) )
	
		if len(sIn) == nSz:
			break
		else:
			#Sleep for a bit, to wait for more data.
			if not bGrowing:
				return None
			time.sleep(0.2)
	
	return sIn

##############################################################################

class SimpleGsePkt(object):
	"""Generic GSE packet objects, Implements the generic rules, is not
	   Juno specific
	"""

	def __init__(self, fIn, bGrowing):
	
		self.offset = fIn.tell()  # Offset in the file object
		
		self.sPkt = ""        # The entire packet as a string
		
		# Prefix broken out fields
		self.nCRC = None
		self.uHTSz = None
		self.uTotalSz = None
		self.nCraft = None
		self.uVer = None
		self.uCont = None
		
		# Header blocks
		self.lBlks = []
		
		# Header block offsets from start of packet
		self.lBlkOff = []
		
		# Data block
		self.sData = ""
		
		# Data block offset
		self.nDataOff = 0
		
		# length repeat at the end
		self.nTrailTSz = None
		
		
		#Read the 1st 16 bytes
		sPre = readUntil(fIn, 16, bGrowing)
		if sPre == None:
			raise StopIteration
		
		# Verify the Sync + ver bytes
		if sPre[:4] != "\xFA\x6C\x27\x41":
			raise RuntimeError("Sync bytes don't match up and this program "+\
			                   "doesn't impliment searching.")
		
		# Verify the version number
		if sPre[14] != '\x04':
			raise RuntimeError("This program expects version '0x04' Waves Ground "+\
			                   "Support Headers")
									 
		# Parse the prefix
		(self.nCRC, self.uHTSz, self.uTotalSz, self.nCraft, self.uVer, self.uCont 
		 ) = struct.unpack(">HHIhBB", sPre[4:16])
		
		
		#Read the rest of the packet
		sTmp = readUntil(fIn, self.uTotalSz - 16, bGrowing)
		if sTmp == None:
			raise StopIteration
			
		self.sPkt = sPre + sTmp
	
		#Get the header blocks
		self.nDataOff = self.uHTSz - 4
		i = 16
		while i < self.nDataOff:
			if self.sPkt[i] == '\x00':
				break
		
			nBlkType = ord(self.sPkt[i])
			nBlkLen = ord(self.sPkt[i+1]) + 1
		
			self.lBlks.append( self.sPkt[i:nBlkLen + i] )
			self.lBlkOff.append(i)
		
			i += nBlkLen
			
		# Get the data block
		self.sData = self.sPkt[self.nDataOff:-4]
	
		# And last the traling length
		self.nTrailTSz = struct.unpack(">I", self.sPkt[-4:])

	#######################################################################
	def getBlock(self, nBlkID):
	
		for sBlk in self.lBlks:
			if ord(sBlk[0]) == nBlkID:
				return sBlk
		
		return None
	
	#######################################################################
	def getProcBlk(self, nProcID):
		
		for sBlk in self.lBlks:
			if ord(sBlk[0]) != 0x70:
				continue
			
			if ord(sBlk[2]) == nProcID:
				return sBlk
		
		return None
	
	
	
	
	
	
	
	
	
	
	
	
