"""Helpers for Juno GSE programing"""

import struct

##############################################################################
def hexPrn(sBytes, sep=" "):
	
	l = list(sBytes)
	for i in xrange(0, len(l)):
		l[i] = "%02X"%ord(l[i])
		
	return sep.join(l)
	

##############################################################################
def multiLineHexPrn(fOut, sBytes, nIndent=3, nGrpSize=4, bSwap=False):
	"""Print a Hex Dump of an arbitrary string, 4 bytes at a time"""
	
	nBytes = len(sBytes)
	sIndent = " "*nIndent

	iBlk = 0
	for i in xrange(0, nBytes, nGrpSize):
	
		if iBlk % 6 == 0:
			if iBlk != 0:
				fOut.write("\n"+sIndent)
			else:
				fOut.write(sIndent)
				
		nStop = nGrpSize + i
		if nStop > nBytes:
			nStop = nBytes
		
		l = list(sBytes[i:nStop])
		for j in xrange(0, len(l)):
			l[j] = "%02X"%ord(l[j])
			
		sBlock = "0x%s "%("".join(l))
		
		fOut.write(sBlock)
		
		iBlk += 1
	
	fOut.write('\n')



##############################################################################
def colString3(nWidth, s1, s2, s3):

	n1 = len(s1)
	n2 = len(s2)
	n3 = len(s3)
	nt = n1 + n2 + n3

	if nWidth < (nt+4):
		nSpace1 = 2
		nSpace2 = 2
	else:
		nSpace = nWidth - nt
		if nSpace % 2 == 0:    #Even
			nSpace1 = nSpace/2
			nSpace2 = nSpace1
		else:                  #Odd
			nSpace1 = nSpace/2
			nSpace2 = nSpace1 + 1
			
	sSpace1 = " "*nSpace1
	sSpace2 = " "*nSpace2
	
	return "%s%s%s%s%s\n"%(s1, sSpace1, s2, sSpace2, s3)

##############################################################################
def colString2(nWidth, nIndent, s1, s2):
	"""Print left adjusted in 2 columns"""
	
	nColSz = (nWidth - (4 + nIndent))/ 2
	
	sFmt = "%s%%-%ds  %%-%ds\n"%(" "*nIndent, nColSz, nColSz)
	
	return sFmt%(s1, s2)


##############################################################################
class ParseError(StandardError):
	def __init__(self, nLine, sMsg):
	
		self.nLine = nLine  # This item is ment to hold a line number
		self.sMsg  = sMsg # This item is ment to hold a message
	
	def __str__(self):
		return self.sMsg

##############################################################################
class UsageError(StandardError):
	'''Generic "this ain't being used right" error.'''
	def __init__(self, sMsg):
		self.sMsg
	
	def __str__(self):
		return self.sMsg

##############################################################################
def splitQuoteWords(sLine):
	"""Given a string, return a list of whitespace seperated items, HOWEVER
	handle using double quotes to string multiple words as a single item.
	Also, handle using # as a comment character.
	"""
	
	iCmt = sLine.find('#')
	if iCmt != -1:
		sLine = sLine[0:iCmt]
		
	lLine = sLine.split()
	
			
	# Go through looking for column boundaries (handles double quotes)
	lBounds = []
	iStart = None
	for j in xrange(0, len(lLine)):
		
		if iStart == None:		
			if lLine[j].startswith('"') and not lLine[j].endswith('"'):
				iStart = j
			else:
				lBounds.append( (j,j+1) )
		else:
			if lLine[j].endswith('"'):
				lBounds.append( (iStart, j+1) )
				iStart = None
	
	if iStart != None:
		raise ParseError( 0, "Quote not closed")
			
	# Go though the list of column boundaries and end up with a list of
	# column strings
	lCols = []
	for j in xrange(0, len(lBounds)):
				
		sCol = ''
		for k in xrange(lBounds[j][0], lBounds[j][1]):
			sCol += lLine[k] + " "
			
				
		lCols.append( sCol.strip('\t "') )

	return lCols

##############################################################################

class DisplayItem(object):
	"""Handles parsing and display for a single housekeeping item.
	
	It is initialized by feeding it a line or more of the setup table
	and then may be used to display a bit of the data out of a chunk
	of bytes.
	
	After initialization, and instance of HskDisplayItem may be used
	repeatedly for different byte chunks.
	"""
	
	# Bits to keep in the first byte based off of the bit offset
	#                    0     1     2     3     4     5     6     7     8(wrap)
	m_l1stByteMask  = [0xFF, 0x7F, 0x3F, 0x1F, 0x0F, 0x07, 0x03, 0x01]
	
	# Bits to keep in the last byte, based off of (size + offset) % 8
	#m_lLastByteMask = [0xFF, 0x80, 0xC0, 0xE0, 0xF0, 0xF8, 0xFC, 0xFE, 0xFF]
	
	#######################################################################
	def __init__(self, lLines, lLineNum):
		"""Given a list of configuration lines for a single HSK item,
		setup for parsing and printing."""
	
		if len(lLines) != len(lLineNum):
			raise ValueError('The line list and line number list must have'+\
			                 ' the same length.')
	
		self.sName = None
		
		self.sType = None #signed, unsigned, 
		
		self.dEnum = None
		self.dLimits = { 'err_lo':None, 'warn_lo':None, 
		                'warn_hi':None,  'err_hi':None  }
		self.sFmt  = None
		self.sDesc = ""
		self.nOffBytes = None
		self.nOffBits = 0
		self.nSzBits = None
		
		self.sUnits = ""
		self.lCoeff = None
		
		self.nVal = None
		self.lVal = None  # Used for the buf type
		
	
		for i in xrange(0, len(lLines)):
			try:
				lCols = splitQuoteWords(lLines[i])
			except ParseError, e:
				raise ParseError(lLineNum[i], e.sMsg)
			
			if len(lCols) == 0:
				continue
				
			if lCols[0][0] in "(<":
				continue
			
			try:
			
				if self.sType == None:
					
					# The Mnemonic
					for sTmp in ['enum_val','err_hi','warn_hi','warn_lo','warn_hi']:
						if lCols[0] == sTmp:
							raise ParseError(lLineNum[i],
							    "unexpected location for keyword %s"%sTmp)
					self.sName = lCols[0]
					
					# The offset(s)
					if lCols[1].find('.') == -1:
						self.nOffBytes = int(lCols[1])
					else:
						lTmp = lCols[1].split('.')
						if len(lTmp) != 2:
							raise ParseError(lLineNum[i], "Syntax Error: "+lCols[1])
						self.nOffBytes = int(lTmp[0])
						self.nOffBits = int(lTmp[1])
					
					if self.nOffBytes < 0 or self.nOffBits < 0:
						raise ParseError(lLineNum[i], "Syntax Error: "+lCols[1])
					
					# The size
					self.nSzBits = int(lCols[2])
					
					# Format code
					sTmp = lCols[3]%1 # Test format string 
					self.sFmt = lCols[3]
					
					# Type
					if lCols[4] not in ['crc','uint','enum','time','int', 'buf']:
						raise ParseError(lLineNum[i], "Syntax Error: "+lCols[4])

					self.sType = lCols[4]
					
					# Double check whole bytes for the buf type
					if (self.sType == 'buf') and \
					   (self.nOffBits != 0 or self.nSzBits % 8 != 0):
						
						raise ParseError(lLineNum[i], "Field type 'buf' must be "
						                 "whole bytes on byte boundaries.")
					
					# Units
					if lCols[5] != '-':
						self.sUnits = lCols[5]
					
					# Coeff
					if lCols[6] != '-':
						self.lCoeff = []
						for sCoeff in lCols[6].split(','):
							self.lCoeff.append( float(sCoeff) )
					
					# Description
					if len(lCols) > 7:
						self.sDesc = " ".join(lCols[7:])
					
				
				else:
					# Only accept enum_val, err_hi, err_lo, warn_hi, warn_lo if
					# the data type has already been set to an appropriate type.
				
					if self.sType == "enum" and lCols[0] == 'enum_val':
						if self.dEnum == None:
							self.dEnum = {}
					
						if len(lCols) < 3:
							raise ParseError(lLineNum[i], 
							      "Missing argument(s) in enum value.")
									
						elif len(lCols) == 3:
							self.dEnum[ int(lCols[1]) ] = (lCols[2], "")
						else:
							self.dEnum[ int(lCols[1]) ] = \
							         (lCols[2], " ".join(lCols[3:len(lCols)]) )
					
					elif self.sType in ['int','uint','time']:
						if len(lCols) < 2:
							raise ParseError(lLineNum[i], 
							   "Missing argument(s) in error or warning value.")
								
						if lCols[0] not in self.dLimits.keys():
							raise ParseError(lLineNum[i],
							                      "Syntax error: '%s'"%lCols[0])
						
						self.dLimits[ lCols[0] ] = float(lCols[1])
					
					else:
						raise ParseError(lLineNum[i], 
						   "Extra information lines following definition of type "+\
							self.sType)
			
			except StandardError, e:
				raise ParseError(lLineNum[i], str(e) + '\n' + lLines[i])
	
	
	#######################################################################
	def findVal(self, sData):
		"""Given a chunk of bytes, move to the right spot, and interperate
		the data.  Return the value as an integer.  Calibrarion polynomial
		is not applied at this point.
		"""
		
		self.nVal = None
		self.lVal = None
		
		# How many bytes to extract
		nExtraBytes = (self.nOffBits + self.nSzBits - 1) / 8
		
		# if we have a bit offset, figure how how much to mask off the
		# first byte
		sBytes = sData[self.nOffBytes : self.nOffBytes + nExtraBytes + 1]
		
		# For buf items, just ord the bytes
		if self.sType == 'buf':
			self.lVal = [ord(c) for c in sBytes]
			return
		

		if self.nOffBits != 0:
			lBytes = list(sBytes)
				
			# Mask off the upper part of the first byte
			lBytes[0] = chr( ord(lBytes[0]) & self.m_l1stByteMask[self.nOffBits])
			
			# Rejoin into a string
			sBytes = ''.join(lBytes)
			
		
		if len(sBytes) < 1 or (len(sBytes) > 4 and self.sType != "buf"):
			raise RuntimeError("Non-buf items that span %d bytes are not supported."%
			                    len(sBytes))
		
		# Now convert to an integer
		nTailBits = (self.nSzBits + self.nOffBits) % 8
		nDownShift = 8 - nTailBits
		
		if self.sType in ['uint', 'enum' ]:
		
			if len(sBytes) == 1:
				nVal = ord(sBytes)
			elif len(sBytes) == 2:
				(nVal, ) = struct.unpack(">H", sBytes)
			elif len(sBytes) == 3:
				(nVal, ) = struct.unpack(">L", '\x00'+sBytes)
			elif len(sBytes) == 4:
				(nVal, ) = struct.unpack(">L", sBytes)
		
			if nDownShift != 8:
				nVal = nVal >> nDownShift
				
		elif self.sType == 'int':
			if self.nOffBits != 0 or nTailBits != 0 or len(sBytes) != 1:
				raise RuntimeError(
				"Error on item %s: Only 1 byte long signed items"%self.sName +\
				"are supported at this time and they must align on byte boundaries."
				)
			(nVal, ) = struct.unpack("b", sBytes)
			
		elif self.sType == 'time':
			if self.nOffBits != 0 or nTailBits != 0 or len(sBytes) != 4:
				raise RuntimeError(
				   "Error on item %s: Time values must be 4 bytes "%self.sName +\
					"long and align on byte boundaries."
				)
			
			(nVal, ) = struct.unpack(">L", sBytes)
			nVal &= 0xFFFFFFFE
			
		elif self.sType == 'crc':
			(nVal, ) = struct.unpack(">H", sBytes)
		
		else:
			raise RuntimeError("self.sType == %s ???"%self.sType)
		
		
		self.nVal = nVal
	
	
	#######################################################################
	def getStr(self, nMnemonicWidth, nTotalWidth):
		"""Prints to string fOut, and returns the number of characters
		printed, including spaces"""
		
											 
		if self.sType == "buf":
			sLastFmt = "%%%ds: %%s"%nMnemonicWidth 
			
			sOut = sLastFmt%(self.sName,' '.join([self.sFmt%n for n in self.lVal]))
			return sOut
		
		
		sLastFmt = "%%%ds: %%-%ds"%(nMnemonicWidth ,
		                            (nTotalWidth - nMnemonicWidth - 2))
		
		if self.sType == "enum":
			if self.dEnum.has_key(self.nVal):
				sOut = sLastFmt%(self.sName, self.dEnum[self.nVal][0])
			else:
				sOut = sLastFmt%(self.sName, "%d (ENUM Missing)"%self.nVal)
			
			return sOut
			
		
		# Convert to float if we have coefficients
		fVal = float(self.nVal)
		if self.lCoeff != None:
			
			fVal = self.lCoeff[0]
			for i in xrange(1, len(self.lCoeff)):
				fVal += self.lCoeff[i] * pow( self.nVal, i)

			
		# This is not eligant, but I want to be done...
		if self.dLimits['err_lo'] != None and fVal <= self.dLimits['err_lo']:
			sWarn = 'LOW!! '
		elif self.dLimits['warn_lo']!=None and fVal<= self.dLimits['warn_lo']:
			sWarn = '(low) '
		elif self.dLimits['err_hi'] !=None and fVal >= self.dLimits['err_hi']:
			sWarn = 'HIGH!!'
		elif self.dLimits['warn_hi']!=None and fVal>= self.dLimits['warn_hi']:
			sWarn = '(high)'
		else:
			sWarn = "      "
		
		if self.sFmt[-1] in 'fg':
			sVal = self.sFmt%fVal
		else:
			sVal = self.sFmt%int(fVal)
		
		
		sOut = sLastFmt%(self.sName, "%s %s %s"%(sVal, self.sUnits, sWarn))
		
		return sOut
				
		
##############################################################################

def mkDisplayItems(fIn):
	"""Given a config file, generate a list of parsers/displayers data blobs
	and example config file for Housekeeping data is in:
	
	   DPU_FSW/trunk/house/hsk.tbl
		
	returns a list of parse item objects, or throws an exception
	"""
		
	lItems = []
	nLine = 0
	lItemLines = []
	lItemLineNo = []
	
	for sLine in fIn:
		
		nLine += 1
		
		# Remove comments
		iCmt = sLine.find('#')
		if iCmt != -1:
			sLine = sLine[0:iCmt]
		
		# strip whitespace from the ends
		sLine = sLine.strip()
		
		if len(sLine) < 1:
			continue
		
		if sLine[0] in "(<":
			continue
		
		bExtra = sLine.startswith('enum_val') or sLine.startswith('err_hi') or \
		         sLine.startswith('warn_hi') or sLine.startswith('warn_lo') or \
					sLine.startswith('err_lo')
		
		# Flush if we have existing stuff and the next line is not a
		# continuation
		if len(lItemLines) > 0 and not bExtra:
			try:
				lItems.append( DisplayItem(lItemLines, lItemLineNo) )
			except ParseError, e:
				print "Line", e.nLine, ':', e.sMsg
				
			lItemLines =  []
			lItemLineNo = []
		
		lItemLines.append(sLine)
		lItemLineNo.append(nLine)
			
	
	# If there is a non flushed line set, handle it
	if len(lItemLines) > 0:
		lItems.append( DisplayItem(lItemLines, lItemLineNo) )
		
	return lItems


##############################################################################
def genericParseAndPrint(lDisplayItems, fOut, sData, nWidth):
	
	# first pass, find out the max mnemonic length
	nMnemonicWidth = 0
	for item in lDisplayItems:
		if len(item.sName) > nMnemonicWidth:
			nMnemonicWidth = len(item.sName)
			
	nColWidth = (nWidth - 2)/2
	
	# Do this two column for now
	lTmp = []
	
	for i in xrange(0, len(lDisplayItems)):
		
		lDisplayItems[i].findVal(sData)
		lTmp.append(lDisplayItems[i].getStr(nMnemonicWidth, nColWidth))
		
		# Buf Items get a whole line to themselves
		if lDisplayItems[i].sType == "buf":
			fOut.write("\n%s"%lTmp[0])
			lTmp = []
			continue
		
		if (i + 1) % 2 == 0:
			fOut.write("\n%s  %s"%tuple(lTmp))
			lTmp = []
		
	
	if len(lTmp) != 0:
		fOut.write("\n%s"%lTmp[0])
		
	fOut.flush()

	
	
	
	
	
	
	
	
	
