/*****************************************************************************/
/* Custom Block Example                                                      */
/* New block type for testing, Contains a file's filesystem info             */

#define FILE_INFO_BLK_ID 0x1F

typedef struct file_info_blk
{	
	GseBlock base;               /* This always has to be the first member */
	
	uint4     st_mode;                 /* Extra info for this block type.  */
	uint4     st_uid;                
	uint4     st_gid;                
	uint8     st_atime;              
	uint8     st_mtime;              
	uint8     st_ctime;				  
	
	char* sPath;
	
	void (*del)(void* vpThis);
	
} FileInfoBlk;


bytebuf_t FileInfoBlk_encode(GseBlock* pBase,  byte uExtraLen, byte* pOffset)
{
	bytebuf_t buf;
	byte uBaseSz;
	byte uExtra;
	size_t nSzPath;
	
	FileInfoBlk* pThis = (FileInfoBlk*)pBase;
	
	if(pThis->sPath == NULL)
		gse_throw_m(GSE_XCEPT,GSE_XENCODE,"Not ready for encoding, path is null");
	
	nSzPath = strlen(pThis->sPath) + 1;
	
	/* More checks could be added */
	gse_storeUinU(4*3 + 8*3 + nSzPath, &uExtra, sizeof(uExtra)); 
	buf = GseBlock_encode(pBase, uExtra, &uBaseSz);  /* Checks to see if <= 256 */
	
	gse_u4ToMSBbytes(buf.ptr + uBaseSz, pThis->st_mode);
	gse_u4ToMSBbytes(buf.ptr + uBaseSz + 4, pThis->st_uid);
	gse_u4ToMSBbytes(buf.ptr + uBaseSz + 8, pThis->st_gid);
	gse_u8ToMSBbytes(buf.ptr + uBaseSz + 12, pThis->st_atime);
	gse_u8ToMSBbytes(buf.ptr + uBaseSz + 20, pThis->st_mtime);
	gse_u8ToMSBbytes(buf.ptr + uBaseSz + 28, pThis->st_ctime);
	
	strcpy((char*)(buf.ptr + uBaseSz + 36), pThis->sPath);
	
	return buf;
}


byte FileInfoBlk_decode(GseBlock* pBase, cbytebuf_t buf)
{
	size_t len;
	FileInfoBlk* pThis = (FileInfoBlk*)pBase;
	byte uBaseOff = GseBlock_decode(pBase, buf);
	
	pThis->st_mode = gse_u4FromMSBbytes(buf.ptr + uBaseOff);
	pThis->st_uid  = gse_u4FromMSBbytes(buf.ptr + 4 + uBaseOff);
	pThis->st_gid  = gse_u4FromMSBbytes(buf.ptr + 8 + uBaseOff);
	pThis->st_atime  = gse_u8FromMSBbytes(buf.ptr + 12 + uBaseOff);
	pThis->st_mtime  = gse_u8FromMSBbytes(buf.ptr + 20 + uBaseOff);
	pThis->st_ctime  = gse_u8FromMSBbytes(buf.ptr + 28 + uBaseOff);
	
	len = strlen((char*)(buf.ptr + 36 + uBaseOff));
	
	if(len + 1 + 36 + uBaseOff > 256)
		gse_throw_m(GSE_XCEPT, GSE_XDECODE, "The end of the path string is "
		            "past the maximum size of a GSE Header Block.");
	
	pThis->sPath = CS((char*)(buf.ptr + 36 + uBaseOff));
	
	/* Return the number of bytes parsed */
	return uBaseOff + 36 + strlen(pThis->sPath) + 1; 
}


void delete_FileInfoBlk(void* vpThis)           /* Memory cleanup routine */
{
	FileInfoBlk* pThis = (FileInfoBlk*)vpThis;
	if(pThis->sPath != NULL) free(pThis->sPath);
	delete_GseBlock(vpThis);
}


FileInfoBlk* new_FileInfoBlk(cbytebuf_t buf)
{
	GseBlock* pBase;
	FileInfoBlk* pThis = (FileInfoBlk*) new_GseBlock(buf, sizeof(FileInfoBlk));
	pBase = (GseBlock*)pThis;
	
	pBase->del      = delete_FileInfoBlk;
	pBase->encode   = FileInfoBlk_encode;
	pBase->decode   = FileInfoBlk_decode;
	pBase->_sName   = "File Info";
	pBase->uType    = FILE_INFO_BLK_ID;
	
	pBase->_uMinLen += 3*4 + 3*8;
	
	if(buf.ptr == NULL)
		return pThis;
	
	/* Decoding, set cleanup handler for our memory if exception is thrown */
	except_cleanup_push(delete_FileInfoBlk, pThis);
	
	FileInfoBlk_decode(pBase, buf);
	
	except_cleanup_pop(except_no_call);
	return pThis;
}


