#define _POSIX_C_SOURCE 200112

#ifdef GSE_POSIX_THREADS
#include <pthread.h>
#endif

#include <stdlib.h>
#include <stdio.h>

/* Add in the mtrace tool, define:                
 *
 *     export MALLOC_TRACE=FILE.log 
 *
 * and then run:
 *
 *   mtrace BINARY FILE.log
 *
 * to see the source of any memory leaks.
 */
#include <mcheck.h>  
#include <strings.h>

#include "../rpwgse/dict.h"
#include "../rpwgse/GseTypes.h"
#include "../rpwgse/GseInit.h"
#include "../rpwgse/GseString.h"
#include "../rpwgse/GseWrap.h"
#include "../rpwgse/GseExcept.h"


/*****************************************************************************/
/* RAW Dictionary Tests */

int compare_int_keys(const void* k1, const void* k2)
{
	int n1, n2;
	n1 = (size_t)k1;
	n2 = (size_t)k2;
	
	if(n1 > n2) return 1;
	if(n2 < n1) return -1;
	return 0;
}

int compare_cchar_keys(const void* k1, const void* k2)
{
	const char* s1 = (const char*)k1;
	const char* s2 = (const char*)k2;

	return strcasecmp(s1,s2);
}


dnode_t* alloc_node(void* ctx)
{
	dnode_t* pNew;
	pNew = (dnode_t*) calloc(1, sizeof(dnode_t) );
	return pNew;
}


void free_node(dnode_t* pNode, void* ctx)
{
	free((char*)dnode_getkey(pNode));
	free(dnode_get(pNode));
	free(pNode);
} 

/*****************************************************************************/
/* Dictionary Object Tests */

/* This is normally in a *.h file */
#define S str_str_dict
#define C StrStrDict
#define K char
#define V char

#include "../rpwgse/ptrDictTplt.h"

#undef S
#undef C
#undef K
#undef V


/* This is normally in a *.c file */
#define C StrStrDict
#define K char
#define V char

#define FREE_KEYS
#define FREE_VALS

#define KEY_COMPARE(a, b) strcasecmp(a, b)

#include "../rpwgse/ptrDictTplt.c"

#undef FREE_KEYS
#undef FREE_VALS
#undef C
#undef K
#undef V


/*****************************************************************************/
int main(int argc, char** argv)
{
	mtrace();
	
	/* RAW Dictionary Tests */
	dict_t dict;
	dnode_t* pNode;

	dict_init(&dict, DICTCOUNT_T_MAX, compare_cchar_keys);
	dict_set_allocator(&dict, alloc_node, free_node, NULL);
	
	if(! dict_alloc_insert(&dict, CS("key1"), CS("This is data") )  ||
		! dict_alloc_insert(&dict, CS("key2"), CS("This is more data") )
	  )
	{
		fprintf(stderr, "Error inserting into dictionary\n");
		return 2;
	}
	
	
	for(pNode = dict_first(&dict); pNode != 0; pNode = dict_next(&dict, pNode)){
		printf("%s = %s\n", (char*)dnode_getkey(pNode), (char*)dnode_get(pNode)); 
	}
	
	fprintf(stderr, "Looking up key1\n");
	
	if( (pNode = dict_lookup(&dict, "key1")) != NULL)
		printf("%s = %s\n", (char*)dnode_getkey(pNode), (char*)dnode_get(pNode)); 
				
	dict_free_nodes(&dict);

	fprintf(stderr, "----------------\n");
	
	
	/* OO Dictionary Tests */
	StrStrDict* pDict;
	const char* sKey;
	const char* sVal;
	
	gse_thread_init(GSE_WARNING, NULL);
	
	pDict = new_StrStrDict("Test");
	pDict->insert(pDict, CS("key1"), CS("This is data"));
	pDict->insert(pDict, CS("key2"), CS("This is more data"));
	
	for(sKey=pDict->first(pDict); sKey!=NULL; sKey=pDict->next(pDict, sKey))
		printf("%s = %s\n", sKey, pDict->get(pDict, sKey));
	
	fprintf(stderr, "Looking up key1\n");
	
	sKey = "key1";
	
	if( (sVal = pDict->get(pDict, sKey)) != NULL)
		printf("%s = %s\n", sKey, sVal); 
	
	pDict->del(pDict);
	
	return 0;
}


