/* Testing I notify */

#include <errno.h>
#include <sys/inotify.h>
#include <sys/ioctl.h>
#include <stdio.h>
#include <stdlib.h>
#include <stdbool.h>
#include <unistd.h>
#include <string.h>


/* size of the event structure, not counting name */
#define EVENT_SIZE  (sizeof (struct inotify_event))

/* reasonable guess as to size of 1024 events */
#define BUF_LEN        (1024 * (EVENT_SIZE + 16))

/*****************************************************************************/
void waitOnFd(int fd)
{
	fd_set fdSet;
	int nReady, nErrno;
	
	FD_ZERO(&fdSet);
	FD_SET(fd, &fdSet);
	
	while(true){
		nReady = select(fd+1, &fdSet, NULL, NULL, NULL);
		nErrno = errno;
		
		if(nReady > 0) break;
		
		if((nReady == -1)&&(nErrno == EINTR)) 
			continue;
		
		if(nReady < 1){
			perror("select");
			exit(2);
		}
	}
}


/*****************************************************************************/
int main(int argc, char** argv)
{
	int fdNotify, wd;
	
	char buf[BUF_LEN];
	int len, i = 0;
	struct inotify_event* pEvent;
	unsigned int nQueueLen;
	
	char sChanges[10];
	
	if(argc<2){
		fprintf(stderr,"Directory argument missing, use -h for help.\n");
		return 1;
	}
	
	if((strcmp(argv[1],"-h")==0) || (strcmp(argv[1],"--help")==0) ){
		printf("Watch a directory tree for changes. Deleting the directory tree\n");
		printf("will end the watch.\n");
		printf("\n");
		printf("Usage:\n");
		printf("     test_inotify DIR_NAME [DIR_NAME DIR_NAME ...]\n\n");
		return 0;
	}
		
	fdNotify = inotify_init();
	if(fdNotify < 0){
		perror("inotify_init");
		return 1;
	}
	
	
	for(i = 1; i<argc; i++){
		wd = inotify_add_watch(fdNotify, argv[i], IN_CREATE|IN_MOVED_TO|IN_DELETE|
				                 IN_MOVED_FROM|IN_MODIFY|IN_DELETE_SELF|IN_MOVE_SELF);
		if(wd < 0){
			perror("inotify_add_watch");
			return 1;
		}
		
		printf("Watching: %s, wd=%d\n", argv[i], wd);
	}
	
	printf("Go ahead change something.\n");
	
	
	/* Start reading the watch directory file descriptor */
	
	while(true){
	
		waitOnFd(fdNotify);
		
		
		if(ioctl(fdNotify, FIONREAD, &nQueueLen) < 0){
			perror("ioctrl");
			return 1;
		}
		
		
		while(true){
			len = read(fdNotify, buf, nQueueLen);
			
			fprintf(stderr, ".");
			if(len < 0){
				if(errno != EINTR){
					perror("Read fdNotify");
					return 1;
				}
				continue;
			}
			else{
				if( len == 0){
					fprintf(stderr, "iNotify buffer length is too small\n");
					return 2;
				}
			}
			break;
		}
		
		i = 0;
		while(i < len){
			
			pEvent = (struct inotify_event*) &buf[i];
			
			strcpy(sChanges, "        ");
			if(pEvent->mask & IN_CREATE)     sChanges[0] = 'C';
			if(pEvent->mask & IN_MOVED_TO)   sChanges[1] = 'c';
			
			if(pEvent->mask & IN_MODIFY)     sChanges[2] = 'M';
			
			if(pEvent->mask & IN_DELETE)     sChanges[3] = 'D';
			if(pEvent->mask & IN_MOVED_FROM) sChanges[4] = 'd';
			
			if(pEvent->mask & IN_DELETE_SELF)sChanges[5] = 'S';
			if(pEvent->mask & IN_MOVE_SELF)  sChanges[6] = 's';
			
			if(pEvent->mask & IN_IGNORED)    sChanges[7] = 'R';
			
			if(pEvent->mask & IN_ISDIR)       sChanges[8] = '/';
			
			
			printf("wd=%d mask=%s cookie=%u len=%u\n", 
			       pEvent->wd, sChanges, pEvent->cookie, pEvent->len);
			
			if(pEvent->len) printf("   File: %s\n", pEvent->name);
		
			i += EVENT_SIZE + pEvent->len;
		}
	
	}
	
	return 0;
}
