/* A program to test large file the effects of large file support.
   To use make two versions of this program on Solaris, one with:
	
	cc -xc99 -g -errwarn test_largefile.c -o test_smallfile
	
	and the other with:
	
	cc -xc99 -g -errwarn $(getconf LFS_CFLAGS) test_largefile.c -o test_largefile
	
	Then you can try to read directories with each and see which one fails
	(if either).
*/

#include <sys/types.h>
#include <dirent.h>
#include <errno.h>
#include <string.h>
#include <stdbool.h>
#include <stdio.h>


int main(int argc, char** argv){

	int nErr = 0;
	DIR* pDir = NULL;
	struct dirent* pEnt = NULL;

	
	if((argc < 2)||(strcmp(argv[1], "-h")==0)||(strcmp(argv[1], "--help")==0)){
		
		printf(
"A program to test effects of compiling in large file support.\n"
"  Usage:  %s  DIR_TO_READ\n"
"If the program works you will see a list of the directory entries in\n"
"DIR_TO_READ, else an error message will be printed\n", argv[0]);
		
		if(argc == 1) return 3;
		else return 0;
	}
	
	errno = 0;
	if( (pDir = opendir(argv[1])) == NULL){
		nErr = errno;
		fprintf(stderr, "Couldn't open %s, reason: %s\n", argv[1], strerror(nErr));
		return 4;
	}
	
	printf("%s\n", argv[1]);
	
	while(true){
		errno = 0;
		pEnt = readdir(pDir);
		if(pEnt == NULL){
			if(errno != 0){
				nErr = errno;
				fprintf(stderr, "Couldn't read directory entry from %s, reason: %s\n",
				        argv[1], strerror(nErr));
				return 5;
			}
			break;
		}
		
		printf(" |-%s\n", pEnt->d_name);
	}
	
	errno = 0;
	if(closedir(pDir) != 0){
		nErr = errno;
		fprintf(stderr, "Couldn't close %s, reason: %s\n", argv[1], strerror(nErr));
		return 6;
	}

	return 0;
}
