/* Test the Gse Field Extractor Config file Lexer */
#define _POSIX_C_SOURCE 200112

#ifdef GSE_POSIX_THREADS
#include <pthread.h>
#endif

#include "../rpwgse/GseTypes.h"
#include "../rpwgse/GseInit.h"
#include "../rpwgse/GseFileStr.h"
#include "../rpwgse/GseExtor.h"

/*****************************************************************************/
/* Stuff that should be in an include file somewhere */

#include <rpwgse/GseExtorParser.h>
#include <rpwgse/GseExtorLexer.h>

/*****************************************************************************/

const char g_sTestConf[] = 
"/* Uses C style comments \n"
"   More here \n"
" */\n"
"\n"
"SRC{\n"
"  off=2 sz=2 ty=U enum=[1:HRS,2:LRS,3:HS] desc=\"Packet Source\"\n" 
"  area=(0,0,4,0) table=(0,4,2,0) span_st=\"<b><i>\" span_st=\"</i></b>\"\n"
"}\n"
"\n"
"HFR_LOCK{off=32 sz=8 ty=S fmt=\"%5.3f\", cal=(0,.00784), desc=\"HFR_LOCK\"}\n"
"\n";

const char g_sTestOut[] = 
"SRC { off = 2 sz = 2 ty = U enum = [ 1 : HRS , 2 : LRS , 3 : HS ] desc = "
"\"Packet Source\" area = ( 0 , 0 , 4 , 0 ) table = ( 0 , 4 , 2 , 0 ) span_st"
" = \"<b><i>\" span_st = \"</i></b>\" } HFR_LOCK { off = 32 sz = 8 ty = S "
"fmt = \"%5.3f\" , cal = ( 0 , .00784 ) , desc = \"HFR_LOCK\" } \n";


int main(int argc, char** argv)
{
	int nLevel, nTok;
	struct gse_extor_pstate pstate;
	GseFileStr* pFStr;
	size_t i, lenTest;
	bool bVerbose;
	
	YYSTYPE yylval_param;
	YYLTYPE yyloc_param;
	
	const char* sConf = g_sTestConf;
	
	if( (argc > 1) && (strncmp(argv[1], "-v", 2) == 0) ){
		bVerbose = true;
		nLevel = GSE_DEBUG;
	}
	else{
		bVerbose = false;
		nLevel = GSE_WARNING;
	}
	
	gse_thread_init(nLevel, NULL);
	
	
	printf("Testing GSE Extractor config file lexer... ");
	
	if(bVerbose){
		printf("\n\nInput Config is:\n%s\n", sConf);
		printf("Expected Token Recognition Output:\n'%s'\n", 
				(const char*)g_sTestOut);
	}
	
	fflush(stdout);
	
	/* Test reading the config string above */
	pFStr = new_GseFileStr(1024);
	
	/* Only the parser uses the Extractors, so safe to pass NULL pointer
	   if the lexer IS CALLED DIRECTLY */
	gse_extor_beg_cfg(&pstate, NULL, sConf);
	
	while((nTok = gse_extor_lex(&yylval_param, &yyloc_param, pstate.scanner))!=0){
		if(nTok < 256){
			pFStr->printf(pFStr, "%c ", (char)nTok);
		}
		else{
			pFStr->printf(pFStr, "%s ", gse_extor_get_text(pstate.scanner));
		}
	}
	
	gse_extor_end_cfg(&pstate);
	
	/* End reading the config string */
	
	pFStr->printf(pFStr, "\n");
	
	if(bVerbose)
		printf("Token recognition output received:\n'%s'\n", pFStr->getstr(pFStr));
	
	lenTest = strlen((const char*)g_sTestOut);
	if(lenTest != pFStr->strlen(pFStr)){
		if(bVerbose)
			printf("...Output length %zd, expected %zd.  Test FAILED!\n",
			       pFStr->strlen(pFStr), lenTest);
		else
			printf("FAILED!\n");
		return 1;
	}
	
	for(i = 0; (i < lenTest) && (i < pFStr->strlen(pFStr)); i++){
		if( pFStr->getstr(pFStr)[i] != g_sTestOut[i]){
			if(bVerbose)
				printf("...Difference on character %zd, '%c'.  Test FAILED!\n",
				       i,  pFStr->getstr(pFStr)[i]);
			else
				printf("FAILED!\n");
			return 1;
		}
	}
	
	if(bVerbose)
		printf("...Test of GSE Extractor config file lexer PASSED.\n");
	else
		printf("PASSED.\n");
	return 0;
}
