/* Test the Gse Field Extractor Config file Parser */
#define _POSIX_C_SOURCE 200112

#ifdef GSE_POSIX_THREADS
#include <pthread.h>
#endif


#include "../rpwgse/GseString.h"
#include "../rpwgse/GseInit.h"
#include "../rpwgse/GseExcept.h"
#include "../rpwgse/GseExtor.h"

/* Should be in an include somewhere */
int gse_extor_parse(struct gse_extor_pstate* pstate);

/*****************************************************************************/
const char g_sTestConf[] = 
"/* Uses C style comments \n"
"   More here \n"
" */\n"
"HFR_LOCK{off=32 sz=8 ty=S fmt=\"%5.3f\" cal=(0,.00784) desc=\"HFR_LOCK\"}\n"
"\n"
"SRC{\n"
"  off=2 sz=2 ty=U enum=[1:HRS,2:LRS,3:HS] desc=\"Packet Source\"\n" 
"  area=(0,0,4,0) table=(0,4,2,0) span_st=\"<b><i>\" span_st=\"</i></b>\"\n"
"}\n"
"WACKED_BUT_LEGAL{\n"
"  off = 45 sz = 32 ty = F \n"
"  list_in_list=[ [\nname=0, values=1], [0.234e-12, enum=[1:DOG,2:CAT] ]\n"
"}\n";


/*****************************************************************************/
int main(int argc, char** argv)
{
	bool bVerbose;
	int nRet, nLevel;
	static const except_id_t catch_spec[] = {{XCEPT_GROUP_ANY, XCEPT_CODE_ANY}};
	except_t* pExcept;
	
	GseFieldExtorAry* pExtors;
	
	if( (argc > 1) && (strncmp(argv[1], "-v", 2) == 0) ){
		bVerbose = true;
		nLevel = GSE_DEBUG;
	}
	else{
		bVerbose = false;
		nLevel = GSE_WARNING;
	}
	
	gse_thread_init(nLevel, NULL);
	
	printf("Testing GSE Extractor config file parser...");
	if(bVerbose){
		printf("\n\nInput Config is:\n%s\n", (const char*)g_sTestConf);
	}
	fflush(stdout);
	
	except_try_push(catch_spec, 1, &pExcept);
	
	if(pExcept == NULL){	
		
		/* The function we are actually testing */
		pExtors = gse_field_mk_extors((const char*)g_sTestConf);
		if(bVerbose){
			printf("Extractors found: %d\n", pExtors->num);
		}
		nRet = 0;
	}
	else{
		if(bVerbose){
			gse_fprint_except(stdout, pExcept);
			printf("...Test of GSE Extractor config file parser FAILED!\n");
		}
		else{
			printf("FAILED!\n");
		}
		nRet = 1;
	}
	
	except_try_pop();
	
	return nRet;
}

