#define _POSIX_C_SOURCE 200112

#ifdef GSE_POSIX_THREADS
#include <pthread.h>
#endif

#include <stdlib.h>
#include <stdio.h>

/* Add in the mtrace tool, define:                
 *
 *     export MALLOC_TRACE=FILE.log 
 *
 * and then run:
 *
 *   mtrace BINARY FILE.log
 *
 * to see the source of any memory leaks.
 */
#include <mcheck.h>

#include "../rpwgse/GseTypes.h"
#include "../rpwgse/GseInit.h"
#include "../rpwgse/GseFileStr.h"
#include "../rpwgse/GseWrap.h"
#include "../rpwgse/GseExcept.h"
#include "../rpwgse/GseParser.h"

/*****************************************************************************/
/* Test making a parser in memory, using it is next */

int main(int argc, char** argv)
{
	GseStrPropDict* pParams;
	GseStrPsrDict* pParsers;
	GseFileStr* pConf; 
	FILE* pIn; 
	char sLine[256]; 
	except_t *pExcept;
	static const except_id_t catch_spec[] = { {GSE_XCEPT, GSE_XSYNTAX} }; 
	
	const GseParser* pParser;
	const GseField*  pField;
	const GseProp*   pProp;
	int nParsers = 0;
	int nFields = 0;
	int nProps = 0;
	const char* sParser;
	const char* sField;
	const char* sProp;
	
	int nRet = 0;
	
	mtrace();
	
	gse_thread_init(GSE_WARNING, NULL);
	
	if( argc < 2){
		gse_critical("Parser Builder config filename missing.\n");
		return 2;
	}
	
	/* Read in the config file */
	
	/* 32 K initial buffer */
	pConf = new_GseFileStr(32768);
   pIn = gse_fopen(argv[1], "rb");
  	
	while( fgets(sLine, 256, pIn) != NULL) 
		pConf->putstr(pConf, sLine);
	
	except_try_push(catch_spec, 1, &pExcept);  /* Start of try-catch */
	
	if(pExcept == NULL){
	
		/* Build a set of parsers from the config data */
		gse_build_parsers(pConf->getstr(pConf), argv[1], &pParams, &pParsers); 
	}
	else{
		/* Just print the error message for syntax stuff */
		fprintf(stderr, "%s\n", except_message(pExcept));
		/* fprintf(stderr, "%s\n", gse_except_backtrace(pExcept)); */
		/* fprintf(stderr, "%s:%d\n", gse_except_srcfile(pExcept), 
				                  gse_except_line(pExcept)); */
		nRet = 1;
	}
	
	except_try_pop();                          /* End of try-catch */
	if(nRet != 0) return nRet;
	
	
	/* Report by iterating over everything */
	if(pParams != NULL){
		for(sProp = pParams->first(pParams); sProp != NULL; 
			sProp = pParams->next(pParams, sProp))
			nProps++;
	}
	
	if(pParsers != NULL){
		for(sParser = pParsers->first(pParsers); 
		    sParser != NULL;
			 sParser = pParsers->next(pParsers, sParser))
		{
					 
			nParsers++;
			pParser = pParsers->get(pParsers, sParser);
			
			for(sProp = pParser->pProps->first(pParser->pProps);
				 sProp != NULL;
				 sProp = pParser->pProps->next(pParser->pProps, sProp))
			{
				nProps++;		 
			}
			
			for(sField = pParser->pFields->first(pParser->pFields);
				 sField != NULL;
				 sField = pParser->pFields->next(pParser->pFields, sField))
			{
				nFields++;
				pField = pParser->pFields->get(pParser->pFields, sField);
				
				for(pProp = pField->first(pField);
				    pProp != NULL;
				    pProp = pField->next(pField, pProp))
					nProps++;		 
			}
		}
	}
	
	printf("The config file contains:\n     ");
	printf("%d properties, %d fields, and %d parsers.\n",nProps,nFields,nParsers);
			 			 
		
	/* Now delete them so we can check for memory leaks */
	if(pParams) pParams->del(pParams);
	if(pParsers) pParsers->del(pParsers);
	
	/* And the config string */
	pConf->del(pConf); 
	
	return 0;
}
