/** file test_read.c test reading generic packets from a file */
#define _POSIX_C_SOURCE 200112

#ifdef GSE_POSIX_THREADS
#include <pthread.h>
#endif

#include <stdio.h>
#include <libgen.h>
#include <string.h>

/* NOTE: Test programs have to insure the get headers from the source 
         tree.  For end user programs #include <rpwgse/Gse.h> is fine.
*/

#include "../rpwgse/GseInit.h"
#include "../rpwgse/GseString.h"
#include "../rpwgse/GseFilePktSrc.h"


void prefix_info(bytebuf_t preBuf);
void packet_info(GsePkt* pPkt);

/*****************************************************************************/
int main(int argc, char** argv){
	GseFilePktSrc* pFileSrc;
	GsePktSrc* pSrc;
	GsePkt* pPkt;
	bytebuf_t preBuf;
	
	if((argc < 2)||(strcmp(argv[1], "-h") == 0)||
	   ( strncmp(argv[1], "--help", 6) == 0)){
		
		printf("\nTest reading packets from a file.  This should not crash "
		       "on any type\nof file, so feel free to feed it MS Word documents "
		       "if you like.\n\n"
		       "     USAGE:  %s FILENAME\n\n", basename(argv[0]));
		return 1;
	}
	
	gse_thread_init(GSE_DEBUG, NULL);
	
	pFileSrc =  new_GseFilePktSrc(argv[1], gse_generic_blocks,
	                                      at_eof_stop);
	pSrc = (GsePktSrc*)pFileSrc;
	
	do{
		pPkt = pSrc->readNext(pSrc);
		prefix_info(preBuf);
		packet_info(pPkt);
		free(preBuf.ptr);
		if(pPkt != NULL) pPkt->del(pPkt);
	}
	while(pPkt != NULL);
	
	pFileSrc->del(pFileSrc);
	return 0;
}

/*****************************************************************************/
void prefix_info(bytebuf_t preBuf){
	if(preBuf.ptr != NULL)
		gse_fmt_hexprn(stdout, const_bytebuf(preBuf), 0, 4);
}

/*****************************************************************************/
void packet_info(GsePkt* pPkt){
	if(pPkt != NULL)
		fprintf(stdout, "Found packet\n");
}
