/* Compile with: 
 *
 *   gcc -Wall -rdynamic test_segcatch.c -o test_segcatch
 */
#define _GNU_SOURCE
#include <execinfo.h>   /* This gives the glibc backtrace funtion */
#include <dlfcn.h>      /* Gives the dladdr function              */

#ifdef GSE_POSIX_THREADS
#include <pthread.h>
#endif

#include <stdlib.h>     /* Gives def of STDERR_FILENO, exit       */
#include <signal.h>     /* Gives def of signal                    */
#include <unistd.h>     /* Gives def of write                     */
#include <string.h>     /* Gives def of strlen                    */
#include <stdio.h>      /* Gives def of sprintf                   */
#include <libgen.h>     /* Gives def of basename                  */

/****************************************************************************/
/* Print a stack trace on an error, malloc is not allowed in a              */
/* signal handler, so static buffers are used instead                       */

#define MAX_STACK_DEPTH 20
#define PRETTY_PRN_BUF_SZ   1024

static void sigsegv_handler(int nSignal){
	int i, nDepth;
	size_t nAdd, nStrLen;
	Dl_info info;
	void* nStackTraceBuf[MAX_STACK_DEPTH];
	const char* sFileNames[MAX_STACK_DEPTH];
	const char* sFunctions[MAX_STACK_DEPTH];
	char* sBuf[PRETTY_PRN_BUF_SZ];
	
#ifdef __linux	
	nDepth = backtrace(nStackTraceBuf, MAX_STACK_DEPTH);
#else
#error Non linux platforms not yet implimented
#endif	
	
	/* Getting the function names */
	nStrLen = 0;
	for(i = 2; i < nDepth - 1; i++){
		sFunctions[i] = "unknown function";
		sFileNames[i] = "unknown file";
				
		if(dladdr(nStackTraceBuf[i], &info)){
			if(info.dli_sname) sFunctions[i] = info.dli_sname;
			if(info.dli_fname) sFileNames[i] = info.dli_fname;
		}
		
		nStrLen += strlen(sFunctions[i]) + strlen(sFileNames[i]) + 15;
	}
	
	/* Printing them */	
	nStrLen = 0;
	for(i = nDepth - 2; i >= 2; i--){
		nAdd = strlen(sFunctions[i]) + strlen(sFileNames[i]) + 15;
		
		if(nStrLen + nAdd >  PRETTY_PRN_BUF_SZ) 
			break;
		
		sprintf( ((char*)sBuf)+nStrLen, "   in %s (from %s)\n",
				  sFunctions[i], sFileNames[i]);
		nStrLen += nAdd;
	}
	
	write(STDERR_FILENO, "Segfault in:\n", 14);
	if(sBuf[0] == '\0')
		write(STDERR_FILENO, "   Stack lost!!!  This is usually caused by a bad"
				" function pointer.\n   Check for unassigned function pointers.\n",
				112);
	else
		write(STDERR_FILENO, sBuf, nStrLen);
	exit(EXIT_FAILURE);
}

/****************************************************************************/
void really_bad(){
	void (*func)() = NULL;
	func();
}
	
void bad(){
	int* pN = NULL;
	
	*pN = 2;
}

int main(int argc, char** argv){

	/* Set a handler for segfaults */
	if( signal(SIGSEGV, sigsegv_handler) == SIG_ERR){
  		write(STDERR_FILENO, "Failure to init SIGSEGV handler\n", 32);
  		exit(EXIT_FAILURE);
	}
	
	if((argc > 1)&&(strncmp(argv[1], "-h", 2) == 0)){
		printf("Test stack trace printing in a SIGSEGV signal handler\n"
		       "   USAGE:  %s [-r]\n"
		       "Where -r is a flag for \"fail really bad\"\n\n", 
		       basename(argv[0]));
		return 0;
	}

	if((argc > 1)&&(strncmp(argv[1], "-r", 2) == 0))
		/* call function that does something really bad */
		really_bad();
	else
		/* call function that does something bad */
		bad();
		
	return 0;
}
