#define _POSIX_C_SOURCE 200112

#ifdef GSE_POSIX_THREADS
#include <pthread.h>
#endif

/* Testing the treewatch object from the RpwGse library to compile use:

   make test
	
	from the main directory
*/

#include <rpwgse/Gse.h>

/*****************************************************************************/
int main(int argc, char** argv)
{
	GseTreeWatch* pTw = NULL;
	GseTreeChange* pChange = NULL;
	const char* sType = NULL;
	int i;
	
	if(argc<2){
		fprintf(stderr,"Directory argument missing, use -h for help.\n");
		return 1;
	}
	
	if((strcmp(argv[1],"-h")==0) || (strcmp(argv[1],"--help")==0) ){
		printf("Watch a directory tree for changes.  Note the directory\n");
		printf("tree need not exist at the time that this program is invoked\n");
		printf("but may instead be created at some indefinite time later.\n");
		printf("\n");
		printf("Usage:\n");
		printf("test_treewatch: DIR_NAME");
		return 0;
	}
	
	gse_thread_init(GSE_WARNING, NULL);
	
	pTw = new_GseTreeWatch(argv[1], GSE_TW_CREATE|GSE_TW_DELETE|GSE_TW_MODIFY
			                          |GSE_TW_FILES_ONLY, NULL,
			                          NULL);
	
	
	while(true){
		
		pTw->waitForChange(pTw);
		
		for(i = pTw->numChanges(pTw); i>0; i--){
			
			pChange = pTw->getChange(pTw);
			
			if(pChange->bIsDir)
				sType = "dir";
			else
				sType = "file";
			
			switch(pChange->cChange){				
				case 'C': printf("Created %s  %s\n", sType, pChange->sPath); break;
				case 'D': printf("Deleted %s  %s\n", sType, pChange->sPath); break;
				case 'M': printf("Modified %s %s\n", sType, pChange->sPath); break;
				case 'I': printf("Names below %s now invalid\n", pChange->sPath); break;
				default:  gse_throw(GSE_XCEPT, GSE_XINCONCEIVABLE);
			}
			
			pChange->del(pChange);
		}
	}
	
	pTw->del(pTw);
	
	return 0;
}
