/* test_read.c
 * Reads files that were writted by test_write.c
 */
#define _POSIX_C_SOURCE 200112

#ifdef GSE_POSIX_THREADS
#include <pthread.h>
#endif

/* test_read.c
 * Reads files that were writted by test_write.c
 */

#include <stdio.h>
#include <libgen.h>
#include <string.h>
#include <sys/types.h>

#ifndef _WIN32
#include <pwd.h>
#include <grp.h>
#endif

#include <time.h>

/* NOTE: Test programs have to insure that headers are from the source 
 *        tree.  For end user programs: 
 *			
 *       #include <rpwgse/Gse.h> 
 *			
 *	      is fine.
 */
#include "../rpwgse/except.h"
#include "../rpwgse/GseExcept.h"
#include "../rpwgse/GseLog.h"
#include "../rpwgse/GseInit.h"
#include "../rpwgse/GseTypes.h"
#include "../rpwgse/GseFile.h"
#include "../rpwgse/GseConv.h"
#include "../rpwgse/GseString.h"
#include "../rpwgse/GseBlock.h"
#include "../rpwgse/GseProcBlk.h"
#include "../rpwgse/GseHdr.h"
#include "../rpwgse/GsePkt.h"
#include "../rpwgse/GsePktSrc.h"
#include "../rpwgse/GseFilePktSrc.h"

/*****************************************************************************/
/* The FileInfoBlk definition, used for testing */
#include "FileInfoBlk.c"

/* Forward declarations */
GseBlock* factory(int2 nSpaceCraft, byte uType, byte uProcess, cbytebuf_t buf);
void prn_info(FileInfoBlk* pBlk);
void extract_file(FileInfoBlk* pBlk, cbytebuf_t data);

#define APP_XCEPT     100
#define APP_XBADMAGIC 1

/*****************************************************************************/
int main(int argc, char** argv){
	GsePktSrc* pSrc = NULL;
	GsePkt* pPkt = NULL;
	GseHdr* pHdr = NULL;
	FileInfoBlk* pBlk = NULL;
	int nBlks, iFile = 1;
	bool bExtractFiles = true;
	
	if((argc < 2)||(strcmp(argv[1], "-h") == 0)||
	   ( strncmp(argv[1], "--help", 6) == 0)||
		( (strcmp(argv[1], "-t") == 0) && (argc < 3))  ){
		
		printf("\nTest unpacking files written by test tar.  This should not "
				 "crash on any type\nof file, so feel free to feed it MS Word "
		       "documents if you like.\n\n"
		       "     USAGE:  %s [-t] FILENAME\n\n"
		       "where -t just extracts the table of contents\n", basename(argv[0]));
		return 1;
	}
	
	if(strcmp(argv[1], "-t") == 0){
		bExtractFiles = false;
		iFile = 2;
	}
	gse_thread_init(GSE_NOTICE, NULL);
	
	pSrc =  (GsePktSrc*)new_GseFilePktSrc(argv[iFile], factory, at_eof_stop);
	
	do{
		if( (pPkt = pSrc->readNext(pSrc)) == NULL)
			continue;
		
		pHdr = pPkt->pHdr;
		if(pHdr->nSpaceCraft != 999){
			gse_notice_v("Skipping packet for unknown Application ID: %d", 
					        pHdr->nSpaceCraft);
			continue;
		}
		if( (nBlks = pHdr->numBlksByType(pHdr, FILE_INFO_BLK_ID, 0)) != 1){
			gse_error_v("Packet Header had %d FileInfoBlocks, expected 1", nBlks);
			continue;
		}
		pBlk = (FileInfoBlk*) pHdr->getBlkByType(pHdr, FILE_INFO_BLK_ID, 0);
		
		if(! bExtractFiles)
			prn_info(pBlk);
		else
			extract_file(pBlk, pPkt->getData(pPkt));
		
		if(pPkt != NULL) pPkt->del(pPkt);
	}
	while(pPkt != NULL);
	
	if(bExtractFiles)
		printf("\nNote: This simple demo does not handle setting user and group "
		       "IDs for extracted files.\n");
	
	pSrc->del(pSrc);
	return 0;
}

/*****************************************************************************/
GseBlock* factory(int2 nSpaceCraft, byte uType, byte uProcess, cbytebuf_t buf)
{
	if(nSpaceCraft != 999)
		gse_throw_fm(APP_XCEPT, APP_XBADMAGIC, "Skipping packet for unknown "
		             "Application ID: %d", nSpaceCraft);
	
	if(uType == FILE_INFO_BLK_ID)
		return (GseBlock*) new_FileInfoBlk(buf);
	
	/* We don't know what this is, let the default generator handle it */
	return gse_generic_blocks(nSpaceCraft, uType, uProcess, buf);
}

/*****************************************************************************/
void prn_info(FileInfoBlk* pBlk)
{
#ifndef _WIN32
	struct passwd* pPw;
	struct group*  pGr;
#endif
	
	if(pBlk->sPath != NULL)
		printf("%s\n", pBlk->sPath);
	
	printf("\tMode:  %o\n", pBlk->st_mode);

#ifndef _WIN32
	if( (pPw = getpwuid(pBlk->st_uid)) != NULL)
		printf("\tUser:  %s\n", pPw->pw_name);
	else
#endif
		printf("\tUID:   %o\n", pBlk->st_uid);

#ifndef _WIN32
	if( (pGr = getgrgid(pBlk->st_gid)) != NULL)
		printf("\tGroup: %s\n", pGr->gr_name);
	else
#endif
		printf("\tGID:   %o\n", pBlk->st_gid);
	
	printf("\tAccesed:    %s", ctime( (time_t*)(&(pBlk->st_atime))) );
	printf("\tModified:   %s", ctime( (time_t*)(&(pBlk->st_mtime))) );
	printf("\tSt. Change: %s", ctime( (time_t*)(&(pBlk->st_ctime))) );
}

/*****************************************************************************/
void extract_file(FileInfoBlk* pBlk, cbytebuf_t data)
{
	if(pBlk->sPath == NULL){
		gse_error("Can't untar file, path information is missing.");
		return;
	}
	
	printf("Extracting: %s (%.0f kB)\n", pBlk->sPath, data.len/1024.0);
	gse_writefile(pBlk->sPath, data, pBlk->st_mode);
}
