/* Testing stacktrace printing on Windows XP and up.                          */
/* This code was ganked from Jochen Kalmbach's excellent StackWalker class,   */
/* if you are fortunate enough to be using C++ instead a C, use his much      */
/* nicer class instead.  You can find it at:                                  */
/*    http://www.codeproject.com/KB/threads/StackWalker.aspx                  */
/*                                                                            */
/* -Chris Piker                                                               */

/* Mingw compile command:  
/*      gcc -Wall -IC:/local/include -c test_win_backtrace.c 
/* Mingw Link command
/*      gcc -Wall -LC:/local/lib test_win_backtrace.o C:/local/lib/dbghelp.lib \
/*          -lKernel32 -o test_win_backtrace 
/* 
/*      and make sure dbghelp.h is in /local/include */

#define WINVER        WindowsXP
#define _WIN32_WINNT  WindowsXP

#include <windows.h>
#include <dbghelp.h>

#include <stdio.h>

/* Supposed to be defined in WinNT.h if WINVER >= WindowsXP, but it ain't */
VOID WINAPI RtlCaptureContext(
  __out  PCONTEXT ContextRecord
);

typedef void (*RtlCaptureContextFunc)(CONTEXT* ContextRecord);

void backtrace(){
	DWORD   mType;
	CONTEXT c;
	STACKFRAME64 s; /* in/out stackframe */
	HANDLE hProcess;
	HANDLE hThread;
	HINSTANCE hKernDLL;
	RtlCaptureContextFunc pRCCF;
	
	IMAGEHLP_SYMBOL * symbol;
	unsigned displacement
	/* hum, static don't like this, must be a windows dll information 
	 * passing limitaton  */
	static char symbolBuffer[ sizeof(IMAGEHLP_SYMBOL) + 255 ];
	
	
	/* Initalize the symbol table to get function names */
	hProcess = GetCurrentProcess();
	SymInitialize( hProcess, 0, true);
	
	/* This returns a unique pseudo handle, if we need a real one call OpenProcess */
	hThread  = GetCurrentThread();
	
	/* Get our current execution context */
	memset(&c, 0, sizeof(CONTEXT)); 
	c.ContextFlags = CONTEXT_FULL;
	
	/* Load the RtlCaptureContext function as is not defined in Kernel32.a */
	hKernDLL = LoadLibrary("Kernel32.dll");
	pRCCF = (RtlCaptureContextFunc)GetProcAddress(hKernDLL,"RtlCaptureContext");
	
	/* Get the current thread context */
	pRCCF(&c);
		
	/* init STACKFRAME for first call */
	memset(&s, 0, sizeof(s));
#ifdef _M_IX86
	/* normally, call ImageNtHeader() and use machine info from PE header */
	mType = IMAGE_FILE_MACHINE_I386;
	s.AddrPC.Offset = c.Eip;
	s.AddrPC.Mode = AddrModeFlat;
	s.AddrFrame.Offset = c.Ebp;
	s.AddrFrame.Mode = AddrModeFlat;
	s.AddrStack.Offset = c.Esp;
	s.AddrStack.Mode = AddrModeFlat;
#elif _M_X64
	mType = IMAGE_FILE_MACHINE_AMD64;
	s.AddrPC.Offset = c.Rip;
	s.AddrPC.Mode = AddrModeFlat;
	s.AddrFrame.Offset = c.Rsp;
	s.AddrFrame.Mode = AddrModeFlat;
	s.AddrStack.Offset = c.Rsp;
	s.AddrStack.Mode = AddrModeFlat;
#elif _M_IA64
	mType = IMAGE_FILE_MACHINE_IA64;
	s.AddrPC.Offset = c.StIIP;
	s.AddrPC.Mode = AddrModeFlat;
	s.AddrFrame.Offset = c.IntSp;
	s.AddrFrame.Mode = AddrModeFlat;
	s.AddrBStore.Offset = c.RsBSP;
	s.AddrBStore.Mode = AddrModeFlat;
	s.AddrStack.Offset = c.IntSp;
	s.AddrStack.Mode = AddrModeFlat;
#else
#error "Platform not supported!"
#endif
	
	while( StackWalk64(mType, hProcess, hThread, &s, &c, NULL, 
			             SymFunctionTableAccess64, SymGetModuleBase64, NULL) ){
		
		/* Declare an image help symbol structure to hold symbol info and   */
		/* name up to 256 chars, this size of the symbol table can change   */
		/* but one of the early members in a length                         */
		memset( symbolBuffer , 0 , sizeof(IMAGEHLP_SYMBOL) + 255 );
		IMAGEHLP_SYMBOL * symbol = (IMAGEHLP_SYMBOL*) symbolBuffer;
		symbol->SizeOfStruct    = sizeof(IMAGEHLP_SYMBOL) + 255;
		symbol->MaxNameLength   = 254;

		/* The displacement from the beginning of the symbol is stored here */
		displacement = 0;

		/* Get the symbol information from the address */
		if(SymGetSymFromAddr(hProcess, s.AddrPC.Offset, (DWORD*) (&displacement),
		                     symbol /*gzout*/)){
			fprintf(stderr, "0x%08llX %s",  s.AddrPC.Offset, symbol->Name);
		}
		else{
			fprintf(stderr, "0x%08llX (unknown function)", s.AddrPC.Offset);
	}
	
	SymCleanup(hProcess);
}

void func2()
{
	backtrace();
}

void func1()
{
	func2();
}

int main(int argc, char** argv)
{
	func1();
	return 0;

}
