/** @file BstRowWriter.h */

#ifndef _BstRowWriter_H_
#define _BstRowWriter_H_

#include <rpwgse/GseTime.h>

#include "JnoPhaser.h"
#include "BstCalib.h"

#ifdef __cplusplus
extern "C" {
#endif

/*****************************************************************************/
#define BSTROWS_E    0
#define BSTROWS_B    1
#define BSTROWS_EUNC 2
#define BSTROWS_BUNC 3
#define BSTROWS_MIX  4
#define BSTROWS_NBS  5

#define BSTROWS_LISTS 6

/****************************************************************************/
/* Lists of measurements */

#define S bst_row_list
#define C BstRowList
#define I BstRow

#include <rpwgse/listTplt.h>

#undef S
#undef C
#undef T

/*****************************************************************************/
typedef struct bst_row_writer{
	
	/** Takes over the memory, no need to add packets in time order */
	bool (*add)(struct bst_row_writer* pThis, BstRow* pRow);
	
	/** Writes the data to a temporary file, keeps it if it's different
	 * throws away the temp file if it's the same.  
	 * 
	 * TODO: Should this return the names of the files that were written or
	 *       retained?
	 */
	
	void (*writeAll)(struct bst_row_writer* pThis, const char* sGroup,
			           JnoPhaser* pPhaser, const gse_caltime_t* pStart);
	
	/* GseStrAry* (*writeAll)(struct bst_row_writer* pThis, const char* sGroup,
			                 JnoPhaser* pPhaser, const gse_caltime_t* pStart); */
	
	/* If add, add, add, ... writeAll is too slow go with the new method:
	 * add, write, add, write, add, write, flush */
	
	void (*del)(void* vpThis);	
	
	/* Lists of measurements to write to disk */
	BstRowList* rows[BSTROWS_LISTS];
	
	/* Number of samples of the longest row in each list */
	int4 maxSamples[BSTROWS_LISTS];
	
	/* Qfactors for each bin */
	float4 fQfAry[BSTROWS_LISTS];
	
	/* file mnemonics for each list */
	const char* mnemonicAry[BSTROWS_LISTS];
	
	/* Start and stop sclks for the whole session */
	float8 dSesStartSclk;
	float8 dSesStopSclk;
	
} BstRowWriter;

/** Initialize a new L2 high rate writer 
 *
 * @param sGroup The data group to add to 
 * @param pStart A time at or before all the packets to be added
 *
 * @return New writer allocated on the heap
 */
BstRowWriter* new_BstRowWriter();


#ifdef __cplusplus
}
#endif

#endif /* _BstRowWriter_H_ */
