/** @file JnoPhaser.h */
#ifndef _JnoPhaser_H_
#define _JnoPhaser_H_

#include <rpwgse/GseTypes.h>
#include <rpwgse/GseTime.h>

#ifdef __cplusplus
extern "C" {
#endif

/*****************************************************************************/
/* A Listing of mission periods */
typedef struct mission_period{
	/* Period Name */
	char sName[64];
	
	/* Period inclusive lower SCET bound */
	gse_caltime_t ctBeg;
	
	/* Period exclusive upper SCET bound */
	gse_caltime_t ctEnd;
	
	/* Period inclusive lower SCLK bound, from ctBeg via spice */
	float8 dSclkBeg;
	
	/* Period exclusive upper SCLK bound, from ctEnd via spice */
	float8 dSclkEnd;
		
	/* equivalent PDS L3 Volume dir name */
	char sSubDir[128];
	
	/* is an orbit period, */
	bool bIsOrbit;
		
}mission_period_t;


#define S period_list
#define C PeriodList
#define I mission_period_t

#include <rpwgse/listTplt.h>

#undef I
#undef C
#undef S

/* Sorting Mission Periods */

int4 period_cmp(const mission_period_t* pOne, const mission_period_t* pTwo);

/*****************************************************************************/
/* Object to handle the phase and orbit table */

typedef struct jno_phaser{
	
	/* Implements the rules of sub directory naming by phase. */
	const mission_period_t* (*getPhase)(const struct jno_phaser* pThis,
			                              float8 dSclk);
	
	/* Same as getPhase, but takes SCETs and converts to dSclk via spice */
	const mission_period_t* (*getPhaseFromScet)(const struct jno_phaser* pThis,
			                                      const gse_caltime_t* pCt);

	
	/* Diagnostic Printers */
	void (*prnPhases)(struct jno_phaser* pThis, FILE* pOut);
	void (*prnOrbits)(struct jno_phaser* pThis, FILE* pOut);
	
	/* Destructor */
	void (*del)(void* vpThis);
	
	PeriodList* _pPhases;
	PeriodList* _pOrbits;
	
}JnoPhaser;

/** Read the list of mission phases from the config file and create a juno
    mission phase handler object */
JnoPhaser* new_JnoPhaser();


#ifdef __cplusplus
}
#endif

#endif /* _JnoPhaser_H_ */
