/** @file SvrCalTbl.h */
#ifndef _SvrCalTbl_H_
#define _SvrCalTbl_H_

#include <rpwgse/GseTypes.h>
#include <rpwgse/dict.h>

#include "caldir_util.h"

#ifdef __cplusplus
extern "C" {
#endif


/*****************************************************************************/
/* Cal Table holder */

typedef struct srv_cal_tbl{
	
	char* sName;  /* Useful for error reporting */
	
	size_t uChan;     /* Number of channels */
	float8* pCenters; /* Bin centers in Hz, size depends on num channels */
	float8* pNbws;    /* Noise band widths, Hz size depends on num channels */
	float8* pAntLen;  /* Effective antenna length, may be channel dependent */
	bool bUseAntLen;  /* If true, the effective antenna length is needed in 
	                     this calibration */
	int4 nPropVer;
	
	size_t uVals;        /* 512 for FT results, 256 for SA Log amp measurements */
	float8* pInstVal;    /* Relates Inst Values to Vrms table indicies, not used
	                        for the HFRH, since the inst value is the tabel idx */
	float8** pChanFrms; /* 1st index is channel, 2nd is the inst value */
	int4 nCalVer;
	
	size_t uAttns;
	attn_cal_t* pAttns;  /* Attenuator calibrations, size is fixed for attn  */
	int4 nAttnVer;       /*  settings on receiver */
	
	uint4 uLoadState;    /* = 0 - 7 depending on which tables have been loaded */
	
	size_t iPos;         /* Temporary counter used during table load */
	
	void (*loadChanProp)(struct srv_cal_tbl* pThis, const char* sPath);
	void (*copyChanProp)(struct srv_cal_tbl* pThis,
	                     const struct srv_cal_tbl* pOther);
	void (*loadConv)(struct srv_cal_tbl* pThis, const char* sPath);
	bool (*isLoaded)(const struct srv_cal_tbl* pThis);
	float8 (*convVal)(const struct srv_cal_tbl* pThis, size_t iChan,
	                  float8 dInstVal);
	void (*del)(void* vpThis);
	
} SrvCalTbl;

/** Make a new calibration table object, and initialize it's internal arrays
 *
 * @public 
 * @memberof SrvCalTbl
 */
SrvCalTbl* new_SrvCalTbl(const char* sName, size_t uChannels, size_t uValues, 
                         size_t uAttns, bool bUseAntLen);


/*****************************************************************************/
/* Cal table dictionaries */

#define S str_srvcaltbl_dict
#define C StrSrvCalTblDict
#define K char
#define V SrvCalTbl

#include <rpwgse/ptrDictTplt.h>

#undef V
#undef K
#undef C
#undef S


/*****************************************************************************/
/* Loading an attenuator settings file to a dictonary of srvcaltables        */

void load_named_attns(StrSrvCalTblDict* pDict, const char* sPath);


#ifdef __cplusplus
}
#endif


#endif /* _SvrCalTbl_H_ */
