/* @file SrvCalib.h */
#ifndef _SrvCalib_H_
#define _SrvCalib_H_

#include <rpwgse/GsePkt.h>

#include "SrvCalTbl.h"

#ifdef __cplusplus
extern "C" {
#endif

/*****************************************************************************/
/* The output of the code, calibrated sections, these are intentionally      */
/* structured so that they don't need a destructor, free will do fine        */

typedef struct srv_row_sec_t{
	
	/* Stuff that goes out the door when making the header */
	
	const SrvCalTbl* pCalTbl; /* Reference to cal table used to get this data */
	const char* sBinPrefix;   /* Used when making channel names */
	
	/* Stuff that goes out the door for each row the Calibrated files */
	
	float8 dSclkOff;
	int4 nPreAttn;    /* 1 = Preamp Attn on, 0 = off */
	int4 nRecAttn;    /* 0 = off, 1 = on for LFR, 1-15 = setting for hfr */
	char sCalVer[10]; /* Bin Prop:Bin:Attn Cal version string */
	
	int4 nNRDon;      /* LFR Only: was Noise Reduc. Algo. turned on? */
	int4 nLfrKind;    /* LFR Only: 1 = S+N, 2 = N, 3 = S+N-N (S), 0 = Not LFR */
	
	int4 nHFRboard;   /* HFR Only: Was this from the HFR (44) or HFWBR (45) */
	
	size_t uChan;     /* Normal Number of Values (nVals) */
	size_t uVals;     /* Number of values in this file */
	float8 dVals[48]; /* Calibrated Data */
	
	/* Stuf that is stored tempararily as an interface */
	float8 dColSclk;
	
}srv_row_sec_t;


/*****************************************************************************/
/* Row Section dictionaries */

#define S pktsec_rowsec_dict
#define C PktSecRowSecDict
#define K uint4
#define V srv_row_sec_t

#include <rpwgse/dictTplt.h>

#undef V
#undef K
#undef C
#undef S


/*****************************************************************************/
/* The calibrator */

typedef struct srv_calib{

	/* Get a measurement template given a packet id, chuck num and NRD setting */
	const srv_row_sec_t* (*getTplt)(const struct srv_calib* pThis, byte uPsid,
	                                byte uSec, byte uBand);
	
	/* Fill in a measurement from data in a packet */
	bool (*cal)(const struct srv_calib* pThis, const GsePkt* pPkt, uint2 uSec,
	            srv_row_sec_t* pOut);
	
	void (*del)(void* vpThis);
	
	/* The calibration table dictionary, the keys are:
	   LFR_LO_B,  LFR_LO_E,  LFR_LO_N,  LFR_HI_E, LFR_HI_N,
		HFR_LO_44, HFR_LO_45, HFR_HI_44, HFR_HI_45 */
	
	StrSrvCalTblDict* pTbls;
	
	/* Section templates, note the key is (PSID << 8 | Section) */
	PktSecRowSecDict* pTplts;
	
	
} SrvCalib;


/*****************************************************************************/
SrvCalib* new_SrvCalib(const char* sGroup);

#ifdef __cplusplus
}
#endif

#endif /* _SrvCalib_H_ */
