/** @file SvrRowWriter.h */
#ifndef _SvrRowWriter_H_
#define _SvrRowWriter_H_

#include <rpwgse/GseTypes.h>
#include <rpwgse/dict.h>

#include "SrvRower.h"
#include "JnoPhaser.h"

#ifdef __cplusplus
extern "C" {
#endif


/*****************************************************************************/

/** Handles all aspects of getting calibrated survey mode data rows into files */
typedef struct srv_row_writer{
	
	/** Write a row to an output file */
	void (*writeRow)(struct srv_row_writer* pThis, const srv_row_t* pRow);
	
	/** Answers the question, how many non-empty row sections have been written */
	int4 (*secsWritten)(const struct srv_row_writer* pThis);
	
	/** Answers the question, how many rows have been written */
	int4 (*rowsWritten)(const struct srv_row_writer* pThis);
	
	/** Do file rotation cleanup if needed on the current output file */
	void (*finish)(struct srv_row_writer* pThis);
	
	/** Destructor */
	void (*del)(void* vpThis);
	
	/* Intrinsic settings */
	const char* sGroup; 
	char* sCat;  /* "E", "B", "EN", "BN", "EALT", "BALT" */
	int4 nMaxSec; /* 1 = B,BN,BALT  2 = EN,EALT  4 = E */
	const JnoPhaser* pPhaser;
	
	/* Current working file */
	float8 dFileBeg;
	float8 dFileEnd;
	char* sPath;
	char* sFile;
	FILE* fOut;
	
	/* Statistics */
	int4 nRowsOut;
	int4 nSectionsOut;
	
}SrvRowWriter;

/** Constructor for survey data row writers 
 * @public
 * @memberof SrvRowWriter
 */
SrvRowWriter* new_SrvRowWriter(const JnoPhaser* pPhaser, const char* sGroup);

/*****************************************************************************/
/* Define a dictionary of SrvRowWriter objects */

#define S str_srvwriter_dict
#define C StrSrvWriterDict
#define K char
#define V SrvRowWriter

#include <rpwgse/ptrDictTplt.h>

#undef V
#undef K
#undef C
#undef S


#ifdef __cplusplus
}
#endif

#endif /* SvrRowWriter */
