/** @file caldir_util.h */

#ifndef _caldir_util_H_
#define _caldir_util_H_

#include <rpwgse/GseString.h>

#ifdef __cplusplus
extern "C" {
#endif

/** Given a set of cal file patterns return updated file paths
 *
 *  This function checks each given calibration file prefix against the 
 *  files listed on the L3 volume and against the config directory.  Files
 *  with a lower version number are updated.  
 *
 * @param sGroup the data group containing the volume.
 *
 * @param pPrefix a set of constant strings indicating the prefecies of 
 *        interest.  Include the name up to the digits in the version number.
 *        For example specifing: WAV_CAL_SRV_HFR_HI_45_V you would get the
 *        the full path to: WAV_CAL_SRV_HFR_HI_45_V01.CSV.
 *
 * @returns an array of string pointers allocated on the heap, pointing to
 *       strings that were allocated on the heap.
 */
GseStrAry* init_calib_dir(const char* sGroup, GseCStrAry* pPrefix);

/*****************************************************************************/
/* Get a version name from a file that ends in nn.CSV */
int4 get_calfile_version(const char* sPath);

/*****************************************************************************/
/* A single attenuation cal row holder */

typedef struct attn_cal{
	/* const char* sSection; */ /* Corresponds to values in srv_row::sSecNames */
	int4 nPaVal;
	int4 nRecVal;
	float4 dDb;
	float4 dFactor;
}attn_cal_t;




#ifdef __cplusplus
}
#endif

#endif /* _caldir_util_H_ */

